/*************************************************************************/ /*!
@File
@Title          fp_switch_host_learn.c
@Copyright      Copyright (c) Milandr. All Rights Reserved
@Description    implemets switch learning API's
@License        Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

/**********************************************************************
 *                                      Include Files
 **********************************************************************/
 #include <fp_library.h>
 #include <fp_switch.h>
 #include <fp_pre_header.h>
 #include <hw_2field_hash_table.h>
 #include <hw_vlan_hash_table.h>
 #include <fp_os_shim.h>

/**********************************************************************
 *                                     Extern Files
 **********************************************************************/
#if 0
/******************************************************************
 * Function Name  : getFwdPortList
 * Description    : serach for DMAC entry in mac hash table
 *                  if not found use vlan table to get the fwd port list
 * Inputs         :
 * Parameters     : pktBuf_t *
 * Outputs        :
 * Parameters     : -
 * Returns        : int
 ********************************************************************/
int getFwdPortList(pktHdrs_t *pktHdr, UINT32 vlanid)
{
	UCHAR *mac, *wsp_base_addr;
	UINT32 action_entry = -1, fwdPortList = 0;
	UINT32 field_entries[5], field_valids = BIT_SET_MAC_2F_MACTBL;
	UINT32 fwd_action = -1;
	struct ether_header *ethHdr = (struct ether_header *)pktHdr->data;

	wsp_base_addr = (UCHAR *)getPciBaseAddr(fp_net_dev);
	if (wsp_base_addr == NULL) {
		FP_DEBUG(DBGP_FEAT_BRIDGE, FP_LOG_EMERG,
			"%s: pci_mem base is NULL\n", __func__);
		return 0;
	}

	mac = ethHdr->ether_shost;

	fp_init_field_entries(mac, vlanid, &field_entries[0]);

	/* update vlan field valid for mac table */
	field_valids |= BIT_SET_VLAN_2F_MACTBL;

	/* search for dmac entry*/
	action_entry = hw_hash_table_search(wsp_base_addr, field_entries,
						field_valids);

	if (-1 != action_entry) {
		/* dmac entry found */
		/* Get fwd action from bridge table entry */
		fwd_action = (action_entry & BRENTRY_UCAST_HIT_ACT_MASK) >>
		BRENTRY_UCAST_HIT_ACT_START_POS;
		fwdPortList = fwd_action << 16;
		fwdPortList |= (action_entry & FWD_PORT_LIST_MASK);
	} else {
		/* dmac entry not found */
		if (0 != vlanid) {
			action_entry = -1;

			action_entry = wsp_vlan_entry_search(wsp_base_addr,
						vlanid);
			if (-1 == action_entry) {
				/* brentry not found */
				/* Un handeled bridge entry,
				take fwd action of fp_global br entry */
				fwd_action = fp_global.fallback_bd_entry.
							ucast_miss_action;

				fwdPortList = fwd_action << 16;
				/* get the forward port list from
							fp_global br entry */
				fwdPortList |= (fp_global.fallback_bd_entry.
					forward_list &
					BRENTRY_FWD_PORT_LIST_MASK);
			} else {
				/* brentry found */
				/* Get fwd action from bridge table entry */
				fwd_action = (action_entry &
					BRENTRY_UCAST_MISS_ACT_MASK) >>
					BRENTRY_UCAST_MISS_ACT_START_POS;

				fwdPortList = fwd_action << 16;
				/* get the forward port list from
							vlan table entry */
				fwdPortList |= (action_entry &
						BRENTRY_FWD_PORT_LIST_MASK);
			}
		}
	}

	return fwdPortList;
}
#endif

static int32_t
fp_switch_host_learn_entry( uint8_t *mac, uint32_t vlanid, uint32_t phy_port, uint32_t punt_reason,
				           uint32_t act_discard, uint32_t static_entry, uint32_t pvlan_prim )
{
	int64_t ret = -1;
	uint32_t fwd_action = 0, action_entry = 0;

	/* check for for already entry existence */
	ret = hw_hash_table_search( mac, vlanid );
	if( ret == -1 ){
		FP_DEBUG( DBGP_FEAT_BRIDGE, FP_LOG_CRIT, "%s: Search Fail\n", __func__ );
		/* fill action_entry's fresh bit */
		action_entry = FRESH_FLAG_SET;
		if( static_entry ){
			action_entry |= STATIC_FLAG_SET;
		}
		if( pvlan_prim ){
			action_entry |= PVLAN_FLAG_SET;
		}
		/* Add SMAC entry with action entry & valid fields */
		/* update action entry with port info on which it arrived */
		action_entry |= ( ( 1 << phy_port ) & FWD_PORT_LIST_MASK );

		/* Get bridge action entry */
		ret = wsp_vlan_entry_search( vlanid );

		if( ret == -1 ){
			/* Un handeled bridge entry, take fwd action of fp_global br entry */
			fwd_action = fp_global.fallback_bd_entry.ucast_hit_action;
		}else{
			/* Get bridge action entry's unicast hit fwd action */
			fwd_action = ( ret & BRENTRY_UCAST_HIT_ACT_MASK ) >> BRENTRY_UCAST_HIT_ACT_START_POS;
		}
		/* Update mac action entry's fwd action */
		action_entry = ( action_entry & ~MAC_2F_ENTRY_FWD_ACT_MASK ) | ( fwd_action << MAC_2F_ENTRY_FWD_ACT_START_POS );

		if( act_discard ){
			action_entry = ( action_entry & ~MAC_2F_ENTRY_FWD_ACT_MASK ) | ( ACT_DISCARD << MAC_2F_ENTRY_FWD_ACT_START_POS );
		}

		/* add mac entry */
		ret = hw_hash_table_add( mac, vlanid, action_entry );
		if( ret == -1 ){
			FP_DEBUG( DBGP_FEAT_BRIDGE, FP_LOG_CRIT, "Learn Fail\n" );
			/* if learn fails just fwd the pkt according to
				egress port list return -1; */
		}
	}else{
		action_entry = ret;

//		/*----- Check and update ACTION FIELD in MAC table, if VALN table unicast hit action is change -----*/
//		/* Get bridge action entry */
//		ret = wsp_vlan_entry_search( vlanid );
//		if( ret == -1 ){
//			/* Un handeled bridge entry, take fwd action of fp_global br entry */
//			fwd_action = fp_global.fallback_bd_entry.ucast_hit_action;
//		}else{
//			/* Get bridge action entry's unicast hit fwd action */
//			fwd_action = ( uint32_t )( ( ret & BRENTRY_UCAST_HIT_ACT_MASK ) >> BRENTRY_UCAST_HIT_ACT_START_POS );
//		}
//
//		if( ( ( action_entry & MAC_2F_ENTRY_FWD_ACT_MASK ) >> MAC_2F_ENTRY_FWD_ACT_START_POS ) != fwd_action ){
//			action_entry = ( action_entry & ~MAC_2F_ENTRY_FWD_ACT_MASK )|( fwd_action << MAC_2F_ENTRY_FWD_ACT_START_POS );
//		}
//		/*----- Check and update ACTION FIELD done -----*/

		/* mac hash entry found */
		switch( punt_reason ){
		case FP_PUNT_SA_IS_ACTIVE:
			if( action_entry & FRESH_FLAG_MASK ){
				action_entry = fp_update_mac_refresh_bit( action_entry );
			}else{
				return 0;
			}
		    break;
		case FP_PUNT_SA_RELEARN:
			/* validate relearn:
			 * check for ingress port match
			 * with action entry port num */
			if( !( ( 1 << phy_port ) & action_entry ) ){
				/* reset old phyno and
					update new phyno into action entry */
				action_entry =	( action_entry & ( ~FWD_PORT_LIST_MASK ) ) |
								( ( 0x1 << phy_port ) & FWD_PORT_LIST_MASK );
			}
		    break;
		case FP_PUNT_SA_MISS:
			return 0;
		default:
		    break;
		} /* End of switch - (pktHdr->punt_reason) */

		/* update new phyno into mac hash table */
		ret = hw_hash_table_update( mac, vlanid, action_entry );

		FP_DEBUG( DBGP_FEAT_BRIDGE, FP_LOG_CRIT, "%s: Update called\n\n", __func__ );
	}

	fp_rc_stats.fwd_cnt++;
	return 0;
}
/******************************************************************************
 * Function Name  : fp_switch_host_learn
 * Description    : lookup on Dest Mac, search fails learn dest mac
			into Hash Mac Table
 * Inputs         :
 * Parameters     : pktBuf_t *
 * Outputs        :
 * Parameters     : -
 * Returns        : int
 *****************************************************************************/
int32_t
fp_switch_host_learn(pktHdrs_t *pktHdr)
{
	uint32_t vlanid = 0, phy_port = pktHdr->phyno, idx = 0;
	uint32_t punt_reason = pktHdr->punt_reason;
	struct ether_header *ethHdr = (struct ether_header *)pktHdr->data;
	uint8_t *mac = ethHdr->ether_shost;
	struct port_s *port = &fp_port[phy_port];
	uint32_t num_sec = port->num_sec;
	/* Assign vlanid of pkt if it is present, else assign port's fallback bdid */

	if( ( pktHdr->vlanid == fp_port[ pktHdr->phyno ].fallback_bd_id ) || ( pktHdr->vlanid == 0 ) ){
		vlanid = fp_port[ pktHdr->phyno ].fallback_bd_id;
	}else{
		vlanid = pktHdr->vlanid;
	}

	FP_DEBUG( DBGP_FEAT_BRIDGE, FP_LOG_INFO, "%s: vlanid %d\n", __func__, vlanid );
	if( !port->pvlan_enable ){
		fp_switch_host_learn_entry( mac, vlanid, phy_port, punt_reason, 0, 0, 0 );
	}else{  /* port->pvlan_enable is set */
		FP_DEBUG( DBGP_FEAT_BRIDGE, FP_LOG_INFO, "%s: phy_port %d, port->pvlan_pri %d, port->pvlan_sec[0] %d \n", __func__, phy_port, port->pvlan_pri, port->pvlan_sec[ 0 ] );
		fp_switch_host_learn_entry( mac, port->pvlan_pri, phy_port, punt_reason, 0, 0, 1 );
		while (num_sec != 0) {

			vlanid = port->pvlan_sec[idx];
			idx++;
			num_sec--;

			if (vlanid == 0){
				continue;
			}
			if (port->pvlan_type == ISOLATED){
				fp_switch_host_learn_entry(mac, vlanid, phy_port, punt_reason, 1, 1, 0);
			}
			else{
				fp_switch_host_learn_entry(mac, vlanid, phy_port, punt_reason, 0, 1, 0);
			}
		}
	}

	return 0;
}

/*******************************************************************************
 * Function Name  : fp_switch_host_mac_refresh
 * Description    : update mac refresh bit
 * Inputs         :
 * Parameters     : UINT
 * Outputs        :
 * Parameters     : -
 * Returns        : int
 ******************************************************************************/
uint32_t
fp_update_mac_refresh_bit(uint32_t action_entry)
{
	action_entry = (action_entry & ~FRESH_FLAG_MASK);
	return action_entry;
}
