#define _GNU_SOURCE
#include <stdio.h>
#include <errno.h>
#include <stdarg.h>
#include <stdlib.h>
#include <memory.h>
#include <malloc.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <regex.h>
#include <getopt.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/mman.h>
#include <dirent.h>

#include "cli.h"
#include "libcli.h"
#include <../l2cli/l2cli.h>

#define CONFIG_L2CLI                        1
#define CONFIG_L3CLI                        1

#define VENDOR_ID                           0x16c3
#define DEVICE_ID                           0x0bad

#ifdef CONFIG_L2CLI
extern void init_l2cli(struct cli_def *cli);
#endif

#define MIL_BANNER \
"\n"\
"#############################################\n"\
"#                                           #\n"\
"#      Welcome to Milandr Smart console     #\n"\
"#                                           #\n"\
"#############################################\n"\
"\n"
#define MIL_PROMPT "MDR"

#define MODE_CONFIG_INT                     10
#define MODE_CONFIG_STRING                  11

struct cli_def *cli;
nakCli_t nkCli;

void
pc (UNUSED (struct cli_def *cli), char *string)
{
    printf ("%s\n", string);
}

int
cmd_config_int_exit (struct cli_def *cli, UNUSED (char *command),
                     UNUSED (char *argv[]), UNUSED (int argc))
{
    cli_set_configmode (cli, MODE_CONFIG, NULL);
    return CLI_OK;
}

int
init_nkCLI(struct cli_def *cli)
{
    memset(&nkCli, 0, sizeof(nkCli));
    nkCli.nk_show = cli_register_command(cli, NULL, "show", NULL,
        PRIVILEGE_UNPRIVILEGED, MODE_EXEC, "Show various configuration items.");

    return (0);
}

char **add_string(char **array, const char *string)
{
    int i = 0;
    while (array[i] != NULL)
        i++;
    
    array = realloc(array, (i + 2) * sizeof(char *));
    array[i] = malloc((strlen(string) + 1));
    array[i+1] = NULL;
    strcpy(array[i], string);
    return array;
}

int
main (int argc, char *argv[])
{
    int s, x, i, dirs, dirsM, id, found_nums = 0;
    struct sockaddr_in servaddr;
    struct dirent **dir, **dirNet;
    int on = 1;
    int port = 0;
    char filename[64] = {0};
    char **devices_found = malloc(sizeof(char *));
    FILE *fd;

    const char *pciFolder = "/sys/bus/pci/devices/";
    const char* short_options = "hp:n:";

    const struct option long_options[] =
    {
        {"help",no_argument,NULL,'h'},
        {"port",required_argument,NULL,'p'},
        {"netdevice",required_argument,NULL,'n'},
        {NULL,0,NULL,0}
    };

    int rez;
	int option_index;

    devices_found[0] = NULL;

	while ((rez=getopt_long(argc,argv,short_options,
		long_options,&option_index))!=-1)
    {
		switch(rez)
        {
			case 'h':
				printf("-p --port <- port number, -n --netdevice <- commutator device, -h --help <- help,\n");
                return 0;
				break;
			case 'p':
                sscanf(optarg, "%d", &port);
				break;
			case 'n':
                strcpy(device_name, optarg);
				break;
		};
	};

    if(!port)
    {
        printf("Error! Set port number with -p or --port\n");
        return 0;
    }

    printf("Search device with VENDOR_ID=%x DEVICE_ID=%x\n", VENDOR_ID, DEVICE_ID);
    dirsM = scandir(pciFolder, &dir, 0, 0);
    for(i=2; i<dirsM; i++)
    {
        memset(filename, 0, sizeof(filename));
        strcat(filename, pciFolder);
        strcat(filename, dir[i]->d_name);
        strcat(filename, "/vendor");
        fd = fopen(filename, "r");
        fseek(fd, 2, SEEK_SET);
        fscanf(fd, "%x", &id);
        fclose(fd);

        if(id == VENDOR_ID)
        {
            memset(filename, 0, sizeof(filename));
            strcat(filename, pciFolder);
            strcat(filename, dir[i]->d_name);
            strcat(filename, "/device");
            fd = fopen(filename, "r");
            fseek(fd, 2, SEEK_SET);
            fscanf(fd, "%x", &id);
            fclose(fd);

            if(id == DEVICE_ID)
            {
                memset(filename, 0, sizeof(filename));
                strcat(filename, pciFolder);
                strcat(filename, dir[i]->d_name);
                strcat(filename, "/net/");
                dirs = scandir(filename, &dirNet, 0, 0);
                if(dirs>0)
                {
                    add_string(devices_found, dirNet[2]->d_name);
                    printf("Netdevice %d name is %s\n", found_nums+1, devices_found[found_nums]);
                    found_nums++;
                    free(dirNet);
                }
            }
        }
    }
    free(dir);

    if(!device_name[0] && found_nums)
        strcpy(device_name, devices_found[0]);
    else
    {
        for(i=0; i<found_nums; i++)
        {
            if(!strcmp(device_name, devices_found[i]))
                break;
        }
        if(i == found_nums)
            memset(device_name, 0, 16);
    }

    for(i=0; i<found_nums; i++)
        free(devices_found[i]);

    if(!device_name[0])
    {
        printf("No correct device name set\n");
        return 0;
    }

    cli  = cli_init();

    cli_set_banner (cli, MIL_BANNER);
    cli_set_hostname (cli, MIL_PROMPT);

    /* common L2 & L3 cli cmds registration */
    init_nkCLI(cli);

#ifdef CONFIG_L2CLI
    init_l2cli(cli);
#endif

#ifdef CONFIG_L3CLI
    /* init all l2cli cmds here */
#endif

    cli_register_command (cli, NULL, "exit", cmd_config_int_exit,
        PRIVILEGE_PRIVILEGED, MODE_CONFIG_INT, "Exit from cli.");

    {
        FILE *fh;

        if ((fh = fopen ("clitest.txt", "r")))
        {
            /* This sets a callback which just displays the cli_print()
             * text to stdout
             */
            cli_print_callback (cli, pc);
            cli_file (cli, fh, PRIVILEGE_UNPRIVILEGED, MODE_EXEC);
            cli_print_callback (cli, NULL);
            fclose (fh);
            /*
             * TODO quit in clitest.txt is not working
             * Add command line arguments
             */
            cli_done(cli);
            return (0);
        }
    }
    if ((s = socket (AF_INET, SOCK_STREAM, 0)) < 0)
    {
        perror ("socket");
        return 1;
    }
    setsockopt (s, SOL_SOCKET, SO_REUSEADDR, &on, sizeof (on));

    memset (&servaddr, 0, sizeof (servaddr));
    servaddr.sin_family = AF_INET;
    servaddr.sin_addr.s_addr = htonl (INADDR_ANY);
    servaddr.sin_port = htons (port);
    if (bind (s, (struct sockaddr *) &servaddr, sizeof (servaddr)) < 0)
    {
        perror ("bind");
        return 1;
    }

    if (listen (s, 50) < 0)                                                     // Only one connection accepted at the moment by cli
    {
        perror ("listen");
        return 1;
    }
    printf ("Listening on port %d\n", port);
    while ((x = accept (s, NULL, NULL)))
    {
        printf("Accept connection on new port %d\n", x);
        cli_loop (cli, x);                                                      // because loop, and one thread
        close (x);
    }
    cli_done (cli);
    return 0;
}
