/*************************************************************************/ /*!
@File
@Title          fp_bd_api.h
@Copyright      Copyright (c) Milandr. All Rights Reserved
@Description    defines for buffer descriports
@License        Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

#ifndef _FP_BD_API_H
#define _FP_BD_API_H

/* buffer descr ctrl flags */
#define BD_CTRL_BD_BUFLEN_MASK   (0xffff)
#define BD_CTRL_PKT_INT_EN       (1 << 0)
#define BD_CTRL_CBD_INT_EN       (1 << 1)
#define BD_CTRL_LIFM             (1 << 2)
#define BD_CTRL_LAST_BD          (1 << 3)
#define BD_CTRL_DIR              (1 << 4)
#define BD_CTRL_PKT_XFER         (1 << 8)
#define BD_CTRL_DESC_EN          (1 << 15)
#define BD_CTRL_PARSE_DISABLE    (1 << 9)
#define BD_CTRL_BRFETCH_DISABLE  (1 << 10)
#define BD_CTRL_RTFETCH_DISABLE  (1 << 11)

#define WB_BD_CTRL_DESC_EN       (1 << 31)

#define NUM_TX_DESCR             64
#define NUM_RX_DESCR             64
#define NUM_TX_WB_DESCR          128
#define NUM_RX_WB_DESCR          128
#define MAX_BUFF_SIZE            MAX_FRAME_SIZE
#define TX_BD_INIT_RING          1
#define RX_BD_INIT_RING          2
#define ALIGN_64_BIT_MASK        64
#define NUM_TOTAL_DESCR          (NUM_TX_DESCR + NUM_RX_DESCR + NUM_WB_DESCR)

struct bd {
    USHORT bd_seqnum;
    USHORT bd_ctrl;
    USHORT bd_buflen;
    USHORT rsvd;
    UINT bd_bufaddr;
    UINT bd_nextptr;
};

struct wb_bd {
    UINT   bd_ctrl; /* rsvd-[31:11] ctrl[10:0] */
    USHORT bd_buflen; /* seq-[31:16] buflen-[15:0] */
    USHORT bd_seqnum;
};

struct bd_ring {
    spinlock_t lock;
    struct bd    *bd_tbl_va;                /* 64-bit aligned */
    struct wb_bd *wb_bd_tbl_va;             /* 64 bit aligned */
    struct wb_bd *wb_read_ptr;
    dma_addr_t   bd_tbl_pa;
    dma_addr_t   wb_bd_tbl_pa;
    dma_addr_t   bd_buff_pa;
    dma_addr_t   dev_addr;
    void         *cpu_addr;
    void         *bd_buff_va;
    struct bd    *head;
    struct bd    *tail;
    INT          index;
    INT          free_bd_cnt;
    INT          wb_seq_num;
    INT          seq_num;
    INT          ring_len;
    INT          size;
    INT          wb_ring_len;
    INT          flag;
};

extern UINT fp_total_ring_size;


struct bd *fp_get_next_free_bd(struct bd_ring *ring);
int fp_get_count_of_free_bd( struct bd_ring *ring );
int fp_check_bd_status(struct bd_ring *ring);
UINT fp_get_curr_bd_index(struct bd_ring *ring, struct bd *bd);
int fp_enque_bd(struct bd_ring *ring, struct bd *);
struct bd *fp_deque_bd(struct bd_ring *ring, UINT *bd_buflen);
void fp_dump_bd_info(struct bd *dump_bd);
int fp_bd_init(fp_private_t *fp);
unsigned int handle_hif(hif_base_t *hif);
void handle_channel(hif_base_t *hif, UINT ch_index);
void fp_disable_hif_ch_interrupts(hif_base_t *hif);
void fp_enable_hif_ch_interrupts(hif_base_t *hif);

#endif /* _FP_BD_TABLE_H */
