/*************************************************************************/ /*
@File
@Title          fp_drv_mcu_tasks.h
@Copyright      Copyright (c) Milandr. All Rights Reserved
@Description    Structures for hw switch
@License        Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

/*!
 *  @file fp_drv_mcu_tasks.h
  */

#ifndef  _FP_DRV_MCU_TASKS_H_
#define  _FP_DRV_MCU_TASKS_H_

#include "fp_macros.h"
#include "fp_drv_mcu.h"

#define DRV_MCU_GET_SFP_EVENTS_EVENT_PORT1_OFFSET	0
#define DRV_MCU_GET_SFP_EVENTS_EVENT_PORT2_OFFSET	1
#define DRV_MCU_GET_SFP_EVENTS_EVENT_PORT3_OFFSET	2
#define DRV_MCU_GET_SFP_EVENTS_EVENT_PORT4_OFFSET	3

#define DRV_MCU_GET_SFP_EVENTS_EVENTS_MASK	0xF
#define DRV_MCU_GET_SFP_EVENTS_EVENT_PORT1	( 1 << DRV_MCU_GET_SFP_EVENTS_EVENT_PORT1_OFFSET )
#define DRV_MCU_GET_SFP_EVENTS_EVENT_PORT2	( 1 << DRV_MCU_GET_SFP_EVENTS_EVENT_PORT2_OFFSET )
#define DRV_MCU_GET_SFP_EVENTS_EVENT_PORT3	( 1 << DRV_MCU_GET_SFP_EVENTS_EVENT_PORT3_OFFSET )
#define DRV_MCU_GET_SFP_EVENTS_EVENT_PORT4	( 1 << DRV_MCU_GET_SFP_EVENTS_EVENT_PORT4_OFFSET )

#define DRV_MCU_GET_SFP_EVENTS_EVENT_PORT1_CONNECT_OFFSET	16
#define DRV_MCU_GET_SFP_EVENTS_EVENT_PORT2_CONNECT_OFFSET	17
#define DRV_MCU_GET_SFP_EVENTS_EVENT_PORT3_CONNECT_OFFSET	18
#define DRV_MCU_GET_SFP_EVENTS_EVENT_PORT4_CONNECT_OFFSET	19

#define DRV_MCU_GET_SFP_EVENTS_EVENT_PORT1_CONNECT	( 1 << DRV_MCU_GET_SFP_EVENTS_EVENT_PORT1_CONNECT_OFFSET )
#define DRV_MCU_GET_SFP_EVENTS_EVENT_PORT2_CONNECT	( 1 << DRV_MCU_GET_SFP_EVENTS_EVENT_PORT2_CONNECT_OFFSET )
#define DRV_MCU_GET_SFP_EVENTS_EVENT_PORT3_CONNECT	( 1 << DRV_MCU_GET_SFP_EVENTS_EVENT_PORT3_CONNECT_OFFSET )
#define DRV_MCU_GET_SFP_EVENTS_EVENT_PORT4_CONNECT	( 1 << DRV_MCU_GET_SFP_EVENTS_EVENT_PORT4_CONNECT_OFFSET )

#define DRV_TASK_RESEULT_PROCESSING_OFFSET			0
#define DRV_TASK_RESEULT_COMPLETE_SUCCESS_OFFSET	1

#define DRV_TASK_RESEULT_PROCESSING					( 1 << DRV_TASK_RESEULT_PROCESSING_OFFSET )
#define DRV_TASK_RESEULT_COMPLETE_SUCCESS			( 1 << DRV_TASK_RESEULT_COMPLETE_SUCCESS_OFFSET )
#define DRV_TASK_RESEULT_COMPLETE_ERROR				0

#define TASKS_QUEUE_SIZE	16

#define TEST_CMD_DATA1_REG_RESULT				0x09080706
#define TEST_CMD_DATA2_REG_RESULT				0x19181700

#define DRV_TASK_MAX_STEPS_LIMIT				40

#define OLD_MCU_FIRMWARE_VERSION				0xFFFFFFFF

enum drv_task_e {
    STATE_IDLE = 0,
    STATE_TASK_PROCESSING,
    STATE_WAIT_CLR_TASK,
};


struct drv_mcu_task_s {
	uint32_t task;
	uint32_t task_cmd;
	uint32_t result;
	enum drv_task_e state;
	uint32_t  *data_ptr;
};

struct drv_mcu_all_tasks_s {
	uint32_t active_task_index;
	uint32_t steps_counter;
	uint32_t task_array[ TASKS_QUEUE_SIZE ];
	uint32_t *tasks_memory;
	struct drv_mcu_task_s task[ MAX_DRV_MCU_CMD ];
};

int32_t init_drv_mcu_all_tasks_struct( uint32_t init );
uint32_t fp_drv_mcu_tasks_init_timer( uint32_t init );
int32_t create_task( enum drv_mcu_transfer_cmd_e cmd, uint32_t param, uint32_t *data );

#endif /*End of _FP_DRV_MCU_TASKS_H_ */
