/*************************************************************************/ /*!
*  @File
*  @Title          fp_mac_ops.c
*  @Copyright      Copyright (c) Milandr. All Rights Reserved
*  @Description    Initializes EAC paramaters
*  @License        Dual MIT/GPLv2
*
*  The contents of this file are subject to the MIT license as set out below.
*
*  Permission is hereby granted, free of charge, to any person obtaining a copy
*  of this software and associated documentation files (the "Software"), to deal
*  in the Software without restriction, including without limitation the rights
*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
*  copies of the Software, and to permit persons to whom the Software is
*  furnished to do so, subject to the following conditions:
*
*  The above copyright notice and this permission notice shall be included in
*  all copies or substantial portions of the Software.
*
*  Alternatively, the contents of this file may be used under the terms of
*  the GNU General Public License Version 2 ("GPL") in which case the provisions
*  of GPL are applicable instead of those above.
*
*  If you wish to allow use of your version of this file only under the terms of
*  GPL, and not to allow others to use your version of this file under the terms
*  of the MIT license, indicate your decision by deleting the provisions above
*  and replace them with the notice and other provisions required by GPL as set
*  out in the file called "GPL-COPYING" included in this distribution. If you do
*  not delete the provisions above, a recipient may use your version of this file
*  under the terms of either the MIT license or GPL.
*
*  This License is also included in this distribution in the file called
*  "MIT-COPYING".
*
*  EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
*  PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
*  BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
*  PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
*  COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
*  IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
*  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

/**********************************************************************
*                                      Include Files
**********************************************************************/
#include <linux/string.h>
#include <linux/netdevice.h>
#include "fp_flow_control.h"
//#include "fp_netd.h"

extern struct net_device  *fp_logical_dev[ FP_MAX_PORTS ];

uint32_t flow_control_disable( uint32_t port_number ){

	uint32_t i;

	fp_reg_write( ( egpi[ port_number ] + EGPI_FW_CONTROL ), 0 );
    if( !get_mac_register( emac[ port_number ], EMAC_NETWORK_CONFIG, &i ) ){
        i = ( i & ~( (1 << 13 )|( 1 << 23 ) ) );
        set_mac_register( emac[ port_number ], EMAC_NETWORK_CONFIG, i );
        fp_reg_write( ( egpi[ port_number ] + EGPI_BMU1_PHY_LOW_WATERMARK ), 0 );
        fp_reg_write( ( egpi[ port_number ] + EGPI_BMU2_PHY_LOW_WATERMARK ), 0 );
        fp_reg_write( ( egpi[ port_number ] + EGPI_BMU1_PHY_HIGH_WATERMARK ), 0 );
        fp_reg_write( ( egpi[ port_number ] + EGPI_BMU2_PHY_HIGH_WATERMARK ), 0 );
        set_mac_register( emac[ port_number ], EMAC_SPEC_ADD1_BOTTOM, 0 );
		set_mac_register( emac[ port_number ], EMAC_SPEC_ADD1_TOP, 0 );
    }else{
        return -1;
    }
	return 0;
}

uint32_t flow_control_enable( uint32_t port_number, struct flow_control_s *fc ){

	uint32_t i;
	uint32_t mac_bottom, mac_top;

	fc->preferred_mode &= 3;

	memcpy( &mac_bottom, &fp_logical_dev[ port_number ]->dev_addr[ 0 ], 4 );
	memcpy( &mac_top,    &fp_logical_dev[ port_number ]->dev_addr[ 4 ], 2 );
	mac_top |= ( 1 << 16 );	//source address!

	set_mac_register( emac[ port_number ], EMAC_TX_PAUSE_QUANTUM, 0xFFFF );
	set_mac_register( emac[ port_number ], EMAC_SPEC_ADD1_BOTTOM, mac_bottom );
    set_mac_register( emac[ port_number ], EMAC_SPEC_ADD1_TOP, mac_top );

	switch(fc->preferred_mode){

		case 0:			//PAUSE = 0, ASM_DIR = 0;
			return -1;	//bad parameters
			break;
		
		case 1:			//tx and rx
		case 2:			//tx only
			fp_reg_write( ( egpi[ port_number ] + EGPI_BMU1_PHY_LOW_WATERMARK ), fc->low_watermark );
    		fp_reg_write( ( egpi[ port_number ] + EGPI_BMU2_PHY_LOW_WATERMARK ), fc->low_watermark );
    		fp_reg_write( ( egpi[ port_number ] + EGPI_BMU1_PHY_HIGH_WATERMARK ), fc->high_watermark );
    		fp_reg_write( ( egpi[ port_number ] + EGPI_BMU2_PHY_HIGH_WATERMARK ), fc->high_watermark );
			fp_reg_write( ( egpi[ port_number ] + EGPI_FW_CONTROL ), 0xA03 );	//enable tx flow control
			if( fc->preferred_mode == 2 ){
				break;
			}
		case 3:		//rx only
			if( !get_mac_register( emac[ port_number ], EMAC_NETWORK_CONFIG, &i ) ){
				i |= ( 1 << 13 )|( 1 << 23 );
				set_mac_register( emac[ port_number ], EMAC_NETWORK_CONFIG, i );	//enable rx flow control
    		}else{
    		    return -2;	//can not set rx flow control
    		}
			break;

//		case 2:			//tx only
//			fp_reg_write( ( egpi[ port_number ] + EGPI_BMU1_PHY_LOW_WATERMARK ), fc->low_watermark );
//    		fp_reg_write( ( egpi[ port_number ] + EGPI_BMU2_PHY_LOW_WATERMARK ), fc->low_watermark );
//    		fp_reg_write( ( egpi[ port_number ] + EGPI_BMU1_PHY_HIGH_WATERMARK ), fc->high_watermark );
//    		fp_reg_write( ( egpi[ port_number ] + EGPI_BMU2_PHY_HIGH_WATERMARK ), fc->high_watermark );
//			fp_reg_write( ( egpi[ port_number ] + EGPI_FW_CONTROL ), 0xA03 );	//enable tx flow control
//			break;

//		case 3:
//			if( !get_mac_register( emac[ port_number ], EMAC_NETWORK_CONFIG, &i ) ){
//        		i |= ( 1 << 13 );
//        		set_mac_register( emac[ port_number ], EMAC_NETWORK_CONFIG, i );	//enable rx flow control
//        		return 0;
//    		}else{
//    		    return -2;	//can not set rx flow control
//    		}
//			break;

		default:
			FP_DEBUG( DBGP_FEAT_HIF, FP_LOG_ERR, "flow_control_enable: should never come here!\n" );
			return -3;
			break;
	}
	return 0;
}

