/*************************************************************************/ /*!
@File
@Title          fp_hif_reg.h
@Copyright      Copyright (c) Milandr. All Rights Reserved
@Description    contains cor data stryctures & defines of hif driver
@License        Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

#ifndef _FP_HIF_REG_H
#define _FP_HIF_REG_H

#include "fp_macros.h"
#ifdef FP_PCI_BM_EN

#define FP_ASSIGNED_HIF 0

#define HIF_CH_INT           (1 << 0)
#define HIF_CH_RXBD_INT      (1 << 1)
#define HIF_CH_RXPKT_INT     (1 << 2)
#define HIF_CH_TXBD_INT      (1 << 3)
#define HIF_CH_TXPKT_INT     (1 << 4)
#define HIF_CH_INT_EN        (1 << 0)
#define HIF_CH_RXBD_INT_EN   (1 << 1)
#define HIF_CH_RXPKT_INT_EN  (1 << 2)
#define HIF_CH_TXBD_INT_EN   (1 << 3)
#define HIF_CH_TXPKT_INT_EN  (1 << 4)

#define HIF_CH_TX_CTRL_DMA_EN     (1 << 0)
#define HIF_CH_TX_BDP_POLL_CNTR_EN (1 << 1)
#define HIF_CH_RX_CTRL_DMA_EN     (1 << 16)
#define HIF_CH_RX_BDP_POLL_CNTR_EN (1 << 17)

#define HIF_CH_TX_START      (1 << 0)
#define HIF_CH_RX_START      (1 << 0)

#define HIF_QUEUE_MAP_VALID_MASK   3
#define NPU_MAX_TX_QUEUES     4

typedef struct channel_reg
{
    volatile uint32 hif_ctrl_ch;
    volatile uint32 hif_rx_bdp_wr_low_addr_ch;
    volatile uint32 hif_rx_bdp_wr_high_addr_ch;
    volatile uint32 hif_rx_bdp_rd_low_addr_ch;
    volatile uint32 hif_rx_bdp_rd_high_addr_ch;
    volatile uint32 hif_tx_bdp_wr_low_addr_ch;
    volatile uint32 hif_tx_bdp_wr_high_addr_ch;
    volatile uint32 hif_tx_bdp_rd_low_addr_ch;
    volatile uint32 hif_tx_bdp_rd_high_addr_ch;
    volatile uint32 hif_rx_wrbk_bd_ch_buffer_size;
    volatile uint32 hif_rx_ch_start;
    volatile uint32 hif_tx_wrbk_bd_ch_buffer_size;
    volatile uint32 hif_tx_ch_start;
    volatile uint32 reserved1[11];
    volatile uint32 hif_ch_int_src;
    volatile uint32 hif_ch_int_en;
    volatile uint32 reserved2[6];
    volatile uint32 hif_tx_rd_curr_bd_low_addr_ch;
    volatile uint32 hif_tx_rd_curr_bd_high_addr_ch;
    volatile uint32 hif_tx_wr_curr_bd_low_addr_ch;
    volatile uint32 hif_tx_wr_curr_bd_high_addr_ch;
    volatile uint32 hif_bdp_ch_tx_fifo_cnt;
    volatile uint32 hif_tx_dma_status_0_ch;
    volatile uint32 hif_tx_status_0_ch;
    volatile uint32 hif_tx_status_1_ch;
    volatile uint32 hif_tx_pkt_cnt0_ch;
    volatile uint32 hif_tx_pkt_cnt1_ch;
    volatile uint32 hif_tx_pkt_cnt2_ch;
    volatile uint32 reserved3[5];
    volatile uint32 hif_rx_rd_curr_bd_low_addr_ch;
    volatile uint32 hif_rx_rd_curr_bd_high_addr_ch;
    volatile uint32 hif_rx_wr_curr_bd_low_addr_ch;
    volatile uint32 hif_rx_wr_curr_bd_high_addr_ch;
    volatile uint32 hif_bdp_ch_rx_fifo_cnt;
    volatile uint32 hif_rx_dma_status_0_ch;
    volatile uint32 hif_rx_status_0_ch;
    volatile uint32 hif_rx_pkt_cnt0_ch0;
    volatile uint32 hif_rx_pkt_cnt1_ch0;
    volatile uint32 reserved4[7];
} channel_reg_t;

struct hif_regs
{
    volatile uint32 hif_version;
    volatile uint32 hif_tx_poll_ctrl;
    volatile uint32 hif_rx_poll_ctrl;
    volatile uint32 hif_misc;
    volatile uint32 hif_timeout_reg;
    volatile uint32 hif_soft_reset;
    volatile uint32 hif_vf_1_0_int_en;
    volatile uint32 hif_vf_3_2_int_en;
    volatile uint32 hif_vf_5_4_int_en;
    volatile uint32 hif_vf_7_6_int_en;
    volatile uint32 hif_msg0;
    volatile uint32 hif_msg1;
    volatile uint32 hif_msg2;
    volatile uint32 hif_msg3;
    volatile uint32 hif_pf_int_en;
    volatile uint32 hif_vf_int_en;
    volatile uint32 hif_int_src;
    volatile uint32 hif_pf_int_coal_timer;
    volatile uint32 hif_vf0_int_coal_timer;
    volatile uint32 hif_vf1_int_coal_timer;
    volatile uint32 hif_vf2_int_coal_timer;
    volatile uint32 hif_vf3_int_coal_timer;
    volatile uint32 hif_vf4_int_coal_timer;
    volatile uint32 hif_vf5_int_coal_timer;
    volatile uint32 hif_vf6_int_coal_timer;
    volatile uint32 hif_vf7_int_coal_timer;
    volatile uint32 hif_err_int_src;
    volatile uint32 hif_err_int_en;
    volatile uint32 hif_tx_fifo_err_int_src;
    volatile uint32 hif_tx_fifo_err_int_en;
    volatile uint32 hif_rx_fifo_err_int_src;
    volatile uint32 hif_rx_fifo_err_int_en;
    volatile uint32 hif_tx_state;
    volatile uint32 hif_tx_actv;
    volatile uint32 hif_tx_curr_ch_no;
    volatile uint32 hif_dxr_tx_fifo_cnt;
    volatile uint32 hif_tx_ctrl_word_fifo_cnt1;
    volatile uint32 hif_tx_ctrl_word_fifo_cnt2;
    volatile uint32 hif_tx_bvalid_fifo_cnt;
    volatile uint32 hif_tx_pkt_cnt1;
    volatile uint32 hif_tx_pkt_cnt2;
    volatile uint32 hif_rx_state;
    volatile uint32 hif_rx_actv;
    volatile uint32 hif_rx_curr_ch_no;
    volatile uint32 hif_dxr_rx_fifo_cnt;
    volatile uint32 hif_rx_ctrl_word_fifo_cnt;
    volatile uint32 hif_rx_bvalid_fifo_cnt;
    volatile uint32 hif_rx_pkt_cnt1;
    volatile uint32 hif_rx_pkt_cnt2;
    volatile uint32 hif_dma_base_addr;
    volatile uint32 hif_dma_burst_size_addr;
    volatile uint32 hif_rx_queue_map_ch_no_addr;
    volatile uint32 reserved[12];
    volatile channel_reg_t ch[NUM_HIF_CHANNELS];
};

/**********************************************************************
 * Function Name  : fp_assign_hif_base_addr
 * Description    : assign hif base address to global fp_private structure
 * Inputs
 *   Parameters   : struct fp_private *
 * Outputs        :
 *   Parameters   : -
 *   Returns      : -
 * Changes        :
 *********************************************************************/
static inline void fp_assign_hif_base_addr(fp_private_t *fp)
{
    UINT addr;

    addr = HIF1_BASE_ADDR;

    fp->hif.regs = (struct hif_regs *)((unsigned long)(addr));
}

/**********************************************************************
 * Function Name  : fp_en_hif_rx_engine
 * Description    : enable hif rx dma engine
 * Inputs
 *   Parameters   : struct fp_private *
 * Outputs        :
 *   Parameters   : -
 *   Returns      : -
 * Changes        :
 *********************************************************************/
static inline void fp_en_hif_rxdma_engine(fp_private_t *fp)
{
    UINT rx_ctrl = 0;
    UINT ch_index = 0;

    /* CLEAR - Poll Counter */
    //fp_reg_write((ULONG)&fp->hif.regs->hif_tx_poll_ctrl, 0x04000400);
    //fp_reg_write((ULONG)&fp->hif.regs->hif_rx_poll_ctrl, 0x04000400);

    fp_reg_write( ( ULONG )&fp->hif.regs->hif_tx_poll_ctrl, 0x00400040 );
    fp_reg_write( ( ULONG )&fp->hif.regs->hif_rx_poll_ctrl, 0x00400040 );

    for(ch_index = 0; ch_index < NUM_HIF_CHANNELS; ch_index++)
    {
        /* Enable the RX DMA Engine & BDP Poll Cntrl */
        rx_ctrl = HIF_CH_RX_CTRL_DMA_EN | HIF_CH_RX_BDP_POLL_CNTR_EN;
        fp_reg_write((ULONG)&fp->hif.regs->ch[ch_index].hif_ctrl_ch, rx_ctrl);

        /* Enable rx write strobe */
        rx_ctrl = HIF_CH_RX_START;
        fp_reg_write((ULONG)&fp->hif.regs->ch[ch_index].hif_rx_ch_start, rx_ctrl);
    }
}

static inline void fp_disable_hif_dma_engine(fp_private_t *fp)
{
    UINT ch_index = 0;

    /* CLEAR - Poll Counter */
    fp_reg_write( ( ULONG )&fp->hif.regs->hif_tx_poll_ctrl, 0x0 );
    fp_reg_write( ( ULONG )&fp->hif.regs->hif_rx_poll_ctrl, 0x0 );

    for(ch_index = 0; ch_index < NUM_HIF_CHANNELS; ch_index++)
    {
        /* Disable the RX DMA Engine & BDP Poll Cntrl */
        fp_reg_write((ULONG)&fp->hif.regs->ch[ch_index].hif_ctrl_ch, 0);
        fp_reg_write((ULONG)&fp->hif.regs->ch[ch_index].hif_rx_ch_start, 0);
        fp_reg_write((ULONG)&fp->hif.regs->ch[ch_index].hif_tx_ch_start, 0);
    }
}

static inline void fp_disable_hif_tx_dma_engine(fp_private_t *fp)
{
    UINT ch_index = 0;

    /* CLEAR - Poll Counter */
    fp_reg_write((ULONG)&fp->hif.regs->hif_tx_poll_ctrl, 0x0);

    for(ch_index = 0; ch_index < NUM_HIF_CHANNELS; ch_index++)
    {
        /* Disable the RX DMA Engine & BDP Poll Cntrl */
        fp_reg_write((ULONG)&fp->hif.regs->ch[ch_index].hif_ctrl_ch, 0x00030000);
        fp_reg_write((ULONG)&fp->hif.regs->ch[ch_index].hif_tx_ch_start, 0);
    }
}

static inline void fp_disable_hif_rx_dma_engine(fp_private_t *fp)
{
    UINT ch_index = 0;

    /* CLEAR - Poll Counter */
    fp_reg_write((ULONG)&fp->hif.regs->hif_rx_poll_ctrl, 0x0);

    for(ch_index = 0; ch_index < NUM_HIF_CHANNELS; ch_index++)
    {
        /* Disable the RX DMA Engine & BDP Poll Cntrl */
        fp_reg_write((ULONG)&fp->hif.regs->ch[ch_index].hif_ctrl_ch, 0x00000000);
        fp_reg_write((ULONG)&fp->hif.regs->ch[ch_index].hif_rx_ch_start, 0);
    }
}



void fp_init_bdp_base_reg(fp_private_t *fp);

#else /* else of FP_PCI_BM_EN */

#define CSR_TX_DMA_EN             (1 << 0)
#define CSR_TX_BDP_POLL_CNTR_EN   (1 << 1)
#define CSR_TX_BDP_CH_START_WSTB  (1 << 2)
#define CSR_RX_DMA_EN             (1 << 0)
#define CSR_RX_BDP_POLL_CNTR_EN   (1 << 1)
#define CSR_RX_BDP_CH_START_WSTB  (1 << 2)
#define HIF_INT        (1 << 0)
#define HIF_RXBD_INT   (1 << 1)
#define HIF_RXPKT_INT  (1 << 2)
#define HIF_TXBD_INT   (1 << 3)
#define HIF_TXPKT_INT  (1 << 4)
#define HIF_INT_EN        (1 << 0)
#define HIF_RXBD_INT_EN   (1 << 1)
#define HIF_RXPKT_INT_EN  (1 << 2)
#define HIF_TXBD_INT_EN   (1 << 3)
#define HIF_TXPKT_INT_EN  (1 << 4)

struct hif_regs
{
    volatile uint32 hif_version;
    volatile uint32 hif_tx_ctrl;
    volatile uint32 hif_tx_curr_bd_addr;
    volatile uint32 hif_tx_alloc;
    volatile uint32 hif_tx_bdp_addr;
    volatile uint32 hif_tx_status;
    volatile uint32 hif_rsvd1[2];
    volatile uint32 hif_rx_ctrl;
    volatile uint32 hif_rx_bdp_addr;
    volatile uint32 hif_rsvd2[2];
    volatile uint32 hif_rx_status;
    volatile uint32 hif_int_src;
    volatile uint32 hif_int_en;
    volatile uint32 hif_poll_ctrl;
    volatile uint32 hif_rx_curr_bd_addr;
    volatile uint32 hif_rx_alloc;
};
#endif
#endif
