/*************************************************************************/ /*!
@File
@Title          fp_init_fastpath.c
@Copyright      Copyright (c) Milandr. All Rights Reserved
@Description    fastpath initilization based on configuration
@License        Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

/**********************************************************************
 *                                      Include Files
 **********************************************************************/
#include <linux/version.h>
#include <linux/module.h>
#include <linux/netdevice.h>
#include <fp_library.h>
#include <hw_2field_hash_table.h>
#include <hw_vlan_hash_table.h>

#include <fp_switch_api.h>

#include "fp_net_driver.h"
#include "fp_mdio_ops.h"
#include "hwinits.h"

/**********************************************************************
 * Function Name  : load_hwreg_table
 * Description    : loading hardware init data
 * Inputs
 * Parameters     : char *, wsp_hwreg_data_t *
 * Outputs        :
 * Parameters     : -
 * Returns        : -
 * Changes        :
 **********************************************************************/
void load_hwreg_table(hwinit_data_t *initdata)
{
	hwinit_data_t *hwdata;

	hwdata = initdata;

	while (hwdata->addr != (ULONG)FP_NULL)
    {
        fp_reg_write(hwdata->addr, hwdata->data);
		hwdata++;
	}
}
/**********************************************************************
 * Function Name  : fp_init_fastpath
 * Description    : initialize PE & non PE based routines based on config
 *                  This routine will called upon net driver init time
 * Inputs
 *   Parameters   : struct net_device *
 * Outputs        :
 *   Parameters   : -
 *   Returns      : int
 * Changes        :
 *********************************************************************/
int fp_init_fastpath(struct net_device *dev)
{
    int i, j, timeout;

#if 0
#ifdef CONFIG_HOST_LEARNING
	fp_switch_init_timer(1);
#endif
#endif

	/*Settings of other PATH of IC*/
	FP_DEBUG(DBGP_FEAT_HIF, FP_LOG_INFO, "%s: hw settings\n", __func__);
	load_hwreg_table(&sw_hwreg_settings[0]);

    for( i = 0; i <16; i++ )
    {
        /* EMAC */
        fp_reg_write(emac[i] + 0x8A0, 0x000000B0);
        fp_reg_write(emac[i] + 0x210, 0x00000020);
        fp_reg_write(emac[i] + 0x21C, 0x00002001);
//        fp_reg_write(emac[i] + 0x004, 0x083F0D1A);      // 1G, SGMII mode, rcv jumbo frames, reject error packets...
        fp_reg_write(emac[i] + 0x004, 0x083F0C12);        // 1G, SGMII mode, jumbo frames disable, 1536 bytes frame disable, reject error packets...
//        fp_reg_write(emac[i] + 0x004, 0x083D0C12);        // 1G, SGMII mode, jumbo frames disable, 1536 bytes frame disable, reject error packets, crc not discard...
        fp_reg_write(emac[i] + 0x028, 0x00010200);        // enable interrupts at: pcs_autoneg_compete, pcs_link_change
        fp_reg_write(emac[i] + 0x1DC, 0x00000014);
        fp_reg_write(emac[i] + 0x0C8, 0xFFFFFFFF);
        fp_reg_write(emac[i] + 0x0CC, 0x0000FFFF);
        fp_reg_write(emac[i] + 0x088, 0x00000000);
        fp_reg_write(emac[i] + 0x08C, 0x00000000);
        fp_reg_write(emac[i] + 0x0C0, 0x80000000);                                                  // enable VLAN processing
        fp_reg_write(emac[i] + 0x048, JUMBO_MAX_LENGTH);                                                  // jambo_max_len

        /* EGPI */
        fp_reg_write(egpi[i] + 0x008, 0x02000001);
        fp_reg_write(egpi[i] + 0x00C, 0x01000030);                                                  //___Q in spec value 0x30
        fp_reg_write(egpi[i] + 0x010, 0x08000080);                                                  //___Q in spec value 0x80
				// if( i < 8 )
				// {
		fp_reg_write(egpi[i] + 0x014, 0xC0100030);
        fp_reg_write(egpi[i] + 0x018, 0xC0100034);
        fp_reg_write(egpi[i] + 0x024, 0xC0620010);
			   //}
			   //else
			   //{
				// fp_reg_write(egpi[i] + 0x014, 0xC0700030);
				// fp_reg_write(egpi[i] + 0x018, 0xC0700034);
			    // fp_reg_write(egpi[i] + 0x024, 0xC0770010);
			   //}
		fp_reg_write(egpi[i] + 0x034, 0x00000100);
        fp_reg_write(egpi[i] + 0x038, 0x00000010);
        fp_reg_write(egpi[i] + 0x060, 0x00000010);
        fp_reg_write(egpi[i] + 0x04C, 0x00000178);
        fp_reg_write(egpi[i] + 0x050, 0x00000050);
				
        if( i < 8 ){
            fp_reg_write(egpi[i] + 0x148, 0xC0700034);
        }else{
		    fp_reg_write(egpi[i] + 0x148, 0xC0100034);
        }
    }

    fp_reg_write(WSP_GLOBAL_BASE_ADDR + 0x044, 0);  //Class 1 processing all ports! (it is only need for first revision of 1923KX028) 

	fp_mdio_port_enable(MDIO_PORT_1, 1);
	fp_mdio_clock_settings(MDIO_PORT_1, 7);	//set MDC clock to ~890 KHz
	fp_mdio_port_enable(MDIO_PORT_2, 1);
	fp_mdio_clock_settings(MDIO_PORT_2, 7);	//set MDC clock to ~890 KHz

    /* Context memory init */
    for( i = 0; i < 17; i++ )
    {

        /* DIRECT ACCESS TO CONTEXT MEMORY */
        #if 0
        fp_reg_write( TMU_BASE_ADDR + 0x2F0, 0x00000001 );
        for( j = 0; j < 8; j++ )
        {

            
            fp_reg_write( TMU_BASE_ADDR + 0x034, ( i << 8 ) | j );                                  // i - номер PHY, j - номер очереди
            fp_reg_write( TMU_BASE_ADDR + 0x038, 0x00000000 );
            fp_reg_write( TMU_BASE_ADDR + 0x03C, 0x00000000 );
            fp_reg_write( TMU_BASE_ADDR + 0x040, 0x00000000 );
            fp_reg_write( TMU_BASE_ADDR + 0x044, 0x00000000 );
            fp_reg_write( TMU_BASE_ADDR + 0x048, 0x00000000 );
            fp_reg_write( TMU_BASE_ADDR + 0x04C, 0x00000000 );
            fp_reg_write( TMU_BASE_ADDR + 0x050, 0x00000000 );
            fp_reg_write( TMU_BASE_ADDR + 0x054, 0x00000000 );                                      //___Q no in spec
            
        }
        #endif

        /*INDIRECT ACCESS TO CONTEXT MEMORY*/
        //fp_reg_write( TMU_BASE_ADDR + 0x2F0, 0x00000000 );
        for( j = 0; j < 64; j++ ){
            fp_reg_write(TMU_BASE_ADDR + TMU_CNTX_ADDR, (i << 16)|j);
            fp_reg_write(TMU_BASE_ADDR + TMU_CNTX_DATA, 0);
            fp_reg_write(TMU_BASE_ADDR + TMU_CNTX_CMD, 3);
            timeout = 100;
            while( ( (fp_reg_read(TMU_BASE_ADDR + TMU_CNTX_CMD) & CSR_CNTX_IND_DONE) == 0 ) && (timeout-- > 0) ){}
            if(timeout == 0){
                FP_DEBUG(DBGP_FEAT_HIF, FP_LOG_ERR, "%s: can not write data to context memory of TMU, phy %d, queue %d, reg %d\n",  __func__, i, (j/8), (j%8) );
            }  
        }
        
    }

	/* Enable all parts of IC */
	FP_DEBUG(DBGP_FEAT_HIF, FP_LOG_INFO, "%s: enable BMU1, BMU2, EGPI1 - EGPI16\n",  __func__);
	load_hwreg_table(&sw_hwreg_enable_bmu_1_2_egpi_1_to_16[0]);

	FP_DEBUG(DBGP_FEAT_HIF, FP_LOG_INFO, "%s: enable HGPI, CLASS_HW1, CLASS_HW2, EMAC1 - EMAC16\n",  __func__);
	load_hwreg_table(&sw_hwreg_enable_hgpi_class_1_2_emac_1_to_16[0]);

	hw_hash_table_init();
	wsp_vlan_hash_table_init();
	fp_hw_sw_init();

	if( fp_hw_vlantable_add() ){
		FP_DEBUG(DBGP_FEAT_HIF, FP_LOG_INFO, "%s: add entry to TABLE1 and TABLE2 failed!\n",  __func__);
	}

	return 0;
}
