/*************************************************************************/ /*!
@File
@Title          fp_ioctl.c
@Copyright      Copyright (c) Milandr. All Rights Reserved
@Description    ioctl API's for wsp module
@License        Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

#ifndef __KERNEL__
    #define __KERNEL__
#endif
#ifndef MODULE
    #define MODULE
#endif

/**********************************************************************
 *                                     Include Files
 **********************************************************************/
#include <linux/pci.h>
#include <linux/module.h>
#include <linux/netdevice.h>
#include <linux/notifier.h>
#include <linux/inetdevice.h>
#include <linux/if_vlan.h>
#include <linux/mii.h>

#include <fp_library.h>
#include <fp_ioctl.h>
#include <fp_stats.h>
#include <fp_interface.h>
#include <fp_poolalloc.h>
#include <fp_netd.h>

#include "nc_conf.h"
#include "fp_bridge_table.h"
#include "fp_router.h"
#include "nc_conf_api.h"

#ifdef _FEATURE_RTP_RELAY_
#include "fp_rtp_relay_table.h"
#include "fp_rtp_relay_ctlm.h"
#include "fp_common_ctlm.h"
#include "fp_rtp_relay_funcs.h"
#endif

#ifndef IPPROTO_TCP
#define IPPROTO_TCP 6
#endif

#ifndef IPPROTO_UDP
#define IPPROTO_UDP 17
#endif

int fp_switch_ioctl_entries_ops(struct net_device *dev,
					struct ifreq *rq, int cmd);
#ifdef FP_ROUTE_IOCTL
int fp_route_ioctl_entries_ops(struct net_device *dev,
					struct ifreq *rq, int cmd);
#endif
struct net_device *fp_pci_dev = FP_NULL;


/* Read h/w info to buff */
void fp_read_hw_data(unsigned char *buff, unsigned int peAddr, unsigned int len)
{
	int i;
	volatile unsigned int *addr;
	volatile unsigned int *data = (unsigned int *)buff;
	unsigned char *pciBaseAddr;

	if (fp_pci_dev == FP_NULL) {
		FP_DEBUG(DBGP_FEAT_ERR, FP_LOG_EMERG, "device pointer is NULL\n");
		return;
	}

	pciBaseAddr = getPciBaseAddr(fp_pci_dev);
	for (i = 0; i < (len/4); i = i + 1) {
		addr = (unsigned int *)(PE_CSR_ADDR + pciBaseAddr);
		*addr = (((peAddr + i * 4) | PE_DMEM_ADDR_OFFSET)) & 0x7FFFFFFF;
		addr  = (unsigned int *)(PE_CSR_DATA + 4 + pciBaseAddr);
		data[i] = *addr;
		/* FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO,
			"Read data at Location 0x%x, value 0x%x\n",
			(((peAddr+i *4) | PE_DMEM_ADDR_OFFSET) & 0x7FFFFFFF),
			data[i]); */
	}
}

/* write buff info to h/w */
void fp_write_hw_data(unsigned char *buff, unsigned int peAddr,
						unsigned int len)
{
	int i;
	volatile unsigned int *addr;
	unsigned int *data = (unsigned int *)buff;
	unsigned char *pciBaseAddr;

	if (fp_pci_dev == FP_NULL) {
		FP_DEBUG(DBGP_FEAT_ERR, FP_LOG_EMERG,
				"device pointer is NULL\n");
		return;
	}

	pciBaseAddr = getPciBaseAddr(fp_pci_dev);
	for (i = 0; i < (len/4); i = i + 1) {
		addr = (unsigned int *)(PE_CSR_DATA + pciBaseAddr);
		*addr = data[i];
		addr = (unsigned int *)(PE_CSR_ADDR + pciBaseAddr);
		*addr = (peAddr + i * 4) | PE_DMEM_ADDR_OFFSET;
	}
}

int ioctl_get_stats(struct net_device *dev, struct ifreq *rq, int cmd)
{
	nc_ioctl_ops_t *kmem_nc_ops;

#ifdef HW_STATS_EN
	if (fp_pci_dev == FP_NULL)
    {
		FP_DEBUG(DBGP_FEAT_ERR, FP_LOG_EMERG, "device pointer is NULL\n");
		return -1;
	}

	fp_stats.dst_be_found = fp_stats.rx_pkt_cnt - fp_host_pkt_cnt;
	fp_stats.dst_be_not_found = fp_host_pkt_cnt;
    fp_host_pkt_cnt = 0;                                                        /* read on clear */
#endif

	kmem_nc_ops = kmalloc( sizeof( nc_ioctl_ops_t ), GFP_KERNEL );
	if(kmem_nc_ops == FP_NULL)
    {
		FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "%s: can not allocate memory for struct nc_ioctl_ops_t\n", __func__);
		return -EFAULT;
	}
	if(copy_from_user(kmem_nc_ops, (nc_ioctl_ops_t *) rq->ifr_ifru.ifru_data, sizeof(nc_ioctl_ops_t)))
    {
		FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "%s: can not copy data from user\n", __func__);
		kfree(kmem_nc_ops);
		return -EFAULT;
	}

	if (copy_to_user(kmem_nc_ops->io_data, &fp_stats, sizeof(fp_stats_t)))
    {
		FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "%s: can not copy data to user\n", __func__);
		kfree(kmem_nc_ops);
		return -EFAULT;
	}

	kfree(kmem_nc_ops);

	return 0;
}

int
ioctl_get_debug_stats(struct net_device *dev, struct ifreq *rq, int cmd) {
	//nc_ioctl_ops_t *pnc_ops = (nc_ioctl_ops_t *) rq->ifr_ifru.ifru_data;
	nc_ioctl_ops_t *kmem_nc_ops;
	UINT i;
	unsigned char *buff;
	fp_debugstats_t *stats;
	//pnc_ops = (nc_ioctl_ops_t *) rq->ifr_ifru.ifru_data;

	buff = kmalloc(sizeof(fp_debugstats_t), GFP_KERNEL );

	if(buff == FP_NULL) {
		FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "%s: can not allocate memory for buff\n", __func__);
		return -1;
	}

	memset(buff, 0, sizeof(fp_debugstats_t));
	stats = (fp_debugstats_t *)buff;
	stats->emac_tx_counters0 = fp_reg_read((ULONG)(0x660108));
	stats->emac_tx_counters1 = fp_reg_read((ULONG)(0x670108));
	stats->emac_rx_counters0 = fp_reg_read((ULONG)(0x660158));
	stats->emac_rx_counters1 = fp_reg_read((ULONG)(0x670158));

	for (i = 0; i < 13; i++) {
		stats->emac_errors0[i] = fp_reg_read((ULONG)(0x660184 + (i * 4)));
		stats->emac_errors1[i] = fp_reg_read((ULONG)(0x670184 + (i * 4)));
	}

	stats->gpi_fifo_count0 = fp_reg_read((ULONG)(0x780058));
	stats->gpi_fifo_count1 = fp_reg_read((ULONG)(0x790058));
	stats->gpi_fifo_count2 = fp_reg_read((ULONG)(0x650058));
	//stats->gpi_fifo_count3 = fp_reg_read(fp_baseAddr, (ULONG)(0x620058));
	stats->gpi_fifo_count3 = 0;
	stats->gpi_drop_count0 = fp_reg_read((ULONG)(0x780058));
	stats->gpi_drop_count1 = fp_reg_read((ULONG)(0x790058));

	for (i = 0; i < 10; i++) {
		stats->class_phy1_count[i] = fp_reg_read((ULONG)(0x62011C + (i * 4)));
		stats->class_phy2_count[i] = fp_reg_read((ULONG)(0x620144 + (i * 4)));
	}

	stats->bmu_buffer_count = fp_reg_read((ULONG)(0x100014));
	stats->bmu_rem_count = fp_reg_read((ULONG)(0x100048));
	stats->bmu_cur_count = fp_reg_read((ULONG)(0x10003C));
	for (i = 0; i < 9; i++)
		stats->bmu_master_based_buffer_counts[i] =
			fp_reg_read((ULONG)(0x100060 + (i * 4)));
	stats->hif1_tx_counters0 = fp_reg_read((ULONG)(0x6400A0));
	stats->hif1_tx_counters1 = fp_reg_read((ULONG)(0x64009C));
	stats->hif1_rx_counters0 = fp_reg_read((ULONG)(0x6400C0));
	stats->hif1_rx_counters1 = fp_reg_read((ULONG)(0x6400BC));
	stats->hif2_tx_counters0 = 0;
	stats->hif2_tx_counters1 = 0;
	stats->hif2_rx_counters0 = 0;
	stats->hif2_rx_counters1 = 0;
	stats->hgpi1_state0 = fp_reg_read((ULONG)(0x650090));
	stats->hgpi1_state1 = fp_reg_read((ULONG)(0x650094));
	stats->hgpi2_state0 = 0;
	stats->hgpi2_state1 = 0;
	stats->egpi1_state0 = fp_reg_read((ULONG)(0x780090));
	stats->egpi1_state1 = fp_reg_read((ULONG)(0x780094));
	stats->egpi2_state0 = fp_reg_read((ULONG)(0x790090));
	stats->egpi2_state1 = fp_reg_read((ULONG)(0x790094));

/*
	if (buff == FP_NULL) {
		FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "ioctl_entries_ops: buff is null\n");
		return -1;
	}
*/

	kmem_nc_ops = kmalloc(sizeof(nc_ioctl_ops_t), GFP_KERNEL );
	if(kmem_nc_ops == FP_NULL){
		FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "%s: can not allocate memory for struct nc_ioctl_ops_t\n", __func__);
		kfree(buff);
		return -EFAULT;
	}
	if(copy_from_user(kmem_nc_ops, (nc_ioctl_ops_t *) rq->ifr_ifru.ifru_data, sizeof(nc_ioctl_ops_t)) ){
		FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "%s: can not copy data from user\n", __func__);
		kfree(buff);
		kfree(kmem_nc_ops);
		return -EFAULT;
	}

/* changed according to integration code */
	//if (copy_to_user(pnc_ops->io_data, buff, sizeof(fp_debugstats_t))){
	if (copy_to_user(kmem_nc_ops->io_data, buff, sizeof(fp_debugstats_t))){
		FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "%s: can not copy data to user\n", __func__);
		kfree(buff);
		kfree(kmem_nc_ops);
		return -EFAULT;
	}

	kfree(kmem_nc_ops);
	kfree(buff);

	return 0;
}




/* extern void fp_disable_fastpath(void); */
int
ioctl_set_fastpath(struct net_device *dev, struct ifreq *rq, int cmd) {
	nc_ioctl_ops_t *pnc_ops = (nc_ioctl_ops_t *) rq->ifr_ifru.ifru_data;

	unsigned int *val;	// = (unsigned int *)pnc_ops->io_data;

	val = kmalloc( sizeof( unsigned int ), GFP_KERNEL );
	if(val == FP_NULL){
		FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "%s: can not allocate memory for val\n", __func__);
		return -1;
	}
	if(copy_from_user(val, (unsigned int *)pnc_ops->io_data, sizeof(unsigned int) ) ){
		FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "%s: can not copy data from user\n", __func__);
		kfree(val);
		return -1;
	}

	if ((*val) == 1) {
		fpathConfig.fastpath = 1;
	} else if ((*val) == 0) {
		/*fp_disable_fastpath();*/
		fpathConfig.fastpath = 0;
	} else if ((*val == 2)) {
		fpathConfig.notifier = 1;
	} else if ((*val) == 3) {
		fpathConfig.notifier = 0;
	}

	kfree(val);
	return 0;
}

/*
 * Function fp_config_ioctl
 */
int
fp_config_ioctl(struct net_device *dev, struct ifreq  *rq,  int  cmd) {

	int result = 0;

	FPATH_TRACE(FP_TRACE_CONFIG, FP_LOG_DEBUG, ("(%s) Received IOCTL CMD = %d\n ", __func__, cmd));

	/* Global dev initialization */
	fp_pci_dev = dev;
	/* FP_DEBUG(DBGP_FEAT_ERR, FP_LOG_INFO, "IOCTL: dev pointr %x\n",
						(unsigned int)fp_pci_dev); */

	switch (cmd) {

	case NC_STATS_GET:
		result = ioctl_get_stats(dev, rq, cmd);
		break;

	case NC_DEBUGSTATS_GET:
		result = ioctl_get_debug_stats(dev, rq, cmd);
		break;

	case NC_DEBUGPRINTS_ENABLE:
		fp_dbgp_flags = 0x1FFF;
		fp_trace_level = 7;
		break;

	case NC_DEBUGPRINTS_DISABLE:
		fp_dbgp_flags = 0x1;
		fp_trace_level = 1;
		break;

	case NC_SWITCH_ENTRIES_GET:
		result = fp_switch_ioctl_entries_ops(dev, rq, cmd);
		break;


//#ifdef FP_ROUTE_IOCTL
//	case NC_ROUTE_ENTRIES_GET:
//		return fp_route_ioctl_entries_ops(dev, rq, cmd);
//		break;
//#endif

	case NC_SET_FASTPATH:
		result = ioctl_set_fastpath(dev, rq, cmd);
		break;

//#ifdef _FEATURE_RTP_RELAY_
//	case PCM_REQ_SET_RTP_RELAY:
//		FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "Received PCM_REQ_SET_RTP_RELAY\n");
//		ioctl_set_rtp_relay(dev, rq, cmd);
//		break;
//#endif

	default:
		FP_DEBUG(DBGP_FEAT_ERR, FP_LOG_INFO, "Received unknown ioctl option :%d\n", cmd);
		result = -EOPNOTSUPP;
		break;
	}

	return result;
}
