/*************************************************************************/ /*!
@File
@Title          fp_net_driver.h
@Copyright      Copyright (c) Milandr. All Rights Reserved
@Description    contains cor data stryctures & defines of net driver
@License        Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

#ifndef _fp_net_driver_
#define _fp_net_driver_

#include <linux/interrupt.h>

#define FP_MAJOR_DRIVER_VERSION 1
#define FP_MINOR_DRIVER_VERSION 11

#define NUM_HIF_CHANNELS         4

#define SWITCH_CONFIG_DONE_POS      0
#define SWITCH_JUMBO_PROCESSING_POS  1

#define SWITCH_CONFIG_DONE        ( 1UL << SWITCH_CONFIG_DONE_POS )
#define SWITCH_JUMBO_PROCESSING   ( 1UL << SWITCH_JUMBO_PROCESSING_POS )

typedef struct
{
    struct bd_ring *txring;
    struct bd_ring *rxring;
    struct tasklet_struct tx_tasklet;
    struct tasklet_struct rx_tasklet;
}hif_chan_t;

 typedef struct
 {
    struct hif_regs *regs;
    hif_chan_t ch[NUM_HIF_CHANNELS];
}hif_base_t;

struct switch_device_stats
{
    u64 bmu1_free;
    u64 bmu2_free;

    u64 class_rx;
    u64 class_l3_fail;
    u64 class_ipv4;
    u64 class_ipv6;
    u64 class_crc_err;
    u64 class_ttl_err;
    u64 class_icmp;
    u64 class_igmp;
    u64 class_tcp;
    u64 class_udp;

    int8_t basisTemp;
    int8_t boardTemp;
};

typedef struct
{
    volatile unsigned char *pcimem_base;        /* Memory mapped physical address */
    volatile unsigned char *pcimem_base_bar4;   /* Memory mapped physical address */
    volatile struct hif_regs *mmio_addr;        /* Memory mapped physical address */
    struct pci_dev  *pci_dev;                   /* PCI device Index */
    struct net_device *dev;
    spinlock_t lock;                            /* Spin lock flag */
    spinlock_t rx_lock;                         /* Spin lock flag */
    hif_base_t hif;
    uint32 txq_stopped;

    /*
    config_flags:
    bit 0: 1 - configuration of 1923KX028 is done; 0 - 1923KX028 in unconfigured state (for proper call of fp_net_release)
    bit 1: 1 - tx / rx jumbo frame enable; 0 - tx / rx jumbo frame disable;
    */
    uint32_t config_flags;                       /* When flag SWITCH_CONFIG_DONE is set, net_open function was called and switch settings was done */
    uint32_t ageing_timeout;                    /* for debug. you can change timeout of ageing timer */
    uint32_t ageing_timer_on;                   /* for debug. you can disable or enable ageing timer */
    uint32_t host_system_loopback;              /* for debug. you can send back all packets that were received by HIF*/
    struct net_device_stats netstats;
    struct switch_device_stats switch_stats;
    uint32_t    mcu_firmware_version;           /* version of MCU firmware */
}fp_private_t;

struct tasklet_data
{
    fp_private_t *fp;
    int ch_index;
};

//#define MIN_FRAME_SIZE 46

//#define MAX_BUF_ALLOCSIZE 2048

#ifdef DEBUG
#define DPRINTK(lvl, fmt, args...) printk(lvl fmt, ##args)
#else
#define DPRINTK(lvl, fmt, args...)
#endif

#define RX_LOOP_THRESH 16
#define ENABLE_HIF_INTERRUPTS 1
#define NFP_IFS  1
void load_hwinit_config(unsigned int peAddr, unsigned int *data, unsigned int len);
int fp_switch_init_timer(int init);
int fp_init_fastpath(struct net_device *dev);
void  fp_cleanup_fastpath(void);
irqreturn_t fp_interrupt(int irq, void *dev);
void fp_rx_tasklet(ulong data);
void fp_en_hif_rx_engine(fp_private_t *fp);
void fp_disable_interrupts(fp_private_t *fp);
int fp_net_xmit(struct sk_buff *skb, struct net_device *dev);
uint32_t fp_hw_sw_init(void);
int fp_hw_vlantable_add(void);
int fp_net_release(struct net_device *dev);
extern const struct net_device_ops fp_netdev_ops;
extern struct ethtool_ops fp_ethtool_ops;
void fp_enable_interrupts(fp_private_t *fp);
void fp_init_rx_tasklet(fp_private_t *fp);
void fp_init_tx_tasklet(fp_private_t *fp);

uint32_t get_pci_cfg_word( uint32_t offset, uint32_t *ptr );
uint32_t set_pci_cfg_word( uint32_t offset, uint32_t value );
void set_mcu_firmware_version( uint32_t value );
uint32_t get_mcu_firmware_version( void );
void set_temperature( uint32_t value );
uint32_t get_temperature( void );

void sfp_disconnected( uint32_t port_num );
uint32_t is_sfp_module_connected( uint32_t port_num );
uint32_t sfp_store_data( uint32_t port_num, uint32_t addr_offset, uint32_t bytes_count, uint32_t *from );
uint32_t sfp_module_connected( uint32_t port_num );
uint32_t sfp_copper_configured( uint32_t port_num );
#endif /* _fp_net_driver_*/
