/*************************************************************************/ /*!
@File
@Title          fp_nw_if.c
@Copyright      Copyright (c) Milandr. All Rights Reserved
@Description    contains functions to register and create logical interfaces
@License        Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

/**********************************************************************
 *                                     Include Files
 **********************************************************************/
/*#include <asm/uaccess.h>*/ /* for copy_from_user */
#include <linux/uaccess.h> /* for copy_from_user */
#include <linux/capability.h>
#include <linux/module.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/skbuff.h>
#include <net/datalink.h>
#include <linux/mm.h>
#include <linux/in.h>
#include <linux/init.h>
#include <linux/if_vlan.h>
#include <net/p8022.h>
#include <net/arp.h>
#include <linux/rtnetlink.h>
#include <linux/notifier.h>
#include <fp_library.h>
#include <fp_netd.h>
#include <fp_types.h>
#include <fp_interface.h>
#include <fp_pktbuff.h>
#include <fp_ether.h>
#include <fp_pre_header.h>

#include <linux/pci.h>
#include "fp_net_driver.h"

#include "phy_i347at4.h"
#include "phy_dp83867.h"

#ifdef HAVE_PTP_1588_CLOCK
#include <gem_ptp.h>
#endif /* HAVE_PTP_1588_CLOCK */

#include <linux/mutex.h>
/**********************************************************************
 *                            Extern Declarations                     *
 **********************************************************************/
unsigned int fp_host_pkt_cnt;
static inline struct net_device_stats *nfp_dev_get_stats(struct net_device *dev);
static int nfp_dev_init(struct net_device *dev);

extern struct mutex mutexProcEMAC;

void nfp_setup(struct net_device *new_dev);

/**********************************************************************
 *				              GLOBAL DECLARATIONS			          *
 **********************************************************************/

static fp_dev_info_t fp_phy[FP_MAX_PORTS];
struct net_device *fp_logical_dev[FP_MAX_PORTS];

static fp_dev_info_t fp_priv[FP_MAX_PORTS];

#ifdef FP_SWITCH_MODE
fp_IfData_t gIfPorts[MAX_PHY_PORTS][MAX_LOGICAL_IFS];
fp_phyPort_t gPhyPorts[MAX_PHY_PORTS];
#endif

nfp_global_config_t fpathConfig;
struct net_device *fp_phy_addr[FP_MAX_PORTS] = {0};

struct fp_phy_stats_ethtool
{
	char stat_string[ETH_GSTRING_LEN];
	int stat_offset;
};

#define FP_PHY_STAT(_name, _stat) { \
	.stat_string = _name, \
	.stat_offset = offsetof(struct phy_statistic, _stat) \
}

/* All stats are u64, so we don't need to track the size of the field. */
static const struct fp_phy_stats_ethtool fp_phy_stats_strings[] = {
    FP_PHY_STAT("tx ok",                tx_ok),
    FP_PHY_STAT("tx broadcast",         tx_broadcast),
    FP_PHY_STAT("tx multicast",         tx_multicast),
    FP_PHY_STAT("tx pause",             tx_pause),
    FP_PHY_STAT("tx 64",                tx_64),
    FP_PHY_STAT("tx 65-127",            tx_65_127),
    FP_PHY_STAT("tx 128-255",           tx_128_255),
    FP_PHY_STAT("tx 256-511",           tx_256_511),
    FP_PHY_STAT("tx 512-1023",          tx_512_1023),
    FP_PHY_STAT("tx 1024-1518",         tx_1024_1518),
    FP_PHY_STAT("tx >1518",             tx_more_1518),
    FP_PHY_STAT("tx underruns",         tx_underruns),
    FP_PHY_STAT("tx single collision",  tx_single_collision),
    FP_PHY_STAT("tx 2-15 collision",    tx_2_15_collision),
    FP_PHY_STAT("tx >15 collision",     tx_more_15_collision),
    FP_PHY_STAT("tx late collision",    tx_late_collision),
    FP_PHY_STAT("tx deferred",          tx_deferred),

    FP_PHY_STAT("rx ok",                rx_ok),
    FP_PHY_STAT("rx broadcast",         rx_broadcast),
    FP_PHY_STAT("rx multicast",         rx_multicast),
    FP_PHY_STAT("rx pause",             rx_pause),
    FP_PHY_STAT("rx 64",                rx_64),
    FP_PHY_STAT("rx 65-127",            rx_65_127),
    FP_PHY_STAT("rx 128-255",           rx_128_255),
    FP_PHY_STAT("rx 256-511",           rx_256_511),
    FP_PHY_STAT("rx 512-1023",          rx_512_1023),
    FP_PHY_STAT("rx 1024-1518",         rx_1024_1518),
    FP_PHY_STAT("rx >1518",             rx_more_1518),
    FP_PHY_STAT("rx <64",               rx_less_64),
    FP_PHY_STAT("rx excessive len",     rx_excessive_len),
    FP_PHY_STAT("rx jabbers",           rx_jabbers),
    FP_PHY_STAT("rx fcs err",           rx_fcs_err),
    FP_PHY_STAT("rx len err",           rx_len_err),
    FP_PHY_STAT("rx symbol err",        rx_symbol_err),
    FP_PHY_STAT("rx aligment err",      rx_aligment_err),
    FP_PHY_STAT("rx recource err",      rx_recource_err),
    FP_PHY_STAT("rx overruns",          rx_overruns),
    FP_PHY_STAT("rx ip crc err",        rx_ip_crc_err),
    FP_PHY_STAT("rx tcp crc err",       rx_tcp_crc_err),
    FP_PHY_STAT("rx udp crc err",       rx_udp_crc_err),

    FP_PHY_STAT("class received",       class_rx),
    FP_PHY_STAT("class L3 fail",        class_l3_fail),
    FP_PHY_STAT("class IPV4",           class_ipv4),
    FP_PHY_STAT("class IPV6",           class_ipv6),
    FP_PHY_STAT("class CRC error",      class_crc_err),
    FP_PHY_STAT("class TTL error",      class_ttl_err),
    FP_PHY_STAT("class ICMP",           class_icmp),
    FP_PHY_STAT("class IGMP",           class_igmp),
    FP_PHY_STAT("class TCP",            class_tcp),
    FP_PHY_STAT("class UDP",            class_udp),
};

#define FP_PHY_GLOBAL_STATS_LEN ARRAY_SIZE(fp_phy_stats_strings)
/* nfp network devices have devices nesting below it, and are a special
 * "super class" of normal network devices; split their locks off into a
 * separate class since they always nest.
 */
#ifdef FP_SWITCH_MODE
int
dump_packet(struct sk_buff *skb)
{
	int i = 0;
	FP_DEBUG(DBGP_FEAT_HIF, FP_LOG_INFO, "DATA Len: %d\n", skb->len);
	for (i = 0; i < skb->len; i++) {
		FP_DEBUG(DBGP_FEAT_HIF, FP_LOG_INFO, "%02x ", skb->data[i] & 0xff);
	}
	FP_DEBUG(DBGP_FEAT_HIF, FP_LOG_INFO, "\n");
} /*End of dump_packe() Function*/
#endif
extern uint8_t control_block_injection_flag_values[ CB_INJ_TX_FLAG_VALUES_LEN ];
/*****************************************************************************
 * Function name     : nfp_dev_get_stats()
 * Input Parameters  : struct net_device*
 * Output Parameters : -
 * Description       : This function returns device statistics
 * Return            : struct net_device_stats*
 * Changes           : -
 ****************************************************************************/

static inline struct net_device_stats *nfp_dev_get_stats(struct net_device *dev)
{
    fp_dev_info_t *nfp_priv = netdev_priv(dev);

	return &(nfp_priv->dev_stats);
}

/*****************************************************************************
 * Function name     : nfp_dev_init()
 * Input Parameters  : struct net_device*
 * Output Parameters : -
 * Description       : Function for device intialisation
 * Return            : int
 * Changes           : -
 ****************************************************************************/
static int
nfp_dev_init(struct net_device *dev)
{
	struct net_device *real_dev = FP_DEV_INFO(dev)->real_dev;
	/* IFF_BROADCAST|IFF_MULTICAST; ??? */
	dev->flags  = real_dev->flags & ~IFF_UP;
	/* dev->flags  |=IFF_BROADCAST; */
	/* dev->flags  |=IFF_MULTICAST; */
	//dev->iflink = real_dev->ifindex;
	dev->state  = (real_dev->state & ((1<<__LINK_STATE_NOCARRIER) | (1<<__LINK_STATE_DORMANT))) | (1<<__LINK_STATE_PRESENT);
	return 0;

} /*End of nfp_dev_init() Function*/

/*****************************************************************************
 * Function name     : nfp_dev_open()
 * Input Parameters  : struct net_device*
 * Output Parameters : -
 * Description       : Function for device open
 * Return            : int
 * Changes           : -
 ****************************************************************************/
int
nfp_dev_open(struct net_device *dev)
{
	fp_dev_info_t *nfp_priv = netdev_priv(dev);
	struct net_device *real_dev = nfp_priv->real_dev;
	int err;

    nfp_priv->link_advertising = ADVERTISED_TP | ADVERTISED_10baseT_Half | ADVERTISED_10baseT_Full |
                                ADVERTISED_100baseT_Half | ADVERTISED_100baseT_Full |
                                ADVERTISED_1000baseT_Half | ADVERTISED_1000baseT_Full |
                                ADVERTISED_Autoneg;

	if (!(real_dev->flags & IFF_UP))
		return -ENETDOWN;
	if (ether_addr_equal(dev->dev_addr, real_dev->dev_addr)) {
		err = dev_uc_add(real_dev, dev->dev_addr);
	if (err < 0)
		return err;
	}
	memcpy(nfp_priv->real_dev_addr, real_dev->dev_addr, ETH_ALEN);
	if (dev->flags & IFF_ALLMULTI)
		dev_set_allmulti(real_dev, 1);
	if (dev->flags & IFF_PROMISC)
		dev_set_promiscuity(real_dev, 1);

    // netif_start_queue(dev);
    FP_DEBUG(DBGP_FEAT_HIF, FP_LOG_INFO, "nfp_dev_open, dev=%d\n", nfp_priv->phy_port_no);
	return 0;

} /*End of nfp_dev_open() Function*/

/*****************************************************************************
 * Function name     : nfp_dev_stop()
 * Input Parameters  : struct net_device*
 * Output Parameters : -
 * Description       : Function for stop device operations
 * Return            : int
 * Changes           : -
 ****************************************************************************/
int
nfp_dev_stop(struct net_device *dev)
{
	struct net_device *real_dev = FP_DEV_INFO(dev)->real_dev;
    fp_dev_info_t *nfp_priv = netdev_priv(dev);

#ifdef HAVE_PTP_1588_CLOCK
	nfp_dev_info_t *nfp_priv = FP_DEV_INFO(dev);

	if (nfp_priv->ptp_tx_skb) {
		dev_kfree_skb_any(nfp_priv->ptp_tx_skb);
		nfp_priv->ptp_tx_skb = NULL;
	}
#endif /* HAVE_PTP_1588_CLOCK */

    memset(&nfp_priv->dev_stats, 0, sizeof(struct net_device_stats));
    memset(&nfp_priv->phy_stats, 0, sizeof(struct phy_statistic));

	dev_mc_unsync(real_dev, dev);

	if (dev->flags & IFF_ALLMULTI)
		dev_set_allmulti(real_dev, -1);
	if (dev->flags & IFF_PROMISC)
		dev_set_promiscuity(real_dev, -1);
	if (ether_addr_equal(dev->dev_addr, real_dev->dev_addr))
		dev_uc_del(real_dev, dev->dev_addr);

    // netif_stop_queue(dev);
    FP_DEBUG(DBGP_FEAT_HIF, FP_LOG_INFO, "nfp_dev_stop, dev=%d\n", nfp_priv->phy_port_no);
	return 0;

} /*End of nfp_dev_stop() Function*/

/*****************************************************************************
 * Function name     : nfp_dev_ioctl()
 * Input Parameters  : struct net_device*,struct ifreq *,int
 * Output Parameters : -
 * Description       : Function for device ioctl  operations
 * Return            : int
 * Changes           : -
 ****************************************************************************/
int
nfp_dev_ioctl(struct net_device *dev, struct ifreq *ifr, int cmd)
{
	struct net_device *real_dev = FP_DEV_INFO(dev)->real_dev;
	struct ifreq ifrr;
	int err = -EOPNOTSUPP;

	strncpy(ifrr.ifr_name, real_dev->name, IFNAMSIZ);
	ifrr.ifr_ifru = ifr->ifr_ifru;
	switch (cmd)
    {
	case SIOCGMIIPHY:
	case SIOCGMIIREG:
	case SIOCSMIIREG:
		if (real_dev->netdev_ops->ndo_do_ioctl && netif_device_present(real_dev))
        	err = real_dev->netdev_ops->ndo_do_ioctl(real_dev, &ifrr, cmd);
	break;
	case SIOCSHWTSTAMP:
		/* TODO */
		return 0;
	break;
	}
	if (!err)
		ifr->ifr_ifru = ifrr.ifr_ifru;
	return err;

} /*End of nfp_dev_ioctl() Function*/

/*****************************************************************************
 * Function name     : nfp_change_rx_flags()
 * Input Parameters  : struct net_device*,int
 * Output Parameters : -
 * Description       : Function for device rx flags
 * Return            : void
 * Changes           : -
 ****************************************************************************/
void
nfp_change_rx_flags(struct net_device *dev, int change)
{
	struct net_device *real_dev = FP_DEV_INFO(dev)->real_dev;
	if (change & IFF_ALLMULTI)
		dev_set_allmulti(real_dev,
				dev->flags & IFF_ALLMULTI ? 1 : -1);
	if (change & IFF_PROMISC)
		dev_set_promiscuity(real_dev,
				dev->flags & IFF_PROMISC ? 1 : -1);

} /*End of nfp_change_rx_flags() Function*/

/*****************************************************************************
 * Function name     : nfp_dev_set_multicast_list()
 * Input Parameters  : struct net_device*
 * Output Parameters : -
 * Description       : Function to set device multicast list
 * Return            : void
 * Changes           : -
 ****************************************************************************/

void
nfp_dev_set_multicast_list(struct net_device *dev)
{

	dev_mc_sync(FP_DEV_INFO(dev)->real_dev, dev);

} /*End of nfp_dev_set_multicast_list() Function*/


/*****************************************************************************
 * Function name     : nfp_dev_hard_start_xmit()
 * Input Parameters  : struct sk_buff *,struct net_device *
 * Output Parameters : -
 * Description       : Function for xmit linux processing packet from nfp
			device to fp0 device.
 * Return            : int
 * Changes           : -
 ****************************************************************************/
int
nfp_dev_hard_start_xmit(struct sk_buff *skb, struct net_device *dev)
{
	// struct net_device_stats *stats = nfp_dev_get_stats(dev);
	// stats->tx_packets++; /* for statics only */

#ifdef DEBUG_HOST
	FP_DEBUG(DBGP_FEAT_HIF, FP_LOG_INFO, "XMIT DUMP %s skb: %p\n", dev->name, skb);

	if ((FP_DEV_INFO(skb->dev)->phyPortNo < 0) && (FP_DEV_INFO(skb->dev)->phyPortNo) > 15) {
		FP_DEBUG(DBGP_FEAT_HIF, FP_LOG_INFO, "nfp_dev_hard_start_xmit: Invalid phyport no at %d \n", FP_DEV_INFO(skb->dev)->phyPortNo);
		return 0;
	}
#endif

#ifdef DEBUG_HOST
	dump_packet(skb);
	FP_DEBUG(DBGP_FEAT_HIF, FP_LOG_INFO, "Completed Printing Tx- Packet\n");
#endif

	/* Store the logical interface on which the packet is received in cb */
//	skb->cb[40] = FP_DEV_INFO(skb->dev)->phy_port_no;
//	skb->cb[CB_PORT_FOR_INJECTION] = FP_DEV_INFO(skb->dev)->phy_port_no;
	skb->cb[ CB_INJ_TX_PORT ] = FP_DEV_INFO(skb->dev)->phy_port_no;
	memcpy( &skb->cb[ CB_INJ_TX_FLAG1 ], control_block_injection_flag_values, CB_INJ_TX_FLAG_VALUES_LEN );

	skb->dev = FP_DEV_INFO(dev)->real_dev;
#ifdef DEBUG_HOST
	FP_DEBUG(DBGP_FEAT_HIF, FP_LOG_INFO, "Transmited to Dev:%s - Packet\n",	skb->dev->name);
#endif

	//FP_DEBUG(DBGP_FEAT_HIF, FP_LOG_INFO, "Transmited to Dev:%s - Packet\n", skb->dev->name);
	if (netif_queue_stopped(skb->dev)){
		netif_wake_queue(skb->dev);
	}
	dev_queue_xmit(skb);
	return 0;

} /*End of nfp_dev_hard_start_xmit() Function*/

/*****************************************************************************
 * Function name     : unregister_nfp_device()
 * Input Parameters  : struct net_device *
 * Output Parameters : -
 * Description       : Function for unregister nfp device
 * Return            : int
 * Changes           : -
 ****************************************************************************/
int
unregister_nfp_device(struct net_device *dev)
{
	dev_put(FP_DEV_INFO(dev)->real_dev);
	unregister_netdev(dev);
	free_netdev(dev);
	return 0;

} /*End of unregister_nfp_device() Function*/

/*****************************************************************************
 * Function name     : register_nfp_device()
 * Input Parameters  : struct net_device *
 * Output Parameters : -
 * Description       : Function for registration of nfp device
 * Return            : fp_dev_info_t*
 * Changes           : -
 ****************************************************************************/
struct net_device*
register_nfp_device(struct net_device *real_dev, unsigned char phyPortNo)
{
	struct net_device *new_dev = NULL;
	char name[IFNAMSIZ];
    fp_dev_info_t *priv;
    fp_private_t *fp = netdev_priv(real_dev);
	/*Set interface names*/
	strcpy(name, "fp_phy%d");

//	new_dev = alloc_netdev(sizeof(nfp_dev_info_t), name, nfp_setup);
	new_dev = alloc_netdev(sizeof(fp_dev_info_t), name, NET_NAME_ENUM, nfp_setup);
	if (new_dev == NULL){
		return NULL; /* -ENOBUFS; */
	}
    priv = netdev_priv(new_dev);

	new_dev->mtu = 1500;/* real_dev->mtu; */
	new_dev->ml_priv = &fp_priv[phyPortNo];

	fp_phy_addr[phyPortNo] = new_dev;
	FP_DEV_INFO(new_dev)->real_dev = real_dev;
	FP_DEV_INFO(new_dev)->phy_port_no = phyPortNo;
	/* new_dev->rtnl_link_ops = &vlan_link_ops; */
	register_netdevice(new_dev);

    priv->real_dev = real_dev;
    priv->phy_port_no = phyPortNo;
    priv->phy_ic_type = PHY_IC_ERR;
    priv->pdev = fp->pci_dev;
    priv->link_is_up = 0;
    priv->link_full_duplex = 0;
    priv->link_speed = 0;
    priv->link_advertising = 0xFF;
    priv->phy_port_state = 1;
	dev_hold(real_dev);

	FP_DEBUG(DBGP_FEAT_ETHTOOL, FP_LOG_INFO, "register_nfp_device phy%d ", fp_phy[phyPortNo].phy_port_no );

	return fp_phy_addr[phyPortNo];

} /*End of register_nfp_device() Function*/

/*****************************************************************************
 * Function name     : fp_create_logical_interfaces
 * Input Parameters  : max num of logical ports
 * Output Parameters : -
 * Description       : create logical interfaces corresponding to each phyport
 * Return            :
 * Changes           : -
 ****************************************************************************/
int fp_create_logical_interfaces(struct net_device *dev, int max_logical_ports)
{
	int portno = 0;

	for (portno = 0; portno < max_logical_ports; portno++)
    {
		if (((FWD_PORT_LIST_MASK >> portno) & 0x1) && (portno != HIF_PORT_NUM))
        {

			fp_logical_dev[portno] = register_nfp_device(dev, portno);

			if (fp_logical_dev[portno] == NULL) {
				FP_DEBUG(DBGP_FEAT_ERR, FP_LOG_EMERG, "fp_nw_if: error while creating logical interface\n");
				return -1;
			}

			/* initialize mac address to each logical interface */
			memcpy( fp_logical_dev[portno]->dev_addr, dev->dev_addr, ETH_ALEN );
			fp_logical_dev[ portno ]->dev_addr[ ETH_ALEN - 1 ] += ( portno + 1 );

			netif_start_queue(fp_logical_dev[portno]);

			FP_DEBUG(DBGP_FEAT_HIF, FP_LOG_INFO, "CREATING LOGICAL INTERFACE at %d\n", portno);
		}
	}

	return 0;
}

/* ---------------------------- ETHTOOL Support ---------------------------- */
static int
fp_nw_get_link_ksettings(struct net_device *netdev, struct ethtool_link_ksettings *cmd)
{
	fp_dev_info_t *adapter = netdev_priv(netdev);
	u32 supported, advertising, lp_advertising = 0;

	// FP_DEBUG(DBGP_FEAT_ETHTOOL, FP_LOG_INFO, "Get settings from phy%d", FP_DEV_INFO(netdev)->phyPortNo );
    FP_DEBUG(DBGP_FEAT_ETHTOOL, FP_LOG_INFO, "Get settings from phy%d IC type = %d", adapter->phy_port_no, adapter->phy_ic_type );

    supported = (SUPPORTED_10baseT_Half |
    			 SUPPORTED_10baseT_Full |
    			 SUPPORTED_100baseT_Half |
    			 SUPPORTED_100baseT_Full |
    			 SUPPORTED_1000baseT_Full|
                 SUPPORTED_1000baseT_Half|
    			 SUPPORTED_Autoneg |
    			 SUPPORTED_TP);
    advertising = adapter->link_advertising;

    cmd->base.port = PORT_TP;
    cmd->base.phy_address = adapter->phy_addr;

    if(adapter->link_is_up)
    {
        FP_DEBUG(DBGP_FEAT_ETHTOOL, FP_LOG_INFO, "phy%d link is UP", adapter->phy_port_no );
        if(adapter->link_full_duplex)
            cmd->base.duplex = DUPLEX_FULL;
        else
            cmd->base.duplex = DUPLEX_HALF;

        cmd->base.speed = adapter->link_speed;
    }
    else
    {
        FP_DEBUG(DBGP_FEAT_ETHTOOL, FP_LOG_INFO, "phy%d link is DOWN", adapter->phy_port_no );
        cmd->base.speed = SPEED_UNKNOWN;
    	cmd->base.duplex = DUPLEX_UNKNOWN;
    }

    if(adapter->link_advertising & ADVERTISED_Autoneg)
        cmd->base.autoneg = AUTONEG_ENABLE;
    else
        cmd->base.autoneg = AUTONEG_DISABLE;

    cmd->base.eth_tp_mdix_ctrl = ETH_TP_MDI_AUTO;

    mutex_lock( &mutexProcEMAC );
    if(adapter->phy_ic_type == PHY_IC_I347AT4)
    {
        lp_advertising = phy_i347at4_get_link_part_abil(adapter->mdio_port, adapter->phy_addr);
        cmd->base.eth_tp_mdix = phy_i347at4_get_mdi_stat(adapter->mdio_port, adapter->phy_addr);
    }
    else if(adapter->phy_ic_type == PHY_IC_DP83867)
    {
        lp_advertising = phy_dp83867_get_link_part_abil(adapter->mdio_port, adapter->phy_addr);
        cmd->base.eth_tp_mdix = phy_dp83867_get_mdi_stat(adapter->mdio_port, adapter->phy_addr);
    }

    ethtool_convert_legacy_u32_to_link_mode(cmd->link_modes.supported,
						supported);
	ethtool_convert_legacy_u32_to_link_mode(cmd->link_modes.advertising,
						advertising);
    ethtool_convert_legacy_u32_to_link_mode(cmd->link_modes.lp_advertising,
                    						lp_advertising);
	mutex_unlock( &mutexProcEMAC );

	return 0;
}

static int
fp_nw_set_link_ksettings(struct net_device *netdev,
				    const struct ethtool_link_ksettings *cmd)
{
    fp_dev_info_t *adapter = netdev_priv(netdev);
    u8 autoneg = cmd->base.autoneg;
	u8 duplex = cmd->base.duplex;
	u32 speed = cmd->base.speed;
	u32 advertising;
	int result = 0;

	FP_DEBUG(DBGP_FEAT_ETHTOOL, FP_LOG_INFO, "Get settings: autoneg=%x, duplex=%x, speed=%x ", cmd->base.autoneg, cmd->base.duplex, cmd->base.speed );
	mutex_lock( &mutexProcEMAC );
    ethtool_convert_link_mode_to_legacy_u32(&advertising,
    						cmd->link_modes.advertising);
    adapter->link_advertising = advertising;

    if (cmd->base.phy_address != adapter->phy_addr){
		result = -EINVAL;
		goto set_link_ksettings_failed;
    }

    if( ( autoneg == AUTONEG_DISABLE ) &&
		( ( ( speed != SPEED_1000 ) &&
			( speed != SPEED_100 ) &&
			( speed != SPEED_10 ) ) ||
		(	( duplex != DUPLEX_HALF ) &&
			( duplex != DUPLEX_FULL ) ) ) ){
		result = -EINVAL;
		goto set_link_ksettings_failed;
    }

    if( autoneg == AUTONEG_ENABLE ){
        adapter->link_advertising |= ADVERTISED_Autoneg;
        if( adapter->phy_ic_type == PHY_IC_I347AT4 ){
            phy_i347at4_speed_set( adapter->mdio_port, adapter->phy_addr, 0, 1 );
        }else if( adapter->phy_ic_type == PHY_IC_DP83867 ){
            phy_dp83867_speed_set( adapter->mdio_port, adapter->phy_addr, speed, duplex, autoneg );
        }
    }else{
        adapter->link_advertising &= ~ADVERTISED_Autoneg;
        if( adapter->phy_ic_type == PHY_IC_I347AT4 ){
            phy_i347at4_speed_set( adapter->mdio_port, adapter->phy_addr, speed, duplex );
        }else if( adapter->phy_ic_type == PHY_IC_DP83867 ){
            phy_dp83867_speed_set( adapter->mdio_port, adapter->phy_addr, speed, duplex, autoneg );
        }
    }

    FP_DEBUG( DBGP_FEAT_ETHTOOL, FP_LOG_INFO, "Set settings to phy%d autoneg=%d, duplex=%d, speed=%d, advertising=%x ",
				adapter->phy_port_no,
				autoneg,
				duplex, speed, advertising );

set_link_ksettings_failed:
	mutex_unlock( &mutexProcEMAC );
    return result;
}

static u32
fp_nw_get_link(struct net_device *netdev)
{
	fp_dev_info_t *adapter = netdev_priv(netdev);

	return adapter->link_is_up;
}

static void
fp_nw_get_drvinfo(struct net_device *netdev, struct ethtool_drvinfo *drvinfo)
{
    fp_dev_info_t *adapter = netdev_priv(netdev);
    uint8_t string[32];
    uint32_t len;

    FP_DEBUG(DBGP_FEAT_ETHTOOL, FP_LOG_INFO, "Get driver info from phy%d", adapter->phy_port_no );
	memset( string, 0, 32 );
	len = snprintf( string, 32, "%d.%d", FP_MAJOR_DRIVER_VERSION, FP_MINOR_DRIVER_VERSION );
	strlcpy(drvinfo->driver, "Milandr 1923KX028 driver", 25);
	strlcpy( drvinfo->version, string, ( len + 1 ) );
	strlcpy( drvinfo->fw_version, "N/A", 4);
	strlcpy( drvinfo->bus_info, pci_name(adapter->pdev), 32);

    drvinfo->n_stats = FP_PHY_GLOBAL_STATS_LEN;
}

static void
fp_nw_get_strings(struct net_device *dev, u32 stringset, u8 *data)
{
	int i;

    FP_DEBUG(DBGP_FEAT_ETHTOOL, FP_LOG_INFO, "Get phy strings, stringset=%d", stringset);

	switch (stringset) {
	case ETH_SS_STATS:
		for (i = 0; i < FP_PHY_GLOBAL_STATS_LEN; i++)
        {
			memcpy(data, fp_phy_stats_strings[i].stat_string, ETH_GSTRING_LEN);
			data += ETH_GSTRING_LEN;
		}
		break;
	}
}

static int
fp_nw_get_sset_count(struct net_device *dev, int sset)
{
    FP_DEBUG(DBGP_FEAT_ETHTOOL, FP_LOG_INFO, "Get phy sset count, sset=%d", sset);

	switch (sset) {
	case ETH_SS_STATS:
		return FP_PHY_GLOBAL_STATS_LEN;
	default:
		return -EOPNOTSUPP;
	}
}

/**
 * fp_get_ethtool_stats - report device statistics
 * @netdev: network interface device structure
 * @stats: ethtool statistics structure
 * @data: pointer to data buffer
 *
 * All statistics are added to the data buffer as an array of u64.
 **/
static void
fp_nw_get_ethtool_stats(struct net_device *netdev, struct ethtool_stats *stats, u64 *data)
{
    fp_dev_info_t *adapter = netdev_priv(netdev);
    struct phy_statistic *phystats = &adapter->phy_stats;
    unsigned int i;
	char *p;

    FP_DEBUG(DBGP_FEAT_ETHTOOL, FP_LOG_INFO, "Get statistics from phy%d", adapter->phy_port_no );

    for (i = 0; i <FP_PHY_GLOBAL_STATS_LEN; i++) {
        p = (char *)phystats + fp_phy_stats_strings[i].stat_offset;
        data[i] =  *(u64 *)p;
    }
}

static int
fp_nw_set_phys_id(struct net_device *netdev,
			     enum ethtool_phys_id_state state)
{
	fp_dev_info_t *adapter = netdev_priv(netdev);

	switch (state)
    {
	case ETHTOOL_ID_ACTIVE:

		return 2;

	case ETHTOOL_ID_ON:
        if(adapter->phy_ic_type == PHY_IC_I347AT4)
            phy_i347at4_setup_led_on(adapter->mdio_port, adapter->phy_addr);
        else if(adapter->phy_ic_type == PHY_IC_DP83867)
            phy_dp83867_led_drive(adapter->mdio_port, adapter->phy_addr, DP83867_LEDS_ON);
		break;

	case ETHTOOL_ID_OFF:
        if(adapter->phy_ic_type == PHY_IC_I347AT4)
            phy_i347at4_setup_led_off(adapter->mdio_port, adapter->phy_addr);
        else if(adapter->phy_ic_type == PHY_IC_DP83867)
            phy_dp83867_led_drive(adapter->mdio_port, adapter->phy_addr, DP83867_LEDS_OFF);
		break;

	case ETHTOOL_ID_INACTIVE:
        if(adapter->phy_ic_type == PHY_IC_I347AT4)
            phy_i347at4_setup_led_default(adapter->mdio_port, adapter->phy_addr);
        else if(adapter->phy_ic_type == PHY_IC_DP83867)
            phy_dp83867_led_drive(adapter->mdio_port, adapter->phy_addr, DP83867_LEDS_SHOW_STATE);
    break;
	}
	return 0;
}

static int
fp_nw_nway_reset(struct net_device *netdev)
{
	fp_dev_info_t *adapter = netdev_priv(netdev);

	if (netif_running(netdev))
	{
        if(adapter->phy_ic_type == PHY_IC_I347AT4)
            phy_i347at4_restart_aneg(adapter->mdio_port, adapter->phy_addr);
        else if(adapter->phy_ic_type == PHY_IC_DP83867)
            phy_dp83867_restart_aneg(adapter->mdio_port, adapter->phy_addr);
    }
	return 0;
}

static const struct net_device_ops nfp_netdev_ops = {
	.ndo_init              = nfp_dev_init,
	.ndo_start_xmit        = nfp_dev_hard_start_xmit,
	.ndo_open              = nfp_dev_open,
	.ndo_stop              = nfp_dev_stop,
	.ndo_do_ioctl          = nfp_dev_ioctl,
	.ndo_get_stats         = nfp_dev_get_stats,
	.ndo_change_rx_flags   = nfp_change_rx_flags,
	.ndo_change_mtu        = NULL

};

static struct ethtool_ops nfp_ethtool_ops = {
	.get_link_ksettings	= fp_nw_get_link_ksettings,
	.set_link_ksettings	= fp_nw_set_link_ksettings,
    .get_link		    = fp_nw_get_link,
    .get_drvinfo        = fp_nw_get_drvinfo,
    .get_strings        = fp_nw_get_strings,
    .get_sset_count     = fp_nw_get_sset_count,
    .get_ethtool_stats	= fp_nw_get_ethtool_stats,
    .set_phys_id		= fp_nw_set_phys_id,
    .nway_reset		    = fp_nw_nway_reset,
};

/*****************************************************************************
 * Function name     : nfp_setup()
 * Input Parameters  : struct net_device *
 * Output Parameters : -
 * Description       : Function to set ethernet propertis to device.
 * Return            : int
 * Changes           : -
 ****************************************************************************/
void
nfp_setup(struct net_device *new_dev)
{
	ether_setup(new_dev);
	new_dev->netdev_ops = &nfp_netdev_ops;
	new_dev->ethtool_ops = &nfp_ethtool_ops;

} /*End of nfp_setup() Function*/
/****************************** End of File ***********************************/
