/*************************************************************************/ /*!
@File
@Title          fp_phy_setup.c
@Copyright      Copyright (c) Milandr. All Rights Reserved
@Description    timer registration for switch ageing
@License        Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

/**********************************************************************
 *                                      Include Files
 **********************************************************************/
 #include <linux/timer.h>
 #include <fp_library.h>
 #include <fp_switch.h>

#include "fp_mdio_ops.h"
/**********************************************************************
 *                              Global Declarations
***********************************************************************/
/* This value is used to see current settings of PHY and MAC speed */
#define FP_PHY_MAC_SETTINGS_REFRESH    (30*HZ)

/* The timer structure used to see current settings of PHY and MAC */
static struct timer_list fp_phy_mac_settings_timer;

/*******************************************************************************
 * Function name:       fp_phy_mac_settings_handler
 * Input Parameters:    The long value passed to every timer handler
 * Output Parameters:   none
 * Result:              none
 * Description:         This functions
 ******************************************************************************/
void
fp_phy_mac_settings_handler(unsigned long data)
{
	// fp_phy_mac_settings();

	/* Update the refresh timer to call this fucntion again */
	fp_phy_mac_settings_timer.expires = jiffies + FP_PHY_MAC_SETTINGS_REFRESH;
	add_timer(&fp_phy_mac_settings_timer);
	FP_DEBUG(DBGP_FEAT_HIF, FP_LOG_EMERG,"END of PHY and MAC function\n");
}


/**********************************************************************
 * Function Name  : fp_switch_init_timer.c
 * Description    : do bridge ageing timer registration
 * Inputs         :
 * Parameters     : INT *data
 * Outputs        :
 * Parameters     : -
 * Returns        : int
 ********************************************************************/
int
fp_phy_mac_settings_init_timer(int init)
{
	int ret = 0;
	if (!init)
		goto del_phy_mac_timer;

	init_timer(&fp_phy_mac_settings_timer);
	fp_phy_mac_settings_timer.expires  = jiffies + FP_PHY_MAC_SETTINGS_REFRESH;
	fp_phy_mac_settings_timer.data     = (unsigned long) &fp_phy_mac_settings_timer;
	fp_phy_mac_settings_timer.function = fp_phy_mac_settings_handler;
	add_timer((struct timer_list *)&fp_phy_mac_settings_timer);
	FP_DEBUG(DBGP_FEAT_HIF, FP_LOG_EMERG, "Init PHY MAC settings timer\n");
	return 0;

del_phy_mac_timer:
	del_timer((struct timer_list *)&fp_phy_mac_settings_timer);
	FP_DEBUG(DBGP_FEAT_HIF, FP_LOG_EMERG, "Delete PHY MAC settings timer\n");
	return ret;
}
