/*************************************************************************/ /*!
@File
@Title          fp_switch_ioctl.c
@Copyright      Copyright (c) Milandr. All Rights Reserved
@Description    ioctl for switch
@License        Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

#ifndef __KERNEL__
    #define __KERNEL__
#endif
#ifndef MODULE
    #define MODULE
#endif

/**********************************************************************
 *                                     Include Files
 **********************************************************************/
#include <linux/pci.h>
#include <linux/module.h>
#include <linux/netdevice.h>
#include <linux/notifier.h>
#include <linux/inetdevice.h>
#include <linux/if_vlan.h>

#include <fp_library.h>
#include <fp_ioctl.h>
#include <fp_stats.h>
#include <fp_interface.h>
#include <fp_poolalloc.h>
#include <fp_netd.h>
#include <hw_2field_hash_table.h>
#include <fp_switch.h>
#include <wsp_vlan_hash_table_L.h>

#include "nc_conf.h"
#include "fp_bridge_table.h"
#include "fp_router.h"
#include "nc_conf_api.h"


fp_bridgeTable_t *fp_bridge_hash_table;
fp_bridgeTable_t *fp_vlan_bridge_hash_table;
fp_bridgeTable_t  fp_bridge_table;
fp_bridgeTable_t  fp_vlan_table;
POOLHEAD_PTR  fp_brPoolHeadPtr;
POOLHEAD  fp_brPoolHead;
POOLHEAD  fp_vlanPoolHead;

/* extern void fp_get_hw_bridge_entries(nc_entries_ops_t *);
extern int br_entry_count; */

void fp_init_bridge_table(struct net_device *dev)
{
	fp_bridge_hash_table = &fp_bridge_table;
	fp_bridge_hash_table->tbl_size = BRIDGE_HASH_SIZE;
	fp_bridge_hash_table->num_entries = 0;
	/* fp_bridge_hash_table->refcnt = 0; */
	fp_bridge_hash_table->table =
		(fp_bridgeEntry_t **)(BASE_ADDR + PE_CONFIG_BTABLE_ADDR);

	/* Need to initilize vlan table */
	fp_vlan_bridge_hash_table = &fp_vlan_table;
	fp_vlan_bridge_hash_table->tbl_size = BRIDGE_HASH_SIZE;
	fp_vlan_bridge_hash_table->num_entries = 0;
	/* fp_vlan_bridge_hash_table->refcnt = 0; */

	fp_vlan_bridge_hash_table->table =
		(fp_bridgeEntry_t **)(BASE_ADDR + PE_CONFIG_VTABLE_ADDR);

	/* Assign bridge poolhead poiter */
	fp_brPoolHeadPtr = (POOLHEAD_PTR)(BASE_ADDR + PE_CONFIG_BRPOOL_HEAD);
}

/*
 * Function: updateBridgeEntry
 */
int
updateBridgeEntry(fp_bridgeEntry_t *srcEntry,  nc_bridge_entry_t *br_entry) {
	int i;
	br_entry->be_pktcount = srcEntry->pktCount;
	br_entry->be_vlanid   = srcEntry->vlanId;
	br_entry->be_state    = srcEntry->entryState;
	br_entry->be_actions  = srcEntry->flowActionFlags;
	br_entry->be_vlanid = *(((UCHAR *)srcEntry) + 18) & 0xFFFF;
	br_entry->be_state    = 2;
	for (i = 0; i < ETH_ALEN-2; i++)
		br_entry->be_macaddr[3-i] =
			*(((UCHAR *)srcEntry) + 8 + i) & 0xFF;

	br_entry->be_macaddr[5] = *(((UCHAR *)srcEntry) + 14) & 0xFF;
	br_entry->be_macaddr[4] = *(((UCHAR *)srcEntry) + 15) & 0xFF;
	FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "********VLANID %x *******\n",
						br_entry->be_vlanid  & 0xFFFF);
	return 0;
}

#define MAC_HASH_TABLE (TWO_FIELD_HASH_ENTRIES + TWO_FIELD_COLL_ENTRIES)

int
fp_get_hw_bridge_entries(nc_entries_ops_t *pent_ops)
{
	int i = 0, temp = 0;
	UCHAR *pcimem_base = FP_NULL;
	UINT word1 = 0, word2 = 0, word3 = 0, word4 = 0;
	volatile unsigned int *addr = 0, cmd_done = 0;
	UINT hw_br_entry_count = 0, br_entry_size = 0, limitReached = 0;
	nc_bridge_entry_t br_entry;

	pcimem_base = (UCHAR *)getPciBaseAddr(fp_net_dev);
	if (pcimem_base == NULL) {
		FP_DEBUG(DBGP_FEAT_BRIDGE, FP_LOG_EMERG,
			"%s: pci_mem base is NULL\n", __func__);
		return 0;
	}

	/*==============================================================*
	* MAC1:MAC[31:0]						*
	*===============================================================*
	* MAC2:ACT_ENTRY[31:29] + VLANID[28:16] + MAC[15:0]		*
	*===============================================================*
	* MAC3:FIELD_VALIDS[31:28] + ACT_ENTRY[27:0]			*
	*===============================================================*
	* MAC4:RSVD[31:28]+VALIDBITS[27:24]+COLPTR[23:8]+PORTNO[7:4]+	*
	*					FIELDVALIDS[3:0]	*
	*===============================================================*/

/*    printk("CMD %x STATUS %x MAC1 %x MAC2 %x\n", REQ_CMD_REG,
		REQ_STATUS_REG, REQ_MAC1_ADDR_REG, REQ_MAC2_ADDR_REG); */

	/* now all hash block registers are direct access from host */
	for (i = 0; i < MAC_HASH_TABLE; i++) {
		/* Issue MEM_READ cmd */
		addr = (unsigned int *)(REQ_CMD_REG + pcimem_base);
		*addr = CMD_MEM_READ | (i << 16);

		/* read STATUS reg - cmd done */
		addr = (unsigned int *)(REQ_STATUS_REG + pcimem_base);
		cmd_done = *addr;

		while (!(cmd_done & 1)) {
			/* read cmd done reg */
			addr = (unsigned int *)(REQ_STATUS_REG + pcimem_base);
			cmd_done = *addr;
		}

		/* get entry valid field */
		addr = (unsigned int *)(REQ_MAC4_ADDR_REG + pcimem_base);
		word4 = *addr; /* word4 reprsents Mac hash data 4th word
						i.e last 4 bytes in mac table */

		if (word4 & 0x08000000) {
			/* Reading MAC1 & MAC2 reg */
			addr = (unsigned int *)(REQ_MAC1_ADDR_REG +
								pcimem_base);
			word1 = *addr;

			addr = (unsigned int *)(REQ_MAC2_ADDR_REG +
								pcimem_base);
			word2 = *addr;

			/* Reading MAC3 which tells Active/Inactive &
					Static/Dynamic state of entry */
			addr = (unsigned int *)(REQ_MAC3_ADDR_REG +
								pcimem_base);
			word3 = *addr;
			memset(&br_entry, 0, sizeof(br_entry));
			/* copy bridge entries to user space */
			if (br_entry_size +
				sizeof(br_entry) > pent_ops->ent_size) {
				limitReached = 1;
				FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_DEBUG ,
				"BREAK: br_entry_size %u", br_entry_size);
				FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_DEBUG ,
				"total_ent_size %u\n", pent_ops->ent_size);
			}

			/* update mac and state to br_entry */
			fp_convert_le2be((char *)&word1, 4);
			memcpy((UINT *)&br_entry.be_macaddr[0],
						(UINT *)&word1, 4);
			br_entry.be_macaddr[5] = word2 & 0xff;
			br_entry.be_macaddr[4] = (word2 >> 8) & 0xff;
			br_entry.be_vlanid = (word2 >> 16 & 0x3ff);
			/* update active state/inactive */
			if (word3 & 0x04000000 /* BRENTRY_STATUS_BIT */) {
				br_entry.be_state = BRIDGE_ENTRY_INACTIVE;
				FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_DEBUG ,
				"ACTIVE STATE\n");
			} else {
				FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_DEBUG ,
				"INACTIVE STATE\n");
				br_entry.be_state = BRIDGE_ENTRY_ACTIVE;
			}
			/*Update entry acrions*/
			/* extract action entry */
			br_entry.be_actions = ((word3 << 3) | (word2 >> 29)) & 0x7FFFFFFF;
			if (copy_to_user(pent_ops->ent_data+br_entry_size,
					&br_entry, sizeof(nc_bridge_entry_t)))
				temp = sizeof(nc_bridge_entry_t);
			FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_DEBUG ,
						"temp %u\n", temp);
			br_entry_size = br_entry_size + sizeof(nc_bridge_entry_t);
			FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_DEBUG,
				"br entry size %u cnt %lu brentry size %lu\n",
				br_entry_size, (unsigned long)
				br_entry_size/(sizeof(br_entry)),
				(unsigned long)sizeof(br_entry));

			if (limitReached) {
				FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_DEBUG ,
						"memory not enough\n");
				/* break; */
			}
			FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_DEBUG ,
			"MAC1 %x MAC2 %x word3 %x\n", word1, word2, word3);
			hw_br_entry_count++;
		}

		/* Clear STATUS reg by writing all 1's */
		addr = (unsigned int *)(REQ_STATUS_REG + pcimem_base);
		*addr = 0xFFFFFFFF;
		word1 = 0; word2 = 0; word3 = 0; word4 = 0;
		cmd_done = 0;
	}

	/* Copy the new size to users */
	if (copy_to_user(&(pent_ops->ent_size), &br_entry_size,
						sizeof(br_entry_size)))
		return -EFAULT;

	/* Copy the new count to users */
	if (copy_to_user(&(pent_ops->ent_count), &hw_br_entry_count,
						sizeof(hw_br_entry_count)))
		return -EFAULT;
	return 0;
}

/* show only bridge entries */
int
fp_switch_ioctl_entries_ops(struct net_device *dev, struct ifreq *rq, int cmd) {

#if 0
	nc_ioctl_ops_t *pnc_ops = (nc_ioctl_ops_t *) rq->ifr_ifru.ifru_data;
	nc_entries_ops_t *pent_ops = (nc_entries_ops_t *)pnc_ops->io_data;

	FPATH_TRACE(FP_TRACE_CONFIG, FP_LOG_DEBUG,
		("pent_ops->ent_cmd = %d\n", pent_ops->ent_cmd));

#endif

	nc_ioctl_ops_t *pnc_ops = (nc_ioctl_ops_t *) rq->ifr_ifru.ifru_data;
	nc_ioctl_ops_t *ker_mem_pnc_ops;
	nc_entries_ops_t *ker_mem_pent_ops;
	nc_entries_ops_t *pent_ops;

	unsigned int result = 0;


	ker_mem_pnc_ops = kmalloc(sizeof(nc_ioctl_ops_t), GFP_ATOMIC);
	if(ker_mem_pnc_ops == NULL){
		FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "%s: can not allocate the memory for NC_OPS!\n", __func__);	
		return -EFAULT;
	}

	if( copy_from_user(ker_mem_pnc_ops, /* to */	pnc_ops, /* from */	sizeof(nc_ioctl_ops_t)) /* how many bytes */ )
	{
		FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "%s Warning! Can not copy data NC_OPS from user space!\n", __func__);
		result = -EFAULT;
		goto memory_err_1;
	}

	pnc_ops = ker_mem_pnc_ops;

	ker_mem_pent_ops = kmalloc(sizeof(nc_entries_ops_t), GFP_ATOMIC);
	if(ker_mem_pent_ops == NULL){
		FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "%s: can not allocate the memory for ENT_OPS!\n", __func__);	
		result = -EFAULT;
		goto memory_err_1;
	}

	if (copy_from_user(ker_mem_pent_ops, /* to */ pnc_ops->io_data, /* from */ sizeof(nc_entries_ops_t)) /* how many bytes */ ){
		FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "%s Warning! Can not copy data ENT_OPS from user space!\n", __func__);
		result = -EFAULT;
		goto memory_err_2;
	}

	pent_ops = ker_mem_pent_ops;

	if(pent_ops->ent_data == NULL){
		FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "%s in ENT_OPS struct field *ent_data is NULL!\n", __func__);
		result = -EFAULT;
		goto memory_err_2;
	}



	switch (pent_ops->ent_cmd) {
	case NC_GET_BRIDGE_ENTRIES:
	case NC_GET_BRIDGE_ENTRIES_EXT:
	case NC_GET_BRIDGE_ENTRIES_TABLE:
	{
/* #if ((defined(SW_HASH_EN)) || (defined(HW_HASH_EN))) */
#if 1
		fp_get_hw_bridge_entries(pent_ops);
#else
		int i, j, limitReached = 0, tSize = 0;
		nc_bridge_entry_t br_entry;
		fp_bridgeEntry_t *list = FP_NULL, peEntry; /* table elements */
		UINT *brEntryPtr = FP_NULL;

		/* to avoid warnings */
		brEntryPtr = (UINT *)list;
		peEntry.next = FP_NULL;

		for (i = 0; i < FP_MAX_BRTBL; i++) {
			for (j = 0; j < BRIDGE_HASH_SIZE; j++) {
				list = (fp_bridgeEntry_t *)
					(((UCHAR *)fp_bridge_hash_table->table)
					+ (j * 4));
				fp_read_hw_data((UCHAR *)&brEntryPtr,
					(ULONG)list, 4);
				list = (fp_bridgeEntry_t *)brEntryPtr;
				/* FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO,
					"Count :%d list %x brEntryPtr %x\n",
					j, (unsigned int)list,
					(unsigned int)brEntryPtr); */
				/* Condition for checking first
						node in linked list*/
				if (list == FP_NULL) {
					/* FP_DEBUG(DBGP_FEAT_IOCTL,
						FP_LOG_INFO, "CONTINUE: Count"
						" :%d list %x brEntryPtr %x\n",
						j, (unsigned int)list,
						(unsigned int)brEntryPtr); */
					continue;
				}
				while (list != FP_NULL) {
					FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO,
					"In the While list :%lx\n",
					(ULONG)list);
					if (tSize + sizeof(br_entry) >
							pent_ops->ent_size) {
						limitReached = 1;
						FP_DEBUG(DBGP_FEAT_IOCTL,
							FP_LOG_INFO, "BREAKE:"
							" Count :%d list %lx"
							"brEntryPtr %lx\n", j,
							(ULONG)list,
							(ULONG)brEntryPtr);
						/* break; */
					}
					fp_read_hw_data((UCHAR *)&peEntry,
						(ULONG)list, sizeof(peEntry));
					list = &peEntry;
					FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO,
						"In the While read h/w:%lx data"
						"%02x%02x%02x%02x%02x%02x\n",
						(ULONG)list,
						peEntry.macEntry.macAddr[0],
						peEntry.macEntry.macAddr[1],
						peEntry.macEntry.macAddr[2],
						peEntry.macEntry.macAddr[3],
						peEntry.macEntry.macAddr[4],
						peEntry.macEntry.macAddr[5]);
					updateBridgeEntry(list,  &br_entry);
					if (copy_to_user(pent_ops->ent_data +
						tSize, &br_entry,
						sizeof(br_entry)))
						return -EFAULT;
					tSize += sizeof(br_entry);
					list = list->next;
					FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO,
						"In the While list next %lx\n",
						(ULONG)list);
				}
				if (limitReached)
					break;
			} /* End of Hash Table */
		} /* End of Toatal tables */
		FPATH_TRACE(FP_TRACE_CONFIG, FP_LOG_DEBUG,
			("pent_ops->ent_size=%d tSize=%d\n",
			pent_ops->ent_size, tSize));
		/* Copy the new size to users */
		if (copy_to_user(&(pent_ops->ent_size), &tSize, sizeof(tSize)))
			return -EFAULT;
#endif
		}
	break;

	case NC_GET_BRIDGE_ENTRIES_COUNT:
/* #if ((defined(SW_HASH_EN)) || (defined(HW_HASH_EN))) */
#if 1
	{
		int entries_count = fp_rc_stats.hw_hash_entry_count;
		if (copy_to_user(pent_ops->ent_data, &entries_count, sizeof(entries_count)) ){
			result = -EFAULT;
			goto memory_err_2;
		}
	}
#else
	{
		int entries_count = 0;
		fp_read_hw_data((UCHAR *)&entries_count,
			(BASE_ADDR + PE_STRUCT_BTABLE_ADDR) + 4, 4);
		if (copy_to_user(pent_ops->ent_data, &entries_count,
				sizeof(entries_count)))
			return -EFAULT;
	}
#endif /* ((defined(SW_HASH_EN)) || (defined(HW_HASH_EN))) */
	break;

	case NC_GET_VLAN_ENTRIES_COUNT:
	break;

	case NC_GET_VLAN_ENTRIES:
	case NC_GET_VLAN_ENTRIES_EXT:
	case NC_GET_VLAN_ENTRIES_TABLE:
	{
		int i, j, limitReached = 0, tSize = 0;
		nc_bridge_entry_t br_entry;
		/* table elements */
		fp_bridgeEntry_t *list = FP_NULL, peEntry;
		UINT *vlanEntryPtr = FP_NULL;

		/* to avoid warnings */
		vlanEntryPtr = (UINT *)list;
		peEntry.next = FP_NULL;
		for (i = 0; i < FP_MAX_VLAN_GRPTBL; i++) {
			for (j = 0; j < BRIDGE_HASH_SIZE; j++) {
				list = (fp_bridgeEntry_t *)(((UCHAR *)
					fp_vlan_bridge_hash_table->table) +
					(j * 4));
				fp_read_hw_data((UCHAR *)&vlanEntryPtr,
					(ULONG)list, 4);
				list = (fp_bridgeEntry_t *)vlanEntryPtr;
				/* Condition for checking
					first node in linked list*/
				if (list == FP_NULL)
					continue;
				while (list != FP_NULL) {
					if (tSize + sizeof(br_entry) >
							pent_ops->ent_size) {
						limitReached = 1;
						/* break; */
					}
					fp_read_hw_data((UCHAR *)&peEntry,
						(ULONG)list, sizeof(peEntry));
					list = &peEntry;
					updateBridgeEntry(list, &br_entry);
					if ( copy_to_user(pent_ops->ent_data+tSize, &br_entry, sizeof(br_entry)) ){
						result = -EFAULT;
						goto memory_err_2;
					}
					tSize += sizeof(br_entry);
					list = list->next;
					FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO,
							"Vlan Tables %d\n", i);
				}
				if (limitReached)
					break;
			} /* End of Hash Table */
			FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO,
						"Vlan Tables %d\n", i);
		} /* End of Toatal tables */
		FPATH_TRACE(FP_TRACE_CONFIG, FP_LOG_DEBUG,
				("pent_ops->ent_size=%d tSize=%d\n",
						pent_ops->ent_size, tSize));
		/* Copy the new size to users */
		if (copy_to_user(&(pent_ops->ent_size), &tSize, sizeof(tSize)) ){
			result = -EFAULT;
			goto memory_err_2;
		}
	}
	break;
	}
memory_err_2:
	kfree(ker_mem_pent_ops);
memory_err_1:
	kfree(ker_mem_pnc_ops);

	return result;
}

