/*
 * switchApi_blocks_ctrlm.c -This file contains function to write various
   vlaues to switch API functions.
 *
 * Copyright (c) 2007-2008, Imagination Technologies.
 * All rights reserved.
 */

/*************************************************************************/ /*!
@File
@Title          switchApi_blocks_ctrlm.c
@Copyright      Copyright (c) Milandr. All Rights Reserved
@Description    contins functn to write various vlaues to switch API functions.
@License        Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

#include <linux/version.h>
#include <linux/types.h>
#include <linux/module.h>
#include <linux/netdevice.h>
#include <linux/timer.h>
#include "fp_types.h"
#include "fp_ioctl.h"
#include "fp_common_msg_header.h"
#include "fp_common_ctlm.h"
#include "fp_port.h"
#include "switchApi_blocks_ctrlm.h"
#include "fp_switch.h"
#include "fp_switch_api.h"
#include "fp_ltc.h"
#include "fp_library.h"

/*************************************************************************
 * Description    : Main function to set tlite config parameters
 *************************************************************************/
int
pcm_switchApi_config_ioctl(struct net_device *dev, struct ifreq *rq, int cmd)
{
	unsigned char *devv = NULL;
	uint action_entry;
	INT64 ret2;
	INT64 *iptr2=NULL;
	nc_ioctl_ops_t /* *pnc_ops,*/ *kmem_nc_ops;

	unsigned int result = 0;
 	pcm_msg_header_t /**pmsg,*/ *kmem_pcm_msg_header;
	switch_br_entry_add_params_t *entry_add, *kmem_entry_add;
	switch_br_entry_add_ports_params_t *kmem_entry_add_ports_param;
	switch_br_entry_port_tag_chg_params_t *ptagchg, *kmem_ptagchg;
	switch_br_entry_farward_action_chg_params_t *kmem_pfachg;
	UINT timediff = 0;
	struct global_s *kmem_Global_br_entry;
	struct port_s *kmem_PortInfo;
	switch_fp_mac_entry_params_t *kmem_fp_mac_entry_params;

 	kmem_nc_ops = kmalloc(sizeof(nc_ioctl_ops_t), GFP_KERNEL );
	if(kmem_nc_ops == FP_NULL){
		FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "%s: can not allocate the memory for NC_OPS!\n", __func__);
		return -EFAULT;
	}

	if( copy_from_user(kmem_nc_ops, (nc_ioctl_ops_t*)rq->ifr_ifru.ifru_data, sizeof(nc_ioctl_ops_t) ) ){
		result = -EFAULT;
		goto mem_error_1;
	}

	//pnc_ops = kmem_nc_ops;

	kmem_pcm_msg_header = kmalloc(sizeof(pcm_msg_header_t), GFP_KERNEL );
	if(kmem_pcm_msg_header == FP_NULL){
		FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "%s: can not allocate the memory for PCM_MSG_HEADER!\n", __func__);
		result = -EFAULT;
		goto mem_error_1;
	}

	if( copy_from_user(kmem_pcm_msg_header, (pcm_msg_header_t*)kmem_nc_ops->io_data, sizeof(pcm_msg_header_t)) ){
		result = -EFAULT;
		goto mem_error_2;
	}

	switch (kmem_pcm_msg_header->pcmh_type)
    {
		case PCM_MSG_SWITCH_BR_ENTRY_ADD:

#if SWITCHAPI_DEBUG
			FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "%s:PCM_MSG_SWITCH_BR_ENTRY_ADD\n", __func__);
#endif

			//switch_br_entry_add_params_t *entry_add = (switch_br_entry_add_params_t *)(((UCHAR *)pnc_ops->io_data) + sizeof(pcm_msg_header_t));


			kmem_entry_add = kmalloc( sizeof(switch_br_entry_add_params_t), GFP_KERNEL );
			if(kmem_entry_add == FP_NULL){
				result = -EFAULT;
				goto mem_error_2;
			}

			if( copy_from_user( kmem_entry_add, ( (switch_br_entry_add_params_t *)(((UCHAR *)kmem_nc_ops->io_data) + sizeof(pcm_msg_header_t)) ), sizeof(switch_br_entry_add_params_t) ) ){
				result = -EFAULT;
				kfree(kmem_entry_add);
				goto mem_error_2;
			}

			entry_add = kmem_entry_add;

			FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO,
					"switch_br_entry_add vlanid = %d"
					" fwdportList = %d untagList = %d"
					" ucastHitAct = %d mcastHitAct = %d"
					" ucastMissAct = %d mcastMissAct = %d"
					" mstpAct = %d\n",
					entry_add->swbea_vlanId,
					entry_add->swbea_fwdPortList,
					entry_add->swbea_untagList,
					entry_add->swbea_ucastHitAct,
					entry_add->swbea_mcastHitAct,
					entry_add->swbea_ucastMissAct,
					entry_add->swbea_mcastMissAct,
					entry_add->swbea_mstpAct);


			/*bridge entry add*/
			if( fp_br_entry_add(
					entry_add->swbea_vlanId,
					entry_add->swbea_fwdPortList,
					entry_add->swbea_untagList,
					entry_add->swbea_ucastHitAct,
					entry_add->swbea_mcastHitAct,
					entry_add->swbea_ucastMissAct,
					entry_add->swbea_mcastMissAct,
					entry_add->swbea_mstpAct) ){
				result = -EFAULT;
			}
			kfree(kmem_entry_add);
		break;

	case PCM_MSG_SWITCH_BR_ENTRY_SEARCH:

		ret2 = fp_br_entry_search(kmem_pcm_msg_header->pcmh_value);
		iptr2 = &ret2;
		if( copy_to_user(kmem_nc_ops->io_data, iptr2, sizeof(UINT64)) ){
			result = -EFAULT;
		}
		break;

	case PCM_MSG_SWITCH_BR_ENTRY_ADD_PORTS:

#if SWITCHAPI_DEBUG
		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "%s:PCM_MSG_SWITCH_BR_ENTRY_ADD_PORTS\n", __func__);
#endif
			//switch_br_entry_add_ports_params_t *entry_add =	(switch_br_entry_add_ports_params_t *) (((UCHAR *)pnc_ops->io_data) + sizeof(pcm_msg_header_t));

		kmem_entry_add_ports_param = kmalloc( sizeof(switch_br_entry_add_ports_params_t), GFP_KERNEL );
		if(kmem_entry_add_ports_param == FP_NULL){
			result = -EFAULT;
			goto mem_error_2;
		}

		if( copy_from_user( kmem_entry_add_ports_param, ( (switch_br_entry_add_ports_params_t *)(((UCHAR *)kmem_nc_ops->io_data) + sizeof(pcm_msg_header_t)) ), sizeof(switch_br_entry_add_ports_params_t) ) ){
			result = -EFAULT;
			kfree(kmem_entry_add_ports_param);
			goto mem_error_2;
		}

		//entry_add = kmem_entry_add;

		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "switch_br_entry_add_ports vlanid = %d portList = 0x%x untagList = 0x%x\n",
			kmem_entry_add_ports_param->swbeap_vlanId,
			kmem_entry_add_ports_param->swbeap_portList,
			kmem_entry_add_ports_param->swbeap_untagList);

		/*bridge entry add ports*/
		if( fp_br_entry_add_ports(kmem_entry_add_ports_param->swbeap_vlanId, kmem_entry_add_ports_param->swbeap_portList, kmem_entry_add_ports_param->swbeap_untagList) ){
			result = -EFAULT;
		}
		kfree(kmem_entry_add_ports_param);
		break;

	case PCM_MSG_SWITCH_BR_ENTRY_PORTLIST_GET:

#if SWITCHAPI_DEBUG
		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "%s:PCM_MSG_SWITCH_BR_ENTRY_PORTLIST_GET\n", __func__);
#endif

		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "switch_br_entry_portlist_get vlanid = %d\n", kmem_pcm_msg_header->pcmh_value);

		action_entry = 0;
		/*get bridge entry port list */

		action_entry = fp_br_entry_portlist_get(kmem_pcm_msg_header->pcmh_value);

		if( copy_to_user(kmem_nc_ops->io_data, &action_entry, sizeof(int)) ){
			result = -EFAULT;
		}
	break;


	case PCM_MSG_SWITCH_BR_ENTRY_UNTAGLIST_GET:

#if SWITCHAPI_DEBUG
		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "%s:PCM_MSG_SWITCH_BR_ENTRY_UNTAGLIST_GET\n", __func__);
#endif

		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "switch_br_entry_untaglist_get vlanid = %d\n", kmem_pcm_msg_header->pcmh_value);
		action_entry = 0;
		/*bridge entry untag list */
		/* action_entry = fp_br_entry_portlist_get(baseaddrr,
							pmsg->pcmh_value); */
		action_entry = fp_br_entry_untaglist_get(kmem_pcm_msg_header->pcmh_value);

		if( copy_to_user(kmem_nc_ops->io_data, &action_entry, sizeof(int)) ){
			result = -EFAULT;
		}
		break;

	case PCM_MSG_SWITCH_BR_ENTRY_PORT_TAG_CHG:

#if SWITCHAPI_DEBUG
		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "%s:PCM_MSG_SWITCH_BR_ENTRY_PORT_TAG_CHG\n", __func__);
#endif

		//switch_br_entry_port_tag_chg_params_t *ptagchg = (switch_br_entry_port_tag_chg_params_t *) (((UCHAR *)pnc_ops->io_data) + sizeof(pcm_msg_header_t));

		kmem_ptagchg = kmalloc(sizeof(switch_br_entry_port_tag_chg_params_t), GFP_KERNEL );
		if(kmem_ptagchg == FP_NULL){
			result = -EFAULT;
			goto mem_error_2;
		}

		if( copy_from_user( kmem_ptagchg, ( (switch_br_entry_port_tag_chg_params_t *) (((UCHAR *)kmem_nc_ops->io_data) + sizeof(pcm_msg_header_t)) ), sizeof(switch_br_entry_port_tag_chg_params_t) ) ){
			result = -EFAULT;
			kfree(kmem_ptagchg);
			goto mem_error_2;
		}

		ptagchg = kmem_ptagchg;

		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "switch_br_entry_port_tag_chg vlanid = %d portList = 0x%X tagValue = 0x%X\n",
				ptagchg->ptchg_vlanId, ptagchg->ptchg_portList, ptagchg->ptchg_tagValue);

		/*bridge entry port tag change*/
		if( fp_br_entry_port_tag_chg(ptagchg->ptchg_vlanId, ptagchg->ptchg_portList, ptagchg->ptchg_tagValue) ){
			result = -EFAULT;
		}
		kfree(kmem_ptagchg);
		break;

	case PCM_MSG_SWITCH_BR_ENTRY_FWD_ACTION_GET:

#if SWITCHAPI_DEBUG
		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "%s:PCM_MSG_SWITCH_BR_ENTRY_FWD_ACTION_GET\n", __func__);
#endif

		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "switch_br_entry_fwd_action_get vlanid = %d\n", kmem_pcm_msg_header->pcmh_value);

		action_entry = 0;
		/*get bridge entry farward action */
		action_entry = fp_br_entry_fwd_action_get(kmem_pcm_msg_header->pcmh_value);
		if( copy_to_user(kmem_nc_ops->io_data, &action_entry, sizeof(int)) ){
			result = -EFAULT;
		}
		break;

	case PCM_MSG_SWITCH_PORT_CONFIG_GET:

#if SWITCHAPI_DEBUG
		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "%s:PCM_MSG_SWITCH_PORT_CONFIG_GET\n", __func__);
#endif
		kmem_PortInfo = kmalloc(sizeof(struct port_s), GFP_KERNEL );

		if(kmem_PortInfo == FP_NULL){
			result = -EFAULT;
			goto mem_error_2;
		}

		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "switch_port_config_get portNo = %d, struct index %d\n", kmem_pcm_msg_header->pcmh_value, (kmem_pcm_msg_header->pcmh_value - 1) );

			/*get port config */
		//fp_port_config_get(baseaddrr, devv, pmsg->pcmh_value, &PortInfo);
		fp_port_config_get(devv, kmem_pcm_msg_header->pcmh_value, kmem_PortInfo);

		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "shutdown = %d\ntpid = 0x%x\naft = %d\nfallback_bd_id = %d\nblock_state = %d\ntrusted = %d\ntcsel_table = %d\npcp2tc_table = %d\ndef_cfi = %d\ndef_pri = %d\ndef_tc = %d\ntc2cos_table = %d\npindex = %d\ncsr_untag_from_btable = %d",
												kmem_PortInfo->shutdown,
												kmem_PortInfo->tpid, kmem_PortInfo->aft,
												kmem_PortInfo->fallback_bd_id,
												kmem_PortInfo->block_state, kmem_PortInfo->trusted,
												kmem_PortInfo->tcsel_table, kmem_PortInfo->pcp2tc_table,
												kmem_PortInfo->def_cfi, kmem_PortInfo->def_pri,
												kmem_PortInfo->def_tc, kmem_PortInfo->tc2cos_table,
												kmem_PortInfo->pindex, kmem_PortInfo->csr_untag_from_btable );

		if( copy_to_user(kmem_nc_ops->io_data, kmem_PortInfo, sizeof(struct port_s)) ){
			result = -EFAULT;
		}

		kfree(kmem_PortInfo);
		break;

	case PCM_MSG_SWITCH_GLOBAL_BR_ENTRY_CONFIG_GET:

#if SWITCHAPI_DEBUG
		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "%s:PCM_MSG_SWITCH_GLOBAL_BR_ENTRY_CONFIG_GET\n", __func__);
#endif

		kmem_Global_br_entry = kmalloc(sizeof(struct global_s), GFP_KERNEL );
		if(kmem_Global_br_entry == FP_NULL){
			result = -EFAULT;
			goto mem_error_2;
		}

		fp_memset( kmem_Global_br_entry, 0, sizeof(struct global_s) );

		/*get global bridge entry config */
		fp_global_br_entry_config_get( kmem_Global_br_entry );

		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "In switch case switch_global_br_entry_config_get\n");

		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "l2_splPunt_enable = %x, fwdPortList = %x, untagList = %x, ucastHitAct = %x, mcastHitAct = %x, ucastMissAct = %x, mcastMissAct = %x, mstpAct = %x",
			kmem_Global_br_entry->l2_special_punt_enable,
			kmem_Global_br_entry->fallback_bd_entry.forward_list,
			kmem_Global_br_entry->fallback_bd_entry.untag_list,
			kmem_Global_br_entry->fallback_bd_entry.ucast_hit_action,
			kmem_Global_br_entry->fallback_bd_entry.mcast_hit_action,
			kmem_Global_br_entry->fallback_bd_entry.ucast_miss_action,
			kmem_Global_br_entry->fallback_bd_entry.mcast_miss_action,
			kmem_Global_br_entry->fallback_bd_entry.mstp_action );

		if( copy_to_user(kmem_nc_ops->io_data, kmem_Global_br_entry, sizeof(struct global_s)) ){
			result = -EFAULT;
		}

		kfree(kmem_Global_br_entry);
		break;

	case PCM_MSG_SWITCH_BR_ENTRY_FWD_ACTION_CHANGE:

#if SWITCHAPI_DEBUG
		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "%s:PCM_MSG_SWITCH_BR_ENTRY_FWD_ACTION_CHANGE\n", __func__);
#endif
		kmem_pfachg = kmalloc( sizeof(switch_br_entry_farward_action_chg_params_t), GFP_KERNEL );
		if(kmem_pfachg == FP_NULL){
			result = -EFAULT;
			goto mem_error_2;
		}

		if( copy_from_user(kmem_pfachg, ( (switch_br_entry_farward_action_chg_params_t *)(((UCHAR *)kmem_nc_ops->io_data) + sizeof(pcm_msg_header_t)) ), sizeof (switch_br_entry_farward_action_chg_params_t) ) ){
			result = -EFAULT;
			kfree(kmem_pfachg);
			goto mem_error_2;
		}

		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "switch_br_entry_fwd_action_change vlanid = %d field = %d Value = %d\n",
			kmem_pfachg->pfachg_vlanId, kmem_pfachg->pfachg_field, kmem_pfachg->pfachg_value );

		/*bridge entry farward action change*/
		if( fp_br_entry_fwd_action_change( kmem_pfachg->pfachg_vlanId, kmem_pfachg->pfachg_field, kmem_pfachg->pfachg_value ) ){
			result = -EFAULT;
		}
		kfree(kmem_pfachg);
		break;

	case PCM_MSG_SWITCH_PORT_CONFIG:

#if SWITCHAPI_DEBUG
		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "%s:PCM_MSG_SWITCH_PORT_CONFIG\n", __func__);
#endif
		kmem_PortInfo = kmalloc( sizeof(struct port_s), GFP_KERNEL );
		if(kmem_PortInfo == FP_NULL){
			result = -EFAULT;
			goto mem_error_2;
		}

		if( copy_from_user( kmem_PortInfo, ( (struct port_s *)(((UCHAR *)kmem_nc_ops->io_data) + sizeof(pcm_msg_header_t)) ), sizeof(struct port_s) ) ){
			result = -EFAULT;
			kfree(kmem_PortInfo);
			goto mem_error_2;
		}

		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "switch_port_config portNo = %d\n", kmem_pcm_msg_header->pcmh_value);
		/*port config */
		//fp_port_config(baseaddrr, pmsg->pcmh_value, Port);
		if( fp_port_config(kmem_pcm_msg_header->pcmh_value, kmem_PortInfo) ){
			result = -EFAULT;
		}
		kfree(kmem_PortInfo);
	break;

	case PCM_MSG_SWITCH_PVLAN_CONFIG:
        FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "%s:PCM_MSG_SWITCH_PVLAN_CONFIG\n", __func__);
        kmem_PortInfo = kmalloc( sizeof(struct port_s), GFP_KERNEL );
        if(kmem_PortInfo == FP_NULL){
        	result = -EFAULT;
        	goto mem_error_2;
        }

        if( copy_from_user(kmem_PortInfo, ((struct port_s *)(((UCHAR *)kmem_nc_ops->io_data) + sizeof(pcm_msg_header_t))), sizeof(struct port_s) ) ){
        	result = -EFAULT;
        	kfree(kmem_PortInfo);
        	goto mem_error_2;
        }

		/*pvlan config */
        if( fp_pvlan_config(1, kmem_pcm_msg_header->pcmh_value, kmem_PortInfo) ){
        	result = -EFAULT;
        }

		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "switch_pvlan_config portNo = %d\n", kmem_pcm_msg_header->pcmh_value);
		kfree(kmem_PortInfo);
		break;

	case PCM_MSG_SWITCH_PVLAN_DEL_CONFIG:
        FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "%s:PCM_MSG_SWITCH_PVLAN_CONFIG\n", __func__);
        kmem_PortInfo = kmalloc( sizeof(struct port_s), GFP_KERNEL );
        if(kmem_PortInfo == FP_NULL){
        	result = -EFAULT;
        	goto mem_error_2;
        }

        if( copy_from_user( kmem_PortInfo, ( (struct port_s *)(((UCHAR *)kmem_nc_ops->io_data) + sizeof(pcm_msg_header_t)) ), sizeof(struct port_s) ) ){
        	result = -EFAULT;
        	kfree(kmem_PortInfo);
        	goto mem_error_2;
        }

		/*pvlan config */
        if( fp_pvlan_config(0, kmem_pcm_msg_header->pcmh_value, kmem_PortInfo) ){
        	result = -EFAULT;
        }
		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "switch_pvlan_config portNo = %d\n",	kmem_pcm_msg_header->pcmh_value);
		kfree(kmem_PortInfo);
		break;

	case PCM_MSG_SWITCH_GLOBAL_BR_ENTRY_CONFIG:

#if SWITCHAPI_DEBUG
		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "%s:PCM_MSG_SWITCH_GLOBAL_BR_ENTRY_CONFIG\n", __func__);
#endif
		kmem_Global_br_entry = kmalloc( sizeof(struct global_s), GFP_KERNEL );
		if(kmem_Global_br_entry == FP_NULL){
			result = -EFAULT;
			goto mem_error_2;
		}

		if( copy_from_user(kmem_Global_br_entry, ( (struct global_s *)(((UCHAR *)kmem_nc_ops->io_data) + sizeof(pcm_msg_header_t)) ), sizeof(struct global_s) ) ){
			result = -EFAULT;
			kfree(kmem_Global_br_entry);
			goto mem_error_2;
		}

		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "switch_global_br_entry_config\n");

		/*global bridge entry config */
		if( fp_global_br_entry_config( kmem_Global_br_entry) ){
			result = -EFAULT;
		}
		kfree(kmem_Global_br_entry);
		break;

	case PCM_MSG_SWITCH_BR_ENTRY_DEL:

#if SWITCHAPI_DEBUG
		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "%s:PCM_MSG_SWITCH_BR_ENTRY_DEL\n", __func__);
#endif
		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "switch_br_entry_del vlanId = %d\n", kmem_pcm_msg_header->pcmh_value);
		/*delete bridge entry */
		if( fp_br_entry_del(kmem_pcm_msg_header->pcmh_value) ){
			result = -EFAULT;
		}
		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "result = fp_br_entry_del = %d\n", result);
		break;

	case PCM_MSG_SWITCH_BR_ENTRY_DEL_PORTS:

#if SWITCHAPI_DEBUG
		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "%s:PCM_MSG_SWITCH_BR_ENTRY_DEL\n", __func__);
#endif
		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "switch_br_entry_del_ports vlanId = %d portList = 0x%X\n",	kmem_pcm_msg_header->pcmh_value, kmem_pcm_msg_header->pcmh_flags);
		/*delete bridge entry ports */
		if( fp_br_entry_del_ports(kmem_pcm_msg_header->pcmh_value, kmem_pcm_msg_header->pcmh_flags) ){
			result = -EFAULT;
		}
		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "switch_br_entry_del_ports return value %x\n", result);
		break;

	case PCM_MSG_SWITCH_MAC_ENTRY_ADD:
		kmem_fp_mac_entry_params = kmalloc( sizeof(switch_fp_mac_entry_params_t), GFP_KERNEL );
		if(kmem_fp_mac_entry_params == FP_NULL){
			result = -EFAULT;
			goto mem_error_2;
		}
		if(copy_from_user(kmem_fp_mac_entry_params, ( (switch_fp_mac_entry_params_t *)(((UCHAR *)kmem_nc_ops->io_data) + sizeof(pcm_msg_header_t)) ), sizeof(switch_fp_mac_entry_params_t) ) ){
			result = -EFAULT;
			kfree(kmem_fp_mac_entry_params);
			goto mem_error_2;
		}
		/*mac entry add */
		action_entry = fp_static_mac_entry_add(kmem_fp_mac_entry_params->addr, kmem_fp_mac_entry_params->swbea_vlanId, kmem_fp_mac_entry_params->swbea_fwdPortList, kmem_fp_mac_entry_params->swbea_hitAct, kmem_fp_mac_entry_params->swbea_tc);
		if( copy_to_user( kmem_nc_ops->io_data, &action_entry, sizeof(int) ) ){
			result = -EFAULT;
		}
		kfree(kmem_fp_mac_entry_params);
		break;

	case PCM_MSG_SWITCH_MAC_ENTRY_DEL:
		kmem_fp_mac_entry_params = kmalloc( sizeof(switch_fp_mac_entry_params_t), GFP_KERNEL );
		if(kmem_fp_mac_entry_params == FP_NULL){
			result = -EFAULT;
			goto mem_error_2;
		}

		if( copy_from_user( kmem_fp_mac_entry_params, ( (switch_fp_mac_entry_params_t *)(((UCHAR *)kmem_nc_ops->io_data) + sizeof(pcm_msg_header_t)) ), sizeof(switch_fp_mac_entry_params_t) ) ){
			result = -EFAULT;
			kfree(kmem_fp_mac_entry_params);
			goto mem_error_2;
		}

		/*mac entry delete */
		fp_mac_entry_del(kmem_fp_mac_entry_params->addr, kmem_pcm_msg_header->pcmh_value);
		break;

	case PCM_MSG_SWITCH_MAC_ENTRY_SEARCH:
		kmem_fp_mac_entry_params = kmalloc( sizeof(switch_fp_mac_entry_params_t), GFP_KERNEL );
		if(kmem_fp_mac_entry_params == FP_NULL){
			result = -EFAULT;
			goto mem_error_2;
		}

		if( copy_from_user( kmem_fp_mac_entry_params, ( (switch_fp_mac_entry_params_t *)(((UCHAR *)kmem_nc_ops->io_data) + sizeof(pcm_msg_header_t)) ), sizeof(switch_fp_mac_entry_params_t) ) ){
			result = -EFAULT;
			kfree(kmem_fp_mac_entry_params);
			goto mem_error_2;
		}

		/*mac entry search */
		action_entry = fp_mac_entry_search(kmem_fp_mac_entry_params->addr, kmem_pcm_msg_header->pcmh_value);

		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "case PCM_MSG_SWITCH_MAC_ENTRY_SEARCH: action_entry = %x\n", action_entry);

		if( copy_to_user(kmem_nc_ops->io_data, &action_entry, sizeof(int)) ){
			result = -EFAULT;
		}
		kfree(kmem_fp_mac_entry_params);
		break;

	case PCM_MSG_SWITCH_MAC_ENTRY_UPDATE:
		kmem_fp_mac_entry_params = kmalloc( sizeof(switch_fp_mac_entry_params_t), GFP_KERNEL );
		if(kmem_fp_mac_entry_params == FP_NULL){
			result = -EFAULT;
			goto mem_error_2;
		}

		if( copy_from_user( kmem_fp_mac_entry_params, ( (switch_fp_mac_entry_params_t *)(((UCHAR *)kmem_nc_ops->io_data) + sizeof(pcm_msg_header_t)) ), sizeof(switch_fp_mac_entry_params_t) ) ){
			result = -EFAULT;
			kfree(kmem_fp_mac_entry_params);
			goto mem_error_2;
		}

		/* mac entry update */
		action_entry = fp_mac_entry_update(kmem_pcm_msg_header->pcmh_flags, kmem_fp_mac_entry_params->addr, kmem_pcm_msg_header->pcmh_value);

		if(copy_to_user(kmem_nc_ops->io_data, &action_entry, sizeof(int)) ){
			result = -EFAULT;
		}
		kfree(kmem_fp_mac_entry_params);
		break;

	case PCM_MSG_SWITCH_LTC_GET_TIMEDIFF:
		timediff = fp_ltc_gettime(kmem_pcm_msg_header->pcmh_value, kmem_pcm_msg_header->pcmh_flags);
		if ((int)timediff < 0){
				timediff = 0;
		}
		/*Send time difference to user */
		if( copy_to_user(kmem_nc_ops->io_data, &timediff, sizeof(UINT)) ){
			result = -EFAULT;
		}
		break;

	case PCM_MSG_SWITCH_LTC_HIF_SECONDS:
		fp_ltc_hif_config_time((kmem_pcm_msg_header->pcmh_value), kmem_pcm_msg_header->pcmh_flags-1, LTC_HIF_SECONDS);
		FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:ltc_set_HIF1seconds\n", __func__);
		FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "dev_no:\t%x seconds:\t%x\n", kmem_pcm_msg_header->pcmh_value, kmem_pcm_msg_header->pcmh_flags);
		break;

	case PCM_MSG_SWITCH_LTC_HIF_NANOSECONDS:
		fp_ltc_hif_config_time((kmem_pcm_msg_header->pcmh_value), kmem_pcm_msg_header->pcmh_flags-1, LTC_HIF_NANOSECONDS);
		FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:ltc_set_HIF1seconds\n", __func__);
		FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "dev_no:\t%x seconds:\t%x\n", kmem_pcm_msg_header->pcmh_value, kmem_pcm_msg_header->pcmh_flags);
		break;

	default:
		FP_DEBUG(DBGP_FEAT_L2API, FP_LOG_INFO, "%s:Unknown message type\n", __func__);
		result = -EOPNOTSUPP;
		break;
	}
mem_error_2:
	kfree(kmem_pcm_msg_header);
mem_error_1:
	kfree(kmem_nc_ops);

	return result;

} /*End of pcm_switchApi_config_ioctl() Function*/
