/*************************************************************************/ /*
@File
@Title          fp_drv_mcu.h
@Copyright      Copyright (c) Milandr. All Rights Reserved
@Description    Structures for hw switch
@License        Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

/*!
 *  @file fp_drv_mcu.h
 *  L2 switch data structures
 */

#ifndef  _FP_DRV_MCU_H_
#define  _FP_DRV_MCU_H_

#include "fp_macros.h"

#define DRV_MCU_TRANSFER_DATA_REGS  2

#define DRV_MCU_STATUS_REG_MASK		3
#define DRV_MCU_STATUS_REG_NO_FLAGS	0
#define DRV_MCU_STATUS_REG_CMPL		1
#define DRV_MCU_STATUS_REG_ERR		2

#define DRV_MCU_CMD_REG_CMD_MASK	0xFF

enum drv_mcu_transfer_cmd_e {
	NO_CMD = 0,
	GET_FW_ID,
	GET_TEMP,
	GET_SFP_EVENTS,
	GET_SFP_DATA,
	SET_PHY_RESET,
	TEST_CMD,
	GET_ETH_PHY_REG_I2C,
	SET_ETH_PHY_REG_I2C,
	MAX_DRV_MCU_CMD
};

enum drv_mcu_transfer_e {
    STATE_NO_CMD = 0,          /*!< No command to execute in CMD_REG */
    STATE_WAIT_STATUS,		   /*!< Waiting flags CMPL or ERR in STATUS_REG, when command is executed */
    STATE_CLEAR_CMD,           /*!< Need to clear CMD_REG */
    STATE_CLEAR_STATUS         /*!< Waiting clear flags CMPL or ERR in STATUS_REG */
};

struct drv_mcu_transfer_s {
  	enum drv_mcu_transfer_e 	state;  /*!< state of the transfer */
	enum drv_mcu_transfer_cmd_e cmd;	/*!< command */
	uint32_t					flags;	/*!< status flags */
  	uint32_t 					result; /*!< 0 - complete; other values - processing of the command */
  	uint32_t 					data[ DRV_MCU_TRANSFER_DATA_REGS ];  /*!< get data from MCU, if transfer complete successfully */
  	uint32_t					write_counter;	/*!< number of trials to write (for debug) */
  	uint32_t					read_counter;	/*!< number of trials to write (for debug) */
};


void init_drv_mcu_ops_struct( void );
int32_t fp_drv_mcu_ops_init_timer( int32_t init );

uint32_t start_drv_mcu_transfer( enum drv_mcu_transfer_cmd_e user_cmd, uint32_t *data );
uint32_t is_transfer_complete( uint32_t *flags );
uint32_t get_transfer_data( uint32_t *data, uint32_t size_in_words );

#endif /*End of _FP_DRV_MCU_H_ */
