/*************************************************************************/ /*!
@File
@Title          fp_ether.h
@Copyright      Copyright (c) Milandr. All Rights Reserved
@Description    data structures for layer 2 header
@License        Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

#ifndef _FP_ETHER_H_
#define _FP_ETHER_H_

#define ETH_ALEN                6            /* Octets in one ethernet addr */
/* Ethernet protocol ID's */
//#define ETHERTYPE_PUP         0x0200       /* Xerox PUP */
#define ETHERTYPE_IP            0x0800       /* IP */
#define ETHERTYPE_ARP           0x0806       /* Address resolution */
#define ETHERTYPE_REVARP        0x8035       /* Reverse ARP */

/* tcpdump header (ether.h) defines ETHER_HDRLEN) */
#ifndef ETHER_HDRLEN 
#define ETHER_HDRLEN 14
#endif

/*
 * This is a name for the 48 bit ethernet address available on many
 * systems
 */ 
struct ether_addr {

  UCHAR ether_addr_octet[ETH_ALEN];

};

/* Ethernet header structure */
struct ether_header {

  UCHAR  ether_dhost[ETH_ALEN];      /* destination eth addr */
  UCHAR  ether_shost[ETH_ALEN];      /* source ether addr    */
  USHORT ether_type;                 /* packet type ID field */

};

/* bridge classification function */
int fp_bridge_classifier(pktBuf_t *in_pkt_buff);

/* Ethernet handle function */
int fp_ethernet_handle(pktBuf_t *in_pkt_buff);

/* Mac to hash calculation */
USHORT fp_ether_mactohash(UCHAR *mac_address, USHORT vLan, USHORT hashMask);

#endif /*End of _FP_ETHER_H_ File*/
                 
