/*************************************************************************/ /*!
@File
@Title          fp_global_addr_map.h
@Copyright      Copyright (c) Milandr. All Rights Reserved
@Description    define all hardware block base addresses
@License        Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

#ifndef _FP_GLOBAL_ADDR_MAP_H_
#define _FP_GLOBAL_ADDR_MAP_H_

#include <linux/kernel.h>

#include "fp_types.h"

#define CBUS_ADDR_BASE			             0x00000000
/* -------------------------------- EMAC ------------------------------------ */
#define EMAC1_BASE_ADDR                      0x660000
#define EMAC1_END_OFFSET                     0x66FFFF

#define EMAC2_BASE_ADDR                      0x670000
#define EMAC2_END_OFFSET                     0x67FFFF

#define EMAC3_BASE_ADDR                      0x680000
#define EMAC3_END_OFFSET                     0x68FFFF

#define EMAC4_BASE_ADDR                      0x690000
#define EMAC4_END_OFFSET                     0x69FFFF

#define EMAC5_BASE_ADDR                      0x6A0000
#define EMAC5_END_OFFSET                     0x6AFFFF

#define EMAC6_BASE_ADDR                      0x6B0000
#define EMAC6_END_OFFSET                     0x6BFFFF

#define EMAC7_BASE_ADDR                      0x6C0000
#define EMAC7_END_OFFSET                     0x6CFFFF

#define EMAC8_BASE_ADDR                      0x6D0000
#define EMAC8_END_OFFSET                     0x6DFFFF

#define EMAC9_BASE_ADDR                      0x6E0000
#define EMAC9_END_OFFSET                     0x6EFFFF

#define EMAC10_BASE_ADDR                     0x6F0000
#define EMAC10_END_OFFSET                    0x6FFFFF

#define EMAC11_BASE_ADDR                     0x760000
#define EMAC11_END_OFFSET                    0x76FFFF

#define EMAC12_BASE_ADDR                     0x710000
#define EMAC12_END_OFFSET                    0x71FFFF

#define EMAC13_BASE_ADDR                     0x720000
#define EMAC13_END_OFFSET                    0x72FFFF

#define EMAC14_BASE_ADDR                     0x730000
#define EMAC14_END_OFFSET                    0x73FFFF

#define EMAC15_BASE_ADDR                     0x740000
#define EMAC15_END_OFFSET                    0x74FFFF

#define EMAC16_BASE_ADDR                     0x750000
#define EMAC16_END_OFFSET                    0x75FFFF

#define EMAC_NETWORK_CONTROL                 0x000
#define EMAC_NETWORK_CONTROL_TX_ENABLE       0x008

#define EMAC_NETWORK_CONFIG                  0x004

#define EMAC_NETWORK_CONFIG_FULL_DUPLEX_BIT_OFFSET  1
#define EMAC_NETWORK_CONFIG_GIGABIT_MODE_ENABLE_BIT_OFFSET  10

#define EMAC_NETWORK_CONFIG_FULL_DUPLEX_BIT             ( 1 << EMAC_NETWORK_CONFIG_FULL_DUPLEX_BIT_OFFSET )
#define EMAC_NETWORK_CONFIG_GIGABIT_MODE_ENABLE_BIT     ( 1 << EMAC_NETWORK_CONFIG_GIGABIT_MODE_ENABLE_BIT_OFFSET )


#define EMAC_NETWORK_STATUS                  0x008

#define EMAC_NETWORK_STATUS_PCS_LINK_STATE_BIT_OFFSET 0
#define EMAC_NETWORK_STATUS_MAC_FULL_DUPLEX_BIT_OFFSET 3


#define EMAC_NETWORK_STATUS_PCS_LINK_STATE_BIT  ( 1 << EMAC_NETWORK_STATUS_PCS_LINK_STATE_BIT_OFFSET )
#define EMAC_NETWORK_STATUS_MAC_FULL_DUPLEX_BIT ( 1 << EMAC_NETWORK_STATUS_MAC_FULL_DUPLEX_BIT_OFFSET )

#define EMAC_INT_STATUS                      0x024

#define EMAC_INT_STATUS_PCS_AUTONEG_COMPLETE_BIT_OFFSET     16
#define EMAC_INT_STATUS_LINK_CHANGE_BIT_OFFSET              9

#define EMAC_INT_STATUS_PCS_AUTONEG_COMPLETE_BIT            ( 1 << EMAC_INT_STATUS_PCS_AUTONEG_COMPLETE_BIT_OFFSET )
#define EMAC_INT_STATUS_LINK_CHANGE_BIT                     ( 1 << EMAC_INT_STATUS_LINK_CHANGE_BIT_OFFSET )

#define EMAC_TX_PAUSE_QUANTUM                0x03C

#define EMAC_JUMBO_MAX_LENGTH                0x048

#define EMAC_SPEC_ADD1_BOTTOM                0x088
#define EMAC_SPEC_ADD1_TOP                   0x08C

#define EMAC_PCS_CONTROL                     0x200
#define EMAC_PCS_STATUS                      0x204
#define EMAC_PCS_AN_ADV                      0x210
#define EMAC_PCS_AN_LP_BASE                  0x214
#define EMAC_PCS_AN_EXP                      0x218
#define EMAC_PCS_AN_NP_TX                    0x21C
#define EMAC_PCS_AN_LP_NP                    0x220
#define EMAC_PCS_AN_EXT_STATUS               0x23C

#define EMAC_PCS_CONTROL_RESTART_AUTONEG_BIT_OFFSET 9
#define EMAC_PCS_CONTROL_RESTART_AUTONEG_BIT        ( 1 << EMAC_PCS_CONTROL_RESTART_AUTONEG_BIT_OFFSET )

/* -------------------------------- EGPI ------------------------------------ */
#define EGPI1_BASE_ADDR                      0x780000
#define EGPI1_END_OFFSET                     0x78FFFF

#define EGPI2_BASE_ADDR                      0x790000
#define EGPI2_END_OFFSET                     0x79FFFF

#define EGPI3_BASE_ADDR                      0x7A0000
#define EGPI3_END_OFFSET                     0x7AFFFF

#define EGPI4_BASE_ADDR                      0x7B0000
#define EGPI4_END_OFFSET                     0x7BFFFF

#define EGPI5_BASE_ADDR                      0x7C0000
#define EGPI5_END_OFFSET                     0x7CFFFF

#define EGPI6_BASE_ADDR                      0x7D0000
#define EGPI6_END_OFFSET                     0x7DFFFF

#define EGPI7_BASE_ADDR                      0x7E0000
#define EGPI7_END_OFFSET                     0x7EFFFF

#define EGPI8_BASE_ADDR                      0x7F0000
#define EGPI8_END_OFFSET                     0x7FFFFF

#define EGPI9_BASE_ADDR                      0x800000
#define EGPI9_END_OFFSET                     0x80FFFF

#define EGPI10_BASE_ADDR                     0x810000
#define EGPI10_END_OFFSET                    0x81FFFF

#define EGPI11_BASE_ADDR                     0x820000
#define EGPI11_END_OFFSET                    0x82FFFF

#define EGPI12_BASE_ADDR                     0x830000
#define EGPI12_END_OFFSET                    0x83FFFF

#define EGPI13_BASE_ADDR                     0x840000
#define EGPI13_END_OFFSET                    0x84FFFF

#define EGPI14_BASE_ADDR                     0x850000
#define EGPI14_END_OFFSET                    0x85FFFF

#define EGPI15_BASE_ADDR                     0x860000
#define EGPI15_END_OFFSET                    0x86FFFF

#define EGPI16_BASE_ADDR                     0x870000
#define EGPI16_END_OFFSET                    0x87FFFF

#define EGPI_BMU1_PHY_LOW_WATERMARK          0x104
#define EGPI_BMU1_PHY_HIGH_WATERMARK         0x108
#define EGPI_BMU2_PHY_LOW_WATERMARK          0x10C
#define EGPI_BMU2_PHY_HIGH_WATERMARK         0x110
#define EGPI_FW_CONTROL                      0x114

#define BMU1_BASE_ADDR                       0x100000
#define BMU1_END_OFFSET                      0x10FFFF

#define BMU2_BASE_ADDR                       0x700000
#define BMU2_END_OFFSET                      0x70FFFF

#define BMU_BUF_CNT                          0x0014
#define BMU_UCAST_CONFIG                     0x0008

#define HIF1_BASE_ADDR                       0x640000
#define HIF1_END_OFFSET                      0x64FFFF

#define HGPI_BASE_ADDR                       0x650000
#define HGPI_END_OFFSET                      0x65FFFF

#define LMEM_BASE_ADDR                       0x200000
#define LMEM_END_OFFSET                      0x23FFFF

#define LMEM2_BASE_ADDR                      0x400000
#define LMEM2_END_OFFSET                     0x43FFFF

#define TMU_BASE_ADDR                        0x600000
#define TMU_END_OFFSET                       0x60FFFF

#define TLITE_BASE_ADDR                      0x600000
#define TLITE_END_OFFSET                     0x61FFFF

#define CLASS_HW_BASE_ADDR                   0x620000
#define CLASS_HW_END_OFFSET                  0x62FFFF

#define CLASS_DAMACHASH_HOST_CMD_REG		 0x390
#define CLASS_DAMACHASH_HOST_MAC_ADDR1_REG	 0x394
#define CLASS_DAMACHASH_HOST_MAC_ADDR2_REG	 0x398
#define CLASS_DAMACHASH_HOST_MAC_ADDR3_REG	 0x39C
#define CLASS_DAMACHASH_HOST_MAC_ADDR4_REG	 0x3a0
#define CLASS_DAMACHASH_HOST_MAC_ADDR5_REG	 0x3a4
#define CLASS_DAMACHASH_HOST_ENTRY_REG		 0x3a8
#define CLASS_DAMACHASH_HOST_STATUS_REG		 0x3aC
#define CLASS_DAMACHASH_HOST_DIRECT          0x3b0

#define CLASS_DAVLANHASH_HOST_CMD_REG        0x3E4
#define CLASS_DAVLANHASH_HOST_MAC_ADDR1_REG  0x3E8
#define CLASS_DAVLANHASH_HOST_MAC_ADDR2_REG  0x3EC
#define CLASS_DAVLANHASH_HOST_MAC_ADDR3_REG  0x3F0
#define CLASS_DAVLANHASH_HOST_MAC_ADDR4_REG  0x3F4
#define CLASS_DAVLANHASH_HOST_MAC_ADDR5_REG  0x3F8
#define CLASS_DAVLANHASH_HOST_ENTRY_REG      0x3FC
#define CLASS_DAVLANHASH_HOST_STATUS_REG     0x400
#define CLASS_DAVLANHASH_HOST_DIRECT         0x404

#define FREE_VLAN_LIST_ENTRIES               0x42c
#define FREE_VLAN_LIST_HEAD_PTR              0x430
#define FREE_VLAN_LIST_TAIL_PTR              0x434

#define CLASS_DOS_CONTRL1                    0x280
#define CLASS_DOS_CONTRL2                    0x284
#define CLASS_DOS_TCP_FLAGCHK_COMBO_VALUE1   0x288
#define CLASS_DOS_ICMPV4_MAX_PKTLEN          0x28C
#define CLASS_PUNT_COS                       0x4C0
#define CLASS_ING_AFULL_THRES                0x290

/* HW MAC TABLE CMDS */
enum
{
    CMD_INIT = 1,
    CMD_ADD,
    CMD_DEL,
    CMD_UPDATE,
    CMD_SEARCH,
    CMD_MEM_READ,
    CMD_MEM_WRITE,
    CMD_FLUSH
};

enum
{
    REG_MAC1 = 0,
    REG_MAC2,
    REG_MAC3,
    REG_MAC4
};

// Биты статусного регистра REQ1_STATUS_REG
#define STAT_CMD_DONE                        0x00000001
#define STAT_ENTR_NFOUND                     0x00000002
#define STAT_INIT_DONE                       0x00000004
#define STAT_ENTR_ADD                        0x00000008
#define STAT_ENTR_MATCH                      0x00000010
#define STAT_REG_CLR                         0xFFFFFFFF

#define CLASS2_HW_BASE_ADDR                  0x770000
#define CLASS2_HW_END_OFFSET                 0x77FFFF

#define WSP_GLOBAL_BASE_ADDR                 0x630000
#define WSP_GLOBAL_END_OFFSET                0x63FFFF
#define WSP_SYS_GENERIC_CONTROL              0x0020

#define SHP_BPS_MODE                         0x00
#define SHP_PPS_MODE                         0x01
#define ING_SHP_MIN_RATE                     1
#define ING_SHP_MAX_RATE                     1000000
#define ING_SHP0_UNIT_MASK                   0x01
#define ING_SHP0_TYPE_MASK                   0x0C
#define ING_SHP1_UNIT_MASK                   0x02
#define ING_SHP1_TYPE_MASK                   0x70
#define ING_SHP_CLKDIV_MASK                  0x1E
#define ING_SHP_MAX_CREDIT_MASK              0x3FFFFF00
#define ING_SHP0_UNIT_POS                    0
#define ING_SHP1_UNIT_POS                    1
#define ING_SHP0_TYPE_POS                    2
#define ING_SHP1_TYPE_POS                    4
#define ING_SHP_ALL                          0
#define ING_SHP_BCAST                        1
#define ING_SHP_MCAST                        2
#define ING_SHP_MAX_WT                       0x7FF
#define ING_SHP_MAX_CLKDIV                   0xF
#define ING_SHP_MAX_CLKDIV_VAL               (1 << ING_SHP_MAX_CLKDIV)
#define ING_SHP_CLKDIV_POS                   1
#define ING_SHP_MAX_CREDIT_POS               8

#define CSR_CNTX_IND_DONE                    0x04

#define EMAC_RX_BIT_MASK                     0x4
#define EMAC_TX_BIT_MASK                     0x8
#define MAX_ING_SHAPERS                      2

#define EMAC_NUMS                            16

#define CLASS_HW_PORT0_STRUC1                0x46c
#define CLASS_HW_PORT0_STRUC2                0x470
#define CLASS_HW_PORT1_STRUC1                0x474
#define CLASS_HW_PORT1_STRUC2                0x478
#define CLASS_HW_PORT2_STRUC1                0x47c
#define CLASS_HW_PORT2_STRUC2                0x480
#define CLASS_HW_PORT3_STRUC1                0x484
#define CLASS_HW_PORT3_STRUC2                0x488
#define CLASS_HW_PORT4_STRUC1                0x48c
#define CLASS_HW_PORT4_STRUC2                0x490
#define CLASS_HW_PORT5_STRUC1                0x494
#define CLASS_HW_PORT5_STRUC2                0x498
#define CLASS_HW_PORT6_STRUC1                0x49c
#define CLASS_HW_PORT6_STRUC2                0x4a0
#define CLASS_HW_PORT7_STRUC1                0x4a4
#define CLASS_HW_PORT7_STRUC2                0x4a8
#define CLASS_HW_PORT8_STRUC1                0x548
#define CLASS_HW_PORT8_STRUC2                0x54c
#define CLASS_HW_PORT9_STRUC1                0x550
#define CLASS_HW_PORT9_STRUC2                0x554
#define CLASS_HW_PORT10_STRUC1               0x558
#define CLASS_HW_PORT10_STRUC2               0x55c
#define CLASS_HW_PORT11_STRUC1               0x560
#define CLASS_HW_PORT11_STRUC2               0x564
#define CLASS_HW_PORT12_STRUC1               0x568
#define CLASS_HW_PORT12_STRUC2               0x56c
#define CLASS_HW_PORT13_STRUC1               0x570
#define CLASS_HW_PORT13_STRUC2               0x574
#define CLASS_HW_PORT14_STRUC1               0x578
#define CLASS_HW_PORT14_STRUC2               0x57c
#define CLASS_HW_PORT15_STRUC1               0x580
#define CLASS_HW_PORT15_STRUC2               0x584
#define CLASS_HW_PORT16_STRUC1               0x588
#define CLASS_HW_PORT16_STRUC2               0x58c

#define CLASS_QOS_PORT01_TC_SEL               0x018
#define CLASS_QOS_PORT23_TC_SEL               0x01C
#define CLASS_QOS_PORT45_TC_SEL               0x020
#define CLASS_QOS_PORT67_TC_SEL               0x024
#define CLASS_QOS_PORT89_TC_SEL               0x5D8
#define CLASS_QOS_PORT1011_TC_SEL             0x5DC
#define CLASS_QOS_PORT1213_TC_SEL             0x5E0
#define CLASS_QOS_PORT1415_TC_SEL             0x5E4
#define CLASS_QOS_PORT1617_TC_SEL             0x5E8

#define CLASS_QOS_PORT0_PCP2TC_MAP            0x044
#define CLASS_QOS_PORT1_PCP2TC_MAP            0x048
#define CLASS_QOS_PORT2_PCP2TC_MAP            0x04C
#define CLASS_QOS_PORT3_PCP2TC_MAP            0x050
#define CLASS_QOS_PORT4_PCP2TC_MAP            0x054
#define CLASS_QOS_PORT5_PCP2TC_MAP            0x058
#define CLASS_QOS_PORT6_PCP2TC_MAP            0x05C
#define CLASS_QOS_PORT7_PCP2TC_MAP            0x060
#define CLASS_QOS_PORT8_PCP2TC_MAP            0x5B4
#define CLASS_QOS_PORT9_PCP2TC_MAP            0x5B8
#define CLASS_QOS_PORT10_PCP2TC_MAP           0x5BC
#define CLASS_QOS_PORT11_PCP2TC_MAP           0x5C0
#define CLASS_QOS_PORT12_PCP2TC_MAP           0x5C4
#define CLASS_QOS_PORT13_PCP2TC_MAP           0x5C8
#define CLASS_QOS_PORT14_PCP2TC_MAP           0x5CC
#define CLASS_QOS_PORT15_PCP2TC_MAP           0x5D0
#define CLASS_QOS_PORT16_PCP2TC_MAP           0x5D4

#define CLASS_QOS_PORT_PID2TC_MAP             0x064
#define CLASS_QOS_PORT1_PID2TC_MAP            0x5EC

#define CLASS_QOS_PORT0_TC2COS_MAP            0x068
#define CLASS_QOS_PORT1_TC2COS_MAP            0x06C
#define CLASS_QOS_PORT2_TC2COS_MAP            0x070
#define CLASS_QOS_PORT3_TC2COS_MAP            0x074
#define CLASS_QOS_PORT4_TC2COS_MAP            0x078
#define CLASS_QOS_PORT5_TC2COS_MAP            0x07C
#define CLASS_QOS_PORT6_TC2COS_MAP            0x080
#define CLASS_QOS_PORT7_TC2COS_MAP            0x084
#define CLASS_QOS_PORT8_TC2COS_MAP            0x590
#define CLASS_QOS_PORT9_TC2COS_MAP            0x594
#define CLASS_QOS_PORT10_TC2COS_MAP           0x598
#define CLASS_QOS_PORT11_TC2COS_MAP           0x59C
#define CLASS_QOS_PORT12_TC2COS_MAP           0x5A0
#define CLASS_QOS_PORT13_TC2COS_MAP           0x5A4
#define CLASS_QOS_PORT14_TC2COS_MAP           0x5A8
#define CLASS_QOS_PORT15_TC2COS_MAP           0x5AC
#define CLASS_QOS_PORT16_TC2COS_MAP           0x5B0

#define CLASS_QOS_GLOBAL_DSCP2TC_MAP0         0x028
#define CLASS_QOS_GLOBAL_DSCP2TC_MAP1         0x02C
#define CLASS_QOS_GLOBAL_DSCP2TC_MAP2         0x030
#define CLASS_QOS_GLOBAL_DSCP2TC_MAP3         0x034
#define CLASS_QOS_GLOBAL_DSCP2TC_MAP4         0x038
#define CLASS_QOS_GLOBAL_DSCP2TC_MAP5         0x03C
#define CLASS_QOS_GLOBAL_DSCP2TC_MAP6         0x040

#define CLASS_GLOBAL_CFG                      0x4AC
#define CLASS_GLOBAL_CFG1                     0x7E8

#define TMU_INQ_WATERMARK                     0x004
#define TMU_AFULL_THRES                       0x11C

#define TMU_CNTX_ADDR                         0x2F4
#define TMU_CNTX_DATA                         0x2F8
#define TMU_CNTX_CMD                          0x2FC

#define TMU_SCH0_CTRL                         0x000
#define TMU_SCH0_Q0_WGHT                      0x020
#define TMU_SCH0_Q1_WGHT                      0x024
#define TMU_SCH0_Q2_WGHT                      0x028
#define TMU_SCH0_Q3_WGHT                      0x02C
#define TMU_SCH0_Q4_WGHT                      0x030
#define TMU_SCH0_Q5_WGHT                      0x034
#define TMU_SCH0_Q6_WGHT                      0x038
#define TMU_SCH0_Q7_WGHT                      0x03C

#define TMU_SCH0_Q_ALLOC0                     0x040
#define TMU_SCH0_Q_ALLOC1                     0x044
#define TMU_SCH0_BIT_RATE                     0x048
#define TMU_SCH0_POS                          0x054

#define TMU_SCH1_CTRL                         0x300
#define TMU_SCH1_Q0_WGHT                      0x320
#define TMU_SCH1_Q1_WGHT                      0x324
#define TMU_SCH1_Q2_WGHT                      0x328
#define TMU_SCH1_Q3_WGHT                      0x32C
#define TMU_SCH1_Q4_WGHT                      0x330
#define TMU_SCH1_Q5_WGHT                      0x334
#define TMU_SCH1_Q6_WGHT                      0x338
#define TMU_SCH1_Q7_WGHT                      0x33C

#define TMU_SCH1_Q_ALLOC0                     0x340
#define TMU_SCH1_Q_ALLOC1                     0x344
#define TMU_SCH1_BIT_RATE                     0x348

#define TMU_SHP0_CTRL                         0x800     
#define TMU_SHP0_WGHT                         0x804  
#define TMU_SHP0_MAX_CREDIT                   0x808
#define TMU_SHP0_CTRL2                        0x80C
#define TMU_SHP0_MIN_CREDIT                   0x810
#define TMU_SHP0_STATUS                       0x814

extern const uint32_t emac[EMAC_NUMS];
extern const uint32_t egpi[EMAC_NUMS];
extern const uint32_t hwPhyCounters[17][14];

extern const uint32_t hwPortStruct1[2][17];
extern const uint32_t hwPortStruct2[2][17];
extern const uint32_t hwQosTcSel[2][17];
extern const uint32_t hwQosPcp2Tc[2][17];
extern const uint32_t hwQosTc2Cos[2][17];
extern const uint32_t Tmu_Scheduler_Addr_Regs[13];
extern const uint32_t Tmu_Shaper_Addr_Regs[6];

#endif /*End of _FP_GLOBAL_ADDR_MAP_H_ File*/
