/*************************************************************************/ /*!
@File
@Title          fp_macros.h
@Copyright      Copyright (c) Milandr. All Rights Reserved
@Description    This file contains macros definitions
@License        Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

#ifndef  _FP_MACROS_H_
#define  _FP_MACROS_H_

//146 - max number of 1514 bytes packets in LMEM
//146-16-1=129 packets in all HOST queues if available 2048 buffers in LMEM
#define INQ_ALMOST_FULL_THRESHOLD				146

//#define INQ_ALMOST_FULL_THRESHOLD				128
//#define INQ_ALMOST_FULL_THRESHOLD				64

#define MAX_BMU_BUFFERS							2048
//#define MAX_BMU_BUFFERS							1024
#define JUMBO_MAX_LENGTH						9000

#define MAX_FRAME_SIZE                          (2 * 1024)
#define MAX_JUMBO_FRAME_SIZE					(9 * 1024)
/************************************************************************
                             MACROS FOR SWAPPING
************************************************************************/
/* Macro definition changing byte order*/
/* enbale below macro for NGW & hw */
//#define DEBUG_HOST                            1
#define DEBUG_PCI                               1
#define HW_STATS_EN                             1
#define BASE_ADDR                               0x00000000

/************************************************************************
                             MACROS FOR SWITCH TABLES
************************************************************************/
#define FP_MAX_PORTS                            17
#define FP_MAX_SWITCH_PORTS                     17

#define NUM_OF_OPTIC_PORTS                      4
#define SIZE_OF_SFP_READ_DATA_IN_BYTES          64

#define FP_PCI_DEV_PHY_CONTROL                  0x814
#define FP_PCI_DEV_MSI_CAP_04H_REG              0x54
#define FP_PCI_DEV_MSI_CAP_08H_REG              0x58
#define FP_PCI_DEV_MSI_CAP_0CH_REG              0x5C
#define FP_PCI_DEV_MSI_CAP_0EH_REG              0x5E

#define FP_CMD_REG                              FP_PCI_DEV_PHY_CONTROL
#define FP_STATUS_REG                           FP_PCI_DEV_MSI_CAP_0CH_REG
#define FP_DATA1_REG                            FP_PCI_DEV_MSI_CAP_08H_REG
#define FP_DATA2_REG                            FP_PCI_DEV_MSI_CAP_04H_REG

#define FWD_PORT_LIST_MASK                      0x1FFFF
#define HOST_PORT_NO                            16            // Physical Port Number of Host
#define HIF_PORT_NUM                            16             // Physical Port Number of Host

#define FP_MAX_BRTBL                            1
#define FP_MAX_VLAN_GRPTBL                      1
#define VLANID_MASK                             0x1FFF
#define BIT_SET_VLAN_1F_VLANTBL                 0x00000100

#define FP_CLASS_MAX_NUMBER                     2   /*Max number of class hw blocks*/

/* VLAN (BD) Hash table action entry map:*
 *=======================================*
 * [19:00]         - forward port list
 * [39:20]         - untag list
 * [42:40]         - ucast_hit_action
 * [45:43]         - mcast_hit_action
 * [48:46]         - ucast_miss_action
 * [51:49]         - mcast_miss_action
 * [54:52]         - mstp
 *=======================================*/
#define BRENTRY_FWD_PORT_LIST_START_POS         0
#define BRENTRY_UNTAG_LIST_START_POS            20
#define BRENTRY_UCAST_HIT_ACT_START_POS         40
#define BRENTRY_MCAST_HIT_ACT_START_POS         43
#define BRENTRY_UCAST_MISS_ACT_START_POS        46
#define BRENTRY_MCAST_MISS_ACT_START_POS        49
#define BRENTRY_MSTP_HIT_ACT_START_POS          52

#define BRENTRY_FWD_PORT_LIST_MASK   FWD_PORT_LIST_MASK      /* 00 - 19 */
#define BRENTRY_UNTAG_LIST_MASK      0x000000FFFFF00000      /* 20 - 39 */
#define BRENTRY_UCAST_HIT_ACT_MASK   0x0000070000000000      /* 40 - 42 */
#define BRENTRY_MCAST_HIT_ACT_MASK   0x0000380000000000      /* 43 - 45 */
#define BRENTRY_UCAST_MISS_ACT_MASK  0x0001C00000000000      /* 46 - 48 */
#define BRENTRY_MCAST_MISS_ACT_MASK  0x000E000000000000      /* 49 - 51 */
#define BRENTRY_MSTP_HIT_ACT_MASK    0x0070000000000000      /* 52 - 54 */

#define BRENTRY_ACTION_ENTRY_MASK               0x007FFFFFFFFFFFFF      /* 00 - 54 */

#define CMD_ENTRY_PORTS_START_POS               16
#define CMD_ENTRY_FIELD_VALID_MASK              0x00001F00

/* !VLAN TABLE MACROS */

/* MAC 2F HASH TABLE MACROs */
/*         MAC Hash table action entry map            *
 * [19:00]           - forward port list              *
 * [22:20]           - TC                             *
 * [25:23]           - fwd_action (ACTION FIELD)      *
 * [26]              - cut through                    *
 * [28:27]           - reserved for feature block     *
 * [29]              - fresh                          *
 * [30]              - static                         */
#define MAC_2F_ENTRY_FWD_PORT_LIST_MASK         FWD_PORT_LIST_MASK
#define MAC_2F_ENTRY_TC_MASK  			 		0x00700000
#define MAC_2F_ENTRY_FWD_ACT_MASK               0x03800000
#define MAC_2F_ENTRY_CUT_THROUGH_MASK           0x04000000
#define MAC_2F_ENTRY_FRESH_MASK                 0x20000000
#define MAC_2F_ENTRY_STATIC_MASK                0x40000000

#define MAC_2F_ACTION_ENTRY_MASK                0x7FFFFFFF

#define MAC_2F_ENTRY_FWD_PORT_LIST_START_POS    0
#define MAC_2F_ENTRY_TC_START_POS 		 		20
#define MAC_2F_ENTRY_FWD_ACT_START_POS          23
#define MAC_2F_ENTRY_CUT_THROUGH_START_POS      26
#define MAC_2F_ENTRY_FRESH_BIT_POS              29
#define MAC_2F_ENTRY_STATIC_BIT_POS             30
/* !MAC 2F HASH TABLE MACROs */

#define VLAN_TABLE_VLAID_COLLPTR_FLAG			4
#define VLAN_TABLE_VLAID_ENTRY_FLAG				8


/************************************************************************
                             MACROS FOR SPANNING STATE
************************************************************************/

#define  fp_ntohs(in_usValue) (((0xFF) & (in_usValue >> 8)) | ((0xFF00) & (in_usValue << 8)))
#define  fp_ntohl(in_usValue) (((0xFF) & (in_usValue >> 24)) | ((0xFF00) & (in_usValue >> 8)) | ((0xFF0000) & (in_usValue << 8)) | ((0xFF000000) & (in_usValue << 24)))

/************************************************************************
                             HW INFO
************************************************************************/
#define FP_MAX_CLASS_BUFFERS                    4
#define FP_CLASS_BUFFSIZE                       0x100
#define FP_CLSBUF_HEADROOM                      0x70
#define FP_CLASS_MASK                           0xf

#ifdef HW_LMEM_ONLY
#define CSR_LMEM_SEC_BUF_OFFSET                 0x10
#define FP_CLASS_DATASIZE                       0x60
#else
#define FP_CLASS_DATASIZE                       0x70
#endif

#define FP_DDR_PKTDATA_OFFSET                   0x100
#define ERR_BIT_1                               0x00010000
#define ERR_BIT_2                               0xF0000000
#define ERR_BIT_3                               0xf0000000

#define BRIDGE_HASH_SIZE                        0x10
#define MAX_BRIDGE_ENTRIES                      0xf
#define MAX_VLAN_ENTRIES                        0xf
#define	BR_HASH_TABLE_POOL_SIZE                 0x40
#define BR_ENTRIES_POOL_SIZE                    0x200

#ifdef ESI32
#ifdef HW_LMEM_BRCM
#define PE_STRUCT_BTABLE_ADDR                   0x00000400
#define PE_CONFIG_BRPOOL_HEAD                   0x00000418
//#define PE_CONFIG_RTABLE_ADDR                 0x00000640
//#define PE_CONFIG_L2TPTABLE_ADDR              0x00000650
//#define PE_CONFIG_PPTPTABLE_ADDR              0x00000660
#define PE_CONFIG_BTABLE_ADDR                   0x00000440
#define PE_CONFIG_BENTRY_ADDR                   0x00000480
#define PE_CONFIG_VTABLE_ADDR                   0x00000680
#define PE_CONFIG_VENTRY_ADDR                   0x000006C0
//#define PE_IFSTATS_ADDR                       0x00000F8E
//#define PE_FP_STATS_ADDR                      0x00000DA0
#define PE_CONFIG_SWITCH_PORT_ADDR              0x000009A0
#define PE_CONFIG_SWITCH_GLOBAL_ADDR            0x000009A0
//#define PE_CONFIG_SWITCH_STATS_ADDR           0x00000e80
#else /*Else of HW_LMEM_BRCM*/
#define PE_STRUCT_BTABLE_ADDR                   0x00000600
#define PE_CONFIG_BRPOOL_HEAD                   0x00000618
#define PE_CONFIG_RTABLE_ADDR                   0x00000640
#define PE_CONFIG_L2TPTABLE_ADDR                0x00000650
#define PE_CONFIG_PPTPTABLE_ADDR                0x00000660
#define PE_CONFIG_BTABLE_ADDR                   0x00000680
#define PE_CONFIG_BENTRY_ADDR                   0x000006C0
#define PE_CONFIG_VTABLE_ADDR                   0x000008C0
#define PE_CONFIG_VENTRY_ADDR                   0x00000900
#define PE_IFSTATS_ADDR                         0x00000F8E
#define PE_FP_STATS_ADDR                        0x00000DA0
#define PE_CONFIG_SWITCH_PORT_ADDR              0x00000be0  /*each port 392bytes,totalports=3 :TOTAL_SIZE:1176bytes,0x498 */
#define PE_CONFIG_SWITCH_GLOBAL_ADDR            0x00000da0  /* TOTAL_SIZE:864bytes,0x360 */
#define PE_CONFIG_SWITCH_STATS_ADDR             0x00000e80  /* TOTAL_SIZE:450bytes,0x1c2 */
#endif /*End of HW_LMEM_BRCM*/
#else /*Else of ESI32*/
#define PE_CONFIG_IFDATA_ADDR  	                0x8f010480
#define PE_CONFIG_PHYPORT_ADDR                  0x8f010400
#define PE_CONFIG_RTABLE_ADDR                   0x8f0108e0
#define PE_CONFIG_BTABLE_ADDR                   0x8f0108f0
#define PE_IFSTATS_ADDR                         0x8f010840
#define PE_CONFIG_SWITCH_PORT_ADDR              0x84000ef0  /*each port 392bytes,totalports=3 :TOTAL_SIZE:1176bytes,0x498 */
#define PE_CONFIG_SWITCH_GLOBAL_ADDR            0x840013a8  /* TOTAL_SIZE:864bytes,0x360 --using 0x88 bytes(136B) */
#define PE_CONFIG_SWITCH_STATS_ADDR             0x840018c0  /* TOTAL_SIZE:450bytes,0x1c2 */
#endif /*End of ESI32*/


#define PCI2PE_ADDR_MASK                        0x00ffffff
#define PE_CSR_DATA                             0x00620104
#define PE_CSR_ADDR                             0x00620100
#define PE_DMEM_ADDR_OFFSET                     0x8f010000
#define PE_PMEM_ADDR_OFFSET                     0x8f008000

#ifdef ESI32
//#define CLASS_BUS_ACCESS_ADDR                 0x00320228
//#define CLASS_BUS_ACCESS_WDATA                0x0032022c
//#define CLASS_BUS_ACCESS_RDATA                0x00320230
//#define CLASS_BUS_ACCESS_BASE_ADDR            0x00320258
#define CLASS_BUS_WDATA_OFFSET                  0x84000000
#define CLASS_BUS_RDATA_OFFSET                  0x04000000
#endif /*End if ESI32*/


/**********************************************************************
			     PROTOCOL NUMBERS
**********************************************************************
*/
#define PROTOCOL_IP	                            0x0800
#define PROTOCOL_IPV6                           0x86DD
#define PROTOCOL_ARP                            0x0806
#define PROTOCOL_RARP                           0x8035
#define PROTOCOL_PPPOE                          0x8864
#define PROTOCOL_VLAN                           0x8100
#define PROTOCOL_IPX                            0x8137
#define PROTOCOL_PPP_IP                         0x0021
#define FP_ICMP                                 0x1
#define FP_IGMP                                 0x2
#define FP_ESP                                  0x32
#define FP_AH                                   0x33
#define PROTOCOL_PPP_IPV4                       0x0021
#define PROTOCOL_PPP_IPV6                       0x0057
#define FP_IPV4                                 0x4
#define FP_IPV6                                 0x6
#define FP_TCP                                  0x6
#define FP_UDP                                  0x11
#define FP_V6ROUTING_HDR                        0x2B
#define FP_ICMPV6                               0x3A
#define FP_IPPROTO_IPV6                         0x29
#define FP_IPPROTO_GRE                          0x2f
#define FP_IPPROTO_IPIP                         0x4

#ifndef IPPROTO_TCP
#define IPPROTO_TCP                             0x6
#endif

#ifndef IPPROTO_UDP
#define IPPROTO_UDP                             0x11
#endif

#define PPPOE_VER_TYPE_CODE                     0x1100 /* ver=1, type=1, code=0 */
#define FP_MSRP_PROTOCOL                        0x22ea
#define FP_MVRP_PROTOCOL                        0x88f5
#define FP_IEE1722_PROTOCOL                     0x22f0

/**********************************************************************
			     HDR SIZES
**********************************************************************/
#define ETH_PROTO_SIZE                          2
#define ETH_ADDR_SIZE                           6
#define ETH_SMAC_DMAC_SIZE                      12
#define	ETH_HDR_SIZE                            14
#define	VLAN_HDR_SIZE                           4
#define	PPPOE_HDR_SIZE                          8
#define	IPV6_HDR_SIZE                           40
#define	IPV4_HDR_SIZE                           20

/**********************************************************************
       			 BRIDGE
***********************************************************************/
#define BRENTRY_STATIC_BIT                      0x80000000         //This bit represents bridge's MAC or dynamic MAC
                                                        //bridge's MACs will have this bit set

#define BRENTRY_STATUS_BIT                      0x00010000         //dynamic learnt MAC entry's status bit
//Macros For BridgeFlow Match Flags
#define MATCH_MAC_ADDR                          0x1		//0
#define MATCH_MAC_VLAN                          0x2		//1
//Entry Bridge Flow Entry State Flags
#define ENTRY_VALID   		                    0x1		//0 Entry valid
#define ENTRY_INVALID                           0x2		//1 Entry need update from slow path
//Macros for Bridge Flow Action Flags
#define INSERT_VLAN                             0x1                                     //0
#define DELETE_VLAN                             0x2                                     //1
#define DO_MCAST                                0x4                                     //2
#define DO_ROUTING                              0x8                                     //3
//Macros for Bridge Flow Class Notify Flags
#define BRIDGE_ENTRY_INACTIVE                   0x1                                     //0 Not Recieving traffic
#define BRIDGE_ENTRY_ACTIVE                     0x2                                     //1 Recieving traffic
#define STATIC_BRIDGE_ENTRY                     0x1                                     //1 Recieving traffic
#define DYNAMIC_BRIDGE_ENTRY                    0x2                                     //1 Recieving traffic

/* tpid defines */
#define VLAN_TAGGED_FRAME                       0x8100
#define PROVIDER_BRIDGING_FRAME                 0x88a8
#define QinQ_9100_FRAME                         0x9100
#define QinQ_9200_FRAME                         0x9200

#define STPID0                                  0x5500                                  /* Not Used by Anyone in IEEE page */
#define STPID1                                  0x5501                                  /* http://standards.ieee.org/develop/regauth/ethertype/eth.txt */
#define STPID2                                  0x5502
#define STPID1_START_POS                        16

/* PHY IC types */
enum
{
    PHY_IC_ERR = -1,
    PHY_IC_I347AT4 = 0,
    PHY_IC_DP83867,
};

/* PHY media types */
enum
{
    PHY_MEDIA_ERR = -1,
    PHY_MEDIA_COPPER = 0,
    PHY_MEDIA_OPTIC,
};

#endif /*End of  _FP_MACROS_H_ File*/
