/*************************************************************************/ /*!
@File
@Title          fp_netd.h
@Copyright      Copyright (c) Milandr. All Rights Reserved
@Description    data structures Host interfaces
@License        Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

#ifndef  _FP_NETD_H_
#define  _FP_NETD_H_

struct phy_statistic
{
    u64 tx_ok;
    u64 tx_broadcast;
    u64 tx_multicast;
    u64 tx_pause;
    u64 tx_64;
    u64 tx_65_127;
    u64 tx_128_255;
    u64 tx_256_511;
    u64 tx_512_1023;
    u64 tx_1024_1518;
    u64 tx_more_1518;
    u64 tx_underruns;
    u64 tx_single_collision;
    u64 tx_2_15_collision;
    u64 tx_more_15_collision;
    u64 tx_late_collision;
    u64 tx_deferred;

    u64 rx_ok;
    u64 rx_broadcast;
    u64 rx_multicast;
    u64 rx_pause;
    u64 rx_64;
    u64 rx_65_127;
    u64 rx_128_255;
    u64 rx_256_511;
    u64 rx_512_1023;
    u64 rx_1024_1518;
    u64 rx_more_1518;
    u64 rx_less_64;
    u64 rx_excessive_len;
    u64 rx_jabbers;
    u64 rx_fcs_err;
    u64 rx_len_err;
    u64 rx_symbol_err;
    u64 rx_aligment_err;
    u64 rx_recource_err;
    u64 rx_overruns;
    u64 rx_ip_crc_err;
    u64 rx_tcp_crc_err;
    u64 rx_udp_crc_err;

    u64 class_rx;
    u64 class_l3_fail;
    u64 class_ipv4;
    u64 class_ipv6;
    u64 class_crc_err;
    u64 class_ttl_err;
    u64 class_icmp;
    u64 class_igmp;
    u64 class_tcp;
    u64 class_udp;
};

/*nfp device private info structure */
typedef struct fp_dev_info_s
{
    struct net_device               *real_dev;
    struct pci_dev                  *pdev;
    u8                              real_dev_addr[ETH_ALEN];
//    u8                              phy_media;
    u8                              phy_port_no;
    s8                              phy_ic_type;                                  // IC PHY type
    u8                              phy_addr;
    u8                              mdio_port;
    u8                              link_is_up;
    u8                              link_full_duplex;   //current duplex
    u16                             link_speed;         //current speed
    u32                             link_advertising;   //ethernrt phy advertise modes (bits 0 - 6), autoneg enable (bit 7)
    u32                             phy_port_state;     //0 - power off; 1 -power on
    struct net_device_stats         dev_stats;
    struct phy_statistic            phy_stats;
    // struct proc_dir_entry           *dent;

#ifdef HAVE_PTP_1588_CLOCK
    unsigned int                    ref_num;
    struct sk_buff                  *ptp_tx_skb;
    unsigned long                   ptp_tx_start;
    u64                             ns;
#endif /* HAVE_PTP_1588_CLOCK */

}fp_dev_info_t;
extern struct net_device *fp_phy_addr[FP_MAX_PORTS];

/*Devices for fp*/
#define FP_DEV_INFO(x) ((fp_dev_info_t *)(x->ml_priv))

typedef struct nfp_global_config {
    unsigned int fastpath; /* enable or disable fastpath */
    unsigned int notifier; /* enable or disable notifier */
} nfp_global_config_t;
extern nfp_global_config_t fpathConfig;
extern struct net_device  *fp_logical_dev[FP_MAX_PORTS];
extern int nfp_skb_recv(struct sk_buff *skb, struct net_device *dev,
                  struct packet_type* ptype, struct net_device *orig_dev);
int unregister_nfp_device(struct net_device *dev);
int fp_create_logical_interfaces(struct net_device *dev, int max_logical_ports);
extern char *getPciBaseAddr(void *dev);
extern int dump_packet(struct sk_buff *skb);
extern int fp_netif_receive_skb(struct sk_buff *skb);
extern int fp_flow_process(struct sk_buff *skb, struct net_device *txDev);

void nfp_change_rx_flags(struct net_device *dev, int change);
void nfp_dev_set_multicast_list(struct net_device *dev);
int nfp_dev_ioctl(struct net_device *dev, struct ifreq *ifr, int cmd);
int nfp_dev_stop(struct net_device *dev);
int nfp_dev_open(struct net_device *dev);
int nfp_dev_hard_start_xmit(struct sk_buff *skb, struct net_device *dev);
int nfp_dev_rebuild_header(struct sk_buff *skb);
struct net_device *register_nfp_device(struct net_device *real_dev,
						unsigned char phyPortNo);
int fp_config_ioctl(struct net_device *, struct ifreq *rq, int cmd);



#endif /*End of _FP_NETD_H_ File */
