/*************************************************************************/ /*!
@File
@Title          fp_pktbuff.h
@Copyright      Copyright (c) Milandr. All Rights Reserved
@Description    contains packet buff data structures
@License        Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

#ifndef  _FP_PKTBUFF_H_
#define  _FP_PKTBUFF_H_

#include "fp_interface.h"
#include "fp_macros.h"
/* macros for differnent memory allocation pools */
#define HASHSIZE              			12
#define POOLSIZE              			10
#define RAW_BUFF_SIZE         			256
#define PKTBUFF_SIZE          			2048
#define BRIDGE_POOLSIZE       			10
#define ROUTER_POOLSIZE       			10

#define MIN_IP_HEADER_LENGTH			5

/*******************************************************************************
                                   PACKET BUFFER STRUCTURE

The packet will be splitted into two parts  
    1. First segment of the packet ( 128bytes + 8 bytes of header )
	    - CPCPCP 112 bytes of packet data (Ethernet Packet) and 16 bytes of
        header (Internal Structure)
    2. Second segment of the packet ( remaining data + 8 bytes of header )

  Classifier can have max of 4 first segements to do classification
      
  Before classification 
      -------------------------
      |   pktBuff             |
      -------------------------
      |   |    |_____________________________________________________________________
      |    -------------------------------|                                          |
      -------------------------------------------------------------------------------
      |  room for pre-pkt(128bytes) | 12bytes header |   116bytes pkt headers        |
      -------------------------------------------------------------------------------

     CPCPCP 16 bytes of header in the above figure. and 112 bytes of packet header
       
  After classification 
      -------------------------
      |   pktBuff             |
      -------------------------
      |   |    |_____________________________________________________________________
      |    -------------------------------|                                          |
      -------------------------------------------------------------------------------
      |  room for pre-pkt(128bytes) | 12bytes header |   116bytes pkt headers        |
      | Buffer Desc
      -------------------------------------------------------------------------------
*****************************************************************************************************/

typedef struct _pktBuf_t_ {

    /*Buffer related members*/
    void     *pNextBuf;     /*pointer to next buffer of the packet*/
    char     *pDataStart;
    char     *pDataEnd;
    char     *pBufStart;
    char     *pBufEnd;
    USHORT   dataLen;
    USHORT   totaldataLen;
    /* packet parsing related members*/
    void     *pLayer2Hdr;
    void     *pLayer3Hdr;
    void     *pLayer4Hdr;
    void     *pAppHdr;
    /*flags are set during on packet parsing*/
    UINT     parseFlags;	
    /* classifier related members*/
    UINT     classFlags;	 
    void     *pL2MatchEntry;
    void     *pV4MatchEntry;
    void     *pV6MatchEntry;
    USHORT   l2Hash;
    USHORT   v4hash;
    USHORT   v6hash;
    USHORT   ifParseFlags; 
    /*input action related members
    output action related members
    vendor related members*/
    void     *pVendorInfo;
    UINT     mediaInfo;          /*contains VPVC, GEMID, SSID etc.*/ 
    USHORT   ifnum,sid; 
    /* unified */
    USHORT   phyno,fid_index;
    USHORT   vlanid;             /* using for egr vlan membership */
    /*port related members*/
    fp_phyPort_t    *pRxPhyPort; /* receved physical port*/
    fp_IfData_t     *pRxIfPort;  /* Interface RX port*/
    /*trace information*/
    void     *tracePtr;	         /* for debugging purpose*/

} pktBuf_t;     // 88bytes

/*******************************************************************************
 The Classifier Engine has 4 Classfiier buffers and each of size about 256bytes.
 First 128 bytes of data is store into classifier buffer and the remaining
 packet will be stored in DDR memory. Below is the structure format for the
 classifier buffer.
 ******************************************************************************/
typedef struct _bd_clbuf_t_ {
    UCHAR num_cpy;              //no of copies to sent out from RO block
    UCHAR dma_len;              //len to be DMAed to DDR mem
    USHORT src_addr;            //class buf address
    UINT  dst_addr;             //DDR memory address
    UINT  res1;                 //reserved 1
    USHORT  res2;               //reserved 2
    USHORT  tsv;                //time stamp val
} bd_clbuf_t;                   //16 bytes

typedef struct _postPktClsHdr_t_ {
    UCHAR start_data_off;       // data start offset
    UCHAR start_buf_off;        // buffer start offset
    USHORT pkt_length;          // total packet lenght
    UCHAR act_phy;              //action block 7:4 and phyno 3:0
    UCHAR queueno;
    USHORT src_mac_msb;         // indicates src_mac 47:32
    UINT src_mac_lsb;           // indicates src_mac 31:0
    UINT vlanid;                //  vlanid
} postPktClsHdr_t;

typedef struct _postPktClsMcastHdr_t_ {
    UCHAR start_data_off;       // data start offset
    UCHAR start_buf_off;        // buffer start offset
    USHORT pkt_length;          // total packet lenght
    UCHAR act_phy;              //action block 7:4 and phyno 3:0
    UCHAR queueno;
    USHORT src_mac_msb;         // indicates src_mac 47:32
    UINT src_mac_lsb;           // indicates src_mac 31:0
    UINT vlanid;                //  vlanid
} postPktClsMcastHdr_t;

typedef struct _packet_t_ {

    bd_clbuf_t     bdClassBuf;
    UCHAR      headroom[FP_CLSBUF_HEADROOM];
    struct packet_t *next_ptr;  // ptr to the start of buffer in DDR memory
    USHORT     length;          // total packet lenght
    USHORT     phyno;           // physical port number
    void       *fp_ptr;         // reserved
    UINT       res;             // reserved
#ifdef HW_LMEM_ONLY
    UINT       res1[4];             // reserved
#endif
    UCHAR      hdr_data[FP_CLASS_DATASIZE]; //First segment of packet

} packet_t;
/*Structres as per hardware design*/
typedef struct _hwparse_t {
    USHORT sid;
    USHORT connid;
    UCHAR  toevec;
    UCHAR  pLayer2Hdr;
    UCHAR  pLayer3Hdr;
    UCHAR  pLayer4Hdr;
    USHORT vlanid;
    USHORT ifParseFlags;
    UINT   parseFlags;
    USHORT srcport;
    USHORT dstport;
    UINT   proto:8;
    UINT   port:4;
    UINT   hash:20;
    UINT64 rte_res_valid:1;
    UINT64 vlan_res_valid:1;
    UINT64 dst_res_valid:1;
    UINT64 src_res_valid:1;
    UINT64 vlan_lookup:20;
    UINT64 dst_lookup:20;
    UINT64 src_lookup:20;
}hwparse_t;

int fp_rx_packet (packet_t *pkt_ptr);
int fp_tx_packet (pktBuf_t *pktBuf, UCHAR port_cnt, UCHAR *port_list);
 
/* packet buffer allocation function */
pktBuf_t* pkt_buff_alloc(void);

/* link layer parsing function */
extern int fp_linklayer_parsing(pktBuf_t *pkt_buff);

/* interface match */
extern int fp_interface_match(pktBuf_t *pkt_buff);

/* init function for fastpath */
extern int fastPathInit(void);

extern int fp_send_packet_host(pktBuf_t *pkt_buff);

extern void fp_linux_tx_packet(UCHAR phyPortNum, pktBuf_t *pktBuf,
                               UINT clone);

#endif /*End of _FP_PKTBUFF_H_ File*/

