/*************************************************************************/ /*!
@File
@Title          fp_pre_header.h
@Copyright      Copyright (c) Milandr. All Rights Reserved
@Description    contains tx/rx prepended headers
@License        Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

#ifndef _FP_PRE_HEADER_H_
#define _FP_PRE_HEADER_H_

#define FP_TX_HDR_LEN           16
#define FP_RX_HDR_LEN           16
#define FP_EGR_TSR_LEN          16

/* Control field */
#define FP_TX_PKT_INJECT_EN     (1 << 0)
#define FP_TX_PKT_PTP_EN        (1 << 2)
#define FP_RX_TS_REPORT_EN      (1 << 4)
#define FP_PUNT_VALID           (1 << 5)
#define FP_RX_TS_VALID          (1 << 6)

#define FP_INJECT_TO_EMAC_1     (1 << 0)
#define FP_INJECT_TO_EMAC_2     (1 << 1)
#define FP_INJECT_TO_HIF_1      (1 << 2)
#define FP_INJECT_TO_HIF_2      (1 << 3)

/* Punt reason codes */
#define FP_PUNT_L2_SPECIAL      (1 << 0)                                        /* L2 protocol frame (e.g. STP BPDU) */
#define FP_PUNT_SA_MISS         (1 << 1)                                        /* SA miss in the MAC table, need to learn MAC */
#define FP_PUNT_SA_RELEARN      (1 << 2)                                        /* SA found on different port, need to relearn MAC */
#define FP_PUNT_SA_IS_ACTIVE    (1 << 3)                                        /* SA is active, need to set Fresh bit to zero */
#define FP_PUNT_SNOOP_UPPER     (1 << 4)                                        /* Upper layer protocol snoop */
#define FP_PUNT_REQ             (1 << 5)
#define FP_PUNT_MGMT            (1 << 6)
#define FP_PUNT_IGMP            (1 << 7)
#define FP_PUNT_FLOOD           (1 << 8)
#define FP_PUNT_PARSE           (1 << 9)

struct tx_header
{
    UINT queue: 4;
    UINT txport_map: 20;
    UINT ctrl: 8;
    USHORT ref_num;
    USHORT rsvd1;
    UINT rsvd2;
    UINT rsvd3;
};

struct rx_header
{
    USHORT punt_reason;
    UCHAR rxport_num;
    UCHAR ctrl;
    UINT rsvd;
    UINT rx_timestamp_nsec;
    UINT rx_timestamp_sec;
};

struct egress_report
{
    UCHAR rsvd[3];
    UCHAR ctrl;
    UINT rsvd1;
    UINT egress_timestamp_nsec;
    UINT egress_timestamp_sec;
    UCHAR rsvd2;
    UCHAR rxport_num;
    USHORT ref_num;
};

/* forward declaration to avoid warnings */
struct sk_buff;

int fp_fill_tx_ptp_header(struct tx_header *txhdr, struct sk_buff *skb);
int fp_add_prepend_tx_header(struct sk_buff *skb);
int fp_process_rx_header(struct sk_buff *skb);

#define CB_INJ_TX_FLAG1_VALUE  0xDE
#define CB_INJ_TX_FLAG2_VALUE  0xAD
#define CB_INJ_TX_FLAG3_VALUE  0xC0
#define CB_INJ_TX_FLAG4_VALUE  0xDE
#define CB_INJ_TX_FLAG5_VALUE  0xDE
#define CB_INJ_TX_FLAG6_VALUE  0xAD
#define CB_INJ_TX_FLAG7_VALUE  0xC0
#define CB_INJ_TX_FLAG8_VALUE  0xDE

#define CB_INJ_TX_FLAG_VALUES_LEN 8

#define CB_INJ_TX_FLAG1  34
#define CB_INJ_TX_FLAG2  35
#define CB_INJ_TX_FLAG3  36
#define CB_INJ_TX_FLAG4  37
#define CB_INJ_TX_FLAG5  38
#define CB_INJ_TX_FLAG6  39
#define CB_INJ_TX_FLAG7  40
#define CB_INJ_TX_FLAG8  41

#define CB_PORT_FOR_INJECTION 43

//#define CB_INJ_TX_FLAG  41
#define CB_INJ_TX_PORT  42
#define CB_TX_SA_MISS   0x40

//#define CB_TX_INJECT_CONSTANT   0xDEADC0DE

void fp_tx_inject_packet( struct sk_buff *skb, int rx_port, int relearn_flag );

#endif  /*End of _FP_PRE_HEADER_H_*/
