/*************************************************************************/ /*!
@File
@Title          fp_router.h
@Copyright      Copyright (c) Milandr. All Rights Reserved
@Description    data structures and APIs for fpath router functionality
@License        Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

#ifndef  _FP_ROUTER_H_
#define  _FP_ROUTER_H_
/*NEW route structure as per new rtl*/ 
typedef struct cacheFlow_s{
  struct cacheFlow_s *next;
  UINT      valid;
  USHORT    srcPort;
  USHORT    dstPort;
  UCHAR     Proto;
  UCHAR     inPhyPortNum;
  USHORT    hash;
  union {
     struct {
        UINT   srcIP;
        UINT   dstIP;
     }v4;
     struct {
        UINT   src_ipv6[4];
        UINT   dst_ipv6[4];
     }v6;
  }u;
  UCHAR     entryState;
  UCHAR     phyPortNum; /* changed res1 to egr phy no as per hardware req */
  /* information updated by the Classifier */
  UCHAR     classNotifyFlags;
  UCHAR     ifPortNum; /*To represent phyport and interface port numbers*/
 /* header construction related infor */
  UCHAR     srcMacAddr[6];
  UCHAR     dstMacAddr[6];
  UINT      flowActionflags;
  USHORT    pppoeSesId;
  USHORT    vlanId;
  UINT      chgSrcIp;
  UINT      chgDstIp;
  USHORT    chgSrcPort;
  USHORT    chgDstPort;
  UCHAR     flag_ipv6;
  UCHAR     matchFlags;
  UCHAR     res1[2];   
  UCHAR     flow_lbl[3]; //need to change endian 
  UCHAR     flow_res; 
  union{
      UINT  sip[4];
      UINT  dip[4];
  }chv6; 
  UINT      res[2];
  /* vendor specific information */
  void      *userInfo[2];
  void      *res_next;
}cacheFlow_t;
/*OLD Route structure*/

/* Hash Table Structure */
typedef struct fp_router_table_s {

    int     tbl_size;         /* size of the table */
    int     num_entries;      /* number of entries in this table */
    int     refcnt;           /* number of references to this table */
    cacheFlow_t **table;  /* table elements */

} fp_router_table_t;

#define MAX_ROUTER_TABLES        12
typedef struct fp_router_tables_s {

    int      num_tables;     /* number of tables created so far */
    fp_router_table_t *fp_router_tables[MAX_ROUTER_TABLES];

} fp_router_tables_t;
#ifdef HASH_ARR_EN
extern cacheFlow_t *grtHashArr;
#endif

extern fp_router_tables_t router_tables;

/* route hash table global variable */
extern fp_router_table_t *fp_router_hash_table;

/* initialize router table array */
extern void init_fp_router_tables(fp_router_tables_t *rtr_tbls);

/* create a router table */
extern fp_router_table_t* fp_create_router_table(int size);

/* Add entry for to hash table */
extern cacheFlow_t* 
fp_add_router_table_entry(USHORT in_usHashValue,
                          cacheFlow_t *in_Entry,
                          fp_router_table_t *in_hashTable);
/*IPV6 */
extern cacheFlow_t*
fp_ipv6_add_router_table_entry(USHORT in_usHashValue,
                           cacheFlow_t *in_Entry,
                           fp_router_table_t *in_hashTable);
extern int 
ipv6_hash5Tuple (cacheFlow_t *in_cacheFlow, unsigned short hashMask);

extern int 
ipv6_hash5Tuple(cacheFlow_t *in_cacheFlow, unsigned short hashMask);

extern cacheFlow_t *
fp_ipv6_search_router_table_entry (USHORT in_usHashValue,
                              cacheFlow_t *in_Entry,
                              fp_router_table_t *in_hashTable);

/* Lookup address in hash table */
extern cacheFlow_t* 
fp_search_router_table_entry(USHORT in_usHashValue,
                             cacheFlow_t *in_Entry,
                             fp_router_table_t *in_hashTable);

/* Display contents of hash table*/
extern void
fp_display_router_table_elements(int in_iTable_Size,
                                 fp_router_table_t *in_hashTable);

/* Count number of addresses available in hash table */
extern int
fp_count_router_table_entries(fp_router_table_t *in_hashTable);

/* Delete Entry from hash table */
extern int
fp_delete_router_table_entry(USHORT in_usHashValue,
                             cacheFlow_t *in_Entry,
                             fp_router_table_t *in_hashTable);

/* delete a router table */
extern void fp_delete_router_table(fp_router_table_t *router_table);

/* Fastpath RouteClassifier */
extern int fp_route_classifier(pktBuf_t *in_pkt_buff);

/* function prototype for calculating hash value of 5tuple*/
int hash5Tuple(cacheFlow_t* in_cacheFlow, USHORT hashMask);

/* function prototype for getting 5tuple ports*/
void fp_get_tuple_ports(cacheFlow_t *in_Entry, pktBuf_t *in_pkt_buff);

/* function prototype for Layer 3 parsing */
int fp_Layer3_Parsing(pktBuf_t *pkt_buff);

/* function prototype for Verifying checksum*/
USHORT verify_checksum(USHORT *buff, USHORT len);

/* udp checksum */
USHORT fp_udp_sum_calc(USHORT len_udp, USHORT src_addr[],
                       USHORT dest_addr[], USHORT padding,
                       USHORT buff[]);

/* tcp checksum */
USHORT fp_tcp_sum_calc(USHORT len_udp, USHORT src_addr[],
                       USHORT dest_addr[], USHORT padding,
                       USHORT buff[]);

void
fp_routeflow_add6rd_header(pktBuf_t *in_pkt_buff,cacheFlow_t *routeEntry,USHORT len,UCHAR ttl);
void
fp_routeflow_add4rd_header(pktBuf_t *in_pkt_buff,cacheFlow_t *routeEntry,USHORT len,UCHAR ttl);

/* function prototype for update stats */
void fp_routeflow_update_stats(pktBuf_t *in_pkt_buff, cacheFlow_t *routeEntry);

/* function prototype for drop pkts */
void fp_routeflow_drop_pkts(pktBuf_t *in_pkt_buff, cacheFlow_t *routeEntry);

/* function prototype for update tcpwindow */
void fp_routeflow_update_tcpwnd(pktBuf_t *in_pkt_buff, cacheFlow_t *routeEntry);

/* function prototype for changing src ip */
void fp_routeflow_change_sipaddr(pktBuf_t *in_pkt_buff,
                                 cacheFlow_t *routeEntry);

/* function prototype for changing source port */
void fp_routeflow_change_sport(pktBuf_t *in_pkt_buff, cacheFlow_t *routeEntry);

/* function prototype for change dipaddr */
void fp_routeflow_change_dipaddr(pktBuf_t *in_pkt_buff,
                                 cacheFlow_t *routeEntry);

/* function prototype for change dport */
void fp_routeflow_change_dport(pktBuf_t *in_pkt_buff, cacheFlow_t *routeEntry);

/* function prototype for change tos */
void fp_routeflow_change_tos(pktBuf_t *in_pkt_buff, cacheFlow_t *routeEntry);

/* function prototype for construct ipheader */
void fp_routeflow_add_iphdr(pktBuf_t *in_pkt_buff, cacheFlow_t *routeEntry);

/* function prototype for construct ah header */
void fp_routeflow_add_ahhdr(pktBuf_t *in_pkt_buff, cacheFlow_t *routeEntry);

/* function prototype for construct esp header */
void fp_routeflow_add_esphdr(pktBuf_t *in_pkt_buff, cacheFlow_t *routeEntry);

/* function prototype for construct ipv6 header */
void fp_routeflow_add_ipv6hdr(pktBuf_t *in_pkt_buff, cacheFlow_t *routeEntry);

/* function prototype for construct ppoe,lan and eth headers */
void fp_routeflow_add_ppoehdr_vlanhdr_ethhdr(pktBuf_t *in_pkt_buff,
                                             cacheFlow_t *routeEntry);

/* function prototype for construct ppoe and eth headers */
void fp_routeflow_add_ppoehdr_ethhdr(pktBuf_t *in_pkt_buff,
                                     cacheFlow_t *routeEntry);

/* function prototype for construct eth and vlan header */
void fp_routeflow_add_vlanhdr_ethhdr(pktBuf_t *in_pkt_buff,
                                     cacheFlow_t *routeEntry);                            
/* function prototype for construct eth header */
void fp_routeflow_add_ethhdr(pktBuf_t *in_pkt_buff, cacheFlow_t *routeEntry);         
extern void fp_add_static_route_tables(void);
extern void fp_print_rtentry(cacheFlow_t *Entry);
extern cacheFlow_t *fp_search_rtentry_local(USHORT in_hash, UCHAR phyno);
extern void fp_update_rtentry_local(cacheFlow_t *rtEntry, USHORT in_hash, UCHAR phyno);

#endif /*End of _FP_ROUTER_H_ file */

