/*
 * fp_stats.h - stats related data structures and APIs for fast path module
 *
 * Copyright (c) 2007-2008, Naksha Technologies, Inc.
 * All rights reserved.
 */

#ifndef  _FP_STATS_H_
#define  _FP_STATS_H_

#define  FP_PARSE_MGMT_MODULE            0x1
#define  FP_PARSE_INRATE_MODULE          0x2
#define  FP_PARSE_OUTRATE_MODULE         0x4
#define  FP_PARSE_INGRVLAN_MODULE        0x8
#define  FP_PARSE_EGRVLAN_MODULE         0x10
#define  FP_PARSE_QOS_MODULE             0x20
#define  FP_PARSE_L2SPECIAL_MODULE       0x40


#define  ALL_STATS          0x1
#define  MGMT_STATS         0x2
#define  INRATE_STATS       0x3
#define  L2SPECIAL_STATS    0x4
#define  INGRVLAN_STATS     0x5
#define  EGRVLAN_STATS      0x6
#define  QOS_STATS          0x7
#define  OUTRATE_STATS      0x8

/* MOdule index for kernel stastics */
#define  FP_MGMT        0x1
#define  FP_L2SPECIAL   0x2
#define  FP_INGRVLAN    0x3
#define  FP_QOS         0x4

typedef struct fp_stats_s
{
    /* protocol stats */
    UINT ip4_pkt_cnt;
    UINT udp_pkt_cnt;
    UINT tcp_pkt_cnt;
    UINT icmp_pkt_cnt;
    UINT igmp_pkt_cnt;
    UINT ip6_pkt_cnt;

    /* packet parsing error counters */
    UINT ip_hdr_len_err_cnt;
    UINT ip_pkt_ttl_err_cnt;
    UINT ip_chksum_err_cnt;

    /* other counters */
    UINT mem_alloc_err_cnt;
    UINT unknown_pkt_cnt; // arp and unknown packets count
    UINT intf_match_cnt;
    UINT intf_mismatch_cnt;
    UINT src_be_found;           /* source bridge entry found */
    UINT dst_be_found;           /* destination bridge entry found */
    UINT src_be_not_found;       /* source bridge entry not found */
    UINT dst_be_not_found;       /* destination bridge entry not found */
    UINT re_found;               /* route entry found */
    UINT re_not_found;           /* route entry not found */
    /*Added for rx/tx/pe-host counters*/
    UINT     rx_pkt_cnt;
} fp_stats_t;

extern fp_stats_t fp_stats;

typedef struct fp_debugstats_s {

    UINT emac_tx_counters0;
    UINT emac_tx_counters1;

    UINT emac_rx_counters0;
    UINT emac_rx_counters1;

    UINT emac_errors0[13];
    UINT emac_errors1[13];

    UINT gpi_fifo_count0;
    UINT gpi_fifo_count1;
    UINT gpi_fifo_count2;
    UINT gpi_fifo_count3;

    UINT gpi_drop_count0;
    UINT gpi_drop_count1;

    UINT class_phy1_count[10];
    UINT class_phy2_count[10];

    UINT bmu_buffer_count;
    UINT bmu_rem_count;
    UINT bmu_cur_count;

    UINT bmu_master_based_buffer_counts[9];

    UINT hif1_tx_counters0;
    UINT hif1_tx_counters1;

    UINT hif1_rx_counters0;
    UINT hif1_rx_counters1;

    UINT hif2_tx_counters0;
    UINT hif2_tx_counters1;

    UINT hif2_rx_counters0;
    UINT hif2_rx_counters1;

    UINT hgpi1_state0;
    UINT hgpi1_state1;

    UINT hgpi2_state0;
    UINT hgpi2_state1;

    UINT egpi1_state0;
    UINT egpi1_state1;

    UINT egpi2_state0;
    UINT egpi2_state1;

} fp_debugstats_t;

#define NFP_STATS(field) do { \
        fp_stats.field++; \
} while(/*CONSTCOND*/0)

    /* l2policies stats */
typedef struct fp_l2policies_stats_s {

    /* policies match success count */
    UINT  smac_match_cnt;
    UINT  dmac_match_cnt;
    UINT  etype_match_cnt;
    UINT  vid_match_cnt;
    UINT  cos_match_cnt;
    UINT  dscp_match_cnt;
    UINT  ipproto_match_cnt;
    UINT  data_match_cnt;
    /* policies match failure count */
    UINT  mismatch_cnt;
    UINT  smac_mismatch_cnt;
    UINT  dmac_mismatch_cnt;
    UINT  vid_mismatch_cnt;
    UINT  cos_mismatch_cnt;
    UINT  dscp_mismatch_cnt;
    UINT  ipproto_mismatch_cnt;

} fp_l2policies_stats_t; /* SIZE - 60 byttes */

#ifndef PE_STRUCT_SIZE
    /* mgmt module stats */
typedef struct fp_mgmt_stats_s {

    UINT  stp_pkt_cnt;
    UINT  igmp_pkt_cnt;
    UINT  policies_succ;
    UINT  policies_fail;
    fp_l2policies_stats_t  mgmt_policies;

} fp_mgmt_stats_t;

    /* inrate module stats */
typedef struct fp_inrate_stats_s {

    UINT  green_pkt_cnt;
    UINT  yellow_pkt_cnt;
    UINT  red_flowctrl_pkt_cnt;
    UINT  red_drop_pkt_cnt;

} fp_inrate_stats_t;


    /* outrate module stats */
typedef struct fp_outrate_stats_s {

    UINT  sched_type;
    UINT  enqueue_pkt_cnt;
    UINT  dequeue_pkt_cnt;
    UINT  drop_pkt_cnt;

} fp_outrate_stats_t;

#endif /*End of ifndef PE_STRUCT_SIZE*/

    /* ingress vlan module stats */
typedef struct fp_ingr_vlan_stats_s {

    UINT  tag_pkt_cnt;
    UINT  untag_pkt_cnt;
    UINT  droptag_pkt_cnt;
    UINT  dropuntag_pkt_cnt;
    UINT  no_mship_cnt;
    UINT  ingr_mship_match_cnt;
    UINT  egr_mship_match_cnt;
    UINT  ingr_mship_mismatch_cnt;
    UINT  egr_mship_mismatch_cnt;
    fp_l2policies_stats_t  ingr_vlan_policies;

} fp_ingr_vlan_stats_t; /* SIZE - 96 bytes */


    /* egress vlan module stats */
typedef struct fp_egr_vlan_stats_s {

    UINT  tag_pkt_cnt;
    UINT  untag_pkt_cnt;
    UINT  droptag_pkt_cnt;
    UINT  dropuntag_pkt_cnt;
    UINT  no_mship_cnt;
    UINT  ingr_mship_match_cnt;
    UINT  egr_mship_match_cnt;
    UINT  ingr_mship_mismatch_cnt;
    UINT  egr_mship_mismatch_cnt;

} fp_egr_vlan_stats_t; /* SIZE - 36 */

#ifndef PE_STRUCT_SIZE
    /* qos module stats */
typedef struct fp_qos_mapmask_stats_s {

    UINT  cos_2primap_cnt;
    UINT  dscp_2primap_cnt;
    UINT  cos_2xmark_cnt;
    UINT  dscp_2xmark_cnt;
    fp_l2policies_stats_t  qos_policies;

} fp_qos_mapmask_stats_t;
#endif /*End of ifndef PE_STRUCT_SIZE*/

    /* bridge module stats */
typedef struct fp_bridge_stats_s {

    UINT  learn_pkt_cnt;
    UINT  flood_pkt_cnt;
    UINT  src_be_found;           /* source bridge entry found */
    UINT  dst_be_found;           /* destination bridge entry found */
    UINT  src_be_not_found;       /* source bridge entry not found */
    UINT  dst_be_not_found;       /* destination bridge entry not found */

} fp_bridge_stats_t; /* SIZE - 24 bytes */

   /* all module stats */
typedef struct fp_all_modules_stats_s {

    UINT  mgmt_pkt_cnt;
    UINT  inrate_pkt_cnt;
    UINT  speciall2_pkt_cnt;
    UINT  outrate_pkt_cnt;
    UINT  ingrvlan_pkt_cnt;
    UINT  rx_pkt_cnt;
    UINT  tx_pkt_cnt;
    UINT  ip_pkt_cnt;
    UINT udp_pkt_cnt;
    UINT tcp_pkt_cnt;
    UINT icmp_pkt_cnt;
    UINT igmp_pkt_cnt;
    UINT arp_pkt_cnt;

} fp_all_modules_stats_t; /* SIZE - 52 */

  /* switch statastics */
typedef struct fp_switch_stats_s {


    /* All module stats */
    fp_all_modules_stats_t  all;

#ifndef PE_STRUCT_SIZE
    /* mgmt module stats */
    fp_mgmt_stats_t  mgmt;

    /* inrate module stats */
    fp_inrate_stats_t  inrate_1;

    /* inrate module stats */
    fp_inrate_stats_t  inrate_2;

    /* outrate module stats */
    fp_outrate_stats_t  outrate;
#endif /*End of ifndef PE_STRUCT_SIZE*/

    /* ingress vlan module stats */
    fp_ingr_vlan_stats_t  ingrvlan;

    /* egress vlan module stats */
    fp_egr_vlan_stats_t  egrvlan;

#ifndef PE_STRUCT_SIZE
    /* qos module stats */
    fp_qos_mapmask_stats_t  qos;
#endif /*End of ifndef PE_STRUCT_SIZE*/

    /* special l2policies module stats */
    fp_l2policies_stats_t  speciall2;

    /* bridge module stats */
    fp_bridge_stats_t  bridge;

    USHORT parse_modules;

} fp_switch_stats_t; /* 52 + 96 + 36 + 60 + 24 + 2  = 270 */

//extern fp_switch_stats_t *sw_stats;
extern fp_switch_stats_t sw_stats;

extern void fp_policy_stats(int module_index, int policy_index);

#endif  /* _FP_STATS_H_ */
