/*************************************************************************/ /*
@File
@Title          hw_2field_hash_table.h
@Copyright      Copyright (c) Milandr. All Rights Reserved
@Description    Structures for hw switch
@License        Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

/*!
 * @file hw_2field_hash_table.h
 * L2 MAC Table API
 */

#ifndef  _HW_HASH_TABLE_H_
#define  _HW_HASH_TABLE_H_

#include "fp_library.h"
#include "fp_global_addr_map.h"

/* Mac Hash table */
#define  FREE_LIST_ENTRIES    (0x3d8)
#define  FREE_LIST_HEAD_PTR   (0x3dc)
#define  FREE_LIST_TAIL_PTR   (0x3e0)

/* MAC HASH table for HOST */
#define  REQ_CMD_REG          (CLASS_HW_BASE_ADDR + 0x390)
#define  REQ_MAC1_ADDR_REG    (CLASS_HW_BASE_ADDR + 0x394)
#define  REQ_MAC2_ADDR_REG    (CLASS_HW_BASE_ADDR + 0x398)
#define  REQ_MAC3_ADDR_REG    (CLASS_HW_BASE_ADDR + 0x39C)
#define  REQ_MAC4_ADDR_REG    (CLASS_HW_BASE_ADDR + 0x3A0)
#define  REQ_MAC5_ADDR_REG    (CLASS_HW_BASE_ADDR + 0x3A4)
#define  REQ_ENTRY_REG        (CLASS_HW_BASE_ADDR + 0x3A8)
#define  REQ_STATUS_REG       (CLASS_HW_BASE_ADDR + 0x3AC)
#define  REQ_DIRECT_REG       (CLASS_HW_BASE_ADDR + 0x3B0)

/* Mac Hash table */
#define  CLASS2_FREE_LIST_ENTRIES    (CLASS2_HW_BASE_ADDR + 0x3d8)
#define  CLASS2_FREE_LIST_HEAD_PTR   (CLASS2_HW_BASE_ADDR + 0x3dc)
#define  CLASS2_FREE_LIST_TAIL_PTR   (CLASS2_HW_BASE_ADDR + 0x3e0)

#define TWO_FIELD_HASH_ENTRIES     0x1000
#define TWO_FIELD_COLL_ENTRIES     0x1000
#define TWO_FIELD_INIT_HEAD_PTR    0x1000
#define TWO_FIELD_INIT_TAIL_PTR    0x1FFF


/* 2-filed mac table valid defines */
#define BIT_SET_MAC_2F_MACTBL     0x00000100
#define BIT_SET_VLAN_2F_MACTBL    0x00000200

/*!
 Initialize hardware MAC hash table.
 **/
void hw_hash_table_init( void );

/*!
 Search the table for a MAC address
 @return                    32-bit brentry (see format below)

 @code
    MAC Hash table brentry (action entry) format

   [07:00]           - forward port list
   [10:8]            - TC
   [13:11]           - fwd_action (ACTION FIELD)
   [28:14]           - Reserved
   [29]              - fresh
   [30]              - static
 @endcode
 **/
int32_t hw_hash_table_search( uint8_t *mac, uint16_t vlaId );

/*!
 Add a MAC address to the table
 @return status code:       0=success, -1=failure
 **/
int32_t hw_hash_table_add( uint8_t *mac, uint16_t vlaId, uint32_t brentry );

/*!
 Delete a MAC address from the table
 @return status code:       0=success, -1=failure
 **/
void hw_hash_table_delete( uint8_t *mac, uint16_t vlanId );
/*!
 Update an existing MAC address entry
 @return status code:       0=success, -1=failure
 **/
int32_t hw_hash_table_update( uint8_t *mac, uint16_t vlanId, uint32_t brentry );

void hw_hash_table_read_entry( uint32_t *entry_data, uint16_t entry );
void hw_hash_table_write_entry( uint32_t *entry_data, uint16_t entry );
int hw_hash_table_read( uint8_t *buff );
#endif /*End of _HW_HASH_TABLE_H_*/
