/*************************************************************************/ /*!
@File
@Title          nc_conf.h
@Copyright      Copyright (c) Milandr. All Rights Reserved
@Description    This file contains prototypes to handle fpath config
@License        Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

#ifndef _FP_CONFIG_H_
#define _FP_CONFIG_H_

/* table information data structure */
typedef struct nc_table_info_s {

    fp_table_type_t     tbl_type;   /* table type */
    u_short     tbl_size;           /* table size */
    u_short     tbl_num_entries;    /* number of entries in this table */
    u_short     tbl_id;             /* table id */
    u_short     tbl_refcnt;         /* TODO: u_int16_t or u_in32_t ?? */

    /* TODO: add port info */

} nc_table_info_t;
typedef struct nc_router_entry_s{
  struct cacheFlow_s *next;
  UINT      valid;
  USHORT    srcPort;
  USHORT    dstPort;
  UCHAR     proto;
  UCHAR     inPhyPortNum;
  USHORT    hash;
  union {
     struct {
        UINT   srcIP;
        UINT   dstIP;
     }v4;
     struct {
        UINT   src_ipv6[4];
        UINT   dst_ipv6[4];
     }v6;
  }u;
  UCHAR     entryState;
  UCHAR     phyPortNum; /* changed res1 to egr phy no as per hardware req */
  /* information updated by the Classifier */
  UCHAR     classNotifyFlags;
  UCHAR     ifPortNum; /*To represent phyport and interface port numbers*/
 /* header construction related infor */
  UCHAR     srcMacAddr[6];
  UCHAR     dstMacAddr[6];
  UINT      flowActionFlags;
  USHORT    pppoeSesId;
  USHORT    vlanId;
  UINT      chgSrcIP;
  UINT      chgDstIP;
  USHORT    chgSrcPort;
  USHORT    chgDstPort;
  u_char    flag_ipv6;
  u_char    res1[3]; 
  UINT      res[9];
  void      *res_next; /*Storing entry pointer in structure*/

}nc_router_entry_t;
#if 0
/* structure for router table entries */
typedef struct nc_router_entry_s {

    u_int     srcIP;
    u_int     dstIP;
    union {
        struct {
            u_short     srcPort;    /* src port for TCP/UDP packets */
            u_short     dstPort;    /* dst port for TCP/UDP packets */
       } layer4Info;
       u_short     fragId;          /* IP fragmentation id */
       u_int       spiIndex;        /* for IPSEC packets */
    } flowSpecific;
    u_int     flowActionFlags;      /* FLOW Action flags */
    u_int     chgSrcIP;
    u_int     chgDstIP;
    u_int     pktCount;             /* EXT */
    u_short   chgSrcPort;
    u_short   chgDstPort;
    u_char    proto;                /* IP protocol */
    u_char    chgRos;
    u_char    rsvd[2];
    /* IPV6 fields added */
    u_int     src_ipv6[4];
    u_int     dst_ipv6[4];
    u_char    flag_ipv6;
    u_char    res_ipv6;

    /* TODO Check if we need to upload the information on flags */
    u_char    srcMacAddr[ETH_ALEN];
    u_char    dstMacAddr[ETH_ALEN];
    u_short   hash;
    u_short   res1;

} nc_router_entry_t;
#endif

#define IF_NAME_LEN 10

/* structure for bridge table entries */
typedef struct nc_bridge_entry_s {

    u_int      be_pktcount;            /* packet count */
    u_short    be_vlanid;              /* VLAN id */
    u_char     be_macaddr[ETH_ALEN];   /* MAC address */
    u_char     be_state;               /* entry state */
    u_int     be_actions;             /* actions on this entry */ 

} nc_bridge_entry_t;

/* data structure for port related information */
typedef struct nc_port_info_s {

    u_int      pidx;          /* port index */
    u_short    media_type;    /* media type */
    u_short    max_num_ifls;  /* max. number of ifls on this physical port */
    u_short    cur_num_ifls;  /* current number of ifls on this port */
    u_int      rx_pkts;       /* rx packet count */
    u_int      tx_pkts;       /* tx packet count */

} nc_port_info_t;

/* data structure for logical interface related information */
typedef struct nc_ifl_info_s {

    u_int     pidx;           /* physical port index */
    u_int     ifidx;          /* interface index */
    u_int     opflags;        /* IF_ROUTE/IF_BRIDGE/IF_VLAN_BRIDGE modes */
    u_int     rx_pkts;        /* rx packet count */
    u_int     tx_pkts;        /* tx packet count */
    u_int     iftype;         /* interface type */
    u_int     ipaddr;         /* ip address, if any */

} nc_ifl_info_t;

#endif /*End of _FP_CONFIG_H_ File*/

