
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <net/if.h>
#include <sys/stat.h>
#include <sys/ioctl.h>

#include "fp_library.h"
#include "libcli.h"
#include "fp_types.h"
#include "nc_conf.h"
#include "nc_conf_api.h"
#include "fp_pktbuff.h"
#include "fp_bridge_table.h"
#include "fp_router.h"
#include "fp_ether.h"
#include "fp_stats.h"
#include "fp_ioctl.h"
#include "fpios.h"

/* TODO REMOVE DEBUG SUPPORT */
#define DEBUG_ENTRIES_UNDEF

int
writeMem(int fp_request, char *argv[])
{
	fpathcmd_t fpCmd;
    int     offset, addr, value;
	int     sockFd;
	struct  ifreq ifr;

    return(0);

	memset(&fpCmd, 0, sizeof(fpathcmd_t));

	/* Create a scoket */
    if((sockFd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
    {
        fprintf(stderr, "Socket:error: %s \n", strerror(errno));
        return (1);
    }

    ifr.ifr_ifru.ifru_data= (char *)&fpCmd;
    strncpy(ifr.ifr_name, device_name, 16);
	fpCmd.cmd  = fp_request;
    switch (fp_request) {
        case  FPATH_WRITE_CONFIG:
            sscanf(argv[1],"%x",&offset);
            sscanf(argv[2],"%x",&value);
	        fpCmd.addr = offset;
            fpCmd.value = value;
            break;
        case FPATH_WRITE_PCI_MEM:
            sscanf(argv[1],"%x",&addr);
            sscanf(argv[2],"%x",&value);
	        fpCmd.addr = addr;
            fpCmd.value = value;
            break;
        default:
        break;
    }
    if(ioctl(sockFd, fp_request, (struct ifreq *)&ifr)<0){
		fprintf(stderr, "ioctl:error :%s \n", strerror(errno));
		close(sockFd);
		return (1);
	}
	close(sockFd);
	return (0);
}


/******************************************************************************
 * Function :: getIoctlInt
 * Args     :: int fp_request int *
 * Comment  :: Obtain an integer values from pci_driver
 * Return   :: 0 on success
 *****************************************************************************/
int
getIoctlIntVals(int fp_request, char *argv[], int *intCnt, void **buffer)
{
	fpathcmd_t fpCmd, *fpc=NULL;
	int     sockFd;
	struct  ifreq ifr;
    int     addr;

	memset(&fpCmd, 0, sizeof(fpathcmd_t));

	/* Create a scoket */
    if((sockFd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
    {
        fprintf(stderr, "Socket:error: %s \n", strerror(errno));
        return (1);
    }
    ifr.ifr_ifru.ifru_data= (char *)&fpCmd;
    strncpy(ifr.ifr_name, device_name, 16);
	fpCmd.cmd  = fp_request;

    switch (fp_request)
    {
        case FPATH_READ_CONFIG:
            sscanf(argv[1],"%x",&addr);
            sscanf(argv[2],"%x",&fpCmd.size);
	        fpCmd.addr = addr;
            *intCnt=fpCmd.size;
            break;
        case FPATH_READ_PCI_MEM:
            sscanf(argv[2],"%x",&addr);
            sscanf(argv[3],"%x",&fpCmd.size);
	        fpCmd.addr = addr;
            *intCnt=fpCmd.size;
            break;
        default:
            fpCmd.size = 1;                                                     /*get One integer value */
            break;
    }

    /* Allocate buffer */
    *buffer = calloc (fpCmd.size, sizeof(int));
    fpCmd.data = (char *)*buffer;

    if(ioctl(sockFd, fp_request, (struct ifreq *)&ifr) < 0)
    {
		fprintf(stderr, "ioctl:error :%s \n", strerror(errno));
		close(sockFd);
		return 1;
	}

    fpc= (fpathcmd_t *)ifr.ifr_ifru.ifru_data;
	if(fpc->cmd == fp_request)
    {
#ifdef DEBUG
        fprintf(stdout, "%d  intVal=%d \n", fp_request, *(int *)*buffer);
#endif
	} else {
		fprintf(stderr, "Error Unknown option. \n");
		close(sockFd);
		return(1);
	}
	close(sockFd);
	return (0);
}

/*****************************************************************************
 * Function :: getEntries
 * Args     :: int fp_request int *
 * Comment  :: Obtain the entries from ioctl and Return
 *          :: entryType  :: FPATH_READ_FLOW_ENTRIES
 *          ::               FPATH_READ_FLOW_ENTIRE_EXT
 *          ::               FPATH_READ_BRIDGE_ENTRIES
 *          ::               FPATH_READ_BRIDGE_ENTIRES_EXT
 *          :: entryNum   :: FPATH_READ_FLOW_NUM
 *                        :: FPATH_READ_BRIDGE_NUM
 * Return   :: 0 on success
 * ***************************************************************************/
int
getEntries(int entryNum, int entryType, int *numEntries, void **entries){
	int     sockFd;
	struct  ifreq ifr;
	fpathcmd_t fpCmd, *fpc=NULL;
    int     entBufSize;

    /* getioctl number of entries */
    {
        void *ptr=NULL;
        int  intCnt;
        getIoctlIntVals(entryNum, NULL, &intCnt, &ptr);
        if(ptr){
            *numEntries=*(int *)ptr;
            free(ptr);
        }
    }
#ifdef DEBUG_ENTRIES
    fprintf(stdout, "DEBUG2 numEntries = %d \n", *numEntries);
#endif
    /* if numEntries == 0 There is nothing we can do, just return */
    if(! *numEntries)
        return (0);

    switch (entryType){
        case FPATH_READ_FLOW_ENTRIES:
        case FPATH_READ_FLOW_ENTRIES_EXT:
            entBufSize = (*numEntries) * 2  * sizeof(nc_router_entry_t);
            *entries=(nc_router_entry_t *)calloc(entBufSize, 1);
            break;
        case FPATH_READ_BRIDGE_ENTRIES:
        case FPATH_READ_BRIDGE_ENTRIES_EXT:
            break;
        default:
            fprintf(stderr, "Unknown option\n");
            return (1);
    }
    if(!(*entries)){
        fprintf(stderr, "Malloc Error : Unable to allocate %d \n", entBufSize);
        return (1);
    }

    ifr.ifr_ifru.ifru_data= (char *)&fpCmd;
    strncpy(ifr.ifr_name, device_name, 16);
    /* Populate fpCmd */
	memset(&fpCmd, 0, sizeof(fpathcmd_t));
	fpCmd.cmd  = entryType;
	fpCmd.size = entBufSize; /* Send the size of the buffer to kernel */
    fpCmd.data = (char *)*entries;

	/* Create a scoket */
    if((sockFd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
        fprintf(stderr, "Socket:error: %s \n", strerror(errno));
        free(*entries); *entries=NULL; *numEntries = 0;
        return (1);
    }
    /* Do the ioctl and obtain the entries from Kernel */
    if(ioctl(sockFd, entryType, (struct ifreq *)&ifr)<0){
		fprintf(stderr, "ioctl:error :%s \n", strerror(errno));
        free(*entries); *entries=NULL; *numEntries = 0;
		close(sockFd);
		return (1);
	}
	close(sockFd);
    /* Obtain the information */
    fpc= (fpathcmd_t *)ifr.ifr_ifru.ifru_data;
#ifdef DEBUG_ENTRIES
    fprintf(stdout, "DEBUG2 Size We Send %d Size we Received %d\n",
            entBufSize, fpc->size);
#endif
	if(fpc->cmd == entryType){
        switch (entryType){
            case FPATH_READ_FLOW_ENTRIES:
            case FPATH_READ_FLOW_ENTRIES_EXT:
                *numEntries = fpc->size/sizeof(nc_router_entry_t);
                break;
            case FPATH_READ_BRIDGE_ENTRIES:
            case FPATH_READ_BRIDGE_ENTRIES_EXT:
                *numEntries = fpc->size/sizeof(nc_bridge_entry_t);
                break;
            default:
                fprintf(stderr, "Unknown option\n");
                free(*entries); *entries=NULL; *numEntries = 0;
                return (1);
        }
	} else {
		fprintf(stderr, "Error Unknown option. \n");
        free(*entries); *entries=NULL; *numEntries = 0;
		return(1);
	}
#ifdef DEBUG_ENTRIES
    fprintf(stdout, "DEBUG2: numEntries = %d \n", *numEntries);
#endif
	return (0);
}


/*******************************************************************************
 * Function :: fp_ioctl_get
 * Args     :: struct fp_stats_t *
 * Comment  :: Obtain stats in fast path
 * Return   :: 0 on success
 ******************************************************************************/
int
fp_ioctl_get(void *vptr, int cmd, int size){
    nc_ioctl_ops_t    nc_ops, *pnc_ops;
	int     sockFd;
	struct  ifreq ifr;

//	memset(&nc_ops, 0, sizeof(nc_ioctl_ops_t));
	/* Create a scoket */
    if((sockFd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
        fprintf(stderr, "Socket:error: %s \n", strerror(errno));
        return (1);
    }

    ifr.ifr_ifru.ifru_data= (char *)&nc_ops;
    strncpy(ifr.ifr_name, device_name, 16);

	nc_ops.io_cmd  = cmd;
	nc_ops.io_size = size;
	nc_ops.io_data = (char *)vptr;

    if(ioctl(sockFd, cmd, (struct ifreq *)&ifr)<0){
		fprintf(stderr, "ioctl:error :%s \n", strerror(errno));
		close(sockFd);
		return (1);
	}

	close(sockFd);
    pnc_ops = (nc_ioctl_ops_t *)ifr.ifr_ifru.ifru_data;
	if(pnc_ops->io_cmd != cmd){
		fprintf(stderr, "Received wrong cmd :Expected: %d, Recieved %d \n",
            cmd, pnc_ops->io_cmd);
		return(1);
	}
	return (0);
}
