#include <stdio.h>

#define _USERSPACE_

#include "libcli.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>

#include <arpa/inet.h>

#include <fp_types.h>

#include <fp_library.h>
#include <fp_stats.h>
#include <fp_ether.h>
#include <nc_conf.h>
#include <fp_bridge_table.h>
#include <fp_ioctl.h>
#include <fp_pktbuff.h>
#include "fp_port.h"

#include "l2cli.h"


extern nakCli_t nkCli;
struct cli_def *cli;

#ifdef __GNUC__
# define UNUSED(d) d __attribute__ ((unused))
#else
# define UNUSED(d) d
#endif

u_int min_table_entries=10;
u_int max_table_entries=100000;
extern int AddSwitch_Cmds (struct cli_def *cli);
/* inet_ntoa wrapper  Note: inet_ntoa returns a static char pointer. */
char *
getCharAddr(int intVal){
    struct in_addr in;
    in.s_addr=intVal;
    return (inet_ntoa(in));
}

void cli_action_field_help(struct cli_def *cli)
{
        cli_print(cli, "\tEnter parameter value for ACT_FORWARD      0");
        cli_print(cli, "\tEnter parameter value for ACT_FLOOD        1");
        cli_print(cli, "\tEnter parameter value for ACT_PUNT         2");
        cli_print(cli, "\tEnter parameter value for ACT_DISCARD      3");
        cli_print(cli, "\tEnter parameter value for ACT_OVERRIDE     4");
        cli_print(cli, "\tEnter parameter value for ACT_FWD_MASK     5");
        cli_print(cli, "\tEnter parameter value for ACT_COS_DISCARD  6");
	return;
}

void cli_action_print(struct cli_def *cli, char *string, UINT32 act)
{
	switch (act) {
		case ACT_FORWARD:
			cli_print(cli, "\t%s : %s\n", string, "ACT_FORWARD");
		break;
		case ACT_FLOOD:
			cli_print(cli, "\t%s : %s\n", string, "ACT_FLOOD");
		break;
		case ACT_PUNT:
			cli_print(cli, "\t%s : %s\n", string, "ACT_PUNT");
		break;
		case ACT_DISCARD:
			cli_print(cli, "\t%s : %s\n", string, "ACT_DISCARD");
		break;
		case ACT_OVERRIDE:
			cli_print(cli, "\t%s : %s\n", string, "ACT_OVERRIDE");
		break;
		case ACT_FWD_MASK:
			cli_print(cli, "\t%s : %s\n", string, "ACT_FWD_MASK");
		break;
		case ACT_COS_DISCARD:
			cli_print(cli, "\t%s : %s\n", string, "ACT_COS_DISCARD");
		default:
			cli_print(cli, "\tInvalid ACTION FIELD\n");
	}
}

int cli_portlist_chk(struct cli_def *cli, UINT32 portlist)
{
	int i = 0;

	if (portlist == 0) {
		cli_print(cli, "\nno ports entered");
		return CLI_ERROR;
	}

	for (i = 0; i < FP_MAX_PORTS; i++) {
		if (((portlist >> i) & 0x1)) {
			if (!((FWD_PORT_LIST_MASK >> i) & 0x1)) {
				cli_print(cli, "\nInvalid port 0x%x", (0x1 << i));
				return CLI_ERROR;
			}
		}
	}

	if (portlist > FWD_PORT_LIST_MASK) {
		cli_print(cli, "\nInvalid port 0x%x", portlist);
		return CLI_ERROR;
	}

	return CLI_OK;
}

int cli_port_chk(struct cli_def *cli, UINT32 portlist)
{
	int i = 0, multiport = 0;

	if (portlist == 0) {
		cli_print(cli, "\nno ports entered");
		return CLI_ERROR;
	}

	for (i = 0; i < FP_MAX_PORTS; i++) {
		if (((portlist >> i) & 0x1)) {
			if (multiport) {
				cli_print(cli, "\nmultiple ports entered");
				return CLI_ERROR;
			}
			multiport = 1;
			if (!((FWD_PORT_LIST_MASK >> i) & 0x1)) {
				cli_print(cli, "\nInvalid port 0x%x", (0x1 << i));
				return CLI_ERROR;
			}
		}
	}

	if (portlist > FWD_PORT_LIST_MASK) {
		cli_print(cli, "\nInvalid port 0x%x", portlist);
		return CLI_ERROR;
	}

	return CLI_OK;
}

int cli_untaglist_chk(struct cli_def *cli, UINT32 portlist)
{
	int i = 0;

	for (i = 0; i < FP_MAX_PORTS; i++) {
		if (((portlist >> i) & 0x1)) {
			if (!((FWD_PORT_LIST_MASK >> i) & 0x1)) {
				cli_print(cli, "\nInvalid port 0x%x", (0x1 << i));
				return CLI_ERROR;
			}
		}
	}

	if (portlist > FWD_PORT_LIST_MASK) {
		cli_print(cli, "\nInvalid port 0x%x", portlist);
		return CLI_ERROR;
	}

	return CLI_OK;
}

void cli_portlist_help(struct cli_def *cli, char *string)
{
	int i = 0;
	cli_print(cli, "Can configure multiple ports, by providing ORed value of each PORT");

	for (i = 0; i < FP_MAX_PORTS; i++) {
		if ((FWD_PORT_LIST_MASK >> i) & 0x1) {
			if (i == HIF_PORT_NUM) {
	        		cli_print(cli, "\tEnter \"%s\" value 0x%x to HIF", string, (0x1 <<i));
			} else {
	        		cli_print(cli, "\tEnter \"%s\" value 0x%x to EMAC%d", string, (0x1 <<i), i+1);
			}
		}
	}

	return;
}

void cli_port_help(struct cli_def *cli, char *string)
{
	int i = 0;

	for (i = 0; i < FP_MAX_PORTS; i++) {
		if ((FWD_PORT_LIST_MASK >> i) & 0x1) {
			if (i == HIF_PORT_NUM) {
	        		cli_print(cli, "\tEnter \"%s\" value 0x%x to HIF", string, (0x1 <<i));
			} else {
	        		cli_print(cli, "\tEnter \"%s\" value 0x%x to EMAC%d", string, (0x1 <<i), i+1);
			}
		}
	}

	return;
}


int cli_print_portlist(struct cli_def *cli, CHAR *string, UINT32 portlist)
{
	int i = 0;

	for (i = 0; i < FP_MAX_PORTS; i++) {
		if (((portlist >> i) & 0x1)) {
			if (i == HIF_PORT_NUM) {
	        		cli_print(cli, "\t%s : HIF", string);
			} else {
	        		cli_print(cli, "\t%s : EMAC%d",string, i+1);
			}

		}
	}

	if (portlist > FWD_PORT_LIST_MASK) {
		cli_print(cli, "\nInvalid port 0x%x", (0x1 << i));
		return CLI_ERROR;
	}

	return CLI_OK;
}

int
displayEntries(struct cli_def *cli, void *entries, int numEntries, int type){
    int i;
    nc_bridge_entry_t *bpCt;

    if(numEntries==0 || !entries){
        cli_print(cli, "There are no entries in the flow table.");
        return CLI_OK;
    }

    if(type == NC_GET_BRIDGE_ENTRIES || type == NC_GET_BRIDGE_ENTRIES_EXT|| type == NC_GET_BRIDGE_ENTRIES_TABLE){
        bpCt =(nc_bridge_entry_t *)entries;
        for(i=0; i<numEntries; i++){
            cli_print(cli, "\nBridge Entry %d", i+1);
            cli_print(cli, "\tVID\t\t:%d\n\tMac Address\t:%02X:%02X:%02X:%02X:%02X:%02X  ",bpCt[i].be_vlanid,
                bpCt[i].be_macaddr[0], bpCt[i].be_macaddr[1], bpCt[i].be_macaddr[2],
                bpCt[i].be_macaddr[3], bpCt[i].be_macaddr[4], bpCt[i].be_macaddr[5]
                );
            cli_print(cli, "\tState\t\t:%s",
                bpCt[i].be_state == BRIDGE_ENTRY_ACTIVE?"Active":"Inactive");
            cli_print(cli, "\tEntry Status\t:%s",
                (bpCt[i].be_actions & MAC_2F_ENTRY_STATIC_MASK)?"STATIC_ENTRY":"DYNAMIC_ENTRY");

	    cli_action_print(cli, "Actions", (bpCt[i].be_actions & MAC_2F_ENTRY_FWD_ACT_MASK)>>MAC_2F_ENTRY_FWD_ACT_START_POS);

	    cli_print_portlist(cli, "FORWARD PORTS", (bpCt[i].be_actions & MAC_2F_ENTRY_FWD_PORT_LIST_MASK)>>MAC_2F_ENTRY_FWD_PORT_LIST_START_POS);
            if(type==NC_GET_BRIDGE_ENTRIES_EXT || type == NC_GET_BRIDGE_ENTRIES_TABLE){
	    	cli_action_print(cli, "Actions", (bpCt[i].be_actions & MAC_2F_ENTRY_FWD_ACT_MASK)>>MAC_2F_ENTRY_FWD_ACT_START_POS);
                cli_print(cli, "\n\tPacket Count\t:%d", bpCt[i].be_pktcount);
            }
        }
    }
    if(type == NC_GET_VLAN_ENTRIES || type == NC_GET_VLAN_ENTRIES_EXT || type == NC_GET_VLAN_ENTRIES_TABLE) {
        bpCt =(nc_bridge_entry_t *)entries;
        for(i=0; i<numEntries; i++){
            cli_print(cli, "\nVLAN Entry %d", i);
            cli_print(cli, "    Mac Address: %02x:%02x:%02x:%02x:%02x:%02x,  VLAN ID=%d",
                bpCt[i].be_macaddr[0], bpCt[i].be_macaddr[1], bpCt[i].be_macaddr[2],
                bpCt[i].be_macaddr[3], bpCt[i].be_macaddr[4], bpCt[i].be_macaddr[5],
                bpCt[i].be_vlanid);
            cli_print(cli, "    State = %s",
                bpCt[i].be_state == BRIDGE_ENTRY_ACTIVE?"Active":"Inactive");
            if(type==NC_GET_VLAN_ENTRIES_EXT){
                cli_print(cli, "    Actions = %d", bpCt[i].be_actions);
                cli_print(cli, "    Packet Count = %d", bpCt[i].be_pktcount);
            }
        }
    }

    return CLI_OK;
}


void printbuf(struct cli_def *cli, char *buff, int size) {
	int i=0;
	for(i=1; i<=size; i++) {
	    cli_print(cli, "%02x ",buff[i-1]&0xff);
        if(((i)%16)==0)
            cli_print(cli, "");
	}
}

void printWords(struct cli_def *cli, unsigned int addr, char *buff, int size) {
	int i=0;
    int *ptr = (int *)buff;
	for(i=0; i< (size/4); i++) {
		cli_print(cli, "%x: %08x ",(addr+i*4), *ptr++);
		if((i> 0)&& ((i)%4)==0)
            cli_print(cli, "");
	}
}

/* Add _ Del _ Flag Values */

#define ADD_TO_BRIDGE 1
#define PROTO_STATS    0x2
#define PARSING_STATS  0x4
#define OTHER_STATS    0x8

int displayStats(struct cli_def *cli, int type, fp_stats_t *fpStats)
{
	cli_print(cli, "%5s %s", "", "STATISTICS:");

	if(type & PROTO_STATS)
    {
		cli_print(cli, "\n%10s %s", "", "PROTOCOL STATS:\n");
		cli_print(cli, "%35s=%u", "IPv4 PACKET COUNT", fpStats->ip4_pkt_cnt);
        cli_print(cli, "%35s=%u", "IPv6 PACKET COUNT", fpStats->ip6_pkt_cnt);
		cli_print(cli, "%35s=%u", "UDP PACKET COUNT", fpStats->udp_pkt_cnt);
		cli_print(cli, "%35s=%u", "TCP PACKET COUNT", fpStats->tcp_pkt_cnt);
		cli_print(cli, "%35s=%u", "ICMP PACKET COUNT", fpStats->icmp_pkt_cnt);
		cli_print(cli, "%35s=%u", "IGMP PACKET COUNT", fpStats->igmp_pkt_cnt);
		//cli_print(cli, "%35s :%u", "ARP PACKET COUNT",
		//				fpStats->igmp_pkt_cnt);
	}

	if(type & PARSING_STATS)
    {
		cli_print(cli, "\n%10s %s", "", "IP PARSING STATS:\n");
		cli_print(cli, "%35s=%u", "IP HEADER LENGTH ERROR COUNT", fpStats->ip_hdr_len_err_cnt);
		cli_print(cli, "%35s=%u", "IP PACKET TTL ERROR COUNT", fpStats->ip_pkt_ttl_err_cnt);
		cli_print(cli, "%35s=%u", "IP CHECKSUM ERROR COUNT", fpStats->ip_chksum_err_cnt);
		// cli_print(cli, "%35s=%u", "MEMORY ALLOCATION ERRORS", fpStats->mem_alloc_err_cnt);
		// cli_print(cli, "%35s=%u", "UNKNOWN_AND_ARP_PKT_CNT", fpStats->unknown_pkt_cnt);
	}

	if( type & OTHER_STATS)
    {
		cli_print(cli, "\n%10s %s", "", "OTHER STATS:\n");
		cli_print(cli, "%35s=%u", "RX-PKT-CNT", fpStats->rx_pkt_cnt);
#ifdef OLD_CLI
		// cli_print(cli, "%35s=%u", "TX-PKT-CNT", fpStats->tx_pkt_cnt);
		// cli_print(cli, "%35s=%u", "PE-HOST-PKT-CNT", fpStats->pe_host_pkt_cnt);
		cli_print(cli, "%35s=%u", "INTERFACE MATCH COUNT", fpStats->intf_match_cnt);
		cli_print(cli, "%35s=%u", "INTERFACE MISMATCH COUNT", fpStats->intf_mismatch_cnt);
		cli_print(cli, "%35s=%u", "SOURCE BRIDGE ENTRY FOUND", 	fpStats->src_be_found);
		cli_print(cli, "%35s=%u", "DESTINATION BRIDGE ENTRY FOUND",	fpStats->dst_be_found);
		cli_print(cli, "%35s=%u", "SOURCE BRIDGE ENTRY NOT FOUND", fpStats->src_be_not_found);
		cli_print(cli, "%35s=%u", "DESTINATION BRIDGE ENTRY NOT FOUND", fpStats->dst_be_not_found);
		cli_print(cli, "%35s=%u", "ROUTE ENTRY FOUND", fpStats->re_found);
		cli_print(cli, "%35s=%u", "ROUTE ENTRY NOT FOUND", fpStats->re_not_found);
#endif
        cli_print(cli, "%35s=%u", "FAST PATH PKT COUNT", fpStats->dst_be_found);
        cli_print(cli, "%35s=%u", "HOST PKT COUNT", fpStats->dst_be_not_found);
	}
	return (0);
}

/*******************************************************************************
 * printStats
 * Arguments : struct cli_def *cli, char *, argv[] , argc
 * Returns   : 0 on success CLI_OK
 * Comments  : Display the stats requested by user.
 ******************************************************************************/
int
printStats(struct cli_def *cli, UNUSED (char *command), char *argv[], int argc)
{
    int printType=0;
    fp_stats_t fpStats;

    if(argc == 2) {
        if(strcmp(argv[1], "zero") == 0) {
            memset(&fpStats, 1, sizeof(fp_stats_t));
            return 0;
        }
    } else {
        memset(&fpStats, 0, sizeof(fp_stats_t));
    }

    if(fp_ioctl_get((void *)&fpStats,  NC_STATS_GET, sizeof(fpStats))){
        cli_print(cli, "Error obtaining stats \n");
        return(CLI_OK);
    }

    /* show fpath stats <VALUES> */
    if(argc==2){
        if(!strcmp(argv[1], "protocol")){
            printType|=PROTO_STATS;
            } else if(!strcmp(argv[1], "parsing-errors")){
                printType|=PARSING_STATS;
            } else if(!strcmp(argv[1], "others")){
                printType|=OTHER_STATS;
            } else if(!strcmp(argv[1], "zero")){
                printType=PROTO_STATS |PARSING_STATS|OTHER_STATS ;
            }
    } else {
        printType=PROTO_STATS |PARSING_STATS|OTHER_STATS ;
    }

    displayStats(cli, printType, &fpStats);
    return (0);
}

int
printDebugStats(struct cli_def *cli, UNUSED (char *command), char *argv[], int argc)
{
    fp_debugstats_t fpStats;


    if(fp_ioctl_get((void *)&fpStats,  NC_DEBUGSTATS_GET, sizeof(fpStats))){
        cli_print(cli, "Error obtaining stats \n");
        return(CLI_OK);
    }

    cli_print(cli, "%s EMAC1=%x  EMAC2=%x\n", "EMAC_TX_COUNTERS", fpStats.emac_tx_counters0, fpStats.emac_tx_counters1);
    cli_print(cli, "%s EMAC1=%x  EMAC2=%x\n", "EMAC_RX_COUNTERS", fpStats.emac_rx_counters0, fpStats.emac_rx_counters1);
    cli_print(cli, "%sUNDERSIZE_FRAMES :%x EXESSIVE_RX_LEN :%x RX_JABBERS :%x FCS_ERR :%x RX_LEN_ERR :%x RX_SYMB_ERR :%x  ALIGN_ERR :%x\nRX_RESOURCE_ERR :%x RX_OVERRUNS :%x  RX_IP_CHECK_ERR :%x  RX_TCP_CHECK_ERR :%x  RX_UDP_CHECK_ERR :%x  AUTO_FLUSH_PACKETS :%x\n", "EMAC-1 ERRORS:-\n", fpStats.emac_errors0[0], fpStats.emac_errors0[1], fpStats.emac_errors0[2], fpStats.emac_errors0[3], fpStats.emac_errors0[4], fpStats.emac_errors0[5], fpStats.emac_errors0[6], fpStats.emac_errors0[7], fpStats.emac_errors0[8], fpStats.emac_errors0[9], fpStats.emac_errors0[10], fpStats.emac_errors0[11], fpStats.emac_errors0[12]);

    cli_print(cli, "%sUNDERSIZE_FRAMES :%x EXESSIVE_RX_LEN :%x RX_JABBERS :%x FCS_ERR :%x RX_LEN_ERR :%x RX_SYMB_ERR :%x  ALIGN_ERR :%x\nRX_RESOURCE_ERR :%x RX_OVERRUNS :%x  RX_IP_CHECK_ERR :%x  RX_TCP_CHECK_ERR :%x  RX_UDP_CHECK_ERR :%x  AUTO_FLUSH_PACKETS :%x\n", "EMAC-2 ERRORS:-\n", fpStats.emac_errors1[0], fpStats.emac_errors1[1], fpStats.emac_errors1[2], fpStats.emac_errors1[3], fpStats.emac_errors1[4], fpStats.emac_errors1[5], fpStats.emac_errors1[6], fpStats.emac_errors1[7], fpStats.emac_errors1[8], fpStats.emac_errors1[9], fpStats.emac_errors1[10], fpStats.emac_errors1[11], fpStats.emac_errors1[12]);

    cli_print(cli, "%s EGPI1 :%x EGPI2 :%x HGPI1 :%x HGPI2 :%x\n", "GPI FIFO COUNT", fpStats.gpi_fifo_count0, fpStats.gpi_fifo_count1, fpStats.gpi_fifo_count2, fpStats.gpi_fifo_count3);
    cli_print(cli, "%s EGPI1 :%x EGPI2 :%x\n", "GPI DROP COUNT", fpStats.gpi_drop_count0, fpStats.gpi_drop_count1);

    cli_print(cli, "%sRX_PKTS :%x TX_PKTS :%x  LP_FAIL_PKTS :%x INTF_FAIL_PKTS :%x  INTF_MATCH_PKTS :%x\nL3_FAIL_PKTS :%x  V4_PKTS :%x V6_PKTS :%x  CHECKSUM_ERRS :%x TTL_ERRS :%x\n", "CLASS_PHY1_COUNTERS:-\n", fpStats.class_phy1_count[0], fpStats.class_phy1_count[1], fpStats.class_phy1_count[2], fpStats.class_phy1_count[3], fpStats.class_phy1_count[4], fpStats.class_phy1_count[5], fpStats.class_phy1_count[6], fpStats.class_phy1_count[7], fpStats.class_phy1_count[8], fpStats.class_phy1_count[9]);

    cli_print(cli, "%sRX_PKTS :%x TX_PKTS :%x  LP_FAIL_PKTS :%x INTF_FAIL_PKTS :%x  INTF_MATCH_PKTS :%x\nL3_FAIL_PKTS :%x  V4_PKTS :%x V6_PKTS :%x  CHECKSUM_ERRS :%x TTL_ERRS :%x\n", "CLASS_PHY2_COUNTERS:-\n", fpStats.class_phy2_count[0], fpStats.class_phy2_count[1], fpStats.class_phy2_count[2], fpStats.class_phy2_count[3], fpStats.class_phy2_count[4], fpStats.class_phy2_count[5], fpStats.class_phy2_count[6], fpStats.class_phy2_count[7], fpStats.class_phy2_count[8], fpStats.class_phy2_count[9]);

    cli_print(cli, "%s BUF_COUNT :%x REM_COUNT :%x CUR_COUNT :%x\n", "BMU_COUNTS", fpStats.bmu_buffer_count, fpStats.bmu_rem_count, fpStats.bmu_cur_count);

    cli_print(cli, "%s MAS0 :%x MAS1 :%x MAS2 :%x MAS3 :%x MAS4 :%x MAS5 :%x MAS6 :%x MAS7 :%x MAS8 :%x\n", "BMU_MASTER_BASED_BUFFER_COUNT", fpStats.bmu_master_based_buffer_counts[0], fpStats.bmu_master_based_buffer_counts[1], fpStats.bmu_master_based_buffer_counts[2], fpStats.bmu_master_based_buffer_counts[3], fpStats.bmu_master_based_buffer_counts[4], fpStats.bmu_master_based_buffer_counts[5], fpStats.bmu_master_based_buffer_counts[6], fpStats.bmu_master_based_buffer_counts[7], fpStats.bmu_master_based_buffer_counts[8]);
    /*In HIF-TX Counters -CSR 0x9c --cnt1 and 0xa0 --cnt2*/
    /*In HIF-RX Counters - CSR 0xbc --cnt1 and 0xc0 --cnt2*/
    cli_print(cli, "%s CNT1 :%x  CNT2 :%x\n", "HIF_TX_COUNTERS", fpStats.hif1_tx_counters1, fpStats.hif1_tx_counters0);
    cli_print(cli, "%s CNT1 :%x  CNT2 :%x\n", "HIF_RX_COUNTERS", fpStats.hif1_rx_counters1, fpStats.hif1_rx_counters0);
    cli_print(cli, "%s CNT1 :%x  CNT2 :%x\n", "HIF2_TX_COUNTERS", fpStats.hif2_tx_counters1, fpStats.hif2_tx_counters0);
    cli_print(cli, "%s CNT1 :%x  CNT2 :%x\n", "HIF2_RX_COUNTERS", fpStats.hif2_rx_counters1, fpStats.hif2_rx_counters0);
    cli_print(cli, "%s STATE0 :0x%x STATE1 :0x%x\n", "EMAC1_GPI_STATE", fpStats.egpi1_state0, fpStats.egpi1_state1);
    cli_print(cli, "%s STATE0 :0x%x STATE1 :0x%x\n", "EMAC2_GPI_STATE", fpStats.egpi2_state0, fpStats.egpi2_state1);
    cli_print(cli, "%s STATE0 :0x%x STATE1 :0x%x\n", "HIF_GPI_STATE", fpStats.hgpi1_state0, fpStats.hgpi1_state1);
    cli_print(cli, "%s STATE0 :0x%x STATE1 :0x%x\n", "HIF_GPI_STATE", fpStats.hgpi2_state0, fpStats.hgpi2_state1);

    return (0);
}

int
get_entries_count(int cmd, struct cli_def *cli, UNUSED(char *command), char *argv[], int argc){
    nc_entries_ops_t ent_ops;
    u_int    ent_count=0;

    memset(&ent_ops, 0, sizeof(nc_entries_ops_t));
    ent_ops.ent_cmd  = cmd;
    ent_ops.ent_size = sizeof(ent_count);
    ent_ops.ent_data = (char *)&ent_count;

    if(fp_ioctl_get((void *)&ent_ops, NC_SWITCH_ENTRIES_GET, sizeof(ent_ops))){
        cli_print(cli, "ioctl error, Unable to obtain necessary data \n");
        return (CLI_OK);
    }
//#ifdef DEBUG
#if 1
    fprintf(stdout, "%s returning ent_count = %d \n", __FUNCTION__, ent_count);
#endif
    return (ent_count);
}

int
show_entries(int ent_cnt_cmd, int ent_type_cmd, struct cli_def *cli,
        UNUSED (char *command), char *argv[], int argc){
    nc_entries_ops_t ent_ops;
    int ent_count;
    nc_router_entry_t  *nc_entries=NULL;
    u_int       ent_buf_size;
  //  int i=0;
    ent_count = get_entries_count(ent_cnt_cmd, cli, command, argv, argc);
//#ifdef DEBUG
#if 1
    fprintf(stdout, "%s ent_count= %d \n", __FUNCTION__, ent_count);
#endif

    if(ent_count < 0){
        cli_print(cli, "Error Unable to obtain number of entries.");
        return CLI_OK;
    }
    switch (ent_type_cmd){
        case NC_GET_FLOW_ENTRIES:
        case NC_GET_FLOW_ENTRIES_EXT:
            ent_buf_size = ent_count * 2 * sizeof(nc_router_entry_t);
            break;
        case NC_GET_BRIDGE_ENTRIES:
        case NC_GET_BRIDGE_ENTRIES_EXT:
        case NC_GET_BRIDGE_ENTRIES_TABLE:
            ent_buf_size = ent_count * sizeof(nc_bridge_entry_t);
            break;
        case NC_GET_VLAN_ENTRIES:
        case NC_GET_VLAN_ENTRIES_EXT:
        case NC_GET_VLAN_ENTRIES_TABLE:
            ent_buf_size = ent_count * 2 * sizeof(nc_bridge_entry_t);
            break;
        default:
            return CLI_OK;
    }
    nc_entries = calloc(ent_count, sizeof(nc_bridge_entry_t));
    if(!nc_entries){
        cli_print(cli, "Error malloc (%d)", ent_buf_size);
        return CLI_OK;
    }

    memset(&ent_ops, 0, sizeof(nc_entries_ops_t));
    ent_ops.ent_cmd  = ent_type_cmd;
    ent_ops.ent_size = ent_buf_size;
    ent_ops.ent_count = ent_count;
    if(ent_type_cmd == NC_GET_BRIDGE_ENTRIES_TABLE)
       ent_ops.ent_index = atoi(argv[3]);

    if(ent_type_cmd == NC_GET_VLAN_ENTRIES_TABLE)
       ent_ops.ent_index = atoi(argv[3]);

    ent_ops.ent_data = (char *)nc_entries;
    if(fp_ioctl_get((void *)&ent_ops, NC_SWITCH_ENTRIES_GET, sizeof(ent_ops))){
        cli_print(cli, "ioctl error, Unable to obtain necessary data \n");
        return (CLI_OK);
    }
    /* Update the count with what ever returned by kernel */

    ent_count = ent_ops.ent_count;

    if(ent_count && nc_entries) {
        displayEntries(cli, nc_entries, ent_count, ent_type_cmd);
    }

    free(nc_entries);
    return (0);
}

static int
show_bridge_entries(struct cli_def *cli,  char *command,
                    char *argv[], int argc){
    return show_entries(NC_GET_BRIDGE_ENTRIES_COUNT, NC_GET_BRIDGE_ENTRIES,
                        cli, command, argv, argc);

}

int
show_bridge_entries_count(struct cli_def *cli, UNUSED (char *command),
       char *argv[], int argc){
    int ent_count;
    ent_count = get_entries_count(NC_GET_BRIDGE_ENTRIES_COUNT,
                cli, command, argv, argc);
    cli_print(cli, "Total Number of bridge entries = %d\n", ent_count);
    return (0);
}
#if 0
static int
show_bridge_entries_ext(struct cli_def *cli,  char *command,
                          char *argv[], int argc){
    return show_entries(NC_GET_BRIDGE_ENTRIES_COUNT, NC_GET_BRIDGE_ENTRIES_EXT,
        cli, command, argv, argc);
}
static int
show_bridge_entries_table(struct cli_def *cli,  char *command,
                          char *argv[], int argc){
    return show_entries(NC_GET_BRIDGE_ENTRIES_COUNT, NC_GET_BRIDGE_ENTRIES_TABLE,
        cli, command, argv, argc);
}
#endif

static int
show_vlan_entries(struct cli_def *cli,  char *command,
                    char *argv[], int argc){
        return show_entries(NC_GET_VLAN_ENTRIES_COUNT, NC_GET_VLAN_ENTRIES,
             cli, command, argv, argc);
}

static int
show_vlan_entries_count(struct cli_def *cli,  char *command,
                    char *argv[], int argc){
int ent_count;
    ent_count = get_entries_count(NC_GET_VLAN_ENTRIES_COUNT,
                cli, command, argv, argc);
    cli_print(cli, "Total Number of vlan entries = %d\n", ent_count);
    return (0);
}

static int
show_vlan_entries_ext(struct cli_def *cli,  char *command,
                    char *argv[], int argc){
        return show_entries(NC_GET_VLAN_ENTRIES_COUNT, NC_GET_VLAN_ENTRIES_EXT,
             cli, command, argv, argc);
}

static int
show_vlan_entries_table(struct cli_def *cli,  char *command,
                          char *argv[], int argc){
    return show_entries(NC_GET_VLAN_ENTRIES_COUNT, NC_GET_VLAN_ENTRIES_TABLE,
        cli, command, argv, argc);
}

int
check_auth (char *username, char *password)
{
  if (strcasecmp (username, "fred") != 0)
    return CLI_ERROR;
  if (strcasecmp (password, "nerk") != 0)
    return CLI_ERROR;
  return CLI_OK;
}

int
check_enable (char *password)
{
    return (0);
  return !strcasecmp (password, "topsecret");
}

int
AddFPathStatsCmds(struct cli_def *cli, nakCli_t *fpCli){

    cli_register_command(cli, fpCli->nk_show, "stats", printStats,
              PRIVILEGE_UNPRIVILEGED, MODE_EXEC, "show statistics ");

	return (0);
}

int
AddDebugStatsCmds(struct cli_def *cli, nakCli_t *fpCli){

    cli_register_command(cli, fpCli->nk_show, "debug_stats", printDebugStats,
              PRIVILEGE_UNPRIVILEGED, MODE_EXEC, "show debug statistics ");

        return (0);
}

int
DebugPrintEnable_Callback (struct cli_def *cli, char *command, char *argv[], int argc) {

    int fastpath = 0;

    if(fp_ioctl_get((void *)&fastpath, NC_DEBUGPRINTS_ENABLE, sizeof(fastpath))){
        fprintf(stderr, "ioctl error, Unable to obtain necessary data \n");
        return (0);
    }
    return (0);


}

int
DebugPrintDisable_Callback (struct cli_def *cli, char *command, char *argv[], int argc) {


    int fastpath = 0;
    if(fp_ioctl_get((void *)&fastpath, NC_DEBUGPRINTS_DISABLE, sizeof(fastpath))){
        fprintf(stderr, "ioctl error, Unable to obtain necessary data \n");
        return (0);
    }

    return (0);
}

int
AddDebugPrintsCmds(struct cli_def *cli, nakCli_t *fpCli){

       /* show debug_enable */

       cli_register_command(cli, fpCli->nk_show, "debug_enable", DebugPrintEnable_Callback,
		PRIVILEGE_UNPRIVILEGED, MODE_EXEC, "Enables debug prints");

       cli_register_command(cli, fpCli->nk_show, "debug_disable", DebugPrintDisable_Callback,
		PRIVILEGE_UNPRIVILEGED, MODE_EXEC, "Disables debug prints");

    return (0);
}

int
AddFPathBrdCmds(struct cli_def *cli, nakCli_t *fpCli){
    struct cli_command *fpEntries;
    nakCli_t fpBrgCli;
    //int    table_idx,port_type_idx=0, unit_idx, unit_type_idx=0;
    //char   helpText[1024];
    memset(&fpBrgCli, 0, sizeof(fpBrgCli));

       fpBrgCli.nk_show = cli_register_command(cli, fpCli->nk_show, "bridge", NULL,
		PRIVILEGE_UNPRIVILEGED, MODE_EXEC, "show bridge entries");

	fpBrgCli.nk_add = cli_register_command(cli, fpCli->nk_add, "bridge", NULL,
		PRIVILEGE_UNPRIVILEGED, MODE_EXEC, "Add bridge entries.");

    /* BEGIN SHOW BRIDGE/VLAN ENTRIES */
	fpEntries = cli_register_command(cli, fpBrgCli.nk_show, "entries",
            show_bridge_entries, PRIVILEGE_UNPRIVILEGED, MODE_EXEC,
        "Display entries in the bridge table tables .");

	cli_register_command(cli, fpEntries, "count", show_bridge_entries_count,
		PRIVILEGE_UNPRIVILEGED, MODE_EXEC,
        "Display number of entries in bridge table.");
#if 0
	cli_register_command(cli, fpEntries, "extensive",
        show_bridge_entries_ext, PRIVILEGE_UNPRIVILEGED, MODE_EXEC,
        "Display detailed information on entries in bridge table.");

	fpEntries = cli_register_command(cli, fpEntries, "table",
        NULL, PRIVILEGE_UNPRIVILEGED, MODE_EXEC,
        "Display given index bridge table entries.");

         for(table_idx=0; table_idx <FP_MAX_BRTBL; ++table_idx){
            /* put table index [0,1] */
            snprintf(buf, sizeof(buf), "%d", table_idx);
            snprintf(helpText, sizeof(helpText), "Enter table index %d",
                       table_idx);
            cli_register_command(cli, fpEntries, buf,
            show_bridge_entries_table, PRIVILEGE_UNPRIVILEGED, MODE_EXEC,
            "Display given index bridge table entries");

          }
#endif
    /* END SHOW BRIDGE ENTRIES */
    return (0);
}

int
AddFPathVlanCmds(struct cli_def *cli, nakCli_t *fpCli){
    struct cli_command *fpEntries;
    int    table_idx;
    char   buf[16], helpText[1024];

    nakCli_t fpVlanCli;
    memset(&fpVlanCli, 0, sizeof(fpVlanCli));

       fpVlanCli.nk_show = cli_register_command(cli, fpCli->nk_show, "vlan", NULL,
		PRIVILEGE_UNPRIVILEGED, MODE_EXEC, "show vlan entries");

       fpVlanCli.nk_add = cli_register_command(cli, fpCli->nk_add, "vlan", NULL,
		PRIVILEGE_UNPRIVILEGED, MODE_EXEC, "Add vlan entries.");

    /* BEGIN SHOW VLAN ENTRIES */
       fpEntries = cli_register_command(cli, fpVlanCli.nk_show, "entries",
            show_vlan_entries, PRIVILEGE_UNPRIVILEGED, MODE_EXEC,
        "Display entries in the vlan tables .");

       cli_register_command(cli, fpEntries, "count", show_vlan_entries_count,
		PRIVILEGE_UNPRIVILEGED, MODE_EXEC,
        "Display number of members in vlan tables.");

       cli_register_command(cli, fpEntries, "extensive",
        show_vlan_entries_ext, PRIVILEGE_UNPRIVILEGED, MODE_EXEC,
        "Display detailed information on entries in vlan table.");

        fpEntries = cli_register_command(cli, fpEntries, "table",
        NULL, PRIVILEGE_UNPRIVILEGED, MODE_EXEC,
        "Display given index vlan table entries.");

         for(table_idx=0; table_idx <FP_MAX_VLAN_GRPTBL; ++table_idx) {
            /* put table index [0,1] */
            snprintf(buf, sizeof(buf), "%d", table_idx);
            snprintf(helpText, sizeof(helpText), "Enter vlan table index %d",
                       table_idx);
            cli_register_command(cli, fpEntries, buf,
            show_vlan_entries_table, PRIVILEGE_UNPRIVILEGED, MODE_EXEC,
            helpText);

          }
    /* END SHOW VLAN ENTRIES */
    return (0);
}

/* enable or disable fastpath mode */
int
set_fastpath_mode(struct cli_def *cli, UNUSED (char *command), char *argv[], int argc)
{

   int fastpath = 0;

   if(strcmp(argv[0],"fastpath") == 0) {
       if(strcmp(argv[1],"enable") == 0) {
           fastpath = 1;
       } else if(strcmp(argv[1], "disable") == 0) {
           fastpath = 0;
       } else {
           cli_print(cli, "unknow fastpath mode\n");
       }
   } else {
       if(strcmp(argv[1],"enable") == 0) {
           fastpath = 2;
       } else if(strcmp(argv[1], "disable") == 0) {
           fastpath = 3;
       } else {
           cli_print(cli, "unknow fastpath mode\n");
       }
   }

   if(fp_ioctl_get((void *)&fastpath, NC_SET_FASTPATH, sizeof(fastpath))) {
       cli_print(cli, "Error obtaining stats \n");
       return(CLI_OK);
   }

    return(CLI_OK);
}


int
AddFastPathCmds(struct cli_def *cli){
    nakCli_t fpCli;

    memset(&fpCli, 0, sizeof(fpCli));
    fpCli.nk_add = nkCli.nk_add;
    fpCli.nk_show = nkCli.nk_show;

    AddFPathStatsCmds(cli, &fpCli);
    AddDebugStatsCmds(cli, &fpCli);
    AddDebugPrintsCmds(cli, &fpCli);
    AddFPathBrdCmds  (cli, &fpCli);
    /* AddFPathVlanCmds  (cli, &fpCli); */

    return (0);
}

void init_l2cli(struct cli_def *cli)
{

    nkCli.nk_tlite = cli_register_command(cli, NULL, "tlite", NULL, // added for Tlite
    PRIVILEGE_UNPRIVILEGED, MODE_EXEC, "Enter Tlite command");

    nkCli.nk_switch = cli_register_command(cli, NULL, "switch", NULL, // added for switch
    PRIVILEGE_UNPRIVILEGED, MODE_EXEC, "Enter switch command");

    AddFastPathCmds(cli);
    AddTlite_Cmds(cli);
    AddSwitch_Cmds(cli);

}
