#ifndef __PHY_DP83867_H
#define __PHY_DP83867_H

#include "fp_mdio_ops.h"
#include <linux/types.h>
#include "fp_port.h"

#define DP83867_PHY_ID          0x2000a231

#define DP83867_MMD3_PCS_CTRL           0

enum
{
   DP83867_BMCR = 0,
   DP83867_BMSR,
   DP83867_PHYIDR1,
   DP83867_PHYIDR2,
   DP83867_ANAR,
   DP83867_ANLPAR,
   DP83867_ANER,
   DP83867_ANNPTR,
   DP83867_ANNPRR,
   DP83867_CFG1,
   DP83867_STS1,
   DP83867_REGCR = 13,
   DP83867_ADDAR,
   DP83867_1KSCR,
   DP83867_PHYCR,
   DP83867_PHYSTS,
   DP83867_MICR,
   DP83867_ISR,
   DP83867_CFG2,
   DP83867_RECR,
   DP83867_BISCR,
   DP83867_STS2,
   DP83867_LEDCR1,
   DP83867_LEDCR2,
   DP83867_LEDCR3,
   DP83867_CFG3 = 30,
   DP83867_CTRL,
   DP83867_TMCH_CTRL = 37,
   DP83867_FLD_CFG = 45,
   DP83867_CFG4 = 49,
   DP83867_RGMIICTL,
   DP83867_RGMIICTL2,
   DP83867_SGMII_ANEG_STS = 55,
   DP83867_100CR = 67,
   DP83867_SKEW_FIFO = 85,
   DP83867_STRAP_STS1 = 110,
   DP83867_STRAP_STS2,
   DP83867_BICSR1 = 113,
   DP83867_BICSR2,
   DP83867_RGMIIDCTL = 134,
   DP83867_SGMIICTL1 = 211,
   DP83867_SYNC_FIFO_CTRL = 233,
   DP83867_LOOPCR = 254,
   DP83867_RXFCFG = 308,
   DP83867_RXFSTS,
   DP83867_RXFPMD1,
   DP83867_RXFPMD2,
   DP83867_RXFPMD3,
   DP83867_RXFSOP1,
   DP83867_RXFSOP2,
   DP83867_RXFSOP3,
   DP83867_RXFPAT1,
   DP83867_RXFPAT2,
   DP83867_RXFPAT3,
   DP83867_RXFPAT4,
   DP83867_RXFPAT5,
   DP83867_RXFPAT6,
   DP83867_RXFPAT7,
   DP83867_RXFPAT8,
   DP83867_RXFPAT9,
   DP83867_RXFPAT10,
   DP83867_RXFPAT11,
   DP83867_RXFPAT12,
   DP83867_RXFPAT13,
   DP83867_RXFPAT14,
   DP83867_RXFPAT15,
   DP83867_RXFPAT16,
   DP83867_RXFPAT17,
   DP83867_RXFPAT18,
   DP83867_RXFPAT19,
   DP83867_RXFPAT20,
   DP83867_RXFPAT21,
   DP83867_RXFPAT22,
   DP83867_RXFPAT23,
   DP83867_RXFPAT24,
   DP83867_RXFPAT25,
   DP83867_RXFPAT26,
   DP83867_RXFPAT27,
   DP83867_RXFPAT28,
   DP83867_RXFPAT29,
   DP83867_RXFPAT30,
   DP83867_RXFPAT31,
   DP83867_RXFPAT32,
   DP83867_RXFPBM1,
   DP83867_RXFPBM2,
   DP83867_RXFPBM3,
   DP83867_RXFPBM4,
   DP83867_RXFPATC = 353,
   DP83867_10M_SGMII_CFG = 367,
   DP83867_IO_MUX_CFG,
   DP83867_GPIO_MUX_CTRL = 370,
   DP83867_TDR_GEN_CFG1 = 384,
   DP83867_ALCD_CTRL = 423,
};

enum
{
   DP83867_LED_LINK_ESTABL = 0,
   DP83867_LED_RXTX_ACT,
   DP83867_LED_TX_ACT,
   DP83867_LED_RX_ACT,
   DP83867_LED_COLLISION,
   DP83867_LED_100BT_LINK,
   DP83867_LED_100BTX_LINK,
   DP83867_LED_10BT_LINK,
   DP83867_LED_10_100BT_LINK,
   DP83867_LED_100_1000BT_LINK,
   DP83867_LED_FULL_DUPLX,
   DP83867_LED_LINK_TXRX_BLINK,
   DP83867_LED_RESERVED,
   DP83867_LED_RXTX_ERR,
   DP83867_LED_RX_ERR
};

enum {
    DP83867_LEDS_ON = 0,
    DP83867_LEDS_OFF,
    DP83867_LEDS_SHOW_STATE
 };

#define DP83867_LED0_POS                        0
#define DP83867_LED1_POS                        4
#define DP83867_LED2_POS                        8
#define DP83867_LED3_POS                        12

#define DP83867_SW_RESET                        0x8000
#define DP83867_SW_RESTART                      0x4000

/* PHY CTRL bits */
#define DP83867_PHYCR_FIFO_DEPTH_SHIFT          14
#define DP83867_PHYCR_FIFO_DEPTH_3_B_NIB        0x00
#define DP83867_PHYCR_FIFO_DEPTH_4_B_NIB        0x01
#define DP83867_PHYCR_FIFO_DEPTH_6_B_NIB        0x02
#define DP83867_PHYCR_FIFO_DEPTH_8_B_NIB        0x03

void phy_dp83867_init( uint8_t ucMdioPort, uint8_t ucMdioAdr, uint16_t speed );
void phy_dp83867_speed_set( uint8_t ucMdioPort, uint8_t ucMdioAdr, uint16_t speed, uint8_t duplex, uint8_t autoneg );
void phy_dp83867_restart_aneg( uint8_t ucMdioPort, uint8_t ucMdioAdr );
uint32_t phy_dp83867_get_link_part_abil( uint8_t ucMdioPort, uint8_t ucMdioAdr );
uint32_t phy_dp83867_get_mdi_stat( uint8_t ucMdioPort, uint8_t ucMdioAdr );
void phy_dp83867_led_drive(  uint8_t ucMdioPort, uint8_t ucMdioAdr, uint8_t ucState );
uint32_t phy_dp83867_mmd_read(uint8_t ucMdioPort, uint8_t ucMdioAdr, uint16_t reg_addr);
void  phy_dp83867_mmd_write(uint8_t ucMdioPort, uint8_t ucMdioAdr, uint16_t reg_addr, uint16_t write_value);
void phy_dp83867_sgmii_loopback_enable( uint8_t ucMdioPort, uint8_t ucMdioAdr);
uint32_t phy_dp83867_get_duplex( uint8_t ucMdioPort, uint8_t ucMdioAdr );
uint32_t phy_dp83867_is_flow_control_available( uint8_t ucMdioPort, uint8_t ucMdioAdr );
uint32_t phy_dp83867_set_fc_parameters( uint8_t ucMdioPort, uint8_t ucMdioAdr, uint32_t param );
uint32_t phy_dp83867_is_flow_control_advertised( uint8_t ucMdioPort, uint8_t ucMdioAdr );
void phy_dp83867_set_power_down( uint8_t ucMdioPort, uint8_t ucMdioAdr );
void phy_dp83867_set_power_up( uint8_t ucMdioPort, uint8_t ucMdioAdr );
void phy_dp83867_set_advertise_speed_duplex( uint8_t ucMdioPort, uint8_t ucMdioAdr, uint32_t advertise );
void phy_dp83867_autoneg_enable( uint8_t ucMdioPort, uint8_t ucMdioAdr );
#endif	//__PHY_DP83867_H
