#include "phy_i347at4.h"
#include "fp_library.h"
#include <linux/ethtool.h>

static void phy_i347at4_reset_page_0_2_3_5_7(u8 ucMdioPort, u8 ucMdioAdr);

void phy_i347at4_init( u8 ucMdioPort, u8 ucMdioAdr, u16 speed )
{
    int tmp;

    FP_DEBUG(DBGP_FEAT_HIF, FP_LOG_INFO, "init phy_i347at4 adr=%d\n", ucMdioAdr);

    /* --- Волшебная инициализация 1 от Intel --- */
    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, I347AT4_PAGE_SELECTOR_REG, 0x00FF );
    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, 24, 0x2800 );
    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, 23, 0x2001 );
    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, I347AT4_PAGE_SELECTOR_REG, PAGE_0 );

    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, I347AT4_PAGE_SELECTOR_REG, 0x00FF );
    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, 23, 0x1001 );
    tmp = fp_mdio_read_phy( ucMdioPort, ucMdioAdr, 25 );

    if( tmp != 0x2800 )
        FP_DEBUG(DBGP_FEAT_HIF, FP_LOG_INFO, "Intel init 1 error\n");

    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, I347AT4_PAGE_SELECTOR_REG, PAGE_0 );

    /* --- Волшебная инициализация 2 от Intel --- */
    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, I347AT4_PAGE_SELECTOR_REG, PAGE_0 );
    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, 29, 0x0003 );
    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, 30, 0x0002 );
    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, 29, 0x0000 );

    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, I347AT4_PAGE_SELECTOR_REG, PAGE_0 );
    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, 29, 0x0003 );
    tmp = fp_mdio_read_phy( ucMdioPort, ucMdioAdr, 30 );

    if( tmp != 0x0002 )
        FP_DEBUG(DBGP_FEAT_HIF, FP_LOG_INFO, "Intel init 2 error\n");

    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, 29, 0x0000 );

    /* --- Включение микросхемы PHY --- */
    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, I347AT4_PAGE_SELECTOR_REG, PAGE_0 );
    tmp = fp_mdio_read_phy( ucMdioPort, ucMdioAdr, P0_COPPER_CNTRL_REG );
    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, P0_COPPER_CNTRL_REG, tmp & 0xF7FF );
    tmp = fp_mdio_read_phy( ucMdioPort, ucMdioAdr, P0_COPPER_SPEC_CNTRL_REG1 );
    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, P0_COPPER_SPEC_CNTRL_REG1, tmp & 0xFFFB );

    /* --- Сброс страниц 0, 2, 3, 5, 7 --- */
    phy_i347at4_reset_page_0_2_3_5_7( ucMdioPort, ucMdioAdr );

    /* --------- Сброс страницы 6 --------- */
    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, I347AT4_PAGE_SELECTOR_REG, PAGE_6 );
    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, P6_GENERAL_CNTRL_REG, 0x8001 );

    /* --- Настройка светодиодов --- */
    phy_i347at4_setup_led_default( ucMdioPort, ucMdioAdr );

   /* ------- Настройка downshift ------- */
   fp_mdio_write_phy( ucMdioPort, ucMdioAdr, I347AT4_PAGE_SELECTOR_REG, PAGE_0 );
   tmp = fp_mdio_read_phy( ucMdioPort, ucMdioAdr, P0_COPPER_SPEC_CNTRL_REG1 );
   tmp |= 0x2800;                                                               // Downshift counter = 3, Downshift enable
   fp_mdio_write_phy( ucMdioPort, ucMdioAdr, P0_COPPER_SPEC_CNTRL_REG1, tmp );
   FP_DEBUG(DBGP_FEAT_HIF, FP_LOG_INFO, "PHY_ADDR %d downshift counter 3x, downshift enable\n", ucMdioAdr);

   phy_i347at4_speed_set( ucMdioPort, ucMdioAdr, 0, 1 );
   FP_DEBUG(DBGP_FEAT_HIF, FP_LOG_INFO, "init phy_i347at4 end\n");
}

void phy_i347at4_reset_page_0_2_3_5_7( u8 ucMdioPort, u8 ucMdioAdr )
{
    u32 tmp;

    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, I347AT4_PAGE_SELECTOR_REG, PAGE_0 );
    tmp = fp_mdio_read_phy( ucMdioPort, ucMdioAdr, P0_COPPER_CNTRL_REG );
    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, P0_COPPER_CNTRL_REG, tmp | 0x8000 );
}

void phy_i347at4_restart_aneg( u8 ucMdioPort, u8 ucMdioAdr )
{
    u32 tmp;

    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, I347AT4_PAGE_SELECTOR_REG, PAGE_0 );
    tmp = fp_mdio_read_phy( ucMdioPort, ucMdioAdr, P0_COPPER_CNTRL_REG );
    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, P0_COPPER_CNTRL_REG, tmp | 0x0200 );
}

void phy_i347at4_speed_set( u8 ucMdioPort, u8 ucMdioAdr, u16 speed, u8 duplex )
{
    u32 tmp;

    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, I347AT4_PAGE_SELECTOR_REG, PAGE_0 );
    switch ( speed )
    {
    case 10:
        tmp = fp_mdio_read_phy( ucMdioPort, ucMdioAdr, P0_COPPER_AUTO_NEG_ADV_REG );
        tmp &= 0x7E1F;
        if( duplex )
        {
            fp_mdio_write_phy( ucMdioPort, ucMdioAdr, P0_COPPER_CNTRL_REG, 0x0100 );
            tmp |= (1 << 6);
        }
        else
        {
            fp_mdio_write_phy( ucMdioPort, ucMdioAdr, P0_COPPER_CNTRL_REG, 0x0000 );
            tmp |= (1 << 5);
        }
        fp_mdio_write_phy( ucMdioPort, ucMdioAdr, P0_COPPER_AUTO_NEG_ADV_REG, tmp );

        tmp = fp_mdio_read_phy( ucMdioPort, ucMdioAdr, P0_1000BASE_T_CNTRL_REG );
        tmp &= ~((1 << 9)|(1 << 8));	                                       // 1000BASE-T Full-Duplex Not Advertise, 1000BASE-T Half-Duplex Not Advirtes
        fp_mdio_write_phy( ucMdioPort, ucMdioAdr, P0_1000BASE_T_CNTRL_REG, tmp );
        break;
    case 100:
        tmp = fp_mdio_read_phy( ucMdioPort, ucMdioAdr, P0_COPPER_AUTO_NEG_ADV_REG );
        tmp &= 0x7E1F;
        if( duplex )
        {
            fp_mdio_write_phy( ucMdioPort, ucMdioAdr, P0_COPPER_CNTRL_REG, 0xA100 );
            tmp |= (1 << 8);
        }
        else
        {
            fp_mdio_write_phy( ucMdioPort, ucMdioAdr, P0_COPPER_CNTRL_REG, 0xA000 );
            tmp |= (1 << 7);
        }
        fp_mdio_write_phy( ucMdioPort, ucMdioAdr, P0_COPPER_AUTO_NEG_ADV_REG, tmp );

        tmp = fp_mdio_read_phy( ucMdioPort, ucMdioAdr, P0_1000BASE_T_CNTRL_REG );
        tmp &= ~((1 << 9)|(1 << 8));	                                       // 1000BASE-T Full-Duplex Not Advertise, 1000BASE-T Half-Duplex Not Advirtes
        fp_mdio_write_phy( ucMdioPort, ucMdioAdr, P0_1000BASE_T_CNTRL_REG, tmp );
        break;
    case 1000:
        tmp = fp_mdio_read_phy( ucMdioPort, ucMdioAdr, P0_1000BASE_T_CNTRL_REG );
        tmp &= ~((1 << 9)|(1 << 8));
        if( duplex )
        {
            fp_mdio_write_phy( ucMdioPort, ucMdioAdr, P0_COPPER_CNTRL_REG, 0x0140 );
            tmp |= 1 << 9;
        }
        else
        {
            fp_mdio_write_phy( ucMdioPort, ucMdioAdr, P0_COPPER_CNTRL_REG, 0x0040 );
            tmp |= 1 << 8;
        }
        fp_mdio_write_phy( ucMdioPort, ucMdioAdr, P0_COPPER_AUTO_NEG_ADV_REG, 0x8001 );
        fp_mdio_write_phy( ucMdioPort, ucMdioAdr, P0_1000BASE_T_CNTRL_REG, tmp );
        break;
    default:                                                                    // Autoneg on
        fp_mdio_write_phy( ucMdioPort, ucMdioAdr, P0_COPPER_CNTRL_REG, 0x1140 );
        fp_mdio_write_phy( ucMdioPort, ucMdioAdr, P0_COPPER_AUTO_NEG_ADV_REG, 0x01E0 );
        tmp = fp_mdio_read_phy( ucMdioPort, ucMdioAdr, P0_1000BASE_T_CNTRL_REG );
        tmp |= 3 << 8;
        fp_mdio_write_phy( ucMdioPort, ucMdioAdr, P0_1000BASE_T_CNTRL_REG, tmp );
        break;
    }
    phy_i347at4_reset_page_0_2_3_5_7( ucMdioPort, ucMdioAdr );
}

u32 phy_i347at4_get_link_part_abil( u8 ucMdioPort, u8 ucMdioAdr )
{
    u32 advertising = 0, tmp;

    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, I347AT4_PAGE_SELECTOR_REG, PAGE_0 );
    tmp = fp_mdio_read_phy( ucMdioPort, ucMdioAdr, P0_COPPER_LINK_PAR_AB_REG_BP );

    if( tmp & ( 1 << 5 ) )
        advertising |= ADVERTISED_10baseT_Half;
    if( tmp & ( 1 << 6 ) )
        advertising |= ADVERTISED_10baseT_Full;
    if( tmp & ( 1 << 7 ) )
        advertising |= ADVERTISED_100baseT_Half;
    if( tmp & ( 1 << 8 ) )
        advertising |= ADVERTISED_100baseT_Full;

    tmp = fp_mdio_read_phy( ucMdioPort, ucMdioAdr, P0_1000BASE_T_STAT_REG );

    if( tmp & ( 1 << 10 ) )
        advertising |= ADVERTISED_1000baseT_Half;
    if( tmp & ( 1 << 11 ) )
        advertising |= ADVERTISED_1000baseT_Full;

    return advertising;
}

u32 phy_i347at4_get_mdi_stat( u8 ucMdioPort, u8 ucMdioAdr )
{
    u32 tmp;

    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, I347AT4_PAGE_SELECTOR_REG, PAGE_0 );
    tmp = fp_mdio_read_phy( ucMdioPort, ucMdioAdr, P0_COPPER_SPEC_STAT_REG1 );

    if( tmp & ( 1 << 6 ) )
        return ETH_TP_MDI_X;
    else
        return ETH_TP_MDI;
}

void phy_i347at4_setup_led_default( u8 ucMdioPort, u8 ucMdioAdr )
{
    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, I347AT4_PAGE_SELECTOR_REG, PAGE_3 );
    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, P3_LED3_0_POL_CNTRL_REG, 0x8800 | 0xFF );                  // LED driver mode
    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, P3_LED3_0_FUNC_CNTRL_REG, 0x2588 );                        // LED action
}

void phy_i347at4_setup_led_on( u8 ucMdioPort, u8 ucMdioAdr )
{
    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, I347AT4_PAGE_SELECTOR_REG, PAGE_3 );
    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, P3_LED3_0_FUNC_CNTRL_REG, 0x9988 );                        // LED[3..2] force ON
}

void phy_i347at4_setup_led_off( u8 ucMdioPort, u8 ucMdioAdr )
{
    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, I347AT4_PAGE_SELECTOR_REG, PAGE_3 );
    fp_mdio_write_phy( ucMdioPort, ucMdioAdr, P3_LED3_0_FUNC_CNTRL_REG, 0x8888 );                        // LED[3..2] force OFF
}
#if 0
/*return value: 0 - speed 10 Mbit/s
				1 - speed 100 Mbit/s
				2 - speed 1000 Mbit/s
				3 - reserved */
unsigned int fp_phy_get_speed(unsigned int mdio_port, unsigned int phy_addr)
{
	unsigned int val;

	mdio_port &= 1;
	phy_addr &= 0x1F;

	val = fp_phy_get_copper_spec_status_register_1(mdio_port, phy_addr);

	return ((val & 0xC000) >> 14);
}

/*return value: 0 - half-duplex
		1 - full-duplex */
unsigned int fp_phy_get_duplex(unsigned int mdio_port, unsigned int phy_addr)
{
	unsigned int val;

	mdio_port &= 1;
	phy_addr &= 0x1F;

	val = fp_phy_get_copper_spec_status_register_1(mdio_port, phy_addr);

	return ((val & 0x2000) >> 13);
}
#endif
