#ifndef __PHY_I347AT4_H
#define __PHY_I347AT4_H

#include "fp_mdio_ops.h"
#include <linux/types.h>

#define I347AT4_PHY_ID                  0x01410DC0

#define I347AT4_PAGE_SELECTOR_REG	      22

/* -------------------------------- Page 0 Registers -------------------------------- */
enum
{
    P0_COPPER_CNTRL_REG = 0,
    P0_COPPER_STAT_REG,
    P0_PHY_ID_1,
    P0_PHY_ID_2,
    P0_COPPER_AUTO_NEG_ADV_REG,
    P0_COPPER_LINK_PAR_AB_REG_BP,
    P0_COPPER_AUTO_NEG_EXP_REG,
    P0_COPPER_NEXT_PAGE_TX_REG,
    P0_COPPER_LINK_PAR_NEXT_PAGE_REG,
    P0_1000BASE_T_CNTRL_REG,
    P0_1000BASE_T_STAT_REG,
    P0_EXTENDED_STAT_REG = 15,
    P0_COPPER_SPEC_CNTRL_REG1,
    P0_COPPER_SPEC_STAT_REG1,
    P0_COPPER_SPEC_INTERRUPT_EN_REG,
    P0_COPPER_INTERRUPT_STAT_REG,
    P0_COPPER_SPEC_CNTRL_REG2,
    P0_COPPER_SPEC_RX_ERR_CNT_REG,
    P0_GLOBAL_INTERRUPT_STATUS = 23,
    P0_COPPER_SPEC_CNTRL_REG3 = 26,
};
/* -------------------------------- Page 1 Registers -------------------------------- */
enum
{
    P1_PHY_ID_1 = 2,
    P1_PHY_ID_2,
    P1_EXTENDED_STAT_REG = 15,
    P1_PRBS_CNTRL = 23,
    P1_PRBSERR_CNT_LSB,
    P1_PRBS_ERR_CNT_MSB
};
/* -------------------------------- Page 2 Registers -------------------------------- */
enum
{
    P2_MAC_SPEC_CNTRL_REG1 = 16,
    P2_MAC_SPEC_INTERRUPT_EN_REG = 18,
    P2_MAC_SPEC_STAT_REG,
    P2_COPPER_RX_ER_BYTE_CAP,
    P2_MAC_SPEC_CNTRL_REG2
};
/* -------------------------------- Page 3 Registers -------------------------------- */
enum
{
    P3_LED3_0_FUNC_CNTRL_REG = 16,
    P3_LED3_0_POL_CNTRL_REG,
    P3_LED_TMR_CNTRL_REG,
    P3_LED5_4_FUNC_POL_CNTRL_REG
};
/* -------------------------------- Page 5 Registers -------------------------------- */
enum
{
    P5_CABLE_TEST_TX_MDI0_RX_COUPLING = 16,
    P5_CABLE_TEST_TX_MDI1_RX_COUPLING,
    P5_CABLE_TEST_TX_MDI2_RX_COUPLING,
    P5_CABLE_TEST_TX_MDI3_RX_COUPLING,
    P5_1000BASE_T_PAIR_SKEW_REG,
    P5_1000BASE_T_PAIR_SWAP_POL,
    P5_CABLE_TEST_CNTRL = 23,
    P5_CABLE_TEST_SAMPLE_POINT_DIST,
    P5_CABLE_TEST_CPPT,
    P5_CABLE_TEST_SPIPT0_1,
    P5_CABLE_TEST_SPIPT2_3,
    P5_CABLE_TEST_SPIPT4_TPC
};
/* -------------------------------- Page 6 Registers -------------------------------- */
enum
{
    P6_PACKET_GENERATION = 16,
    P6_CRC_CNT,
    P6_CHECKER_CNTRL,
    P6_GENERAL_CNTRL_REG = 20,
    P6_LATE_COLLIS_CNT1_2 = 23,
    P6_LATE_COLLIS_CNT3_4,
    P6_LATE_COLLIS_WIND_ADJ_LINK_DISCNT,
    P6_MISC_TEST
};

enum
{
    PAGE_0 = 0,
    PAGE_1,
    PAGE_2,
    PAGE_3,
    PAGE_4,
    PAGE_5,
    PAGE_6,
    PAGE_7
};

enum
{
 PHY_PORT_0	= 0,
 PHY_PORT_1,
 PHY_PORT_2,
 PHY_PORT_3
};

void phy_i347at4_init( u8 ucMdioPort, u8 ucMdioAdr, u16 speed );
void phy_i347at4_speed_set( u8 ucMdioPort, u8 ucMdioAdr, u16 speed, u8 duplex );
void phy_i347at4_restart_aneg( u8 ucMdioPort, u8 ucMdioAdr );
u32 phy_i347at4_get_link_part_abil( u8 ucMdioPort, u8 ucMdioAdr );
u32 phy_i347at4_get_mdi_stat( u8 ucMdioPort, u8 ucMdioAdr );
void phy_i347at4_setup_led_default( u8 ucMdioPort, u8 ucMdioAdr );
void phy_i347at4_setup_led_on( u8 ucMdioPort, u8 ucMdioAdr );
void phy_i347at4_setup_led_off( u8 ucMdioPort, u8 ucMdioAdr );

#endif	//__PHY_I347AT4_H
