/*************************************************************************/ /*!
@File
@Title		tlite_blocks_ctrlm.c
@Copyright	Copyright (c) Milandr. All Rights Reserved
@Description	registers of tlite blocks and update configure values to
		hardware
@License	Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

/************************************************************************
				Include Files
*************************************************************************/
#include <linux/version.h>
#include <linux/types.h>
#include <linux/module.h>
#include <linux/netdevice.h>
#include <linux/timer.h>
#include <fp_types.h>
#include <fp_ioctl.h>
#include <fp_common_msg_header.h>
#include <fp_common_ctlm.h>
#include <tlite_blocks_ctrl_api.h>
#include <tlite_blocks_ctrlm.h>
#include <wsp_vlan_hash_table_L.h>
#include "fp_global_addr_map.h"
/************************************************************************
				GLOBAL VARIABLES
*************************************************************************/
static char *baseaddr;
static pcm_msg_header_t gpmsg;

/* added for tlite stats */
/*static*/ uint32_t Qposition[TLITE_SWITCH_MAX_DEV_NUM][TLITE_MAX_SCHED_NUM][TLITE_MAX_QUE_NO];

/**********************************************************************
 * Description	:Context memory register write function Writes given value
 *				in given address.
 *********************************************************************/
void write_fasw_context_memory(uint32_t data, uint32_t que_no, uint32_t phy_no, uint32_t reg_no) {

	int32_t tmp;
	/* selecting register */
	fp_reg_write( (uint32_t)(TLITE_CNTX_ADDR), (reg_no + que_no * TOTAL_CONTEXT_REGS) + (phy_no << SHIFT_BY16));
	/* initialising the register with 0 */
	fp_reg_write( (uint32_t)(TLITE_CNTX_DATA_ADDR), data);
	/* csr_cntx_ind_cmd -> 1 and csr_cntx_ind_start->1 */
	fp_reg_write( (uint32_t)(TLITE_CNTX_CMD_ADDR), WRITE_CMD);
	fp_reg_write( (uint32_t)(TLITE_CNTX_CMD_ADDR), WRITE_CMD + START_CMD);

	tmp = 100;
	/* poll for csr_cntx_ind_done bit */
	while ( ( !(fp_reg_read( (uint32_t)(TLITE_CNTX_CMD_ADDR) ) & DONE_BIT) ) && (tmp-- > 0) );

	if( ( !(fp_reg_read( (uint32_t)(TLITE_CNTX_CMD_ADDR) ) & DONE_BIT) ) && (tmp == 0) ){
		FP_DEBUG(DBGP_FEAT_ERR, FP_LOG_EMERG, "%s: can not write data to context memory of TMU, phy %d, queue %d, reg %d\n",  __func__, phy_no, que_no, reg_no );
	}
	/* End of while loop */

} /*End of write_fasw_context_memory() Function*/


/**********************************************************************
 * Description    : Hardware register read function read value
 *                  from given address.
 *********************************************************************/
uint32_t
read_fasw_context_memory(uint32_t que_no, uint32_t phy_no, uint32_t reg_no) {
	uint32_t data = 0, tmp;
	/* selecting register */
	fp_reg_write( (uint32_t)(TLITE_CNTX_ADDR), (reg_no + que_no * TOTAL_CONTEXT_REGS) + (phy_no << SHIFT_BY16) );
	/* csr_cntx_ind_cmd -> 0 and csr_cntx_ind_start->1 */
	fp_reg_write( (uint32_t)(TLITE_CNTX_CMD_ADDR), READ_CMD);
	fp_reg_write( (uint32_t)(TLITE_CNTX_CMD_ADDR), READ_CMD + START_CMD);

	tmp = 100;
	/* poll for csr_cntx_ind_done bit */
	while( ( !(fp_reg_read( (uint32_t)(TLITE_CNTX_CMD_ADDR)) & DONE_BIT) ) && (tmp-- > 0) );
	
	if( ( !(fp_reg_read( (uint32_t)(TLITE_CNTX_CMD_ADDR) ) & DONE_BIT) ) && (tmp == 0) ){
		FP_DEBUG(DBGP_FEAT_ERR, FP_LOG_EMERG, "%s: can not read data from context memory of TMU, phy %d, queue %d, reg %d\n",  __func__, phy_no, que_no, reg_no );
		return 0xFFFFFFFF;
	}
	/* initialising the register with 0 */
	data = fp_reg_read( (uint32_t)(TLITE_CNTX_DATA_ADDR));

	return data;
} /*End of read_fasw_context_memory() Function*/


/*************************************************************************
 * Description    : This function enables the dynamic configuration
 *************************************************************************/
int32_t pfe_tlite_dynamic_config_enable(uint32_t hw_type)
{
	uint32_t read_val = 0, write_val = 0, phy_no = 0;
	hw_type = TLITE_SWITCH;

	FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:\n", __func__);
	for (phy_no = 0; phy_no < TLITE_SWITCH_MAX_DEV_NUM; phy_no++) {
		/*making the hw_en -> 0 */
		read_val = fp_reg_read( (uint32_t)(TLITE_PHY0_TDQ_CTRL_ADDR + (REG_LEN * phy_no)) );
		write_val = read_val & BIT1_ZERO;
		fp_reg_write( (uint32_t)(TLITE_PHY0_TDQ_CTRL_ADDR + (REG_LEN * phy_no)), write_val);
	}

	return 0;
}


/*************************************************************************
 * Description    : This function configures given scheduler in tlite module
 *************************************************************************/
int32_t
pcm_tlite_scheduler_config(tlite_schd_params_t *psch, uint32_t hw_type) {

	uint32_t read_val = 0;
	/*Validate scheduler config parameters*/

	/*Scheduler number validation*/
	if( ( psch->tlsd_schd_id < TLITE_MIN_SCHED_NUM ) || ( psch->tlsd_schd_id > TLITE_MAX_SCHED_NUM ) ) {
		FP_DEBUG( DBGP_FEAT_ERR, FP_LOG_EMERG, "%s:TLITE_ERR_MSG_SCH_NUMBER %d\n", __func__, psch->tlsd_schd_id );
		return TLITE_ERR_MSG_SCH_NUMBER;
	}

	/*Scheduler Algo type validation*/
	if( ( psch->tlsd_algo < TLITE_MIN_ALGO_NUM ) || ( psch->tlsd_algo > TLITE_MAX_ALGO_NUM ) ) {
		FP_DEBUG( DBGP_FEAT_ERR, FP_LOG_EMERG, "%s:TLITE_ERR_MSG_SCH_ALGO %d\n", __func__, psch->tlsd_algo );
		return TLITE_ERR_MSG_SCH_ALGO;
	}
	/*Scheduler device id validation*/
	if( ( psch->tlsd_dev_id < TLITE_MIN_DEV_NUM ) || ( psch->tlsd_dev_id > TLITE_SWITCH_MAX_DEV_NUM ) ){
		FP_DEBUG( DBGP_FEAT_ERR, FP_LOG_EMERG, "%s:TLITE_ERR_MSG_SCH_DEV port %d\n", __func__, psch->tlsd_dev_id );
		return TLITE_ERR_MSG_SCH_DEV;
	}

	/*Set original values*/
	psch->tlsd_algo--;
	psch->tlsd_schd_id--;
	psch->tlsd_dev_id--;
	psch->tlsd_position--;

	/*checking for allw_tdq_prog-> bit 4 in PHY0_TDQ_CTRL register */
	read_val = fp_reg_read( (uint32_t)(TLITE_PHY0_TDQ_CTRL_ADDR + (REG_LEN * psch->tlsd_dev_id)));

	/* checking for allw_tdq_prog -> 1 and hw_en -> 0 */
	if ( !( ((read_val & BIT4_ONE) == BIT4_ONE) && ((read_val & BIT1_ONE) == 0) ) ){
		return TLITE_ERR_HW;
	}

	if( psch->tlsd_schd_id == 0 ){
		/* writing the queue number of sch1, where sch0 was connected */
		if( ( Qposition[ psch->tlsd_dev_id ][ 1 ][ psch->tlsd_position ] != 0 ) &&
			( Qposition[ psch->tlsd_dev_id ][ 1 ][ psch->tlsd_position ] != 9 ) ){
			FP_DEBUG( DBGP_FEAT_ERR, FP_LOG_EMERG,	"%s: port_%d, sch_1 input_%d is busy!\n"
													"Free sch_1 input_%d first and try again configuration\n",
													__func__, psch->tlsd_dev_id, psch->tlsd_position, psch->tlsd_position );
			return TLITE_ERR_MSG_SCH_INPUT;
		}
		/* clean last configuration of the sch0 to sch1 connection */
		for( read_val = 0; read_val < 8; read_val++ ){
			if( Qposition[ psch->tlsd_dev_id ][ 1 ][ read_val ] == 9 ){
				Qposition[ psch->tlsd_dev_id ][ 1 ][ read_val ] = 0;
				break;
			}
		}
		fp_reg_write( ( uint32_t )( TLITE_SCH0_POS_ADDR + ( DEV_ADDRESS_MASK * ( psch->tlsd_dev_id + 1 ) ) ), psch->tlsd_position );
		fp_reg_write( ( uint32_t )( TLITE_SCH1_Q0_WGHT_ADDR + ( REG_LEN * psch->tlsd_position ) + ( DEV_ADDRESS_MASK * ( psch->tlsd_dev_id + 1 ) ) ), psch->tlsd_weight );
		/* write new configuration of the sch0 to sch1 connection */
		Qposition[ psch->tlsd_dev_id ][ 1 ][ psch->tlsd_position ] = 9;
	}

		/* writing algorithm type */
	fp_reg_write( (uint32_t)(TLITE_SCH0_CTRL_ADDR + (SHED_ADDRESS_MASK * psch->tlsd_schd_id) + (DEV_ADDRESS_MASK * (psch->tlsd_dev_id + 1))),	psch->tlsd_algo);

	fp_tdev_list[ psch->tlsd_dev_id ].sched_list[ psch->tlsd_schd_id ] = 1;

	return 0;
}

/*************************************************************************
 * Description    : To get shaper config parameters calculated based on
 *                : rate and frequency
 *************************************************************************/
int32_t
tlite_shaper_config_parameters(uint32_t rate_units, uint32_t rate,
		uint32_t clk_frequency, uint32_t *intwt, uint32_t *frcwt,
							uint32_t *clkd){
	uint32_t sysclk_khz = 0, temp = 0;
	uint32_t clkdiv = 0, trydiv = 0, w_i = 0, w_f = 0;
	uint64_t wt = 0;

	/*Clock division intial value*/
	clkdiv = 2;

	/*Changing Mhz to KHz*/
	sysclk_khz = clk_frequency * 1000;

	/* find the clkdiv value that gives us the largest valid wt value */
	while (clkdiv < MAX_CLKDIV) {
		trydiv = clkdiv << 1;
		wt = ( (uint64_t)rate * trydiv * (4096 / 8)) / (sysclk_khz) ;
		/* wt = ((( (uint64_t)rate * trydiv)/sysclk_khz) * (4096 / 8)); */
		if (wt > MAX_WT){
			break;
		}
		clkdiv = trydiv;
		temp++;
	}

	wt = ( (uint64_t)rate * clkdiv * (4096 / 8)) / (sysclk_khz);

	w_i = (wt >> SHIFT_BY12) & 0xFF;
	w_f = wt & 0xFFF;

	FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%lld:bps Shaper wt -----\n", wt);
	FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%d:bps Clk div -----\n", clkdiv);
	FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%d:bps temp -----\n", temp);

	/*Updating values*/

	*clkd = temp;
	*frcwt = w_f;
	*intwt = w_i;
	return 0;
}

/*************************************************************************
 * Description    : To get shaper config parameters calculated based on
 *                : rate and frequency
 *************************************************************************/
int32_t
tlite_shaper_config_parameters_pps(uint32_t rate_units, uint32_t rate,
		uint32_t clk_frequency, uint32_t *intwt, uint32_t *frcwt,
					uint32_t *clkd){
	uint32_t sysclk_khz = 0, temp = 0;
	uint32_t clkdiv = 0, trydiv = 0, w_i = 0, w_f = 0;
	uint64_t wt = 0;

	/*Clock division intial value*/
	clkdiv = 2;

	/*Changing Mhz to KHz*/
	//sysclk_khz = clk_frequency * 1000;

	/*Changing Mhz to Hz*/
	sysclk_khz = clk_frequency * 1000 * 1000;

	/* find the clkdiv value that gives us the largest valid wt value */
	while (clkdiv < MAX_CLKDIV) {
		trydiv = clkdiv << SHIFT_BY1;
		wt = ( ( (uint64_t)rate * trydiv * 4096 ) /sysclk_khz);
		if (wt > MAX_WT)
			break;
		clkdiv = trydiv;
		temp++;
	}

	wt = (( (uint64_t)rate * clkdiv * 4096 ) / sysclk_khz );
	w_i = (wt >> SHIFT_BY12) & 0xFF;
	w_f = wt & 0xFFF;

	FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%lld:pps Shaper wt -----\n", wt);
	FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%d:pps Clk div -----\n", clkdiv);
	FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%d:pps temp -----\n", temp);

	/*Updating values*/
	*clkd = temp;
	*frcwt = w_f;
	*intwt = w_i;
	return 0;
}



/*************************************************************************
 * Description    : This function configures given shaper in tlite module
 *************************************************************************/
int32_t
pcm_tlite_shaper_config( tlite_shpr_params_t *pshp, uint32_t hw_type, uint32_t dynamic ){

	uint32_t read_val = 0;
	uint32_t intwt, frc_wt, clk_div;
	/* config_info_t r_cfg; */

	FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:entered...\n", __func__);

	/*Validate shaper config parameters*/
	/*Shaper number validation*/
	if( ( pshp->tlsp_shpr_id < TLITE_MIN_SHPR_NUM ) || ( pshp->tlsp_shpr_id > TLITE_MAX_SHPR_NUM ) ){
		FP_DEBUG( DBGP_FEAT_ERR, FP_LOG_EMERG, "%s:TLITE_ERR_MSG_SHP_NUMBER %d\n", __func__, pshp->tlsp_shpr_id );
		return TLITE_ERR_MSG_SHP_NUMBER;
	}

	/*Shaper device id validation*/
	if( ( pshp->tlsp_dev_id < TLITE_MIN_DEV_NUM ) || ( pshp->tlsp_dev_id > TLITE_SWITCH_MAX_DEV_NUM ) ){
		FP_DEBUG( DBGP_FEAT_ERR, FP_LOG_EMERG, "%s:TLITE_ERR_MSG_SHP_DEV switch %d\n", __func__, pshp->tlsp_dev_id );
		return TLITE_ERR_MSG_SHP_DEV;
	}

	/*Shaper rate validation*/
	if( ( pshp->tlsp_rate < TLITE_MIN_RATE ) || ( pshp->tlsp_rate > TLITE_MAX_RATE ) ) {
		FP_DEBUG( DBGP_FEAT_ERR, FP_LOG_EMERG, "%s:TLITE_ERR_MSG_SHP_RATE %d\n", __func__, pshp->tlsp_rate );
		return TLITE_ERR_MSG_SHP_RATE;
	}

	/*Shaper rate units validation*/
	if( ( pshp->tlsp_rate_units < TLITE_MIN_RATE_UNITS ) || ( pshp->tlsp_rate_units > TLITE_MAX_RATE_UNITS ) ){
		FP_DEBUG( DBGP_FEAT_ERR, FP_LOG_EMERG, "%s:TLITE_ERR_MSG_SHP_RATE_UNITS %d\n", __func__, pshp->tlsp_rate_units );
		return TLITE_ERR_MSG_SHP_RATE_UNITS;
	}

	/*Shaper frequency validation*/
	if( ( pshp->tlsp_clk_frequency < TLITE_MIN_CLOCK_FREQ ) || ( pshp->tlsp_clk_frequency > TLITE_MAX_CLOCK_FREQ ) ){
		FP_DEBUG( DBGP_FEAT_ERR, FP_LOG_EMERG, "%s:TLITE_ERR_MSG_SHP_CLK_FREQUENCY %d\n", __func__, pshp->tlsp_clk_frequency );
		return TLITE_ERR_MSG_SHP_CLK_FREQUENCY;
	}

	/*Shaper min credit validation*/
	if( ( pshp->tlsp_min_credit < MIN_MAX_CREDIT ) || ( pshp->tlsp_min_credit > MAX_MAX_CREDIT ) ){
		FP_DEBUG( DBGP_FEAT_ERR, FP_LOG_EMERG, "%s:TLITE_ERR_MSG_SHP_MIN_CREDIT %d\n", __func__, pshp->tlsp_min_credit );
		return TLITE_ERR_MSG_SHP_MIN_CREDIT;
	}

	/*Shaper position validation*/
	if( ( pshp->tlsp_position < TLITE_MIN_SHP_POSITON ) || ( pshp->tlsp_position > TLITE_MAX_SHP_POSITON ) ){
		FP_DEBUG( DBGP_FEAT_ERR, FP_LOG_EMERG, "%s:TLITE_ERR_MSG_SHP_POSITION %d\n", __func__, pshp->tlsp_position );
		return TLITE_ERR_MSG_SHP_POSITION;
	}

	/*Shaper mode validation*/
	if( ( pshp->tlsp_mode < TLITE_MIN_MODE ) || ( pshp->tlsp_mode > TLITE_MAX_MODE ) ){
		FP_DEBUG( DBGP_FEAT_ERR, FP_LOG_EMERG, "%s:TLITE_ERR_MSG_SHP_MODE %d\n", __func__, pshp->tlsp_mode );
		return TLITE_ERR_MSG_SHP_MODE;
	}

	/*Check shaper updates flag and then update rate units*/

	/*Set original values*/
	pshp->tlsp_shpr_id--;
	pshp->tlsp_dev_id--;
	pshp->tlsp_rate_units--;
	pshp->tlsp_position--;

	/*To get shpaer config parameters
		calculated based on rate and frequency*/
	if( pshp->tlsp_rate_units ){
		tlite_shaper_config_parameters_pps( pshp->tlsp_rate_units, pshp->tlsp_rate, pshp->tlsp_clk_frequency, &intwt, &frc_wt, &clk_div );
	}else{
		tlite_shaper_config_parameters( pshp->tlsp_rate_units, pshp->tlsp_rate, pshp->tlsp_clk_frequency, &intwt, &frc_wt, &clk_div );
	}

	FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_INFO, "intwt\t%x\n", intwt );

	/*checking for allw_tdq_prog-> bit 4 in PHY0_TDQ_CTRL register */
	read_val = fp_reg_read( ( uint32_t )( TLITE_PHY0_TDQ_CTRL_ADDR + ( REG_LEN * pshp->tlsp_dev_id ) ) );

	if (!dynamic) {
		/* checking  for allw_tdq_prog-> 1 and hw_en -> 0 */
		if( !( ( ( read_val & BIT4_ONE ) == BIT4_ONE ) && ( ( read_val & BIT1_ONE ) == 0 ) ) ){
			return TLITE_ERR_HW;
		}
	}

	read_val = fp_reg_read( ( uint32_t )( TLITE_SHP0_CTRL_ADDR + ( SHPR_ADDRESS_MASK * pshp->tlsp_shpr_id ) + ( DEV_ADDRESS_MASK * ( pshp->tlsp_dev_id + 1 ) ) ) );
	FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_INFO, "%s phy%d_ctrl_register:\t%x\n", __func__, pshp->tlsp_dev_id, read_val );

	/* writing clock division value in TMU_SHP0_CTRL register */
	read_val = SHPR_EN_BIT + ( clk_div << SHIFT_BY1 );

	FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_INFO, "%s:SHPR_EN_BIT + (clk_div << SHIFT_BY1) %x\n", __func__, read_val );
	fp_reg_write( ( uint32_t )( TLITE_SHP0_CTRL_ADDR + ( SHPR_ADDRESS_MASK * pshp->tlsp_shpr_id ) + ( DEV_ADDRESS_MASK * ( pshp->tlsp_dev_id + 1 ) ) ), read_val );

	FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_INFO, "%s:writing clockdiv value %d\n", __func__, clk_div );
	FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_INFO, "shp%d_ctrl_register addr:0x%X\tval %x\n", pshp->tlsp_shpr_id, ( TLITE_SHP0_CTRL_ADDR + ( SHPR_ADDRESS_MASK * pshp->tlsp_shpr_id ) + ( DEV_ADDRESS_MASK * ( pshp->tlsp_dev_id + 1 ) ) ),
				fp_reg_read( ( uint32_t )( TLITE_SHP0_CTRL_ADDR + ( SHPR_ADDRESS_MASK * pshp->tlsp_shpr_id ) + ( DEV_ADDRESS_MASK * ( pshp->tlsp_dev_id + 1 ) ) ) ) );
	/* writing fractional wght and int_wgt in TMU_SHP0_WGHT register */

	read_val = ( frc_wt & BIT_0TO11 ) | ( ( intwt & BIT_0TO7 ) << SHIFT_BY12 );
	fp_reg_write( ( uint32_t )( TLITE_SHP0_WGHT_ADDR + ( SHPR_ADDRESS_MASK * pshp->tlsp_shpr_id ) + ( DEV_ADDRESS_MASK * ( pshp->tlsp_dev_id + 1 ) ) ), read_val );

	FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_INFO, "%s:writing frac %x and int weight %x\n", __func__, frc_wt, intwt );
	FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_INFO, "shp%d_wght_register:\t%x\n", pshp->tlsp_shpr_id, fp_reg_read( ( uint32_t )( TLITE_SHP0_WGHT_ADDR + ( SHPR_ADDRESS_MASK * pshp->tlsp_shpr_id ) + ( DEV_ADDRESS_MASK * ( pshp->tlsp_dev_id + 1 ) ) ) ) );

	/* writing max credit value in TMU_SHP0_MAX_CREDIT register */
	read_val = ( ( pshp->tlsp_max_credit ) << SHIFT_BY10 );
	fp_reg_write( ( uint32_t )( TLITE_SHP0_MAX_CREDIT_ADDR + ( SHPR_ADDRESS_MASK * pshp->tlsp_shpr_id ) + ( DEV_ADDRESS_MASK * ( pshp->tlsp_dev_id + 1 ) ) ), read_val );

	FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_INFO, "%s:max credit %x\n", __func__, pshp->tlsp_max_credit );
	FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_INFO, "shp%d_max_credit_register:\t%x\n", pshp->tlsp_shpr_id, fp_reg_read( ( uint32_t )( TLITE_SHP0_MAX_CREDIT_ADDR + ( SHPR_ADDRESS_MASK * pshp->tlsp_shpr_id ) + ( DEV_ADDRESS_MASK * ( pshp->tlsp_dev_id + 1 ) ) ) ) );

	/* writing rate units and position in TMU_SHP0_CTRL2 register */
	read_val =  ( ( pshp->tlsp_rate_units ) | ( ( pshp->tlsp_position) << SHIFT_BY1 ) | ( ( pshp->tlsp_mode ) << SHIFT_BY6 ) );
	fp_reg_write( ( uint32_t )( TLITE_SHP0_CTRL2_ADDR + ( SHPR_ADDRESS_MASK * pshp->tlsp_shpr_id ) + ( DEV_ADDRESS_MASK * ( pshp->tlsp_dev_id + 1 ) ) ), read_val );

	/* writing min credit in TMU_SHP0_MIN_CREDIT register */
	read_val =  pshp->tlsp_min_credit;
	fp_reg_write( ( uint32_t )( TLITE_SHP0_MIN_CREDIT + ( SHPR_ADDRESS_MASK * pshp->tlsp_shpr_id ) + ( DEV_ADDRESS_MASK * ( pshp->tlsp_dev_id + 1 ) ) ), read_val );

	FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_INFO, "%s:rate units %x and positon %x\n", __func__, pshp->tlsp_rate_units, pshp->tlsp_position );
	FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_INFO, "shp%d_ctrl2_register:\t%x\n", pshp->tlsp_shpr_id, fp_reg_read( ( uint32_t )( TLITE_SHP0_CTRL2_ADDR + ( SHPR_ADDRESS_MASK * pshp->tlsp_shpr_id ) + ( DEV_ADDRESS_MASK * ( pshp->tlsp_dev_id + 1 ) ) ) ) );

	/*Updating the dev list structure */
	fp_tdev_list[ pshp->tlsp_dev_id ].shaper_list[ pshp->tlsp_shpr_id ] = pshp->tlsp_position + 1;

	return 0;
} /*End of pcm_tlite_shaper_config() Function*/


/*************************************************************************
 * Description    : This function disable given shaper in tlite module
 *************************************************************************/
int32_t
pcm_tlite_shaper_disable(tlite_shpr_params_t *pshp, uint32_t hw_type, uint32_t dynamic) {

	uint32_t read_val = 0;

	FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:entered...\n", __func__);

	/*Validate shaper config parameters*/
	/*Shaper number validation*/
	if (pshp->tlsp_shpr_id < TLITE_MIN_SHPR_NUM || pshp->tlsp_shpr_id > TLITE_MAX_SHPR_NUM){
		FP_DEBUG(DBGP_FEAT_ERR, FP_LOG_EMERG, "%s:TLITE_ERR_MSG_SHP_NUMBER %d\n", __func__, pshp->tlsp_shpr_id);
		return TLITE_ERR_MSG_SHP_NUMBER;
	}

	/*Shaper device id validation*/
	if (pshp->tlsp_dev_id < TLITE_MIN_DEV_NUM || pshp->tlsp_dev_id > TLITE_SWITCH_MAX_DEV_NUM) {
		FP_DEBUG(DBGP_FEAT_ERR, FP_LOG_EMERG, "%s:TLITE_ERR_MSG_SHP_DEV switch %d\n", __func__, pshp->tlsp_dev_id);
		return TLITE_ERR_MSG_SHP_DEV;
	}
	/*Check shaper updates flag and then update rate units*/

	/*Set original values*/
	pshp->tlsp_shpr_id--;
	pshp->tlsp_dev_id--;

	/*checking for allw_tdq_prog-> bit 4 in PHY0_TDQ_CTRL register */
	read_val = fp_reg_read( (uint32_t)(TLITE_PHY0_TDQ_CTRL_ADDR + (REG_LEN * pshp->tlsp_dev_id)) );
	if( !dynamic ){
		/* checking  for allw_tdq_prog-> 1 and hw_en -> 0 */
		if( !( ( ( read_val & BIT4_ONE ) == BIT4_ONE ) && ( ( read_val & BIT1_ONE ) == 0 ) ) ){
			return TLITE_ERR_HW;
		}
	}
	read_val = fp_reg_read( ( uint32_t )( TLITE_SHP0_CTRL_ADDR + ( SHPR_ADDRESS_MASK * pshp->tlsp_shpr_id ) + ( DEV_ADDRESS_MASK * ( pshp->tlsp_dev_id + 1 ) ) ) );
	FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s phy%d_ctrl_register:\t%x\n", __func__, pshp->tlsp_dev_id, read_val);

	/* writing clock division value in TMU_SHP0_CTRL register */
	read_val = 0;
	fp_reg_write( ( uint32_t )( TLITE_SHP0_CTRL_ADDR + ( SHPR_ADDRESS_MASK * pshp->tlsp_shpr_id ) + ( DEV_ADDRESS_MASK * ( pshp->tlsp_dev_id + 1 ) ) ), read_val );
	fp_reg_write( ( uint32_t )( TLITE_SHP0_WGHT_ADDR + ( SHPR_ADDRESS_MASK * pshp->tlsp_shpr_id ) + ( DEV_ADDRESS_MASK * ( pshp->tlsp_dev_id + 1 ) ) ), read_val );
	/* writing max credit value in TMU_SHP0_MAX_CREDIT register */
	fp_reg_write( ( uint32_t )( TLITE_SHP0_MAX_CREDIT_ADDR + ( SHPR_ADDRESS_MASK * pshp->tlsp_shpr_id ) + ( DEV_ADDRESS_MASK * ( pshp->tlsp_dev_id + 1 ) ) ), ( 0x3FFE00 << SHIFT_BY10 ) );
	/* writing rate units and position in TMU_SHP0_CTRL2 register */
	fp_reg_write( ( uint32_t )( TLITE_SHP0_CTRL2_ADDR + ( SHPR_ADDRESS_MASK * pshp->tlsp_shpr_id ) + ( DEV_ADDRESS_MASK * ( pshp->tlsp_dev_id + 1 ) ) ), 0 );
	/* writing min credit in TMU_SHP0_MIN_CREDIT register */
	fp_reg_write( ( uint32_t )( TLITE_SHP0_MIN_CREDIT + ( SHPR_ADDRESS_MASK * pshp->tlsp_shpr_id ) + ( DEV_ADDRESS_MASK * ( pshp->tlsp_dev_id + 1 ) ) ), 0x3FFE00 );

	/*Updating the dev list structure */
	fp_tdev_list[ pshp->tlsp_dev_id ].shaper_list[ pshp->tlsp_shpr_id ] = 0;

	return 0;
} /*End of pcm_tlite_shaper_config() Function*/


/*************************************************************************
 * Description    : Helper function for pcm_classq_add() Function
 *************************************************************************/
int32_t
tlite_add_queue2scheduler(tlite_classq_params_t *que) {

	uint32_t read_val = 0;
	uint32_t read_val1 = 0, read_val2 = 0;
	/*Loop for number of achedulers*/
	/*Read Alloc0 register*/
	read_val1 = fp_reg_read( (uint32_t)(TLITE_SCH0_Q_ALLOC0_ADDR +
				SHED_ADDRESS_MASK * que->tlcq_schd_id +
				(DEV_ADDRESS_MASK * (que->tlcq_dev_id + 1) ) ) );
	/*Read alloc1 register*/
	read_val2 = fp_reg_read( (uint32_t)(TLITE_SCH0_Q_ALLOC1_ADDR +
				SHED_ADDRESS_MASK * que->tlcq_schd_id +
				(DEV_ADDRESS_MASK * (que->tlcq_dev_id + 1) ) ) );

	FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:", __func__);
	FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "sch%d, slot %d", que->tlcq_schd_id, que->tlcq_schd_slot);

	/*Switch to select que priority*/
	switch (que->tlcq_schd_slot) {
		case SCH_SLOT_ZERO:
			/*Update alloc register with given queue number*/
			read_val1 = (read_val1 & NIBBLE0_ZERO) | (que->tlcq_classq_id & NIBBLE0_ONE);
			break;
		case SCH_SLOT_ONE:
			/*Update alloc register with given queue number*/
			read_val1 = (read_val1 & NIBBLE2_ZERO) | ((que->tlcq_classq_id & NIBBLE0_ONE) << SHIFT_BY8);
			break;
		case SCH_SLOT_TWO:
			/*Update alloc register with given queue number*/
			read_val1 = (read_val1 & NIBBLE4_ZERO) | ((que->tlcq_classq_id & NIBBLE0_ONE) << SHIFT_BY16);
			break;
		case SCH_SLOT_THREE:
			/*Update alloc register with given queue number*/
			read_val1 = (read_val1 & NIBBLE6_ZERO) | ((que->tlcq_classq_id & NIBBLE0_ONE) << SHIFT_BY24);
			break;
		case SCH_SLOT_FOUR:
			/*Update alloc register with given queue number*/
			read_val2 = (read_val2 & NIBBLE0_ZERO) | (que->tlcq_classq_id & NIBBLE0_ONE);
			break;
		case SCH_SLOT_FIVE:
			/*Update alloc register with given queue number*/
			read_val2 = (read_val2 & NIBBLE2_ZERO) | ((que->tlcq_classq_id & NIBBLE0_ONE) << SHIFT_BY8);
			break;
		case SCH_SLOT_SIX:
			/*Update alloc register with given queue number*/
			read_val2 = (read_val2 & NIBBLE4_ZERO) | ((que->tlcq_classq_id & NIBBLE0_ONE) << SHIFT_BY16);
			break;
		case SCH_SLOT_SEVEN:
			/*Update alloc register with given queue number*/
			read_val2 = (read_val2 & NIBBLE6_ZERO) | ((que->tlcq_classq_id & NIBBLE0_ONE) << SHIFT_BY24);
			break;
		default:
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:Invalid scheduler slot\n", __func__);
			break;
	} /*End of scheduler slot switch statement*/


	/*checking for allw_tdq_prog-> bit 4 in PHY0_TDQ_CTRL register */
	read_val = fp_reg_read( ( uint32_t )( TLITE_PHY0_TDQ_CTRL_ADDR + ( REG_LEN * que->tlcq_dev_id ) ) );

	/* checking  for allw_tdq_prog-> 1 and hw_en -> 0 */
	if( !( ( ( read_val & BIT4_ONE ) == BIT4_ONE ) && ( ( read_val & BIT1_ONE ) == 0 ) ) ) {
		FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_INFO, "TLITE_ERR_HW\n :" );
		return TLITE_ERR_HW;
	}

	/*Update que weight*/
	fp_reg_write( ( uint32_t )( TLITE_SCH0_Q0_WGHT_ADDR + ( REG_LEN * que->tlcq_schd_slot ) + ( SHED_ADDRESS_MASK * que->tlcq_schd_id ) + ( DEV_ADDRESS_MASK * ( que->tlcq_dev_id + 1 ) ) ), que->tlcq_weight );

	FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "qweight %x\n :", fp_reg_read( (uint32_t)(TLITE_SCH0_Q0_WGHT_ADDR + REG_LEN * que->tlcq_classq_id + SHED_ADDRESS_MASK * que->tlcq_schd_id + (DEV_ADDRESS_MASK * (que->tlcq_dev_id + 1)))));

	/*Write que number into hardware*/
	if (que->tlcq_schd_slot < SCH_SLOT_FOUR){
		fp_reg_write( (uint32_t)(TLITE_SCH0_Q_ALLOC0_ADDR + (SHED_ADDRESS_MASK * que->tlcq_schd_id) + (DEV_ADDRESS_MASK * (que->tlcq_dev_id + 1))), read_val1);
		FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "sch%d, sched slot 0-3 %x\n :", que->tlcq_schd_id, fp_reg_read( (uint32_t)(TLITE_SCH0_Q_ALLOC0_ADDR + SHED_ADDRESS_MASK * que->tlcq_schd_id + (DEV_ADDRESS_MASK * (que->tlcq_dev_id + 1)))));
	} else {
		fp_reg_write( (uint32_t)(TLITE_SCH0_Q_ALLOC1_ADDR + SHED_ADDRESS_MASK * que->tlcq_schd_id + (DEV_ADDRESS_MASK * (que->tlcq_dev_id + 1))), read_val2);
		FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "sch%d, sched slot 4-7 %x\n :", que->tlcq_schd_id, fp_reg_read( (uint32_t)(TLITE_SCH0_Q_ALLOC1_ADDR + SHED_ADDRESS_MASK * que->tlcq_schd_id + (DEV_ADDRESS_MASK * (que->tlcq_dev_id + 1)))));
	}

	return 0;
}

/*************************************************************************
 * Description    : This function return bit mask of used queues
 *************************************************************************/
int32_t interface_queues_used( uint8_t port_number, uint32_t *queue_mask ){
	uint32_t Tc2Queue, queue_number, i;
	if( !( port_number < FP_MAX_PORTS )){
		FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_ERR, "%s: Invalid port number %d\n", __func__, port_number );
		return -1;	//Invalid port_number!
	}

	*queue_mask = 0;
	Tc2Queue = fp_reg_read( hwQosTc2Cos[ 0 ][ port_number ] );
	FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_DEBUG, "%s: port %d Tc -> Queue: 0x%X\n", __func__, port_number, Tc2Queue );
	for( i = 0; i < TLITE_MAX_QUE_NO; i++ ){
		queue_number = ( Tc2Queue >> ( i * 3 ) ) & 7;
		*queue_mask |= 1 << queue_number;
	}

	if( port_number == HIF_PORT_NUM ){
		Tc2Queue = fp_reg_read( ( CLASS_HW_BASE_ADDR + CLASS_PUNT_COS ) );
		FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_DEBUG, "%s: port %d CLASS_PUNT_COS -> Queue: 0x%X\n", __func__, port_number, Tc2Queue );
		//we have 7 punt events for HOST port
		for( i = 0; i < 7; i++ ){
			queue_number = ( Tc2Queue >> ( i << 2 ) ) & 7;
			*queue_mask |= 1 << queue_number;
		}
	}
	FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_DEBUG, "%s: port %d queue mask: 0x%X\n", __func__, port_number, *queue_mask );
	return 0;
}

/***************************************************************************************
 * Description    : This function checks the maximum value of packets in all HOST queues
 ***************************************************************************************/
int32_t check_max_packets_in_host_queues( tlite_classq_params_t *que ){
	uint32_t queue_size, queue_uses, tmp, read_val, i;

	if( que->tlcq_classq_id != HIF_PORT_NUM ){
		return 0;
	}

	interface_queues_used( HIF_PORT_NUM, &queue_uses );
	read_val = fp_reg_read( ( CLASS_HW_BASE_ADDR + CLASS_ING_AFULL_THRES ) );
	queue_size = 0;

	for( i = 0; i < 8; i++ ){
		/* HOST interface size of queues */
		if( ( queue_uses >> i ) & 1 ){
			//if we use current queue
			tmp = read_fasw_context_memory( i, que->tlcq_dev_id, CURQ_QMAX_QMIN_CFG );
			if( ( tmp & 3 ) == 0 ){
				return -1;
			}
			queue_size += ( tmp >> SHIFT_BY11 ) & 0x1FF;
		}
	}
	if( ( read_val - 17 ) < ( queue_size + que->tlcq_max_len ) ){
		FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_ALERT, "%s: INVALID max packets in all host queues %d. Max packets value in all host queues should less then %d\n", __func__, ( queue_size + que->tlcq_max_len ), ( read_val - 17 ) );
		return -1;
	}
	return 0;
}

/**************************************************************************************
 * Description    : This function calculate host queues depth to write in hw registers
 *************************************************************************************/
int32_t host_queues_settings( uint16_t *queues_depth ){
	uint32_t queues_mask, threshold, delta, div, i;

	interface_queues_used( HIF_PORT_NUM, &queues_mask );
    threshold = fp_reg_read( CLASS_HW_BASE_ADDR + CLASS_ING_AFULL_THRES );
    div = 1;
    delta = threshold - 16 - 1;
    memset( queues_depth, 0, ( sizeof( uint16_t ) * 8 ) );
    for( i = 0; i < 8; i++ ){
        /* HOST interface size of queues */
        if( ( queues_mask >> i ) & 1 ){
            //if we use current queue
            queues_depth[ i ] = ( threshold - 16 - 1 ) >> ( div );
            div++;
            if( !queues_depth[ i ] ){
                queues_depth[ i ] = 1;
            }
            delta -= queues_depth[ i ];
        }
        FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_INFO, "%s: queues_depth[%d] = %d\n", __func__, i, queues_depth[ i ] );
    }

    for( i = 0; i < 8; i++ ){
    	if( queues_depth[ i ] ){
    		queues_depth[ i ] += delta;
    		break;
    	}
	}
    return 0;
}

/*************************************************************************
 * Description    : This function adds classq for a given scheduler
 *************************************************************************/
int32_t
pcm_tlite_classq_add(tlite_classq_params_t *que, uint32_t hw_type){
	uint32_t write_val = 0;

	/*Validate classq config parameters*/
	/*classq number validation*/
	if( ( que->tlcq_classq_id < TLITE_MIN_QUE_NO ) || ( que->tlcq_classq_id > TLITE_MAX_QUE_NO ) ){
		FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_INFO, "%s:TLITE_QUE_NUM_ERR %d\n", __func__, que->tlcq_classq_id );
		return TLITE_QUE_NUM_ERR;
	}

	hw_type = TLITE_SWITCH;
	/*classq device id validation*/
	if( ( que->tlcq_dev_id < TLITE_MIN_DEV_NUM ) || ( que->tlcq_dev_id > TLITE_SWITCH_MAX_DEV_NUM ) ){
		FP_DEBUG( DBGP_FEAT_ERR, FP_LOG_EMERG, "%s:TLITE_ERR_MSG_SHP_DEV switch %d\n", __func__, que->tlcq_dev_id );
		return TLITE_ERR_MSG_SHP_DEV;
	}

	/*classq scheduuler id validation*/
	if( ( que->tlcq_schd_id < TLITE_MIN_SCHED_NUM ) || ( que->tlcq_schd_id > TLITE_MAX_SCHED_NUM ) ){
		FP_DEBUG( DBGP_FEAT_ERR, FP_LOG_EMERG, "%s:TLITE_ERR_MSG_SCH_NUMBER %d\n", __func__, que->tlcq_schd_id );
		return TLITE_ERR_MSG_SCH_NUMBER;
	}

	/*classq scheduuler slot validation*/
	if( ( que->tlcq_schd_slot < TLITE_MIN_SCHED_POSITION ) || ( que->tlcq_schd_slot > TLITE_MAX_SCHED_POSITION ) ){
		FP_DEBUG( DBGP_FEAT_ERR, FP_LOG_EMERG, "%s:TLITE_ERR_MSG_SCH_SLOT %d\n", __func__, que->tlcq_schd_slot );
		return TLITE_ERR_MSG_SCH_SLOT;
	}

	/*Sett al id's to original val*/
	que->tlcq_classq_id--;
	que->tlcq_dev_id--;
	que->tlcq_schd_id--;
	que->tlcq_schd_slot--;

	if( check_max_packets_in_host_queues( que ) ){
		FP_DEBUG( DBGP_FEAT_ERR, FP_LOG_EMERG, "%s:TLITE_ERR_MAX_PCK_IN_HOST_QUEUES. Update \"Max queue size\" and \"Queue management\" parameters and try again\n", __func__ );
		return TLITE_ERR_MAX_PCK_IN_HOST_QUEUES;	
	}

	if( que->tlcq_qmgmt == TLITE_QUE_TAIL_DROP ){
		/* writing queue managment in register */
		/* Qmax: Qmin : cfg = ([19:11],[10:2],[1:0]) */
		write_val = ( ( que->tlcq_max_len ) << SHIFT_BY11 ) + que->tlcq_qmgmt;
		write_fasw_context_memory( write_val, que->tlcq_classq_id, que->tlcq_dev_id, CURQ_QMAX_QMIN_CFG );

	/* if queue management is WRED */
	}else if( que->tlcq_qmgmt == TLITE_QUE_WRED ){
		/* writing queue managment , Qmax and Qmin values to register */
		/* Qmax: Qmin : cfg = ([19:11],[10:2],[1:0]) */
		write_val = ( ( que->tlcq_max_len ) << SHIFT_BY11 ) + ( ( que->tlcq_min_len ) << SHIFT_BY2 ) + que->tlcq_qmgmt;
		write_fasw_context_memory( write_val, que->tlcq_classq_id, que->tlcq_dev_id, CURQ_QMAX_QMIN_CFG );

		/* writing probability values to prob_cfg registers */
		write_val = ( ( que->tlcq_prob_cfg5 ) << SHIFT_BY25 ) +
					( ( que->tlcq_prob_cfg4 ) << SHIFT_BY20 ) +
					( ( que->tlcq_prob_cfg3 ) << SHIFT_BY15 ) +
					( ( que->tlcq_prob_cfg2 ) << SHIFT_BY10 ) +
					( ( que->tlcq_prob_cfg1 ) << SHIFT_BY5  ) +
						que->tlcq_prob_cfg0;

		write_fasw_context_memory( write_val, que->tlcq_classq_id, que->tlcq_dev_id, CURQ_HW_PROB_CFG_TBL0 );

		FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_INFO, "prob_cfg req %x = write_val %x\n", read_fasw_context_memory( que->tlcq_classq_id, que->tlcq_dev_id, CURQ_HW_PROB_CFG_TBL0 ), write_val );

		write_val = ( ( que->tlcq_prob_cfg7 ) << SHIFT_BY5 ) + que->tlcq_prob_cfg6;
		write_fasw_context_memory( write_val, que->tlcq_classq_id, que->tlcq_dev_id, CURQ_HW_PROB_CFG_TBL1 );

		FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_INFO, "prob_cfg2 req %x = write_val %x\n", read_fasw_context_memory( que->tlcq_classq_id, que->tlcq_dev_id, CURQ_HW_PROB_CFG_TBL1 ), write_val );
	}else{
		write_fasw_context_memory( 0, que->tlcq_classq_id, que->tlcq_dev_id, CURQ_QMAX_QMIN_CFG );
	}
	write_val = read_fasw_context_memory( que->tlcq_classq_id, que->tlcq_dev_id, CURQ_QMAX_QMIN_CFG );
	FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_INFO, "qmgmt %x; algo: %d; Qmin: %d; Qmax: %d\n ",
		write_val,
		( ( write_val >> 0 ) & 3 ),
		( ( write_val >> 2 ) & 0x1FF ),
		( ( write_val >> 11 ) & 0x1FF ) );

	/*if scheduler_2 input is occupied by sheduler_1 output - return*/
	if( que->tlcq_schd_id == 1 ){
		write_val = fp_reg_read( ( uint32_t )( TLITE_SCH0_POS_ADDR + ( DEV_ADDRESS_MASK * ( que->tlcq_dev_id + 1 ) ) ) );
		if( write_val == que->tlcq_schd_slot ){
			FP_DEBUG( DBGP_FEAT_ERR, FP_LOG_EMERG,	"%s: try to connect queue_%d to input_%d of sch_1,\n"
													"but sch_0 output is connected to same input of sch_1 too!\n"
													"Free input_%d of sch_1 and try again configuration\n",
				__func__,
				que->tlcq_classq_id + 1,
				que->tlcq_schd_slot,
				que->tlcq_schd_slot );
			return TLITE_ERR_MSG_SCH_SLOT_BUSY;
		}
	}

	/*if any queue is connected to spcify sch and specify input */
	if( ( Qposition[ que->tlcq_dev_id ][ que->tlcq_schd_id ][ que->tlcq_schd_slot ] != 0 ) &&
		( Qposition[ que->tlcq_dev_id ][ que->tlcq_schd_id ][ que->tlcq_schd_slot ] != ( que->tlcq_classq_id + 1 ) ) ){
		FP_DEBUG( DBGP_FEAT_ERR, FP_LOG_EMERG,	"%s: connected queue_%d to input_%d of sch_%d\n"
												"but queue_%d is connected to same input of sch_%d too!\n"
												"Free input_%d of sch_%d and try again configuration\n", 
				__func__,
				que->tlcq_classq_id + 1,
				que->tlcq_schd_slot,
				que->tlcq_schd_id,
				Qposition[ que->tlcq_dev_id ][ que->tlcq_schd_id ][ que->tlcq_schd_slot ],
				que->tlcq_schd_id,
				que->tlcq_schd_slot,
				que->tlcq_schd_id );
		return TLITE_ERR_MSG_SCH_SLOT_BUSY;
	}

	/*try delete queue from scheduler 1 and scheduler 2*/
	pcm_tlite_classq_delete( ( que->tlcq_dev_id + 1 ), ( que->tlcq_classq_id + 1 ), TLITE_SWITCH );

	/*Add queue to scheduler*/
	tlite_add_queue2scheduler( que );
	/*Updating the dev list structure */
	fp_tdev_list[ que->tlcq_dev_id ].classq_list[ que->tlcq_classq_id ] = que->tlcq_schd_id + 1;
	Qposition[ que->tlcq_dev_id ][ que->tlcq_schd_id ][ que->tlcq_schd_slot ] = que->tlcq_classq_id + 1;/* 1 to 8 */
	return 0;
} /*End of pcm_tlite_classq_add() Function*/



/*************************************************************************
 * Description    : This function deletes que number from a given scheduler
 *************************************************************************/
int32_t
pcm_tlite_classq_delete(uint32_t dev_id, uint32_t qnum, uint32_t hw_type) {
	uint32_t que_flag = 0, i = 0;
	uint32_t read_val = 0;
	uint32_t read_val1 = 0, read_val2 = 0;
	int32_t sch_slot = 0;

	/*Validate classq config parameters*/
	/*classq number validation*/
	if (qnum < TLITE_MIN_QUE_NO || qnum > TLITE_MAX_QUE_NO){
		return TLITE_QUE_NUM_ERR;
	}

	hw_type = TLITE_SWITCH;
	/*classq device id validation*/
	if (dev_id < TLITE_MIN_DEV_NUM || dev_id > TLITE_SWITCH_MAX_DEV_NUM){
		return TLITE_ERR_MSG_SHP_DEV;
	}

	/*Set to original val*/
	dev_id = dev_id - 1;
	qnum = qnum - 1;

	/*Delete que from scheduler*/
	for (i = 0; i < TLITE_MAX_SCHED_NUM; i++) {

		read_val1 = fp_reg_read( (uint32_t)(TLITE_SCH0_Q_ALLOC0_ADDR + (SHED_ADDRESS_MASK * i) + (DEV_ADDRESS_MASK * (dev_id + 1))) );
		read_val2 = fp_reg_read( (uint32_t)(TLITE_SCH0_Q_ALLOC1_ADDR + (SHED_ADDRESS_MASK * i) + (DEV_ADDRESS_MASK * (dev_id + 1))) );
		/*Que search in scheduler alloc registers*/
		if ((read_val1 & NIBBLE0_ONE) == qnum){
			sch_slot = SCH_SLOT_ZERO;
		}else if (((read_val1 >> SHIFT_BY8) & NIBBLE0_ONE) == qnum){
			sch_slot = SCH_SLOT_ONE;
		}else if (((read_val1 >> SHIFT_BY16) & NIBBLE0_ONE) == qnum){
			sch_slot = SCH_SLOT_TWO;
		}else if (((read_val1 >> SHIFT_BY24) & NIBBLE0_ONE) == qnum){
			sch_slot = SCH_SLOT_THREE;
		}else if ((read_val2 & NIBBLE0_ONE) == qnum){
			sch_slot = SCH_SLOT_FOUR;
		}else if (((read_val2 >> SHIFT_BY8) & NIBBLE0_ONE) == qnum){
			sch_slot = SCH_SLOT_FIVE;
		}else if (((read_val2 >> SHIFT_BY16) & NIBBLE0_ONE) == qnum){
			sch_slot = SCH_SLOT_SIX;
		}else if (((read_val2 >> SHIFT_BY24) & NIBBLE0_ONE) == qnum){
			sch_slot = SCH_SLOT_SEVEN;
		}else{
			sch_slot = -1;
		}
		/*Find scheduler slot*/
		/*Delete scheduler slot que*/
		switch (sch_slot) {
			case SCH_SLOT_ZERO:
				read_val1 = (read_val1 & NIBBLE0_ZERO) + INVALID_QUEUE0;
				que_flag = 1;
				break;
			case SCH_SLOT_ONE:
				read_val1 = (read_val1 & NIBBLE2_ZERO) + INVALID_QUEUE1;
				que_flag = 1;
				break;
			case SCH_SLOT_TWO:
				read_val1 = (read_val1 & NIBBLE4_ZERO) + INVALID_QUEUE2;
				que_flag = 1;
				break;
			case SCH_SLOT_THREE:
				read_val1 = (read_val1 & NIBBLE6_ZERO) + INVALID_QUEUE3;
				que_flag = 1;
				break;
			case SCH_SLOT_FOUR:
				read_val2 = (read_val2 & NIBBLE0_ZERO) + INVALID_QUEUE4;
				que_flag = 1;
				break;
			case SCH_SLOT_FIVE:
				read_val2 = (read_val2 & NIBBLE2_ZERO) + INVALID_QUEUE5;
				que_flag = 1;
				break;
			case SCH_SLOT_SIX:
				read_val2 = (read_val2 & NIBBLE4_ZERO) + INVALID_QUEUE6;
				que_flag = 1;
				break;
			case SCH_SLOT_SEVEN:
				read_val2 = (read_val2 & NIBBLE6_ZERO) + INVALID_QUEUE7;
				que_flag = 1;
				break;
			default:
				que_flag = 0;
				FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s: Sch%d Invalid sch_slot\n", __func__, i);
				break;
		} /*End of scheduler slot switch statement*/

		/*checking for allw_tdq_prog-> bit 4 in PHY0_TDQ_CTRL register*/
		read_val = fp_reg_read( (uint32_t)(TLITE_PHY0_TDQ_CTRL_ADDR + (REG_LEN * dev_id)));

		/* checking  for allw_tdq_prog-> 1 and shp_cntrl_en -> 0 */
		if ( !(((read_val & BIT4_ONE) == BIT4_ONE) && ((read_val & BIT1_ONE) == 0) ) ){
			return TLITE_ERR_HW;
		}

		/*Check slot to selct alloc0 or alloc1*/
		if(que_flag == 1){
			if (sch_slot < SCH_SLOT_FOUR) {
				/*Write Alloc0 register*/
				fp_reg_write( (uint32_t)( TLITE_SCH0_Q_ALLOC0_ADDR + (SHED_ADDRESS_MASK * i) + (DEV_ADDRESS_MASK * (dev_id + 1)) ), read_val1 );
			}else{
				/*Write Alloc1 register*/
				fp_reg_write( (uint32_t)( TLITE_SCH0_Q_ALLOC1_ADDR + (SHED_ADDRESS_MASK * i) + (DEV_ADDRESS_MASK * (dev_id + 1)) ), read_val2 );
			}
			/*Updating the dev list structure */
			fp_tdev_list[ dev_id ].classq_list[ qnum ] = 0;
			Qposition[ dev_id ][ i ][ sch_slot ] = 0;
		}else{
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s: Sch%d no change, invalid sch_slot\n", __func__, i);
		}

	} /*Scheduler loop*/
	return 0;
} /*End of pcm_tmu_classq_delete() Function*/




/*************************************************************************
 * Description    : This function set inter frame gap value of given device
 *************************************************************************/
int32_t
pfe_tlite_set_ifg(uint32_t dev_id, uint32_t ifg, uint32_t hw_type) {
	uint32_t r_ifg = 0;
	uint32_t read_val = 0;

	/*dev_id  validation*/
	hw_type = TLITE_SWITCH;

	if (dev_id < TLITE_MIN_DEV_NUM || dev_id > TLITE_SWITCH_MAX_DEV_NUM){
		return TLITE_ERR_MSG_DEV_NUM;
	}

	/* ifg validation*/
	if (ifg < TLITE_MIN_IFG || ifg > TLITE_MAX_IFG){
		return TLITE_ERR_MSG_IFG;
	}

	dev_id--;
	/*Writing ifg value in tdq global register */
	r_ifg = ifg & 0xFF;

	/*checking for allw_tdq_prog-> bit 4 in PHY0_TDQ_CTRL register */
	read_val = fp_reg_read( (uint32_t)(TLITE_PHY0_TDQ_CTRL_ADDR + (REG_LEN * dev_id)));

	/* checking  for allw_tdq_prog-> 1 and shp_cntrl_en -> 0 */
	if ( !(((read_val & BIT4_ONE) == BIT4_ONE) && ((read_val & BIT1_ONE) == 0)) ){
		return TLITE_ERR_HW;
	}

	fp_reg_write( (uint32_t)( TLITE_PHY0_TDQ_IIFG_ADDR + (REG_LEN * dev_id) ), r_ifg);

	FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:Set ifg config done\n", __func__);
	return 0;
} /*End of pfe_tlite_set_ifg() Function*/


/*************************************************************************
 * Description    : To get meter config parameters calculated based on
 *                : rate and frequency
 *************************************************************************/
int32_t
tlite_meter_config_parameters(uint32_t rate, uint32_t clk_frequency,
			uint32_t rate_units, uint32_t *intwt, uint32_t *frcwt) {
	uint32_t   wt = 0;
	uint32_t   sysclk_hz = 0, clkdiv = 0;
	uint32_t   w_i = 0, w_f = 0;

	/*Clock division intial value*/
	clkdiv = 64;

	/*Changing khz to Hz*/
	/* sysclk_hz = clk_frequency * 1000; */
	sysclk_hz = clk_frequency;

	if (rate_units) {/* packets per second */
		wt = (rate * 65536 * clkdiv)/(sysclk_hz);
	}else{
		wt = (rate * 8192 * clkdiv)/(sysclk_hz);
		/* wt =(rate *8192 * clkdiv)/sysclk_hz; */
	}

	FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "wt\t%d\n", wt);

	w_i = (wt >> SHIFT_BY16);
	w_f = wt & 0xffff;

	/*Updating values*/
	*frcwt = w_f;
	*intwt = w_i;
	return 0;
}

/*************************************************************************
 * Description    : This function set intialization done, so that hardware
 *                : will start to process
 *************************************************************************/
int32_t pfe_tlite_dynamic_config_done()
{

	uint32_t phy_no;
	/* HW enable for the tdq fsm */
	for (phy_no = 0; phy_no < TLITE_SWITCH_MAX_DEV_NUM; phy_no++) {
		fp_reg_write( (uint32_t)(TLITE_PHY0_TDQ_CTRL_ADDR + (REG_LEN * phy_no)), TLITE_HW_EN);
	}

	return 0;
}

/*************************************************************************
 * Description    : This function reads the transmit packet count and current
 *                : queue drop count send to user side
 *************************************************************************/

pcm_msg_header_t *pfe_tlite_que_stats(uint32_t dev_id,
				uint32_t que_number, uint32_t hw_type)
{

	uint32_t read_val = 0, read_val2 = 0, read_val3 = 0;

	/*Validate config parameters*/
	/*classq number validation*/
	if (que_number < TLITE_MIN_QUE_NO || que_number > TLITE_MAX_QUE_NO) {
		gpmsg.pcmh_flags = TLITE_QUE_NUM_ERR;
		return (pcm_msg_header_t *)&gpmsg;
	}

	hw_type = TLITE_SWITCH;
	/*classq device id validation*/
	if (dev_id < TLITE_MIN_DEV_NUM || dev_id > TLITE_SWITCH_MAX_DEV_NUM) {
		gpmsg.pcmh_flags = TLITE_ERR_MSG_SHP_DEV;
		return (pcm_msg_header_t *)&gpmsg;
	}

	dev_id--;
	que_number--;

	/*xmit packet count*/
	read_val = read_fasw_context_memory(que_number, dev_id, CURQ_TRANS_CNT);

	/*Read current que dropcount*/
	read_val2 = read_fasw_context_memory(que_number, dev_id, CURQ_DROP_CNT);

	/*Read pkt count */
	read_val3 = read_fasw_context_memory(que_number, dev_id, CURQ_PKT_CNT);

	/*Reset message header */
	memset(&gpmsg, 0, sizeof(pcm_msg_header_t));

	/*Read pkt count*/
	gpmsg.pcmh_flags = read_val3;

	/*Update tx packet count*/
	gpmsg.pcmh_length = read_val;

	/*Message header value contrains given que number dropcount*/
	gpmsg.pcmh_value = read_val2;


	return (pcm_msg_header_t *)&gpmsg;

}

/*************************************************************************
 * Description    : This function reads the inq write pointer,
 *                : inq read pointer and fifo count send to user side
 *************************************************************************/

pcm_msg_header_t *pfe_tlite_inq_stats()
{

	uint32_t read_val = 0;

	/*Read inq stats register*/
	read_val = fp_reg_read( (uint32_t)(TLITE_PHY_INQ_STAT_ADDR) );

	/*Reset message header */
	memset(&gpmsg, 0, sizeof(pcm_msg_header_t));

	/*Message header flags contains inq write pointer*/
	gpmsg.pcmh_flags = read_val & BIT_0TO7;

	/*Message header length contains inq read pointer*/
	gpmsg.pcmh_length = (read_val >> SHIFT_BY8) & BIT_0TO7;

	/*Message header vlaue contains fifo count*/
	gpmsg.pcmh_value = (read_val >> SHIFT_BY16) & BIT_0TO15;

	return (pcm_msg_header_t *)&gpmsg;
}

/*************************************************************************
 * Description    : This function reads the configured schedulers
 *                : and send to user
 *************************************************************************/

uint32_t *pfe_tlite_sch_list(uint32_t dev_id)
{
	uint32_t *iptr;
	tlite_dev_list_t *pdev_list;

	pdev_list = &fp_tdev_list[dev_id];
	iptr = pdev_list->sched_list;
	return iptr;
}

/*************************************************************************
 * Description    : This function reads the configured shapers
 *                : and send to user
 *************************************************************************/

uint32_t *pfe_tlite_shp_list(uint32_t dev_id)
{
	uint32_t *iptr;
	tlite_dev_list_t *pdev_list;

	pdev_list = &fp_tdev_list[dev_id];
	iptr = pdev_list->shaper_list;
	return iptr;
}


/*************************************************************************
 * Description    : This function reads the configured queue
 *                : and send to user
 *************************************************************************/

uint32_t *pfe_tlite_classq_list(uint32_t dev_id)
{
	uint32_t *iptr;
	tlite_dev_list_t *pdev_list;

	pdev_list = &fp_tdev_list[dev_id];
	iptr = pdev_list->classq_list;

	return iptr;
}


/*************************************************************************
 * Description    : This function reads the configured queue
 *                : and send to user
 *************************************************************************/

uint32_t *pfe_tlite_classq_per_sched_list(uint32_t dev_id)
{
	uint32_t *iptr;
	tlite_dev_list_t *pdev_list;
	pdev_list = &fp_tdev_list[dev_id];
	iptr = pdev_list->classq_list;
	return iptr;
}



/*************************************************************************
 * Description    : Main function to intialize all parameters in tlite
 *************************************************************************/
int32_t
pfe_tlite_init(struct net_device *dev)
{

	uint32_t max_dev_no = 0, phy_no = 0, sched_no = 0, shp_no = 0;
	/*Get pci base address*/
	baseaddr = getPciBaseAddr(dev);

	FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s\n", __func__);

	memset(fp_tdev_list, 0, TLITE_SWITCH_MAX_DEV_NUM * sizeof(tlite_dev_list_t) );

	memset(Qposition, 0, TLITE_SWITCH_MAX_DEV_NUM * TLITE_MAX_SCHED_NUM * TLITE_MAX_QUE_NO * sizeof(uint32_t) );
	if (baseaddr == NULL) {
		FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_ERR, "base addr is NULL\n");
		return -1;
	}

	max_dev_no = TLITE_SWITCH_MAX_DEV_NUM;

	/* making the default queues as invalid */
	for( phy_no = 0; phy_no < max_dev_no; phy_no++ ){
		for( sched_no = 0; sched_no < TLITE_MAX_SCHED_NUM; sched_no++ ){
			fp_reg_write( ( ( uint32_t )( TLITE_SCH0_Q_ALLOC0_ADDR + ( SHED_ADDRESS_MASK * sched_no ) + ( DEV_ADDRESS_MASK * ( phy_no + 1 ) ) ) ), INVALID_QUEUES );
			fp_reg_write( ( ( uint32_t )( TLITE_SCH0_Q_ALLOC1_ADDR + ( SHED_ADDRESS_MASK * sched_no ) + ( DEV_ADDRESS_MASK * ( phy_no + 1 ) ) ) ), INVALID_QUEUES );
		}
	}

	/* making the default shapers as invalid */
	for( phy_no = 0; phy_no < max_dev_no; phy_no++ ){
		for( shp_no = 0; shp_no < TLITE_MAX_SHPR_NUM; shp_no++ ){
			//fp_reg_write( (uint32_t)(TLITE_SHP0_CTRL2_ADDR + (SHPR_ADDRESS_MASK * shp_no) + (DEV_ADDRESS_MASK * (phy_no + 1))), 0x22); /* 100010 */
			fp_reg_write( (uint32_t)(TLITE_SHP0_CTRL_ADDR + (SHPR_ADDRESS_MASK * shp_no) + (DEV_ADDRESS_MASK * (phy_no + 1))), 0x22); /* 100010 */
		}
	}

	/* writel(INQ_WMARK, TMU_INQ_WATERMARK); */

	return 0;
} /*End of pfe_tlite_init() Function*/

/*************************************************************************
 * Description    : This function intialize schedulers, shapers and queues
 *                : with default configuration
 *************************************************************************/
int32_t
pfe_tlite_default_config(void) {

	uint32_t i = 0, j = 0;
	uint32_t read_val = 0, write_val = 0, phy_no = 0;
	uint16_t queue_depth[ TLITE_MAX_QUE_NO ];
	uint32_t ret_val = 0;
	tlite_schd_params_t tsch;
	tlite_shpr_params_t tshp;
	tlite_classq_params_t tque;

	FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_INFO, "%s\n", __func__ );

	memset( &tsch, 0, sizeof( tlite_schd_params_t   ) );
	memset( &tshp, 0, sizeof( tlite_shpr_params_t   ) );
	memset( &tque, 0, sizeof( tlite_classq_params_t ) );

	for( phy_no = 0; phy_no < TLITE_SWITCH_MAX_DEV_NUM; phy_no++ ){
		/*making the hw_en -> 0 */
		read_val = fp_reg_read( ( uint32_t )( TLITE_PHY0_TDQ_CTRL_ADDR + ( REG_LEN * phy_no ) ) );
		write_val = read_val & BIT1_ZERO;
		fp_reg_write( ( uint32_t )( TLITE_PHY0_TDQ_CTRL_ADDR + ( REG_LEN * phy_no ) ), write_val );

		/* polling for allw_tdq_prog -> 1 */
		//while ( !((fp_reg_read((ULONG)(TLITE_PHY0_TDQ_CTRL_ADDR + (REG_LEN * phy_no))) & BIT4_ONE) == BIT4_ONE) ){} /* End of while loop */
		i = 100;
		while ( ( ( fp_reg_read( ( uint32_t )( TLITE_PHY0_TDQ_CTRL_ADDR + ( REG_LEN * phy_no ) ) ) & BIT4_ONE ) != BIT4_ONE ) && ( i-- > 0 ) ){} /* End of while loop */
		if(i == 0){
			FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_ERR, "timeout occure when allw_tdq_prog polling!\n" );
		}
	}

	/*Scheduler config*/
	for( j = 1; j <= TLITE_SWITCH_MAX_DEV_NUM; j++ ){
		for( i = 1; i <= TLITE_MAX_SCHED_NUM; i++ ){
			if( i == 1 ){
				tsch.tlsd_position = 1;
				tsch.tlsd_algo = 5; /* WRR */
			}else{
				tsch.tlsd_algo = 1; /* priority queuing */
			}
			tsch.tlsd_dev_id = j;
			tsch.tlsd_schd_id = i;
			tsch.tlsd_weight = 0x500;
			ret_val = pcm_tlite_scheduler_config( &tsch, TLITE_SWITCH );
			if( ret_val != 0 ){
				FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_ERR, "%s return error code for PORT %d, sheduler %d!\n", __func__, j, i );
			}

		}
	}

	/*Shaper config*/
	for( j = 1; j <= TLITE_SWITCH_MAX_DEV_NUM; j++ ){
		for( i = 1; i <= TLITE_MAX_SHPR_NUM; i++ ){
			tshp.tlsp_max_credit = 0x3FFE00;
			tshp.tlsp_rate_units = 1;
			/* 200 Mhz system clock */
			tshp.tlsp_clk_frequency  = 200; //200 Mhz system clock
			tshp.tlsp_rate  = 50000; /* 50Mbps */
			tshp.tlsp_dev_id = j;
			tshp.tlsp_shpr_id = i;
			/* shaper 0 and 1 are connected to SCH1, input 6 and 7 */
			/* shaper 2 and 3 disable */
			if( i < 3 ){
				tshp.tlsp_mode = 0x1; /* 0x1-credit based mode */
				tshp.tlsp_position = i + 7;
			}else{
				tshp.tlsp_mode = 0x0;
				tshp.tlsp_position = i + 10;
			}
			if( tshp.tlsp_mode == 0 ){
				tshp.tlsp_min_credit = ( tshp.tlsp_max_credit );
			}
			else{
				tshp.tlsp_min_credit = tshp.tlsp_max_credit;
			}
			FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_INFO, "\n\n" );
			if( i < 3 ){
				ret_val = pcm_tlite_shaper_config( &tshp, TLITE_SWITCH, 0 );
				if( ret_val < 0 ){
					FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_ERR, "%s shaper config failed! port %d, shaper %d!\n", __func__, ( j - 1 ), ( i - 1 ) );
				}
			}else{
				ret_val = pcm_tlite_shaper_disable( &tshp, TLITE_SWITCH, 0 ); /* shaper 3, 4 disable */
				if( ret_val < 0 ){
					FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_ERR, "%s shaper disable failed! port %d, shaper %d!\n", __func__, ( j - 1 ), ( i - 1 ) );
				}
			}
		}
	}

	/* Real HOST interface queue uses */
	// interface_queues_used( HIF_PORT_NUM, &j );
	// read_val = fp_reg_read( ( CLASS_HW_BASE_ADDR + CLASS_ING_AFULL_THRES ) );

	// ret_val = 1;
	// delta = read_val - 16 - 1;
	// for( i = 0; i < 8; i++ ){
	// 	/* HOST interface size of queues */
	// 	queue_depth[ i ] = 0;
	// 	if( ( j >> i ) & 1 ){
	// 		//if we use current queue
	// 		queue_depth[ i ] = ( read_val - 16 - 1 ) >> ( ret_val );
	// 		ret_val++;
	// 		if( !queue_depth[ i ] ){
	// 			queue_depth[ i ] = 1;
	// 		}
	// 		delta -= queue_depth[ i ];
	// 	}
	// 	FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_DEBUG, "%s: queue_depth[%d] = %d\n", __func__, i, queue_depth[ i ] );
	// }
	// queue_depth[ 0 ] += delta;
	host_queues_settings( queue_depth );

	for( j = 1; j <= TLITE_SWITCH_MAX_DEV_NUM; j++ ){
		read_val = 0;
		if( j == ( HIF_PORT_NUM + 1 ) ){
			read_val = TLITE_QUE_TAIL_DROP;
		}
		for( i = 1; i <= TLITE_MAX_QUE_NO; i++ ){
			tque.tlcq_classq_updates = TLITE_QUE_TAIL_DROP;
			tque.tlcq_max_len = 100;
			tque.tlcq_weight = 0x500;
			tque.tlcq_qmgmt = read_val;
			if( read_val ){
				tque.tlcq_qmgmt = TLITE_QUE_TAIL_DROP;
				tque.tlcq_max_len = queue_depth[ i - 1 ];
			}
			tque.tlcq_dev_id = j;
			tque.tlcq_classq_id = i;
			if( ( i == 3 ) || ( i == 4 ) ){
				tque.tlcq_schd_id = 2;
				/* Actuval Q2 and Q3 connected to
						schedular 1, input_6 and input_7 */
				tque.tlcq_schd_slot = i + 4;
			}else{
				tque.tlcq_schd_id = 1;
				if( i < 5 ){
					tque.tlcq_schd_slot = i;
				}else{
					tque.tlcq_schd_slot = i - 2;
				}
			}
			ret_val = pcm_tlite_classq_add( &tque, TLITE_SWITCH );
			if( ret_val < 0 ){
				FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_ERR, "%s queue add failed! port %d, queue %d!\n", __func__, ( j - 1 ), ( i - 1 ) );
			}
		}
	}

	/* HW enable for the tdq fsm */
	pfe_tlite_dynamic_config_done();

	return 0;

} /*End of pfe_tlite_default_config() Function*/




/*************************************************************************
 * Description    : This function configures credit mode shapers for
 *                  given device
 *                :
 *************************************************************************/
int32_t
pcm_tlite_creditmode_shaper_config(tlite_creditmode_params_t *ptdp)
{
	uint32_t  j = 0, k = 0, l = 0;
	uint32_t ret_val = 0;
	tlite_shpr_params_t tshp;

	memset(&tshp, 0, sizeof(tlite_shpr_params_t));
	pfe_tlite_dynamic_config_enable(1);

	FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:\n", __func__);


	/*Shaper config*/
	for (j = 0; j < TLITE_SWITCH_MAX_DEV_NUM; j++) {
		for (k = 0; k < TLITE_MAX_SCHED_NUM; k++) {
			for (l = 0; l < TLITE_MAX_QUE_NO; l++) {
				if (Qposition[j][k][l] == ptdp->tld_traffic_class) {
					/* 2 to 17 */
					tshp.tlsp_position = l + ((1 - k) * 8) + 1 + 1;
				}
			}
		}
	}

	tshp.tlsp_rate  = ptdp->tld_idle_slope;
	tshp.tlsp_dev_id = ptdp->tld_port_no;
	tshp.tlsp_shpr_id = ptdp->tld_shaper_id;
	tshp.tlsp_max_credit = 0x3FFE00;
	tshp.tlsp_min_credit = 0x3FFE00;
	tshp.tlsp_mode = 0x1;
	tshp.tlsp_rate_units = 1;
	//tshp.tlsp_clk_frequency  = 39;
	tshp.tlsp_clk_frequency  = 200;
	ret_val = pcm_tlite_shaper_config(&tshp, TLITE_SWITCH, 0);
	pfe_tlite_dynamic_config_done();
	return ret_val;

} /*End of pcm_tlite_creditmode_shaper_config Function*/

/*************************************************************************
* Description    : This function set regenerate  pcp for
*                  given divice
*                :
*************************************************************************/
void pfe_tlite_set_regenerate_pcp(uint32_t devId, uint32_t pcp_value)
{
	if( devId < TLITE_SWITCH_MAX_DEV_NUM){
		fp_reg_write( (uint32_t)(CLASS_PORT0_STRUC3 + (REG_LEN * devId)), pcp_value);
		fp_reg_write( (uint32_t)(CLASS2_PORT0_STRUC3 + (REG_LEN * devId)), pcp_value);
	}
}

/*************************************************************************
* Description    : This function set regenerate  pcp for
*                  given divice
*		 :
*************************************************************************/

uint32_t pfe_tlite_get_regenerate_pcp(uint32_t devId)
{
	uint32_t read_val = 0;
	if(devId < TLITE_SWITCH_MAX_DEV_NUM){
		read_val = fp_reg_read( (uint32_t)(CLASS_PORT0_STRUC3 + (REG_LEN * devId)));
	}
	return read_val;
}

/*************************************************************************
 * Description    : This function enables taildrop
 *************************************************************************/
int32_t
pfe_tlite_taildrop_enable(uint32_t dev_id, uint32_t que_no, uint32_t value) {
	uint32_t write_val = 0;

	/*dev_id  validation*/
	if (dev_id < TLITE_MIN_DEV_NUM || dev_id > TLITE_SWITCH_MAX_DEV_NUM){
		return TLITE_ERR_MSG_DEV_NUM;
	}

	/*classq number validation*/
	if (que_no < TLITE_MIN_QUE_NO || que_no > TLITE_MAX_QUE_NO){
		return TLITE_QUE_NUM_ERR;
	}

	/* queue validation*/
	if (value < TLITE_MIN_TAILDROP_VALUE || value > TLITE_MAX_TAILDROP_VALUE){
		return TLITE_ERR_MSG_TAILDROP_VALUE;
	}

	dev_id--;
	que_no--;
	value--;

	write_val =  (value << SHIFT_BY11) | TLITE_QUE_TAIL_DROP;
	write_fasw_context_memory(write_val, que_no, dev_id, CURQ_QMAX_QMIN_CFG);
	return 0;
} /*End of pfe_tlite_taildrop_enable() Function*/


/*************************************************************************
 * Description    : This function disables taildrop
 *************************************************************************/
int32_t
pfe_tlite_taildrop_disable(uint32_t dev_id, uint32_t que_no) {

	/*dev_id  validation*/
	if (dev_id < TLITE_MIN_DEV_NUM || dev_id > TLITE_SWITCH_MAX_DEV_NUM){
		return TLITE_ERR_MSG_DEV_NUM;
	}

	/*classq number validation*/
	if (que_no < TLITE_MIN_QUE_NO || que_no > TLITE_MAX_QUE_NO){
		return TLITE_QUE_NUM_ERR;
	}

	dev_id--;
	que_no--;

	write_fasw_context_memory(0, que_no, dev_id, CURQ_QMAX_QMIN_CFG);

	return 0;
} /*End of pfe_tlite_taildrop_disable() Function*/


/*************************************************************************
 * Description    : This function get taildrop value
 *************************************************************************/
int32_t
pfe_tlite_get_taildrop_value(uint32_t dev_id, uint32_t que_no) {
	uint32_t read_val = 0;

	/*dev_id  validation*/
	if (dev_id < TLITE_MIN_DEV_NUM || dev_id > TLITE_SWITCH_MAX_DEV_NUM){
		return TLITE_ERR_MSG_DEV_NUM;
	}

	/*classq number validation*/
	if (que_no < TLITE_MIN_QUE_NO || que_no > TLITE_MAX_QUE_NO){
		return TLITE_QUE_NUM_ERR;
	}

	dev_id--;
	que_no--;

	read_val = read_fasw_context_memory( que_no, dev_id, CURQ_QMAX_QMIN_CFG );

	return read_val;
} /*End of pfe_tlite_get_taildrop_value() Function*/



/*************************************************************************
 * Description    : This function set shaper mode
 *************************************************************************/
int32_t
pfe_tlite_shaper_mode_config(uint32_t dev_id, uint32_t shaper_no, uint32_t mode) {
	uint32_t read_val = 0;
	/*dev_id  validation*/
	if (dev_id < TLITE_MIN_DEV_NUM || dev_id > TLITE_SWITCH_MAX_DEV_NUM){
		return TLITE_ERR_MSG_DEV_NUM;
	}
	/*Shaper number validation*/
	if (shaper_no < TLITE_MIN_SHPR_NUM || shaper_no > TLITE_MAX_SHPR_NUM) {
		FP_DEBUG(DBGP_FEAT_ERR, FP_LOG_EMERG, "%s:TLITE_ERR_MSG_SHP_NUMBER %d\n", __func__, shaper_no);
		return TLITE_ERR_MSG_SHP_NUMBER;
	}
	if (mode < TLITE_MIN_MODE || mode > TLITE_MAX_MODE){
		return TLITE_ERR_MSG_SHP_MODE;
	}
	dev_id--;
	shaper_no--;
	pfe_tlite_dynamic_config_enable(1);
	/* writing rate units and position in TMU_SHP0_CTRL2 register */
	read_val = fp_reg_read( (uint32_t)(TLITE_SHP0_CTRL2_ADDR + (SHPR_ADDRESS_MASK * shaper_no) + (DEV_ADDRESS_MASK * (dev_id + 1))) );
	read_val =  (read_val & 0x3F) | (mode << SHIFT_BY6);
	fp_reg_write( (uint32_t)(TLITE_SHP0_CTRL2_ADDR + (SHPR_ADDRESS_MASK * shaper_no) + (DEV_ADDRESS_MASK * (dev_id + 1))), read_val );
	read_val = fp_reg_read( (uint32_t)(TLITE_SHP0_CTRL2_ADDR + (SHPR_ADDRESS_MASK * shaper_no) + (DEV_ADDRESS_MASK * (dev_id + 1))) );
	pfe_tlite_dynamic_config_done();
	return read_val;
} /*End of pfe_tlite_shaper_mode_config () Function*/

/*************************************************************************
* Description    : Main function to set tlite config parameters
*************************************************************************/

int32_t
pcm_tlite_config_ioctl(struct net_device *dev, struct ifreq *rq, int cmd)
{

	uint32_t retval = 0;
	uint32_t /*ret,*/ *iptr, pcpval = 0;

	nc_ioctl_ops_t *kmem_pnc, *pnc_ops = (nc_ioctl_ops_t *) rq->ifr_ifru.ifru_data;
	pcm_msg_header_t *pmsg = (pcm_msg_header_t *)pnc_ops->io_data;
	pcm_msg_header_t *kmem_pmsg;
	tlite_schd_params_t *kmem_schd_params;
	tlite_shpr_params_t *kmem_shpr_params;
	tlite_classq_params_t *kmem_classq_params;
	//tlite_meter_params_t *kmem_meter_params;
	tlite_creditmode_params_t *kmem_creditmode_params;

	kmem_pnc = kmalloc(sizeof(nc_ioctl_ops_t), __GFP_DMA|GFP_ATOMIC);
	if(kmem_pnc == FP_NULL){
		FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "%s: can not allocate memory for NC_IOCTL_OPS!\n", __func__);
		return -EFAULT;
	}

	if( copy_from_user(kmem_pnc, (nc_ioctl_ops_t *)rq->ifr_ifru.ifru_data, sizeof(nc_ioctl_ops_t)) ){
		FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "%s: can not copy data from user!\n", __func__);
		kfree(kmem_pnc);
		return -EFAULT;
	}

	pnc_ops = kmem_pnc;

	kmem_pmsg = kmalloc(sizeof(pcm_msg_header_t), __GFP_DMA|GFP_ATOMIC);

	if(kmem_pmsg == FP_NULL){
		FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "%s: can not allocate memory for PCM_MSG_HEADER!\n", __func__);
		kfree(kmem_pnc);
		return -EFAULT;
	}


	if( copy_from_user(kmem_pmsg, (pcm_msg_header_t *)pnc_ops->io_data, sizeof(pcm_msg_header_t)) ){
		FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "%s: can not copy data from user!\n", __func__);
		kfree(kmem_pmsg);
		kfree(kmem_pnc);
		return -EFAULT;
	}


	pmsg = kmem_pmsg;

	/*Get pci base address*/
	baseaddr = getPciBaseAddr(dev);

	switch (pmsg->pcmh_type) {

		case PCM_MSG_TLITE_DYNAMIC_CONFIG_ENABLE:
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:dynamic config enable\n", __func__);
			/*dynamic configuration enable for all schedulers and shapers*/
			//retval = pfe_tlite_dynamic_config_enable(pmsg->pcmh_value);
			retval = pfe_tlite_dynamic_config_enable(1);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:dynamic config enable\n", __func__);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "hw_type:\t%x\n", 1);
			break;

		case PCM_MSG_TLITE_SCHD_CONFIG:
			kmem_schd_params = kmalloc(sizeof(tlite_schd_params_t), __GFP_DMA|GFP_ATOMIC);
			if(kmem_schd_params == FP_NULL){
				FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "%s: req: PCM_MSG_TLITE_SCHD_CONFIG can not allocate memory for tlite_schd_params_t!\n", __func__);
				retval = -EFAULT;
				goto mem_clear;
			}
			if( copy_from_user(kmem_schd_params, (tlite_schd_params_t *)(((uint8_t *)pnc_ops->io_data) + sizeof(pcm_msg_header_t)), sizeof(tlite_schd_params_t)) ){
				FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "%s: req: PCM_MSG_TLITE_SCHD_CONFIG; can not copy data from user\n", __func__);
				retval = -EFAULT;
				kfree(kmem_schd_params);
				goto mem_clear;
			}
			//retval = pcm_tlite_scheduler_config(kmem_schd_params, pmsg->pcmh_value);
			retval = pcm_tlite_scheduler_config(kmem_schd_params, 1);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "schconfig return value:%x\n", retval);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "%s:Scheduler config\n", __func__);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "sch_id:\t%x\n", kmem_schd_params->tlsd_schd_id);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "dev_id:\t%x\n", kmem_schd_params->tlsd_dev_id);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "algo  :\t%x\n", kmem_schd_params->tlsd_algo);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "position:\t%x\n", kmem_schd_params->tlsd_position);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "weight:\t%x\n", kmem_schd_params->tlsd_weight);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "hw_type:\t%x\n", pmsg->pcmh_value);
			kfree(kmem_schd_params);
			break;

		case PCM_MSG_TLITE_SHPR_CONFIG:
			kmem_shpr_params = kmalloc(sizeof(tlite_shpr_params_t), __GFP_DMA|GFP_ATOMIC);
			if(kmem_shpr_params == FP_NULL){
				FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "%s: req: PCM_MSG_TLITE_SHPR_CONFIG can not allocate memory for tlite_shpr_params_t!\n", __func__);
				retval = -EFAULT;
				goto mem_clear;
			}
			if( copy_from_user(kmem_shpr_params, (tlite_shpr_params_t *)(((uint8_t *)pnc_ops->io_data) + sizeof(pcm_msg_header_t)), sizeof(tlite_shpr_params_t) ) ){
				FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "%s: req: PCM_MSG_TLITE_SHPR_CONFIG can not copy data from user\n", __func__);
				retval = -EFAULT;
				kfree(kmem_shpr_params);
				goto mem_clear;
			}
			//retval = pcm_tlite_shaper_config(kmem_shpr_params, pmsg->pcmh_value, 0);
			retval = pcm_tlite_shaper_config(kmem_shpr_params, 1, 0);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:Shaper config\n", __func__);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "shp_update:\t%x\n", kmem_shpr_params->tlsp_shpr_updates);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "shpr_id:\t%x\n", kmem_shpr_params->tlsp_shpr_id);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "dev_id  :\t%x\n", kmem_shpr_params->tlsp_dev_id);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "clk_freq:\t%x\n", kmem_shpr_params->tlsp_clk_frequency);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "rate:\t%x\n", kmem_shpr_params->tlsp_rate);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "r_units:\t%x\n", kmem_shpr_params->tlsp_rate_units);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "max_credit:\t%x\n", kmem_shpr_params->tlsp_max_credit);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "shp_position:\t%x\n", kmem_shpr_params->tlsp_position);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "hw_type:\t%x\n", pmsg->pcmh_value);

			kfree(kmem_shpr_params);
			break;

		case PCM_MSG_TLITE_CLASSQ_ADD:
			kmem_classq_params = kmalloc(sizeof(tlite_classq_params_t), __GFP_DMA|GFP_ATOMIC);
			if(kmem_classq_params == FP_NULL){
				FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "%s: req: PCM_MSG_TLITE_CLASSQ_ADD can not allocate memory for tlite_classq_params_t!\n", __func__);
				retval = -EFAULT;
				goto mem_clear;
			}
			if(copy_from_user(kmem_classq_params, (tlite_classq_params_t *)(((uint8_t *)pnc_ops->io_data) + sizeof(pcm_msg_header_t)), sizeof(tlite_classq_params_t)) ){
				FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "%s: req: PCM_MSG_TLITE_CLASSQ_ADD can not copy data from user\n", __func__);
				retval = -EFAULT;
				kfree(kmem_classq_params);
				goto mem_clear;
			}
			//retval = pcm_tlite_classq_add(kmem_classq_params, pmsg->pcmh_value);
			retval = pcm_tlite_classq_add(kmem_classq_params, 1);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:Class que add\n", __func__);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "dev_id\t%x\n", kmem_classq_params->tlcq_dev_id);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "q_id:\t%x\n", kmem_classq_params->tlcq_classq_id);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "sch_id  :\t%x\n", kmem_classq_params->tlcq_schd_id);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "schd_slot:\t%x\n", kmem_classq_params->tlcq_schd_slot);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "max_len:\t%x\n", kmem_classq_params->tlcq_max_len);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "q_wt:\t%x\n", kmem_classq_params->tlcq_weight);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "q_mgmt:\t%x\n", kmem_classq_params->tlcq_qmgmt);
			if ((kmem_classq_params->tlcq_qmgmt) == 2) {
				FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "min_length:\t%x\n", kmem_classq_params->tlcq_min_len);
				FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "prob0:\t%x\n", kmem_classq_params->tlcq_prob_cfg0);
				FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "prob1:\t%x\n", kmem_classq_params->tlcq_prob_cfg1);
				FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "prob2:\t%x\n", kmem_classq_params->tlcq_prob_cfg2);
				FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "prob3:\t%x\n", kmem_classq_params->tlcq_prob_cfg3);
				FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "prob4:\t%x\n", kmem_classq_params->tlcq_prob_cfg4);
				FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "prob5:\t%x\n", kmem_classq_params->tlcq_prob_cfg5);
				FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "prob6:\t%x\n", kmem_classq_params->tlcq_prob_cfg6);
				FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "prob7:\t%x\n", kmem_classq_params->tlcq_prob_cfg7);
			}
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "hw_type:\t%x\n", pmsg->pcmh_value);
			kfree(kmem_classq_params);
			break;

		case PCM_MSG_TLITE_CLASSQ_DELETE:
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:Cls q del  dev:%x q_num:%x hw_type:%x\n", __func__, pmsg->pcmh_flags, pmsg->pcmh_value, pmsg->pcmh_length);
			//retval = pcm_tlite_classq_delete( (uint32_t)pmsg->pcmh_flags, (uint32_t)pmsg->pcmh_value, pmsg->pcmh_length);
			retval = pcm_tlite_classq_delete( (uint32_t)pmsg->pcmh_flags, (uint32_t)pmsg->pcmh_value, 1);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:Class Que delete dev:%x q_num:%x hw_type:%x\n", __func__, pmsg->pcmh_flags, pmsg->pcmh_value, pmsg->pcmh_length);
			break;

		case PCM_MSG_TLITE_SET_IFG:
			/*Set ifg value based on device id*/
			//retval = pfe_tlite_set_ifg( (uint32_t)pmsg->pcmh_flags, (uint32_t)pmsg->pcmh_value, pmsg->pcmh_length);
			retval = pfe_tlite_set_ifg( (uint32_t)pmsg->pcmh_flags, (uint32_t)pmsg->pcmh_value, 1);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:set ifg\n", __func__);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "dev_id\t%x\n", pmsg->pcmh_flags);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "ifg:\t%x\n", pmsg->pcmh_value);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "hw_type:\t%x\n", pmsg->pcmh_length);
			break;

		case PCM_MSG_TLITE_METER_CONFIG:
			/*
			kmem_meter_params = kmalloc(sizeof(tlite_meter_params_t), __GFP_DMA|GFP_ATOMIC);
			if(kmem_meter_params == FP_NULL){
				FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "%s: req: PCM_MSG_TLITE_METER_CONFIG can not allocate memory for tlite_meter_params_t!\n", __func__);
				retval = -EFAULT;
				goto mem_clear;
			}
			if(copy_from_user(kmem_meter_params, (tlite_meter_params_t *)(((uint8_t *)pnc_ops->io_data) + sizeof(pcm_msg_header_t)), sizeof(tlite_meter_params_t) ) ){
				FP_DEBUG(DBGP_FEAT_IOCTL, FP_LOG_INFO, "%s: req: PCM_MSG_TLITE_METER_CONFIG can not copy data from user\n", __func__);
				retval = -EFAULT;
				kfree(kmem_meter_params);
				goto mem_clear;
			}
			*/
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:meter_config\nThis function not supported\n", __func__);
			//retval = pcm_tlite_fasw_meter_config(kmem_meter_params);
			/*
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:meter_config\n", __func__);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "meter_no\t%x\n", kmem_meter_params->tlmt_meter_no);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "cloc_freq:\t%x\n", kmem_meter_params->tlmt_clock_freq);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "max_credit:\t%x\n", kmem_meter_params->tlmt_max_credit);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "enable:\t%x\n", kmem_meter_params->tlmt_enable);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "ifg_add:\t%x\n", kmem_meter_params->tlmt_ifg_add);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "rate_units:\t%x\n", kmem_meter_params->tlmt_rate_units);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "clear:\t%x\n", kmem_meter_params->tlmt_clear);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_DEBUG, "rate :\t%x\n", kmem_meter_params->tlmt_rate);
			*/
			/* kfree(kmem_meter_params); */
			break;

		case PCM_MSG_TLITE_METER_GLOBAL_ENABLE:
			/*Enable or disable based on input*/
			//retval = pfe_tlite_fasw_meter_enable((ULONG)pmsg->pcmh_value);
			//FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:meter_global_enable\n", __func__);
			//FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "enable:\t%x\n", pmsg->pcmh_value);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:meter_global_enable\nThis function not supported\n", __func__);
			break;

		case PCM_MSG_TLITE_DYNAMIC_CONFIG_DONE:
			/*Set init done bit, after intialization of all schedulers and shapers*/
			retval = pfe_tlite_dynamic_config_done();
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:dynamic_config_done\n", __func__);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "hw_type:\t%x\n", pmsg->pcmh_value);
			break;

		case PCM_MSG_TLITE_QUE_STATS:
			//pmsg = pfe_tlite_que_stats(pmsg->pcmh_value, pmsg->pcmh_length, pmsg->pcmh_flags);
			pmsg = pfe_tlite_que_stats(pmsg->pcmh_value, pmsg->pcmh_length, 1);
			/*Update information to user*/
			if(copy_to_user(pnc_ops->io_data, pmsg, sizeof(pcm_msg_header_t)) ){
				FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s: req: PCM_MSG_TLITE_QUE_STATS; can not copy data to user\n", __func__);
				retval = -EFAULT;
			}
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:get_que_stats\n", __func__);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "tx packet count\t%x\n", pmsg->pcmh_length);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "dropcount:\t%x\n", pmsg->pcmh_value);
			break;

		case PCM_MSG_TLITE_INQ_STATS:
			pmsg =  pfe_tlite_inq_stats();
			/*Update inq stats information to user*/
			if( copy_to_user(pnc_ops->io_data, pmsg, sizeof(pcm_msg_header_t)) ){
				FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s: req: PCM_MSG_TLITE_INQ_STATS; can not copy data to user\n", __func__);
				retval = -EFAULT;
			}
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:get_inq_stats\n", __func__);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "hw_type:\t%x\n", pmsg->pcmh_value);
		break;

		case PCM_MSG_TLITE_CLASSQ_CHECK:
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "dev_id:\t%x\n", pmsg->pcmh_flags);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "classq_id:\t%x\n", pmsg->pcmh_length);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "sched_id:\t%x\n", pmsg->pcmh_value);
			if (fp_tdev_list[pmsg->pcmh_flags].classq_list[pmsg->pcmh_length] == 0) {
				pmsg->pcmh_length = CLASSQ_NOTCONNECTED;
				FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s: CLASSQ_NOTCONNECTED\n", __func__);
			} else {
				FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:CLASSQ_CONNECTED\n", __func__);
				pmsg->pcmh_value = fp_tdev_list[pmsg->pcmh_flags].classq_list[pmsg->pcmh_length];
				pmsg->pcmh_length = CLASSQ_CONNECTED;
			}
			if(copy_to_user(pnc_ops->io_data, pmsg, sizeof(pcm_msg_header_t)) ){
				FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s: req: PCM_MSG_TLITE_CLASSQ_CHECK; can not copy data to user\n", __func__);
				retval = -EFAULT;
			}
			break;

		case PCM_MSG_TLITE_SCH_LIST:
			iptr = pfe_tlite_sch_list((pmsg->pcmh_value) - 1);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:get_sch_list\n", __func__);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, ":\t%x \t%x\n", *iptr, *(iptr + 1));
			if( copy_to_user(pnc_ops->io_data, iptr, TLITE_MAX_SCHED_NUM * sizeof(uint32_t)) ){
				FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s: req: PCM_MSG_TLITE_SCH_LIST; can not copy data to user\n", __func__);
				retval = -EFAULT;
			}
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:get_sch_list\n", __func__);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "dev_id:\t%x\n", pmsg->pcmh_value);
			break;

		case PCM_MSG_TLITE_SHP_LIST:
			iptr = pfe_tlite_shp_list((pmsg->pcmh_value) - 1);
			if( copy_to_user(pnc_ops->io_data, iptr, TLITE_MAX_SHPR_NUM * sizeof(uint32_t)) ){
				FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s: req: PCM_MSG_TLITE_SHP_LIST; can not copy data to user\n", __func__);
				retval = -EFAULT;
			}
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:get_shp_list\n", __func__);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "dev_id:\t%x\n", pmsg->pcmh_value);
			break;

		case PCM_MSG_TLITE_CLASSQ_LIST:
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "dev_id:\t%x\n", pmsg->pcmh_value);
			iptr = pfe_tlite_classq_list((pmsg->pcmh_value) - 1);
			if( copy_to_user(pnc_ops->io_data, iptr, TLITE_MAX_QUE_NO * sizeof(uint32_t)) ){
				FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s: req: PCM_MSG_TLITE_CLASSQ_LIST; can not copy data to user\n", __func__);
				retval = -EFAULT;
			}
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:get_classq_list\n", __func__);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "dev_id:\t%x\n", pmsg->pcmh_value);
			break;

		case PCM_MSG_TLITE_CLASSQ_PER_SCH_LIST:
			iptr = pfe_tlite_classq_per_sched_list( (pmsg->pcmh_value) - 1);
			if( copy_to_user(pnc_ops->io_data, iptr, TLITE_MAX_QUE_NO * sizeof(uint32_t)) ){
				FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s: req: PCM_MSG_TLITE_CLASSQ_PER_SCH_LIST; can not copy data to user\n", __func__);
				retval = -EFAULT;
			}
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:get_classq_per_sch_list\n", __func__);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "dev_id:\t%x\n", pmsg->pcmh_value);
			break;

		case PCM_MSG_TLITE_CREDITMODE_SHP_CONFIG:
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:get_classq_per_sch_list\n", __func__);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "dev_id:\t%x\n", pmsg->pcmh_value);
			kmem_creditmode_params = kmalloc(sizeof(tlite_creditmode_params_t), __GFP_DMA|GFP_ATOMIC);
			if(kmem_creditmode_params == FP_NULL){
				retval = -EFAULT;
				FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s: req: PCM_MSG_TLITE_CREDITMODE_SHP_CONFIG; can not allocate memory for tlite_creditmode_params_t!\n", __func__);
				goto mem_clear;
			}
			if( copy_from_user(kmem_creditmode_params, (tlite_creditmode_params_t *)(((uint8_t *)pnc_ops->io_data) + sizeof(pcm_msg_header_t)), sizeof(tlite_creditmode_params_t ) ) ){
				retval = -EFAULT;
				kfree(kmem_creditmode_params);
				FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s: req: PCM_MSG_TLITE_CREDITMODE_SHP_CONFIG; can not copy data from user\n", __func__);
				goto mem_clear;
			}
			retval = pcm_tlite_creditmode_shaper_config(kmem_creditmode_params);
			kfree(kmem_creditmode_params);
			break;

		case PCM_MSG_TLITE_SET_REGENERATE_PCP:
			pfe_tlite_set_regenerate_pcp( (pmsg->pcmh_flags) - 1, pmsg->pcmh_value );
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:set_regenerate_pcpvalue\n", __func__);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "port_no:\t%x value:\t%x\n", pmsg->pcmh_flags, pmsg->pcmh_value);
			break;

		case PCM_MSG_TLITE_GET_REGENERATE_PCP:
			pcpval = pfe_tlite_get_regenerate_pcp( (pmsg->pcmh_flags) - 1 );
			iptr = &pcpval;
			if (iptr == NULL){
				break;
			}
			if(copy_to_user(pnc_ops->io_data, iptr, sizeof(uint32_t))){
				retval = -EFAULT;
				FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s: req: PCM_MSG_TLITE_GET_REGENERATE_PCP; can not copy data to user\n", __func__);
			}
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:get_regenerate_pcpvalue\n", __func__);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "value:\t%x\n", *iptr);
			break;

		case PCM_MSG_TLITE_TAILDROP_ENABLE:
			pfe_tlite_taildrop_enable( pmsg->pcmh_value, pmsg->pcmh_length, pmsg->pcmh_flags );
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:taildrop_enable\n", __func__);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "dev_id:\t%x classq_id:\t%x value:\t%x\n", pmsg->pcmh_value, pmsg->pcmh_length, pmsg->pcmh_flags);
			break;

		case PCM_MSG_TLITE_TAILDROP_DISABLE:
			pfe_tlite_taildrop_disable( pmsg->pcmh_value, pmsg->pcmh_length );
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:taildrop_disable\n", __func__);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "dev_id:\t%x classq_id:\t%x\n", pmsg->pcmh_value, pmsg->pcmh_length);
			break;

		case PCM_MSG_TLITE_GET_TAILDROP_VALUE:
			retval = pfe_tlite_get_taildrop_value( pmsg->pcmh_value, pmsg->pcmh_length );
			iptr = &retval;
			if (iptr == NULL){
				retval = 0;
				break;
			}
			if(copy_to_user(pnc_ops->io_data, iptr, sizeof(uint32_t))){
				retval = -EFAULT;
				FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s: req: PCM_MSG_TLITE_GET_TAILDROP_VALUE; can not copy data to user\n", __func__);
			}
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:get_taildrop_value\n", __func__);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "dev_id:\t%x classq_id:\t%x\n", pmsg->pcmh_value, pmsg->pcmh_length);
			break;

		case PCM_MSG_TLITE_SHAPER_SET_MODE:
			retval = pfe_tlite_shaper_mode_config( pmsg->pcmh_value, pmsg->pcmh_length, pmsg->pcmh_flags );
			iptr = &retval;
			if (iptr == NULL){
				retval = 0;
				break;
			}
			if( copy_to_user(pnc_ops->io_data, iptr, sizeof(uint32_t))){
				retval = -EFAULT;
				FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s: req: PCM_MSG_TLITE_SHAPER_SET_MODE; can not copy data to user\n", __func__);
			}
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:shaper_mode_config\n", __func__);
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "dev_id:\t%x shaper_id:\t%x mode value:\t%x\n", pmsg->pcmh_value, pmsg->pcmh_length, pmsg->pcmh_flags);
			break;

		default:
			FP_DEBUG(DBGP_FEAT_TMU, FP_LOG_INFO, "%s:Unknown message type\n", __func__);
			retval = PCM_ERR_MSG_TMU_UNKNOWN_TYPE;
			break;

	} /*End of message type switch*/

mem_clear:
	kfree(kmem_pmsg);
	kfree(kmem_pnc);
	return retval;
} /*End of pcm_tmu_config_ioctl() Function*/
