#include "fp_proc_fs.h"
#include "fp_library.h"

#include "fp_netd.h"
#include "fp_net_driver.h"
#include "fp_stats.h"
#include "fp_switch.h"
#include "fp_switch_api.h"
#include "fp_global_addr_map.h"

#include "hw_2field_hash_table.h"
#include "hw_vlan_hash_table.h"
#include "fp_pre_header.h"
#include "phy_dp83867.h"
#include <fp_common_msg_header.h>
#include <tlite_blocks_ctrl_api.h>
#include <tlite_blocks_ctrlm.h>
#include "fp_flow_control.h"
#include "fp_mac_ops.h"
#include "fp_bd_api.h"

#include <linux/mutex.h>

#define MAX_SZIE_MAC_TABLE_IN_BYTES 1228800
#define MAX_SZIE_VLAN_TABLE_IN_BYTES 40960

#define CREATE_PROC_FILE_ID( proc_entry, name, permission, dir, fops, id_arr, id_pos )      id_arr[id_pos] = id_pos; \
                                                                                            proc_entry = proc_create_data( name, permission, dir, fops, &id_arr[id_pos] );

enum
{
    FILE_STAT_EM1 = 0,
    FILE_STAT_EM2,
    FILE_STAT_EM3,
    FILE_STAT_EM4,
    FILE_STAT_EM5,
    FILE_STAT_EM6,
    FILE_STAT_EM7,
    FILE_STAT_EM8,
    FILE_STAT_EM9,
    FILE_STAT_EM10,
    FILE_STAT_EM11,
    FILE_STAT_EM12,
    FILE_STAT_EM13,
    FILE_STAT_EM14,
    FILE_STAT_EM15,
    FILE_STAT_EM16,
    FILE_STAT_SWITCH,
    MAX_FILES_STAT
};
static uint32_t id_stat[MAX_FILES_STAT];

enum
{
    FILE_HASH_MAC_SHOW_TABLE = 0,
    FILE_HASH_MAC_ADD,
    FILE_HASH_MAC_SEARCH,
    FILE_HASH_MAC_DELETE,
    FILE_HASH_MAC_CLEAN_TABLE,
    FILE_HASH_VLAN_SHOW_TABLE,
    FILE_HASH_VLAN_ADD,
    FILE_HASH_VLAN_SEARCH,
    FILE_HASH_VLAN_DELETE,
    FILE_HASH_VLAN_CLEAN_TABLE,
    FILE_HASH_VLAN_GLOBAL_VLAN_ENTRY,
    MAX_FILES_HASH
};
static uint32_t id_hash[MAX_FILES_HASH];

enum
{
    FILE_SETTINGS_PORT1 = 0,
    FILE_SETTINGS_PORT2,
    FILE_SETTINGS_PORT3,
    FILE_SETTINGS_PORT4,
    FILE_SETTINGS_PORT5,
    FILE_SETTINGS_PORT6,
    FILE_SETTINGS_PORT7,
    FILE_SETTINGS_PORT8,
    FILE_SETTINGS_PORT9,
    FILE_SETTINGS_PORT10,
    FILE_SETTINGS_PORT11,
    FILE_SETTINGS_PORT12,
    FILE_SETTINGS_PORT13,
    FILE_SETTINGS_PORT14,
    FILE_SETTINGS_PORT15,
    FILE_SETTINGS_PORT16,
    FILE_SETTINGS_SWITCH,
    MAX_FILES_SETTINGS_PORT
};
static uint32_t id_port_settings[MAX_FILES_SETTINGS_PORT];

enum
{
    FILE_PORT1_QUEUES = 0,
    FILE_PORT2_QUEUES,
    FILE_PORT3_QUEUES,
    FILE_PORT4_QUEUES,
    FILE_PORT5_QUEUES,
    FILE_PORT6_QUEUES,
    FILE_PORT7_QUEUES,
    FILE_PORT8_QUEUES,
    FILE_PORT9_QUEUES,
    FILE_PORT10_QUEUES,
    FILE_PORT11_QUEUES,
    FILE_PORT12_QUEUES,
    FILE_PORT13_QUEUES,
    FILE_PORT14_QUEUES,
    FILE_PORT15_QUEUES,
    FILE_PORT16_QUEUES,
    FILE_HOST_QUEUES,
    MAX_FILES_QUEUE_SETTINGS
};
static uint32_t id_queue_settings[MAX_FILES_QUEUE_SETTINGS];

enum
{
    FILE_DEBUG_PRINT = 0,
    FILE_DEBUG_WR_PACKET,
    FILE_DEBUG_AGEING_TIMEOUT,
    FILE_DEBUG_BMU_OCCUPIED_BUFFERS,
    FILE_DEBUG_FAR_END_LOOPBACK,
    FILE_DEBUG_HOST_SYSTEM_LOOPBACK,
    FILE_DEBUG_MDIO_SPEED,
    FILE_DEBUG_JUMBO_ENABLE,
    FILE_DEBUG_PHY_TEST_MODE,
    FILE_DEBUG_MCU_FW_VERSION,
    MAX_FILES_DEBUG
};
static uint32_t id_debug[MAX_FILES_DEBUG];

enum
{
    FILE_PORT1_SCHEDULER = 0,
    FILE_PORT2_SCHEDULER,
    FILE_PORT3_SCHEDULER,
    FILE_PORT4_SCHEDULER,
    FILE_PORT5_SCHEDULER,
    FILE_PORT6_SCHEDULER,
    FILE_PORT7_SCHEDULER,
    FILE_PORT8_SCHEDULER,
    FILE_PORT9_SCHEDULER,
    FILE_PORT10_SCHEDULER,
    FILE_PORT11_SCHEDULER,
    FILE_PORT12_SCHEDULER,
    FILE_PORT13_SCHEDULER,
    FILE_PORT14_SCHEDULER,
    FILE_PORT15_SCHEDULER,
    FILE_PORT16_SCHEDULER,
    FILE_HOST_SCHEDULER,
    MAX_FILES_SCHEDULER_SETTINGS
};
static uint32_t id_scheduler_settings[MAX_FILES_SCHEDULER_SETTINGS];

enum
{
    FILE_PORT1_SHAPERS = 0,
    FILE_PORT2_SHAPERS,
    FILE_PORT3_SHAPERS,
    FILE_PORT4_SHAPERS,
    FILE_PORT5_SHAPERS,
    FILE_PORT6_SHAPERS,
    FILE_PORT7_SHAPERS,
    FILE_PORT8_SHAPERS,
    FILE_PORT9_SHAPERS,
    FILE_PORT10_SHAPERS,
    FILE_PORT11_SHAPERS,
    FILE_PORT12_SHAPERS,
    FILE_PORT13_SHAPERS,
    FILE_PORT14_SHAPERS,
    FILE_PORT15_SHAPERS,
    FILE_PORT16_SHAPERS,
    FILE_HOST_SHAPERS,
    MAX_FILES_PORT_SHAPERS
};
static uint32_t id_port_shaper[MAX_FILES_PORT_SHAPERS];

enum{
    FILE_DOS_CONTROL = 0,
    MAX_FILES_DOS_CONTROL
};
static uint32_t id_dos_control[MAX_FILES_DOS_CONTROL];

enum
{
    FILE_PVLAN_PORT1 = 0,
    FILE_PVLAN_PORT2,
    FILE_PVLAN_PORT3,
    FILE_PVLAN_PORT4,
    FILE_PVLAN_PORT5,
    FILE_PVLAN_PORT6,
    FILE_PVLAN_PORT7,
    FILE_PVLAN_PORT8,
    FILE_PVLAN_PORT9,
    FILE_PVLAN_PORT10,
    FILE_PVLAN_PORT11,
    FILE_PVLAN_PORT12,
    FILE_PVLAN_PORT13,
    FILE_PVLAN_PORT14,
    FILE_PVLAN_PORT15,
    FILE_PVLAN_PORT16,
    FILE_PVLAN_HOST_PORT,
    MAX_FILE_PVLAN_PORT
};
static uint32_t id_port_pvlan[MAX_FILE_PVLAN_PORT];

enum
{
    FILE_PCKS_TO_QUEUES_PORT1 = 0,
    FILE_PCKS_TO_QUEUES_PORT2,
    FILE_PCKS_TO_QUEUES_PORT3,
    FILE_PCKS_TO_QUEUES_PORT4,
    FILE_PCKS_TO_QUEUES_PORT5,
    FILE_PCKS_TO_QUEUES_PORT6,
    FILE_PCKS_TO_QUEUES_PORT7,
    FILE_PCKS_TO_QUEUES_PORT8,
    FILE_PCKS_TO_QUEUES_PORT9,
    FILE_PCKS_TO_QUEUES_PORT10,
    FILE_PCKS_TO_QUEUES_PORT11,
    FILE_PCKS_TO_QUEUES_PORT12,
    FILE_PCKS_TO_QUEUES_PORT13,
    FILE_PCKS_TO_QUEUES_PORT14,
    FILE_PCKS_TO_QUEUES_PORT15,
    FILE_PCKS_TO_QUEUES_PORT16,
    FILE_PCKS_TO_QUEUES_HOST_PORT,
    FILE_PCKS_TO_QUEUES_GLOBAL_DSCP_2_TC,
    MAX_FILE_PCKS_TO_QUEUES
};
static uint32_t id_pcks_to_queues[MAX_FILE_PCKS_TO_QUEUES];

enum
{
    FILE_FLOW_CONTROL_PORT1 = 0,
    FILE_FLOW_CONTROL_PORT2,
    FILE_FLOW_CONTROL_PORT3,
    FILE_FLOW_CONTROL_PORT4,
    FILE_FLOW_CONTROL_PORT5,
    FILE_FLOW_CONTROL_PORT6,
    FILE_FLOW_CONTROL_PORT7,
    FILE_FLOW_CONTROL_PORT8,
    FILE_FLOW_CONTROL_PORT9,
    FILE_FLOW_CONTROL_PORT10,
    FILE_FLOW_CONTROL_PORT11,
    FILE_FLOW_CONTROL_PORT12,
    FILE_FLOW_CONTROL_PORT13,
    FILE_FLOW_CONTROL_PORT14,
    FILE_FLOW_CONTROL_PORT15,
    FILE_FLOW_CONTROL_PORT16,
    MAX_FILE_FLOW_CONTROL
};
static uint32_t id_flow_control[MAX_FILE_FLOW_CONTROL];

enum
{
    FILE_ETH_PHY_SETTINGS_PORT1 = 0,
    FILE_ETH_PHY_SETTINGS_PORT2,
    FILE_ETH_PHY_SETTINGS_PORT3,
    FILE_ETH_PHY_SETTINGS_PORT4,
    FILE_ETH_PHY_SETTINGS_PORT5,
    FILE_ETH_PHY_SETTINGS_PORT6,
    FILE_ETH_PHY_SETTINGS_PORT7,
    FILE_ETH_PHY_SETTINGS_PORT8,
    FILE_ETH_PHY_SETTINGS_PORT9,
    FILE_ETH_PHY_SETTINGS_PORT10,
    FILE_ETH_PHY_SETTINGS_PORT11,
    FILE_ETH_PHY_SETTINGS_PORT12,
    FILE_ETH_PHY_SETTINGS_PORT13,
    FILE_ETH_PHY_SETTINGS_PORT14,
    FILE_ETH_PHY_SETTINGS_PORT15,
    FILE_ETH_PHY_SETTINGS_PORT16,
    MAX_FILE_ETH_PHY_SETTINGS_PORT
};
static uint32_t id_eth_phy_settings_port[MAX_FILE_ETH_PHY_SETTINGS_PORT];

enum
{
    FILE_OPTIC_MODULE_PORT1 = 0,
    FILE_OPTIC_MODULE_PORT2,
    FILE_OPTIC_MODULE_PORT3,
    FILE_OPTIC_MODULE_PORT4,
    MAX_FILE_OPTIC_MODULE_PORT
};
static uint32_t id_optic_module_port[ MAX_FILE_OPTIC_MODULE_PORT ];

static struct proc_dir_entry *basis_dir;
static struct proc_dir_entry *mem_dir;
static struct proc_dir_entry *statistics_dir;
static struct proc_dir_entry *port_settings_dir;
static struct proc_dir_entry *hash_tables_dir;
static struct proc_dir_entry *mac_dir, *vlan_dir;
static struct proc_dir_entry *queue_settings_dir;
static struct proc_dir_entry *debug_dir;
static struct proc_dir_entry *scheduler_dir;
static struct proc_dir_entry *shaper_dir;
static struct proc_dir_entry *dos_dir;
static struct proc_dir_entry *private_vlan_dir;
static struct proc_dir_entry *pcks_to_queues_dir;
static struct proc_dir_entry *flow_control_dir;
static struct proc_dir_entry *eth_phy_settings_port_dir;
static struct proc_dir_entry *optic_module_dir;

static uint32_t regAddr = BMU1_BASE_ADDR;
static uint8_t  queue_number[MAX_FILES_QUEUE_SETTINGS];
static uint8_t  shaper_number[MAX_FILES_PORT_SHAPERS];
static uint32_t  debug_phy_test_mode[FP_MAX_SWITCH_PORTS];
static uint8_t searchMac[6];
static uint32_t searchVlanIdMac, searchVlanIdVlan;
static uint32_t phy_far_loopback;

const unsigned char act[8][20] = {"ACT_FORWARD", "ACT_FLOOD", "ACT_PUNT", "ACT_DISCARD", "ACT_OVERRIDE", "ACT_FWD_MASK", "ACT_COS_DISCARD", "UNKNOWN"};

extern uint8_t fp_dbgp_arr[FP_MAX_DBGP][FP_MAX_DBGP_LEN];
extern uint32_t fp_dbgp_flags;
extern uint32_t fp_trace_level;

extern uint32_t Qposition[TLITE_SWITCH_MAX_DEV_NUM][TLITE_MAX_SCHED_NUM][TLITE_MAX_QUE_NO];
extern uint8_t control_block_injection_flag_values[ CB_INJ_TX_FLAG_VALUES_LEN ];

struct mutex mutexProcTMU, mutexProcMem, mutexProcStat, mutexProcHash, mutexProcClass, mutexProcEMAC;

int32_t update_class_tmu_threshold( uint32_t action, uint32_t jumbo_size_in_bytes ){
    uint32_t new_threshold;

    new_threshold = INQ_ALMOST_FULL_THRESHOLD;
    if( action ){
        //enable jumbo
        // 1 + ( ( jumbo_size_in_bytes - 64 ) + 111 ) / 112 <--- number of buffers to allocate 1 jumbo packets
        new_threshold = MAX_BMU_BUFFERS / ( 1 + ( ( jumbo_size_in_bytes - 64 ) + 111 ) / 112 );     
    }
    fp_reg_write(  CLASS_HW_BASE_ADDR + CLASS_ING_AFULL_THRES, new_threshold );
    fp_reg_write( CLASS2_HW_BASE_ADDR + CLASS_ING_AFULL_THRES, new_threshold );
    fp_reg_write( TMU_BASE_ADDR + TMU_INQ_WATERMARK, new_threshold );
    fp_reg_write( TMU_BASE_ADDR + TMU_AFULL_THRES, new_threshold );
    FP_DEBUG( DBGP_FEAT_HIF, FP_LOG_INFO, "%s: class, tmu new thresholds: %d\n", __func__, new_threshold );    
    return 0;
}


static int temperature_read( struct seq_file *f, void *v ) {
    fp_dev_info_t *logical_priv;
    fp_private_t *real_priv;
    
    if( fp_logical_dev[0] == NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, the device doesn't OPEN!\n", __func__ );
        return 0;
    }

    logical_priv = netdev_priv( fp_logical_dev[ 0 ] );
    real_priv  = netdev_priv( logical_priv->real_dev );

    seq_printf( f, "Temperature:\n" );
    if( real_priv->switch_stats.basisTemp != 0xFFFFFFFF ){
        seq_printf( f, "  Basis: %d\n", real_priv->switch_stats.basisTemp );
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: basis: %d\n", __func__, real_priv->switch_stats.basisTemp );
    }else{
        seq_printf( f, "  Basis: UNAVAILABLE\n" );
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: basis: unavailable\n", __func__ );
    }
    if( real_priv->switch_stats.boardTemp != 0xFFFFFFFF ){
        seq_printf( f, "  Board: %d\n", real_priv->switch_stats.boardTemp );
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: board: %d\n", __func__, real_priv->switch_stats.boardTemp );
    }else{
        seq_printf( f, "  Board: UNAVAILABLE\n" );
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: board: unavailable\n", __func__ );
    }
    return 0;
}

static int temperature_open( struct inode *inode, struct file *file ){
    return single_open( file, temperature_read, NULL );
}

static const struct file_operations temperature_fops = {
    .owner = THIS_MODULE,
    .open  = temperature_open,
    .read  = seq_read,
    .llseek = seq_lseek,
    .release = single_release,
};

static int mem_read( struct seq_file *f, void *v ) {
    uint32_t regData;

    mutex_lock( &mutexProcMem );
    regData = fp_reg_read( regAddr );
    seq_printf( f, "0x%06x = 0x%08x\n", regAddr, regData );
    mutex_unlock( &mutexProcMem );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: 0x%06x=0x%08x\n", __func__, regAddr, regData );

    return 0;
}

static ssize_t mem_write( struct file *file, const char *buf,
                           size_t count, loff_t *ppos ) {
    int res, regData;
    uint8_t *buff;
    size_t result;

    result = 0;
    mutex_lock( &mutexProcMem );
    buff = FP_NULL;
    buff = kmalloc( PROC_BUF_SIZE, GFP_KERNEL );
    if( buff == FP_NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "MEM_WRITE: can not allocate memory for buffer\n" );
        result = -ENOMEM;
        goto exit_mem_write;
    }
    if( count > PROC_BUF_SIZE ){
        count = PROC_BUF_SIZE;
    }

    res = copy_from_user( buff, ( void* )buf, count );
    if( res ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "MEM_WRITE: can not copy data from user\n" );
        result = -EFAULT;
        goto exit_mem_write;
    }

    buff[ count - 1 ] = '\0';
    result = count;

    res = sscanf( buff, "%x = %x",  &regAddr, &regData );
    if( res == 1 ){
        regAddr &= ~3;
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "MEM_WRITE: update address\n" );
    }else if( res == 2 ){
        regAddr &= ~3;
        result = count;
        fp_reg_write( regAddr, regData );
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "MEM_WRITE: done\n" );
    }else{
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "MEM_WRITE: can not get data from buffer\n" );
    }

exit_mem_write:
    if( buff != FP_NULL ){
        kfree( buff );
    }
    mutex_unlock( &mutexProcMem );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "MEM_WRITE: end\n" );

    return result;
}

static int mem_open( struct inode *inode, struct file *file ){
    return single_open( file, mem_read, NULL );
}

static const struct file_operations mem_fops = {
    .owner = THIS_MODULE,
    .open  = mem_open,
    .read  = seq_read,
    .llseek = seq_lseek,
    .release = single_release,
    .write = mem_write,
};

/* ------------------------------------------- Statistics --------------------------------------- */
static int stat_read( struct seq_file *f, void *v ) {
    fp_dev_info_t *logical_priv;
    int file_id;

    if( fp_logical_dev[0] == NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, the device doesn't OPEN!\n", __func__ );
        return 0;
    }

    file_id = *( uint32_t* )PDE_DATA( file_inode( f->file ) );
    if( file_id >= MAX_FILES_STAT){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, bad file id!\n", __func__ );
        return 0;
    }

    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: read from file id %d\n", __func__, file_id );

    mutex_lock( &mutexProcStat );
    if( file_id <= FILE_STAT_EM16 ){
        logical_priv = netdev_priv( fp_logical_dev[ file_id ] );

        seq_printf( f, "Statistic for EMAC%d\n", ( file_id + 1 ) );
        seq_printf(   f, "  TX statistic\n" );
        seq_printf( f, "    OK: %lld\n", logical_priv->phy_stats.tx_ok );
        seq_printf( f, "    Broadcast: %lld\n", logical_priv->phy_stats.tx_broadcast );
        seq_printf( f, "    Multicast: %lld\n", logical_priv->phy_stats.tx_multicast );
        seq_printf( f, "    Pause: %lld\n", logical_priv->phy_stats.tx_pause );
        seq_printf( f, "    64 bytes packets: %lld\n", logical_priv->phy_stats.tx_64 );
        seq_printf( f, "    65-127 bytes packets: %lld\n", logical_priv->phy_stats.tx_65_127 );
        seq_printf( f, "    128-255 bytes packets: %lld\n", logical_priv->phy_stats.tx_128_255 );
        seq_printf( f, "    256-511 bytes packets: %lld\n", logical_priv->phy_stats.tx_256_511 );
        seq_printf( f, "    512-1023 bytes packets: %lld\n", logical_priv->phy_stats.tx_512_1023 );
        seq_printf( f, "    1024-1518 bytes packets: %lld\n", logical_priv->phy_stats.tx_1024_1518 );
        seq_printf( f, "    >1518 bytes packets: %lld\n", logical_priv->phy_stats.tx_more_1518 );
        seq_printf( f, "    underruns: %lld\n", logical_priv->phy_stats.tx_underruns );
        seq_printf( f, "    single collision: %lld\n", logical_priv->phy_stats.tx_single_collision );
        seq_printf( f, "    2-15 collisions: %lld\n", logical_priv->phy_stats.tx_2_15_collision );
        seq_printf( f, "    >15 collisions: %lld\n", logical_priv->phy_stats.tx_more_15_collision );
        seq_printf( f, "    late collision: %lld\n", logical_priv->phy_stats.tx_late_collision );
        seq_printf( f, "    deferred: %lld\n", logical_priv->phy_stats.tx_deferred );
        seq_printf( f, "  RX statistic\n" );
        seq_printf( f, "    OK: %lld\n", logical_priv->phy_stats.rx_ok );
        seq_printf( f, "    Broadcast: %lld\n", logical_priv->phy_stats.rx_broadcast );
        seq_printf( f, "    Multicast: %lld\n", logical_priv->phy_stats.rx_multicast );
        seq_printf( f, "    Pause: %lld\n", logical_priv->phy_stats.rx_pause );
        seq_printf( f, "    <64 bytes packets: %lld\n", logical_priv->phy_stats.rx_less_64 );
        seq_printf( f, "    64 bytes packets: %lld\n", logical_priv->phy_stats.rx_64 );
        seq_printf( f, "    65-127 bytes packets: %lld\n", logical_priv->phy_stats.rx_65_127 );
        seq_printf( f, "    128-255 bytes packets: %lld\n", logical_priv->phy_stats.rx_128_255 );
        seq_printf( f, "    256-511 bytes packets: %lld\n", logical_priv->phy_stats.rx_256_511 );
        seq_printf( f, "    512-1023 bytes packets: %lld\n", logical_priv->phy_stats.rx_512_1023 );
        seq_printf( f, "    1024-1518 bytes packets: %lld\n", logical_priv->phy_stats.rx_1024_1518 );
        seq_printf( f, "    >1518 bytes packets: %lld\n", logical_priv->phy_stats.rx_more_1518 );
        seq_printf( f, "    excessive lenght: %lld\n", logical_priv->phy_stats.rx_excessive_len );
        seq_printf( f, "    jabbers: %lld\n", logical_priv->phy_stats.rx_jabbers );
        seq_printf( f, "    fcs errors: %lld\n", logical_priv->phy_stats.rx_fcs_err );
        seq_printf( f, "    lenght errors: %lld\n", logical_priv->phy_stats.rx_len_err );
        seq_printf( f, "    symbol errors: %lld\n", logical_priv->phy_stats.rx_symbol_err );
        seq_printf( f, "    aligment errors: %lld\n", logical_priv->phy_stats.rx_aligment_err );
        seq_printf( f, "    recource errors: %lld\n", logical_priv->phy_stats.rx_recource_err );
        seq_printf( f, "    overruns: %lld\n", logical_priv->phy_stats.rx_overruns );
        seq_printf( f, "    IP CRC errors: %lld\n", logical_priv->phy_stats.rx_ip_crc_err );
        seq_printf( f, "    TCP CRC errors: %lld\n", logical_priv->phy_stats.rx_tcp_crc_err );
        seq_printf( f, "    UDP CRC errors: %lld\n", logical_priv->phy_stats.rx_udp_crc_err );
        seq_printf( f, "    dropped: %ld\n", logical_priv->dev_stats.rx_dropped );
        seq_printf( f, "  Classificator statistic\n" );
        seq_printf( f, "    RX: %lld\n", logical_priv->phy_stats.class_rx );
        seq_printf( f, "    L3 fail: %lld\n", logical_priv->phy_stats.class_l3_fail );
        seq_printf( f, "    IPV4: %lld\n", logical_priv->phy_stats.class_ipv4 );
        seq_printf( f, "    IPV6: %lld\n", logical_priv->phy_stats.class_ipv6 );
        seq_printf( f, "    CRC errors: %lld\n", logical_priv->phy_stats.class_crc_err );
        seq_printf( f, "    TTL errors: %lld\n", logical_priv->phy_stats.class_ttl_err );
        seq_printf( f, "    ICMP: %lld\n", logical_priv->phy_stats.class_icmp );
        seq_printf( f, "    IGMP: %lld\n", logical_priv->phy_stats.class_igmp );
        seq_printf( f, "    TCP: %lld\n", logical_priv->phy_stats.class_tcp );
        seq_printf( f, "    UDP: %lld\n", logical_priv->phy_stats.class_udp );
        seq_printf( f, "  Bytes statistic\n" );
        seq_printf( f, "    TX: %ld\n", logical_priv->dev_stats.tx_bytes );
        seq_printf( f, "    RX: %ld\n", logical_priv->dev_stats.rx_bytes );
    }else if( file_id == FILE_STAT_SWITCH ){
        seq_printf( f, "Statistic for switch:\n" );
        seq_printf( f, "  fwd cnt = %d\n", fp_rc_stats.fwd_cnt );
        seq_printf( f, "  hash entry cnt = %d\n", fp_rc_stats.hw_hash_entry_count );
        seq_printf( f, "  sa miss cnt = %d\n", fp_rc_stats.sa_miss_cnt );
        seq_printf( f, "  sa relearn cnt = %d\n", fp_rc_stats.sa_relearn_cnt );
        seq_printf( f, "  sa is active cnt = %d\n", fp_rc_stats.sa_is_active_cnt );
        seq_printf( f, "  sa request cnt = %d\n", fp_rc_stats.requested_cnt );
        seq_printf( f, "  sa igmp cnt = %d\n", fp_rc_stats.igmp_cnt );
        seq_printf( f, "  sa flood cnt = %d\n", fp_rc_stats.flood_cnt );
        seq_printf( f, "  sa parse cnt = %d\n", fp_rc_stats.parse_cnt );
    }
    mutex_unlock( &mutexProcStat );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: read %s statistics complete\n", __func__,
                ( file_id == FILE_STAT_SWITCH )?"SWITCH":"EMAC" );
    return 0;
}

/* Write to statistic mean reset counters */
static ssize_t stat_write( struct file *file, const char *buf,
                           size_t count, loff_t *ppos ) {
    int file_id;
    fp_dev_info_t *logical_priv;

    file_id = *( uint32_t* )PDE_DATA( file_inode( file ) );

    if( fp_logical_dev[ 0 ] == NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, the device doesn't OPEN!\n", __func__ );
        return 0;
    }

    if( file_id >= MAX_FILES_STAT ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, bad file id!\n", __func__ );
        return 0;
    }

    if( file_id <= FILE_STAT_EM16 ){
        mutex_lock( &mutexProcStat );
        logical_priv = netdev_priv( fp_logical_dev[ file_id ] );
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: reset EMAC%d statistics\n", __func__, ( file_id + 1 ) );
        memset( &logical_priv->dev_stats, 0, sizeof( struct net_device_stats ) );
        memset( &logical_priv->phy_stats, 0, sizeof( struct phy_statistic ) );
        mutex_unlock( &mutexProcStat );
        return count;
    }
    return count;
}

static int statistics_open( struct inode *inode, struct file *file ){
    return single_open( file, stat_read, NULL );
}

static const struct file_operations stat_fops = {
    .owner = THIS_MODULE,
    .open  = statistics_open,
    .read  = seq_read,
    .llseek = seq_lseek,
    .release = single_release,
    .write = stat_write,
};

static int hash_read( struct seq_file *f, void *v ){
    
    int32_t res, file_id, cnt;
    int64_t llres;
    uint32_t params[ 2 ];

    if( fp_logical_dev[0] == FP_NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, the device doesn't OPEN!\n", __func__ );
        return 0;
    }

    file_id = *( uint32_t* )PDE_DATA( file_inode( f->file ) );
    if( ( file_id >= MAX_FILES_HASH )||
        ( file_id == FILE_HASH_MAC_SHOW_TABLE )||
        ( file_id == FILE_HASH_VLAN_SHOW_TABLE ) ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, bad file id %d!\n", __func__, file_id );
        return 0;
    }

    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: start\n", __func__ );

    mutex_lock( &mutexProcHash );
    switch ( file_id ){
        case FILE_HASH_MAC_ADD:
            seq_printf( f,  "MAC:     00:23:45:67:89:AB\n"
                            "VLAN ID: 1\n"
                            "Port:    1\n"
                            "Action:  0\n"
                            "Static:  0\n"
                            "Tc:   0\n" 
                            "----------------------------------------------------------\n"
                            "MAC: mac address to add in MAC table (range: 00:00:00:00:00:00 - FF:FF:FF:FF:FF:FF)\n"
                            "VLAN ID: number of vlan to specify in first string mac address (range: 1 - 4094)\n"
                            "Port: for static MAC entry Port field is bit mask (for example: 0x1)\n"
                            "      for dynamic MAC entry Port field is number of port (for example: 2)\n"
                            "Action: action with packets (range: 0 - Forward; 1 - Flood; 2 - Punt; 3 - Discard; 4 - Override; 5 - Fwd Mask; 6 - COS discard)\n"
                            "Static: 0 - dynamic entry, 1 - static entry\n"
                            "Tc: traffic lable to receive packets (range: 0 - 7)\n" );
        break;
        case FILE_HASH_MAC_SEARCH:
            seq_printf( f, "MAC:     %02x:%02x:%02x:%02x:%02x:%02x\nVLAN ID: %d\n",
                            searchMac[0], searchMac[1],
                            searchMac[2], searchMac[3],
                            searchMac[4], searchMac[5],
                            searchVlanIdMac );

            res = fp_mac_entry_search( searchMac, searchVlanIdMac );
            if( res == -1 ){
                seq_printf( f, "\nResult:  Not found\n" );
            }else{
                seq_printf( f, "\nResult:  Found\n\n" );
                if( res & ( 1 << 29 ) ){
                    seq_printf( f, "Fresh bit: Set\n" );
                }else{
                    seq_printf( f, "Fresh bit: Reset\n" );
                }
                if( res & ( 1 << 30 ) ){
                    seq_printf( f, "Type:      Static\n" );
                }else{
                    seq_printf( f, "Type:      Dynamic\n" );
                }
                seq_printf( f, "Action:    %s\n", act[ ( res >> 23 ) & 7 ] );
                if( res & ( 1 << 30 ) ){
                    seq_printf( f, "Ports list:      0x%X\n\n", res & 0x0001FFFF );
                }else{
                    cnt = 1;
                    res &= 0x0001FFFF;
                    while( ( res & 1 ) != 1 ){
                        res >>= 1;
                        cnt++;
                    }
                    seq_printf( f, "Port:      %d\n", cnt );
                }
            }
        break;
        case FILE_HASH_MAC_DELETE:
            seq_printf( f, "MAC:     11:22:33:44:55:66\nVLAN ID: 1\n" );
        break;
        case FILE_HASH_MAC_CLEAN_TABLE:
            seq_printf( f, "Write anything in file to clean all entries\n" );
        break;
        case FILE_HASH_VLAN_ADD:
            seq_printf( f, "VLAN ID:       1\n"
                         "Forward list:  0x00000\n"
                         "Untag list:    0x00000\n"
                         "UnicastHit:    0\n"
                         "UnicastMiss:   0\n"
                         "MulticastHit:  0\n"
                         "MulticastMiss: 0\n"
                         "Mstp:          0\n"
                         "----------------------------------------------------------\n"
                         "VLAN ID - number of vlan in dec format\n"
                         "Forward list - bit mask of ports in this VLAN domain\n"
                         "Untag list - bit mask of ports to remove the vlan tag from packet at egress\n"
                         "Fields UnicastHit, UnicastMiss, ... is actions\n"
                         "Available actions:\n"
                         "  0 - Forward\n"
                         "  2 - Punt\n"
                         "  3 - Discard\n"
                         "  6 - COS discard\n"
                         "  1, 4, 5 - same as Forward\n" );
        break;
        case FILE_HASH_VLAN_SEARCH:
            seq_printf( f, "VLAN ID: %d\n", searchVlanIdVlan );
            llres = wsp_vlan_entry_search( searchVlanIdVlan );
            if( llres != -1 ){
                seq_printf( f, "Forward list:  0x%05X\n", ( uint32_t )( llres & 0xFFFFF ) );
                seq_printf( f, "Untag list:    0x%05X\n", ( uint32_t )( ( ( llres >> 20 ) & 0xFFFFF ) ) );
                seq_printf( f, "UnicastHit:    %s\n", act[ ( llres >> 40 ) & 7 ] );
                seq_printf( f, "UnicastMiss:   %s\n", act[ ( llres >> 46 ) & 7 ] );
                seq_printf( f, "MulticastHit:  %s\n", act[ ( llres >> 43 ) & 7 ] );
                seq_printf( f, "MulticastMiss: %s\n", act[ ( llres >> 49 ) & 7 ] );
                seq_printf( f, "Mstp:          %s\n", act[ ( llres >> 52 ) & 7 ] );
            }else{
                seq_printf( f, "Not found\n" );
            }
        break;
        case FILE_HASH_VLAN_DELETE:
            seq_printf( f, "VLAN ID: 1\n" );
        break;
        case FILE_HASH_VLAN_CLEAN_TABLE:
            seq_printf( f, "Write anything in file to clean all entries\n" );
        break;
        case FILE_HASH_VLAN_GLOBAL_VLAN_ENTRY:
            params[ 0 ] = fp_reg_read( ( CLASS_HW_BASE_ADDR + CLASS_GLOBAL_CFG ) );   //get class_global_cfg
            params[ 1 ] = fp_reg_read( ( CLASS_HW_BASE_ADDR + CLASS_GLOBAL_CFG1 ) );   //get class_global_cfg1       
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: CLASS_GLOBAL_CFG  0x%08X\n", __func__, params[ 0 ] );
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: CLASS_GLOBAL_CFG1 0x%08X\n", __func__, params[ 1 ] );
            seq_printf( f, "Forward list:  0x%05X\n"
                           "Untag list:    0x%05X\n"
                           "UnicastHit:    %d\n"
                           //"UnicastHit:    %s\n"
                           "UnicastMiss:   %d\n"
                           //"UnicastMiss:   %s\n"
                           "MulticastHit:  %d\n"
                           //"MulticastHit:  %s\n"
                           "MulticastMiss: %d\n"
                           //"MulticastMiss: %s\n"
                           "Mstp:          %d\n"
                           //"Mstp:          %s\n"
                           "L2_spec_punt_enable: %d\n"
                           "----------------------------------------------------------\n"
                           "L2_spec_punt_enable - enable or disable punt to the HOST all special L2 packets (1 - enabel, 0 - disable)\n"
                           "Available actions:\n"
                           "  0 - Forward\n"
                           "  2 - Punt\n"
                           "  3 - Discard\n"
                           "  6 - COS discard\n"
                           "  1, 4, 5 - same as Forward\n",
                           ( ( params[ 0 ] >> 1 ) & 0x1FFFF ),
                           ( ( ( params[ 0 ] >> 21 ) & 0x7FF ) | ( ( params[ 1 ] & 0x1FF) << 11 ) ), 
                           //act[ ( ( params[ 1 ] >> 9 ) & 7 ) ],
                           //act[ ( ( params[ 1 ] >> 15 ) & 7 ) ],
                           //act[ ( ( params[ 1 ] >> 12 ) & 7 ) ],
                           //act[ ( ( params[ 1 ] >> 18 ) & 7 ) ],
                           //act[ ( ( params[ 1 ] >> 21 ) & 7 ) ],
                           ( ( params[ 1 ] >> 9 ) & 7 ),
                           ( ( params[ 1 ] >> 15 ) & 7 ),
                           ( ( params[ 1 ] >> 12 ) & 7 ),
                           ( ( params[ 1 ] >> 18 ) & 7 ),
                           ( ( params[ 1 ] >> 21 ) & 7 ),
                           ( params[ 0 ] & 1 ) );
        break;
    }
    mutex_unlock( &mutexProcHash );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: end\n", __func__ );
    return 0;
}

/* ----------------------------------------- Hash tables ---------------------------------------- */
static ssize_t hash_write( struct file *file, const char *buf,
                           size_t count, loff_t *ppos ) {
    int32_t res, file_id;
    uint8_t *buff;
    uint32_t *params;
    uint8_t *hash_write_buffer;
    size_t result = count;

    if( fp_logical_dev[ 0 ] == NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "HASH_WRITE: the device doesn't OPEN!\n" );
        goto hash_write_exit;
    }

    file_id = *( uint32_t* )PDE_DATA( file_inode( file ) );

    if( file_id >= MAX_FILES_HASH ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "HASH_WRITE: bad file id!\n" );
        goto hash_write_exit;
    }

    mutex_lock( &mutexProcHash );
    buff = FP_NULL;
    params = FP_NULL;
    hash_write_buffer = FP_NULL;

    hash_write_buffer = kmalloc( PROC_BUF_SIZE, GFP_KERNEL );
    if( hash_write_buffer == FP_NULL ){
        FP_DEBUG(DBGP_FEAT_PROCFS, FP_LOG_ERR, "HASH_WRITE: can not allocate memory for user data\n" );        
        goto hash_write_clean_and_exit;
    }

    if( count > PROC_BUF_SIZE ){
        count = PROC_BUF_SIZE;
    }

    res = copy_from_user( hash_write_buffer, (void*)buf, count );
    if( res ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "HASH_WRITE: can not get data from user\n" );
        goto hash_write_clean_and_exit;
    }
    result = count;

    switch ( file_id ){
         /* Add static record add mode */
        case FILE_HASH_MAC_ADD:
            buff = kmalloc( ( sizeof( uint8_t ) * 6 ), GFP_KERNEL );
            if( buff == FP_NULL ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "HASH_WRITE: can not allocate memory for mac, FILE_HASH_MAC_ADD!\n" );
                goto hash_write_clean_and_exit;
            }

            params = kmalloc( ( sizeof( uint32_t ) * 6 ), GFP_KERNEL );
            if( params == FP_NULL ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "HASH_WRITE: can not allocate memory for params, FILE_HASH_MAC_ADD!\n" );
                goto hash_write_clean_and_exit;
            }

            res = 0;
            params[ 5 ] = 0;
            while( res == 0 ){
                res = sscanf( &hash_write_buffer[ params[ 5 ] ], "Static:  %d", &params[ 3 ] );
                if( res != 1 ){
                    res = 0;
                    while(  ( hash_write_buffer[ params[ 5 ] ] != '\n' ) &&
                            ( params[ 5 ] < count ) ){
                        params[ 5 ]++;
                    }
                    if( params[ 5 ] == count ){
                        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "HASH_WRITE: bad file format for MAC ADD!\n" );
                        break;
                    }
                    params[ 5 ]++;
                    continue;
                }
                res = 1;
                params[ 3 ] &= 1;
            }

            if( !res ){
                break;
            }

            if( params[ 3 ] == 0 ){ 
                //add dynamic entry
                res = sscanf( hash_write_buffer, "MAC:     %hhx:%hhx:%hhx:%hhx:%hhx:%hhx\n"
                                                 "VLAN ID: %d\n"
                                                 "Port:    %d\n"
                                                 "Action:  %d\n"
                                                 "Static:  %d\n"
                                                 "Tc:      %d",
                                                &buff[ 0 ], &buff[ 1 ], &buff[ 2 ], &buff[ 3 ], &buff[ 4 ], &buff[ 5 ],
                                                &params[ 0 ],
                                                &params[ 1 ],
                                                &params[ 2 ],
                                                &params[ 3 ],
                                                &params[ 4 ] );
            }else{
                //add static entry
                res = sscanf( hash_write_buffer, "MAC:     %hhx:%hhx:%hhx:%hhx:%hhx:%hhx\n"
                                                 "VLAN ID: %d\n"
                                                 "Port:    %x\n"
                                                 "Action:  %d\n"
                                                 "Static:  %d\n"
                                                 "Tc:      %d",
                                                &buff[ 0 ], &buff[ 1 ], &buff[ 2 ], &buff[ 3 ], &buff[ 4 ], &buff[ 5 ],
                                                &params[ 0 ],
                                                &params[ 1 ],
                                                &params[ 2 ],
                                                &params[ 3 ],
                                                &params[ 4 ] );
            }

            if( res == 11 ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "MAC_ADD mac %hhx:%hhx:%hhx:%hhx:%hhx:%hhx, VLAN ID %d, port %d, action %d, static %d, tc %d\n",
                        buff[ 0 ], buff[ 1 ], buff[ 2 ], buff[ 3 ], buff[ 4 ], buff[ 5 ], params[ 0 ], params[ 1 ], params[ 2 ], params[ 3 ], params[ 4 ] );
        
                params[ 3 ] &= 1; //staticEntry
                params[ 4 ] &= 7; //TC
                params[ 2 ] &= 7; //action
    
                buff[ 0 ] &= 0xFF;
                buff[ 1 ] &= 0xFF;
                buff[ 2 ] &= 0xFF;
                buff[ 3 ] &= 0xFF;
                buff[ 4 ] &= 0xFF;
                buff[ 5 ] &= 0xFF;
        
                if( ( params[ 0 ] > 0 ) && ( params[ 0 ] < 4096 ) ){   //vlan
                    if( params[ 3 ] == 0 ){ //static flag
                        if( ( params[ 1 ] > 0 ) && ( params[ 1 ] < 18 ) ){  //port number
                            fp_mac_entry_add( ( params[ 1 ] - 1 ), buff, params[ 0 ] );
                        }
                    }else{
                        if( ( params[ 1 ] > 0 ) && ( params[ 1 ] <= FWD_PORT_LIST_MASK ) ){  //ports list
                            fp_static_mac_entry_add( buff, params[ 0 ], params[ 1 ], params[ 2 ], params[ 4 ] );
                        }
                    }
                }
            }
        break;
        case FILE_HASH_MAC_SEARCH:

            buff = kmalloc( ( sizeof( uint8_t ) * 6 ), GFP_KERNEL );
            if( buff == FP_NULL ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "HASH_WRITE: can not allocate memory for mac, FILE_HASH_MAC_SEARCH!\n" );
                goto hash_write_clean_and_exit;
            }

            params = kmalloc( sizeof( uint32_t ), GFP_KERNEL );
            if( params == FP_NULL ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "HASH_WRITE: can not allocate memory for params, FILE_HASH_MAC_SEARCH!\n");
                goto hash_write_clean_and_exit;
            }

            res = sscanf( hash_write_buffer, "MAC:     %hhx:%hhx:%hhx:%hhx:%hhx:%hhx\nVLAN ID: %d",
                &buff[ 0 ], &buff[ 1 ], &buff[ 2 ], &buff[ 3 ], &buff[ 4 ], &buff[ 5 ], &params[ 0 ] );
    
            if( res == 7 ){
                searchMac[ 0 ] = buff[ 0 ] & 0xFF;
                searchMac[ 1 ] = buff[ 1 ] & 0xFF;
                searchMac[ 2 ] = buff[ 2 ] & 0xFF;
                searchMac[ 3 ] = buff[ 3 ] & 0xFF;
                searchMac[ 4 ] = buff[ 4 ] & 0xFF;
                searchMac[ 5 ] = buff[ 5 ] & 0xFF;
    
                searchVlanIdMac = params[ 0 ] & 0xFFF;
                result = count;
            }
        break;
        case FILE_HASH_MAC_DELETE:

            buff = kmalloc( ( sizeof( uint8_t ) * 6 ), GFP_KERNEL );
            if( buff == FP_NULL ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "HASH_WRITE: can not allocate memory for mac, FILE_HASH_MAC_DELETE!\n");
                goto hash_write_clean_and_exit;
            }

            params = kmalloc( sizeof( uint32_t ), GFP_KERNEL );
            if( params == FP_NULL ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "HASH_WRITE: can not allocate memory for params, FILE_HASH_MAC_DELETE!\n");
                goto hash_write_clean_and_exit;
            }

            res = sscanf( hash_write_buffer, "MAC:     %hhx:%hhx:%hhx:%hhx:%hhx:%hhx\nVLAN ID: %d",
                &buff[ 0 ], &buff[ 1 ], &buff[ 2 ], &buff[ 3 ], &buff[ 4 ], &buff[ 5 ], &params[ 0 ] );
    
            if( res == 7 ){
                buff[ 0 ] &= 0xFF;
                buff[ 1 ] &= 0xFF;
                buff[ 2 ] &= 0xFF;
                buff[ 3 ] &= 0xFF;
                buff[ 4 ] &= 0xFF;
                buff[ 5 ] &= 0xFF;
                params[ 0 ] &= 0xFFF;
    
                fp_mac_entry_del( buff, params[ 0 ] );
                result = count;
            }
        break;
        case FILE_HASH_MAC_CLEAN_TABLE:
            hw_hash_table_init( );
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "HASH_WRITE: clean MAC HASH table\n" );
            result = count;
        break;
        case FILE_HASH_VLAN_ADD:
            params = kmalloc( ( sizeof( uint32_t ) * 8 ), GFP_KERNEL );
            if( params == FP_NULL ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "HASH_WRITE: can not allocate memory for params, FILE_HASH_VLAN_ADD!\n");
                goto hash_write_clean_and_exit;
            }

            res = sscanf( hash_write_buffer,    "VLAN ID:       %d\n"\
                                                "Forward list:  %x\n"\
                                                "Untag list:    %x\n"\
                                                "UnicastHit:    %d\n"\
                                                "UnicastMiss:   %d\n"\
                                                "MulticastHit:  %d\n"\
                                                "MulticastMiss: %d\n"\
                                                "Mstp:          %d",
                                                &params[ 0 ],
                                                &params[ 1 ],
                                                &params[ 2 ],
                                                &params[ 3 ],
                                                &params[ 4 ],
                                                &params[ 5 ],
                                                &params[ 6 ],
                                                &params[ 7 ] );
            if( res == 8 ){
                params[ 0 ] &= 0xFFF;
                params[ 1 ] &= 0x1FFFF;
                params[ 2 ] &= 0x1FFFF;
                params[ 3 ] &= 7;
                params[ 4 ] &= 7;
                params[ 5 ] &= 7;
                params[ 6 ] &= 7;
                params[ 7 ] &= 7;
                fp_br_entry_add( params[ 0 ],
                                 params[ 1 ],
                                 params[ 2 ],
                                 params[ 3 ],
                                 params[ 5 ],
                                 params[ 4 ],
                                 params[ 6 ],
                                 params[ 7 ] );
                result = count;
            }
        break;
        case FILE_HASH_VLAN_SEARCH:
            params = kmalloc( sizeof( uint32_t ), GFP_KERNEL );
            if( params == FP_NULL ){
                FP_DEBUG(DBGP_FEAT_PROCFS, FP_LOG_ERR, "HASH_WRITE: can not allocate memory for params, FILE_HASH_VLAN_SEARCH!\n");
                goto hash_write_clean_and_exit;
            }

            res = sscanf( hash_write_buffer, "VLAN ID: %d\n", &params[ 0 ] );
            if( res == 1 ){
                searchVlanIdVlan = params[ 0 ] & 0xFFF;
                result = count;
            }
        break;
        case FILE_HASH_VLAN_DELETE:
            params = kmalloc( sizeof( uint32_t ), GFP_KERNEL );
            if( params == FP_NULL ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "HASH_WRITE: can not allocate memory for params, FILE_HASH_VLAN_DELETE!\n" );
                goto hash_write_clean_and_exit;
            }
            res = sscanf( hash_write_buffer, "VLAN ID: %d\n", &params[ 0 ] );
            if( res == 1 ){
                params[ 0 ] &= 0xFFF;
                fp_br_entry_del( params[ 0 ] );
                result = count;
            }
        break;
        case FILE_HASH_VLAN_CLEAN_TABLE:
            wsp_vlan_hash_table_init( );
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "HASH_WRITE: clean VLAN HASH table\n" );
            result = count;
        break;
        case FILE_HASH_VLAN_GLOBAL_VLAN_ENTRY:
            params = kmalloc( ( sizeof( uint32_t ) * 8 ), GFP_KERNEL );
            if( params == FP_NULL ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "HASH_WRITE: can not allocate memory for params, FILE_HASH_VLAN_DELETE!\n" );
                goto hash_write_clean_and_exit;
            }
            res = sscanf( hash_write_buffer,    "Forward list:  %x\n"
                                                "Untag list:    %x\n"
                                                "UnicastHit:    %d\n"
                                                "UnicastMiss:   %d\n"
                                                "MulticastHit:  %d\n"
                                                "MulticastMiss: %d\n"
                                                "Mstp:          %d\n"
                                                "L2_spec_punt_enable: %d",
                                            &params[ 0 ],
                                            &params[ 1 ],
                                            &params[ 2 ],
                                            &params[ 3 ],
                                            &params[ 4 ],
                                            &params[ 5 ],
                                            &params[ 6 ],
                                            &params[ 7 ] );
            if(res == 8){
                fp_global.fallback_bd_entry.forward_list = params[ 0 ] & FWD_PORT_LIST_MASK;
                fp_global.fallback_bd_entry.untag_list = params[ 1 ] & FWD_PORT_LIST_MASK;

                /*if( ( params[ 2 ] == 0 ) || ( params[ 2 ] == 2 ) || ( params[ 2 ] == 3 ) ){*/
                    fp_global.fallback_bd_entry.ucast_hit_action = params[ 2 ];    
                //}else{
                //    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "HASH_WRITE: global_vlan_entry, UCastHit bad value, no change in ucast_hit_action of global_vlan_entry!\n");
                //}
                
                //if( ( params[ 3 ] == 0 ) || ( params[ 3 ] == 2 ) || ( params[ 3 ] == 3 ) ){
                    fp_global.fallback_bd_entry.ucast_miss_action = params[ 3 ];    
                //}else{
                //    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "HASH_WRITE: global_vlan_entry, UCastMiss bad value, no change in ucast_miss_action of global_vlan_entry!\n");
                //}

                //if( ( params[ 4 ] == 0 ) || ( params[ 4 ] == 2 ) || ( params[ 4 ] == 3 ) ){
                    fp_global.fallback_bd_entry.mcast_hit_action = params[ 4 ];    
                //}else{
                //    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "HASH_WRITE: global_vlan_entry, MCastHit bad value, no change in mcast_hit_action of global_vlan_entry!\n");
                //}

                //if( ( params[ 5 ] == 0 ) || ( params[ 5 ] == 2 ) || ( params[ 5 ] == 3 ) ){
                    fp_global.fallback_bd_entry.mcast_miss_action = params[ 5 ];    
                //}else{
                //    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "HASH_WRITE: global_vlan_entry, MCastMiss bad value, no change in mcast_miss_action of global_vlan_entry!\n");
                //}

                //if( ( params[ 6 ] == 0 ) || ( params[ 6 ] == 2 ) || ( params[ 6 ] == 3 ) ){
                    fp_global.fallback_bd_entry.mstp_action = params[ 6 ];    
                //}else{
                //    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "HASH_WRITE: global_vlan_entry, MSTP bad value, no change in mstp_action of global_vlan_entry!\n");
                //}

                fp_global.l2_special_punt_enable = params[ 7 ] & 1;
            
                params[ 0 ] = ( fp_global.l2_special_punt_enable ) |
                            ( fp_global.fallback_bd_entry.forward_list << 1 ) |
                            ( fp_global.fallback_bd_entry.untag_list << 21 );
            
                params[ 1 ] = ( ( fp_global.fallback_bd_entry.untag_list >> 11 ) & 0x1FF ) |
                            ( fp_global.fallback_bd_entry.ucast_hit_action << 9 ) |
                            ( fp_global.fallback_bd_entry.mcast_hit_action << 12 ) |
                            ( fp_global.fallback_bd_entry.ucast_miss_action << 15 ) |
                            ( fp_global.fallback_bd_entry.mcast_miss_action << 18 ) |
                            ( fp_global.fallback_bd_entry.mstp_action << 21 );
            
                fp_reg_write( (CLASS_HW_BASE_ADDR + CLASS_GLOBAL_CFG), params[ 0 ]);
                fp_reg_write( (CLASS_HW_BASE_ADDR + CLASS_GLOBAL_CFG1), params[ 1 ]);
            
                fp_reg_write( (CLASS2_HW_BASE_ADDR + CLASS_GLOBAL_CFG), params[ 0 ]);
                fp_reg_write( (CLASS2_HW_BASE_ADDR + CLASS_GLOBAL_CFG1), params[ 1 ]);
            
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "global_vlan_entry_write write CLASS_GLOBAL_CFG  0x%08X\n", params[ 0 ] );
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "global_vlan_entry_write write CLASS_GLOBAL_CFG1 0x%08X\n", params[ 1 ] );
                result = count;
            }else{
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "HASH_WRITE: global_vlan_entry, can not get values from file!\n");
            }
        break;
    }

hash_write_clean_and_exit:
    if( params != FP_NULL ){
        kfree( params );    
    }
    if( buff != FP_NULL ){
        kfree( buff );
    }
    if( hash_write_buffer != FP_NULL ){
        kfree( hash_write_buffer );
    }
    mutex_unlock( &mutexProcHash );
hash_write_exit:
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "HASH_WRITE end\n" );
    return result;
}

static int hash_open( struct inode *inode, struct file *file ){
    return single_open( file, hash_read, NULL );
}

static const struct file_operations hash_fops = {
    .owner = THIS_MODULE,
    .open  = hash_open,
    .read  = seq_read,
    .llseek = seq_lseek,
    .release = single_release,
    .write = hash_write,
};


static int hash_mac_show_read( struct seq_file *f, void *v ){
    
    int32_t file_id;
    uint32_t params[ 4 ];
    uint32_t counter, tmp;
    int entryAddress = 0, entryNumber = 0;

    if( fp_logical_dev[0] == FP_NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, the device doesn't OPEN!\n", __func__ );
        return 0;
    }

    file_id = *( uint32_t* )PDE_DATA( file_inode( f->file ) );
    if( file_id != FILE_HASH_MAC_SHOW_TABLE )  {
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, bad file id %d!\n", __func__, file_id );
        return 0;
    }
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: start\n", __func__ );

    mutex_lock( &mutexProcHash );
    while( entryAddress < 8192 ){
        hw_hash_table_read_entry( params, entryAddress );
        if( params[ REG_MAC4 ] & HASH_ENTRY_VALID_FLAG ) {
            entryNumber++;
            seq_printf( f, "Record %d:\n", entryNumber );
            seq_printf( f, "VLAN ID:   %d\nMAC:       ", ( params[ REG_MAC2 ] >> 16 ) & 0xFFF );
            seq_printf( f, "%02x:", ( params[ REG_MAC1 ] >> 24 ) & 0xFF );
            seq_printf( f, "%02x:", ( params[ REG_MAC1 ] >> 16 ) & 0xFF );
            seq_printf( f, "%02x:", ( params[ REG_MAC1 ] >> 8 ) & 0xFF );
            seq_printf( f, "%02x:", ( params[ REG_MAC1 ] & 0xFF ) );
            seq_printf( f, "%02x:", ( params[ REG_MAC2 ] >> 8 ) & 0xFF );
            seq_printf( f, "%02x\n", ( params[ REG_MAC2 ] & 0xFF) );
            
            if( params[ REG_MAC3 ] & HASH_ENTRY_FRESH_FLAG ){
                seq_printf( f, "Fresh bit: Set\n" );
            }
            else{
                seq_printf( f, "Fresh bit: Reset\n" );
            }
            
            if( params[REG_MAC3] & HASH_ENTRY_STATIC_FLAG ){
                seq_printf( f, "Type:      Static\n" );
            }
            else{
                seq_printf( f, "Type:      Dynamic\n" );
            }

            seq_printf( f, "Action:    %s\n", act[ ( params[ REG_MAC3 ] >> 20) & 7 ] );
            
            tmp = ( ( params[ REG_MAC3 ] << 3 ) | ( params[ REG_MAC2 ] >> 29 ) ) & 0x1FFFF;
            if( params[ REG_MAC3 ] & HASH_ENTRY_STATIC_FLAG ){
                // Static entry - it is fwd_portlist mask (example, for multicast MAC address)!
                seq_printf( f, "Ports list:      0x%X\n\n", tmp );
            }else{
                // Dynamic entry - only 1 bit can be setup in fwd_portlist 
                counter = 1;
                while( ( tmp & 1 ) != 1 ){
                    tmp = tmp >> 1;
                    counter++;
                }
                seq_printf( f, "Port:      %d\n\n", counter );
            }
        }
        entryAddress++;
    }
    mutex_unlock( &mutexProcHash );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: end\n", __func__ );
    return 0;
}

static int hash_mac_show_open( struct inode *inode, struct file *file ){
    return single_open_size( file, hash_mac_show_read, NULL, MAX_SZIE_MAC_TABLE_IN_BYTES );
}

static const struct file_operations hash_mac_show_fops = {
    .owner = THIS_MODULE,
    .open  = hash_mac_show_open,
    .read  = seq_read,
    .llseek = seq_lseek,
    .release = single_release,
};

static int hash_vlan_show_read( struct seq_file *f, void *v ){
    int32_t file_id;
    uint32_t params[ 4 ];
    int entryAddress = 0, entryNumber = 0;

    if( fp_logical_dev[0] == FP_NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, the device doesn't OPEN!\n", __func__ );
        return 0;
    }

    file_id = *( uint32_t* )PDE_DATA( file_inode( f->file ) );
    if( file_id != FILE_HASH_VLAN_SHOW_TABLE ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, bad file id %d!\n", __func__, file_id );
        return 0;
    }
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: start\n", __func__ );            
    
    mutex_lock( &mutexProcHash );
    while( entryAddress < 128 ){
        wsp_read_entry_vlan_hash_table( params, entryAddress );
        if( params[ REG_MAC4 ] & VLAN_TABLE_VLAID_ENTRY_FLAG ){
            entryNumber++;
            seq_printf( f, "Record %d:\n", entryNumber );
            seq_printf( f, "VLAN ID:   %d\n", params[ REG_MAC1 ] & VLANID_MASK );
            seq_printf( f, "Forward list:  0x%05X\n", 
                            ( ( params[ REG_MAC1 ] >> 13 )|( ( params[ REG_MAC2 ] & 1 ) << 19 ) ) & 0xFFFFF );
            seq_printf( f, "Untag list:    0x%05X\n",
                            ( params[ REG_MAC2 ] >> 1 ) & 0xFFFFF );
            seq_printf( f, "UnicastHit:    %s\n",
                            act[ ( params[ REG_MAC2 ] >> 21 ) & 7 ] );
            seq_printf( f, "UnicastMiss:   %s\n", 
                            act[ ( params[ REG_MAC2 ] >> 27 ) & 7 ] );
            seq_printf( f, "MulticastHit:  %s\n",
                            act[ ( params[ REG_MAC2 ] >> 24 ) & 7 ] );
            seq_printf( f, "MulticastMiss: %s\n", 
                            act[ ( ( params[ REG_MAC2 ] >> 30 ) & 3 )|( ( params[ REG_MAC3 ] & 1 ) << 2 ) ] );
            seq_printf( f, "Mstp:          %s\n", 
                            act[ ( params[ REG_MAC3 ] >> 1 ) & 7 ] );
            seq_printf( f, "True field:    0x%02X\n",
                            ( params[ REG_MAC3 ] >> 4 ) & 0xFF );
            seq_printf( f, "Port field:    0x%X\n",
                            ( params[ REG_MAC3 ] >> 12 ) & 0xF );
            seq_printf( f, "Collision ptr: 0x%04X\n",
                            ( params[ REG_MAC3 ] >> 16 ) & 0xFFFF );
            seq_printf( f, "Flags:         0x%X\n\n",
                            params[ REG_MAC4 ] & 0xF );
        }
        entryAddress++;
    }
    mutex_unlock( &mutexProcHash );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: end\n", __func__ );
    return 0;
}

static int hash_vlan_show_open( struct inode *inode, struct file *file ){
    return single_open_size( file, hash_vlan_show_read, NULL, MAX_SZIE_VLAN_TABLE_IN_BYTES );
}

static const struct file_operations hash_vlan_show_fops = {
    .owner = THIS_MODULE,
    .open  = hash_vlan_show_open,
    .read  = seq_read,
    .llseek = seq_lseek,
    .release = single_release,
};

/*--- Ports settings ---*/
static int port_settings_read( struct seq_file *f, void *v ){
    int file_id;
    uint32_t val;

    if( fp_logical_dev[ 0 ] == NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, the device doesn't OPEN!\n", __func__ );
        return 0;
    }

    file_id = *( uint32_t* )PDE_DATA( file_inode( f->file ) );
    if( file_id >= MAX_FILES_SETTINGS_PORT ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, invalid file id %d!\n", __func__, file_id );
        return 0;
    }
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: read from file id %d\n", __func__, file_id );
    
    mutex_lock( &mutexProcClass );
    val = fp_reg_read( hwPortStruct1[ 0 ][ file_id ] );   //get csr_portX_tpid and csr_portX_fallback_bd_id
    seq_printf( f, "TPID: %04X\nFallback_BD_id (dec): %d\n",
                ( val & PORT_TPID_MASK ), 
                ( ( val & PORT_FALLBACK_BDID_MASK ) >> PORT_FALLBACK_BDID_START_POS ) );
    
    val = fp_reg_read( hwPortStruct2[ 0 ][ file_id ] );   //get shutdown, aft, blockstate, def_cfi, def_pri, def_tc, trusted, vid_prefix, untag_from_btable
    seq_printf( f, "shutdown: %d\naft: %d\nblockstate: %d\ndefault_pcp: %d\nuntag_from_btable: %d\n",
                ( ( val & PORT_SHUTDOWN_MASK ) >> PORT_SHUTDOWN_START_POS ),
                ( ( val & PORT_AFT_MASK ) >> PORT_AFT_START_POS ),
                ( ( val & PORT_BLOCKSTATE_MASK ) >> PORT_BLOCKSTATE_START_POS ),
                ( ( val & PORT_DEF_PRI_MASK ) >> PORT_DEF_PRI_START_POS ),
                ( ( val & PORT_UNTAG_FROM_BTABLE_MASK ) >> PORT_UNTAG_FROM_BTABLE_START_POS ) );
    
    seq_printf( f,  "----------------------------------------------------------\n"
                    "TPID - tagged protocol ID (in most case 8100, hex. format, range 0 - FFFF)\n"
                    "Fallback_BD_id - default vlan id for untag packets (dec. format, range 1 - 4094)\n"
                    "shutdown - 0 (port enable); 1 (port disable)\n"
                    "aft - acceptable frame type; 0 (tagged/untagged); 1 (tagged only); 2 (untagged only)\n"
                    "blockstate - 0 (forwarding); 1 (blocked); 2 (learn only)\n"
                    "default_pcp - 0 to 7 (default pcp field value, if VLAN tag is pushing to packet)\n"
                    "untag_from_btable - 1 (use untag list from vlan table entry or global_vlan_entry); 0 (untag list from vlan entry or global_vlan_entry is unused)\n" );

    mutex_unlock( &mutexProcClass );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: read complete\n", __func__ );
    return 0;
}

static ssize_t port_settings_write( struct file *file, const char *buf,
                           size_t count, loff_t *ppos ) {
    int res, file_id;
    size_t result;
    uint8_t *buff;
    uint32_t *params;

    file_id = *( uint32_t* )PDE_DATA( file_inode( file ) );

    if( fp_logical_dev[ 0 ] == NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PORT_SETTINGS_WRITE: the device doesn't OPEN!\n");
        return count;
    }

    if( file_id >= MAX_FILES_SETTINGS_PORT ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PORT_SETTINGS_WRITE: the device doesn't OPEN!\n");
        return count;
    }

    mutex_lock( &mutexProcClass );
    result = 0;
    buff = FP_NULL;
    params = FP_NULL;
    buff = kmalloc( PROC_BUF_SIZE, GFP_KERNEL );
    if( buff == FP_NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PORT_SETTINGS_WRITE: can not allocate memory for user data\n" );
        goto exit_port_settings_write;
    }

    params = kmalloc( ( sizeof( uint32_t ) * 9 ), GFP_KERNEL );
    if( params == FP_NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PORT_SETTINGS_WRITE: can not allocate memory for params\n" );
        goto exit_port_settings_write;
    }

    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "PORT_SETTINGS_WRITE: start\n");

    if( count > PROC_BUF_SIZE ){
        count = PROC_BUF_SIZE;
    }

    res = copy_from_user( buff, (void*)buf, count );
    if( res ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PORT_SETTINGS_WRITE: can not get data from user\n" );
        goto exit_port_settings_write;   
    }

    buff[ count -1 ] = '\0';
    result = count;
    res = sscanf( buff, "TPID: %x\n"
                        "Fallback_BD_id (dec): %d\n"
                        "shutdown: %d\n"
                        "aft: %d\n"
                        "blockstate: %d\n"
                        "default_pcp: %d\n"
                        "untag_from_btable: %d",
                        &params[ 0 ], &params[ 1 ], &params[ 2 ],
                        &params[ 3 ], &params[ 4 ], &params[ 5 ], &params[ 6 ] );

    if( res != 7 ){
        FP_DEBUG(DBGP_FEAT_PROCFS, FP_LOG_ERR, "PORT_SETTINGS_WRITE: can not get values from file!\n");
        result = 0;
        goto exit_port_settings_write;
    }
    result = count;

    params[ 7 ] =   ( ( params[ 0 ] << PORT_TPID_START_POS ) & PORT_TPID_MASK )|
                    ( ( params[ 1 ] << PORT_FALLBACK_BDID_START_POS ) & PORT_FALLBACK_BDID_MASK );  
    params[ 8 ] =   ( ( params[ 2 ] << PORT_SHUTDOWN_START_POS ) & PORT_SHUTDOWN_MASK )|
                    ( ( params[ 3 ] << PORT_AFT_START_POS ) & PORT_AFT_MASK )|
                    ( ( params[ 4 ] << PORT_BLOCKSTATE_START_POS ) & PORT_BLOCKSTATE_MASK )|
                    ( ( params[ 5 ] << PORT_DEF_PRI_START_POS ) & PORT_DEF_PRI_MASK )|
                    ( ( params[ 6 ] << PORT_UNTAG_FROM_BTABLE_START_POS ) & PORT_UNTAG_FROM_BTABLE_MASK );
    
    /*------------------------------------------------------------------------*/
    fp_port[file_id].shutdown = params[ 2 ] & ( PORT_SHUTDOWN_MASK >> PORT_SHUTDOWN_START_POS );
    fp_port[file_id].tpid = params[ 0 ] & ( PORT_TPID_MASK >> PORT_TPID_START_POS );
    fp_port[file_id].aft = params[ 3 ] & ( PORT_AFT_MASK >> PORT_AFT_START_POS );
    fp_port[file_id].fallback_bd_id = params[ 1 ] & ( PORT_FALLBACK_BDID_MASK >> PORT_FALLBACK_BDID_START_POS );
    fp_port[file_id].block_state = params[ 4 ] & ( PORT_BLOCKSTATE_MASK >> PORT_BLOCKSTATE_START_POS );
    fp_port[file_id].def_pri = params[ 5 ] & ( PORT_DEF_PRI_MASK >> PORT_DEF_PRI_START_POS );
    fp_port[file_id].csr_untag_from_btable = params[ 6 ] & ( PORT_UNTAG_FROM_BTABLE_MASK >> PORT_UNTAG_FROM_BTABLE_START_POS );
    
    /*do this for 2 classifiers */
    for( res = 0; res < FP_CLASS_MAX_NUMBER; res++ ){
        fp_reg_write( hwPortStruct1[ res ][ file_id ], params[ 7 ] );
        fp_reg_write( hwPortStruct2[ res ][ file_id ], params[ 8 ] );
    }
    FP_DEBUG(DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "PORT_SETTINGS_WRITE: new settings\n\tport struc 1: 0x%08X\n\tport struc 2: 0x%06X\n", params[ 7 ], params[ 8 ] );

exit_port_settings_write:
    if( buff != FP_NULL ){
        kfree( buff );
    }
    if( params != FP_NULL ){
        kfree( params );
    }
    mutex_unlock( &mutexProcClass );
    FP_DEBUG(DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "PORT_SETTINGS_WRITE: end\n" );
    return result;
}

static int port_settings_open( struct inode *inode, struct file *file ){
    return single_open( file, port_settings_read, NULL );
}

static const struct file_operations port_settings_fops = {
    .owner = THIS_MODULE,
    .open  = port_settings_open,
    .read  = seq_read,
    .llseek = seq_lseek,
    .release = single_release,
    .write = port_settings_write,
};

//static const struct file_operations port_settings_fops = {
//    .owner = THIS_MODULE,
//    .read  = port_settings_read,
//    .write = port_settings_write,
//
//};

/* TMU Settings write*/
static ssize_t port_queues_write( struct file *file, const char *buf,
                           size_t count, loff_t *ppos ) {
    size_t result;
    int file_id, res, index;
    uint32_t add, j, clr_cnt;
    tlite_classq_params_t *pque;
    uint8_t *buff;

    file_id = *( uint32_t* )PDE_DATA( file_inode( file ) );

    if( fp_logical_dev[ 0 ] == NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PORT_QUEUES_WRITE: the device doesn't OPEN!\n" );
        return 0;
    }

    if( file_id >= MAX_FILES_QUEUE_SETTINGS ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PORT_QUEUES_WRITE: bad file id!\n" );
        return 0;
    }

    mutex_lock( &mutexProcTMU );
    result = 0;
    pque = FP_NULL;
    buff = FP_NULL;
    buff = kmalloc( PROC_BUF_SIZE, GFP_KERNEL );
    if( buff == FP_NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PORT_QUEUES_WRITE: can not allocate memory for user data\n" );
        goto exit_port_queues_write;
    }

    pque = kmalloc( sizeof( tlite_classq_params_t ), GFP_KERNEL );
    if( pque == NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PORT_QUEUES_WRITE: can not allocate memory for tlite_classq_params_t structure\n" );
        goto exit_port_queues_write;
    }

    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "PORT_QUEUES_WRITE: start\n" );
    if( count > PROC_BUF_SIZE ){
        count = PROC_BUF_SIZE;
    }

    res = copy_from_user( buff, (void*)buf, count );
    if( res ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PORT_QUEUES_WRITE: can not get data from user\n" );
        goto exit_port_queues_write;   
    }
    buff[ count - 1 ] = '\0';

    index = 0;
    result = count;
    res = sscanf( buff, "Queue add: %d", &add );
    if( res != 1 ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PORT_QUEUES_WRITE: can not get \"Queue add\" value from file!\n" );
        goto exit_port_queues_write;
    }
    add &= 3;

    pque->tlcq_classq_updates = 1;
    pque->tlcq_dev_id = ( file_id + 1 );

    while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ) {}

    res = sscanf( &buff[ index ], "Queue number: %d", &j );
    if( res != 1 ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PORT_QUEUES_WRITE: can not get \"Queue number\" value from file!\n" );
        goto exit_port_queues_write;
    }

    queue_number[ file_id ] = ( j - 1 ) & 7;
    pque->tlcq_classq_id = j;
    clr_cnt = 0;

    if( add == 1 ){
        while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
        res = sscanf( &buff[ index ], "Queue weight: %d", &pque->tlcq_weight );
        if( res != 1 ){
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PORT_QUEUES_WRITE: can not get \"Queue weight\" value from file!\n" );
            goto exit_port_queues_write;
        }
    
        while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
        res = sscanf( &buff[ index ], "Scheduler number: %d", &pque->tlcq_schd_id );
        if( res != 1 ){
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PORT_QUEUES_WRITE: can not get \"Scheduler number\" value from file!\n" );
            goto exit_port_queues_write;
        }
        
        while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
        res = sscanf( &buff[ index ], "Scheduler input: %d", &pque->tlcq_schd_slot );
        if( res != 1 ){
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PORT_QUEUES_WRITE: can not get \"Scheduler input\" value from file!\n" );
            goto exit_port_queues_write;
        }
    
        while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
        res = sscanf( &buff[ index ], "Queue management: %d", &pque->tlcq_qmgmt );
        if( res != 1 ){
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PORT_QUEUES_WRITE: can not get \"Queue management\" value from file!\n" );
            goto exit_port_queues_write;
        }
    
        if( pque->tlcq_qmgmt == TLITE_QUE_TAIL_DROP ){
            while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
            res = sscanf( &buff[ index ], "Max queue size: %d", &pque->tlcq_max_len );
            if( res != 1 ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PORT_QUEUES_WRITE: can not get \"Max queue size\" value from file!\n" );
                goto exit_port_queues_write;
            }

            for( j = 0; j < 3; j++ ){
                index++;                
                while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
            }

            res = sscanf( &buff[ index ], "Clear queue counters: %d", &clr_cnt );
            if( res != 1 ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PORT_QUEUES_WRITE: can not get \"Clear queue counters\" value from file!\n" );
                goto exit_port_queues_write;
            }

        }else if( pque->tlcq_qmgmt == TLITE_QUE_WRED ){
            while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
            res = sscanf( &buff[ index ], "Max queue size: %d", &pque->tlcq_max_len );
            if( res != 1 ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PORT_QUEUES_WRITE: can not get \"Max queue size\" value from file!\n" );
                goto exit_port_queues_write;
            }
    
            while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
            res = sscanf( &buff[ index ], "Min queue size: %d", &pque->tlcq_min_len );
            if( res != 1 ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PORT_QUEUES_WRITE: can not get \"Min queue size\" value from file!\n" );
                goto exit_port_queues_write;
            }
    
            while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
            res = sscanf( &buff[ index ], "Drop probability: %d %d %d %d %d %d %d %d",
                                                        &pque->tlcq_prob_cfg0,
                                                        &pque->tlcq_prob_cfg1,
                                                        &pque->tlcq_prob_cfg2,
                                                        &pque->tlcq_prob_cfg3,
                                                        &pque->tlcq_prob_cfg4,
                                                        &pque->tlcq_prob_cfg5,
                                                        &pque->tlcq_prob_cfg6,
                                                        &pque->tlcq_prob_cfg7 );
            if( res != 8 ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PORT_QUEUES_WRITE: can not get \"Drop probability\" values from file!\n");
                goto exit_port_queues_write;
            }
            
            while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
            res = sscanf( &buff[ index ], "Clear queue counters: %d", &clr_cnt );
            if( res != 1 ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PORT_QUEUES_WRITE: can not get \"Clear queue counters\" value from file!\n" );
                goto exit_port_queues_write;
            }
        }else{
            for( j = 0; j < 4; j++ ){
                index++;
                while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
            }
            res = sscanf( &buff[ index ], "Clear queue counters: %d", &clr_cnt );
            if( res != 1 ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PORT_QUEUES_WRITE: can not get \"Clear queue counters\" value from file!\n" );
                goto exit_port_queues_write;
            }
        }
    }else{
        for( j = 0; j < 8; j++ ){
            index++;
            while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
        }
        res = sscanf( &buff[ index ], "Clear queue counters: %d", &clr_cnt );
        if( res != 1 ){
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PORT_QUEUES_WRITE: can not get \"Clear queue counters\" value from file!\n" );
            goto exit_port_queues_write;
        }
    }

    result = count;

    pfe_tlite_dynamic_config_enable( TLITE_SWITCH );

    if( clr_cnt == 1 ){
        /* Clear queue statistics */
        write_fasw_context_memory( 0, ( ( pque->tlcq_classq_id - 1 ) & 7 ), (pque->tlcq_dev_id - 1), CURQ_TRANS_CNT );
        write_fasw_context_memory( 0, ( ( pque->tlcq_classq_id - 1 ) & 7 ), (pque->tlcq_dev_id - 1), CURQ_DROP_CNT );
        write_fasw_context_memory( 0, ( ( pque->tlcq_classq_id - 1 ) & 7 ), (pque->tlcq_dev_id - 1), CURQ_PKT_CNT );
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "PORT_QUEUES_WRITE: clear port_%d queue_%d counters\n", pque->tlcq_dev_id, pque->tlcq_classq_id );
    }

    if( add == 1 ){
        /*connected queue to scheduler input*/
        pcm_tlite_classq_add( pque, TLITE_SWITCH );
    }else if( add == 2 ){
        /*remove queue at scheduler input*/
        pcm_tlite_classq_delete( pque->tlcq_dev_id, pque->tlcq_classq_id, TLITE_SWITCH );
    }

    pfe_tlite_dynamic_config_done( );

exit_port_queues_write:
    if( buff != FP_NULL ){
        kfree( buff );
    }
    if( pque != FP_NULL ){
        kfree( pque );
    }
    mutex_unlock( &mutexProcTMU );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "PORT_QUEUES_WRITE: end\n" );
    return result;
}

/* TMU Settings read*/
static int port_queues_read( struct seq_file *f, void *v ) {
    int32_t file_id;
    uint32_t arr, que, sch, sch_input;

    if( fp_logical_dev[ 0 ] == NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, the device doesn't OPEN!\n", __func__ );
        return 0;
    }

    file_id = *( uint32_t* )PDE_DATA( file_inode( f->file ) );
    if( file_id >= MAX_FILES_QUEUE_SETTINGS ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, invalid file id %d!\n", __func__, file_id );
        return 0;
    }

    mutex_lock( &mutexProcTMU );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: start\n", __func__ );
    que = queue_number[ file_id ];   //queue number
    sch = fp_tdev_list[ file_id ].classq_list[ que ];

    if( sch ){
        for( sch_input = 0; sch_input < 8; sch_input++ ){
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: sch_input = %d\n", __func__, sch_input );
            if( Qposition[ file_id ][ sch - 1 ][ sch_input ] == ( que + 1 ) ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "PORT_QUEUES_READ: Qposition was found: %d in sch_%d, input %d!\n", Qposition[ file_id ][ sch - 1 ][ sch_input ], sch, ( sch_input + 1 ) );
                break;
            }
        }
        if( sch_input == 8 ){
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ALERT, "%s: Oops! Qposition not found!\n", __func__ );
        }
    }

    /*--------------------------------------------------------------------------------------------------*/
    seq_printf( f, "Queue add: %s\n", !sch?"0":"1" );
    /*--------------------------------------------------------------------------------------------------*/
    seq_printf( f, "Queue number: %d\n", ( que + 1 ) );
    /*--------------------------------------------------------------------------------------------------*/
    if( sch ){
        arr = fp_reg_read( ( uint32_t )( TLITE_SCH0_Q0_WGHT_ADDR + ( REG_LEN * sch_input ) + ( SHED_ADDRESS_MASK * ( sch - 1 ) + ( DEV_ADDRESS_MASK * ( file_id + 1 ) ) ) ) );
        seq_printf( f, "Queue weight: %u\n", arr );
    }else{
        seq_printf( f, "Queue weight: 0\n" );
    }
    /*--------------------------------------------------------------------------------------------------*/
    seq_printf( f, "Scheduler number: %d\n", sch );
    /*--------------------------------------------------------------------------------------------------*/
    if( sch ){
        seq_printf( f, "Scheduler input: %d\n", ( sch_input + 1 ) );
    }else{
        seq_printf( f, "Scheduler input: 0\n" );
    }
    /*--------------------------------------------------------------------------------------------------*/
    arr = read_fasw_context_memory( que, file_id, CURQ_QMAX_QMIN_CFG );

    if( ( arr & 3 ) == TLITE_QUE_TAIL_DROP ){
        seq_printf( f,  "Queue management: 1\n"
                        "Max queue size: %d\n"
                        "Min queue size: 0\n"
                        "Drop probability: 0 0 0 0 0 0 0 0\n",
                        ( ( arr >> SHIFT_BY11 ) & 0x1FF ) );
    }else if( ( arr & 3 ) == TLITE_QUE_WRED ){
        seq_printf( f,  "Queue management: 2\n" 
                        "Max queue size: %d\n"
                        "Min queue size: %d\n",
                        ( ( arr >> SHIFT_BY11 ) & 0x1FF ),
                        ( ( arr >>  SHIFT_BY2 ) & 0x1FF ) );
        arr = read_fasw_context_memory( que, file_id, CURQ_HW_PROB_CFG_TBL0 );
        seq_printf( f,  "Drop probability: %d %d %d %d %d %d ", 
                        ( arr & BIT_0TO4),
                        ( ( arr >> SHIFT_BY5 ) & BIT_0TO4 ),
                        ( ( arr >> SHIFT_BY10 ) & BIT_0TO4 ),
                        ( ( arr >> SHIFT_BY15 ) & BIT_0TO4 ),
                        ( ( arr >> SHIFT_BY20 ) & BIT_0TO4 ),
                        ( ( arr >> SHIFT_BY25 ) & BIT_0TO4 ) );
        arr = read_fasw_context_memory( que, file_id, CURQ_HW_PROB_CFG_TBL1 );
        seq_printf( f,  "%d %d\n",
                        ( arr & BIT_0TO4 ),
                        ( ( arr >> SHIFT_BY5 ) & BIT_0TO4 ) );
    }else{
        seq_printf( f,    "Queue management: 0\n"
                        "Max queue size: 0\n"
                        "Min queue size: 0\n"
                        "Drop probability: 0 0 0 0 0 0 0 0\n" );
    }
    seq_printf( f, "Clear queue counters: 0\n" );
    arr = read_fasw_context_memory( que, file_id, CURQ_TRANS_CNT );
    seq_printf( f, "Queue tx counter: %u\n", arr );
    arr = read_fasw_context_memory( que, file_id, CURQ_DROP_CNT );
    seq_printf( f, "Queue drop counter: %u\n", arr );
    arr = read_fasw_context_memory( que, file_id, CURQ_PKT_CNT );
    seq_printf( f, "Queue pck counter: %u\n", arr );

    seq_printf( f,
        "----------------------------------------------------------\n"
        "Queue add - at write: add (1), remove (2) or read settings (0) of current queue\n"
        "- at read:  0 - queue not serviced, 1 - queue serviced\n"
        "Queue number - queue number for setup or read settings (range: 1 to 8)\n"
        "Queue weight - queue weight (range: 1 to 4294967295)\n"
        "Scheduler number - scheduler number to which current queue is going (1 - Scheduler1,"
        " 2 - Scheduler2, 0 - queue not connected to scheduler)\n"
        "Scheduler input - scheduler input to which current queue is going (range: 1 to 8,"
        " 0 - queue not connected to scheduler)\n"
        "Queue management - management algorithm for current queue (0 - no management, 1 -"
        " Tail Drop, 2 - Weighted Random Early Detection (WRED))\n"
        "Max queue size - only for Tail Drop and WRED, maximum queue size in packets\n"
        "Min queue size - only for WRED, queue size in packets when packet can be dropped\n"
        "Drop probability - probability of packets drop, when current queue size above min"
        " queue size, but below max queue size\n"
        "Clear queue counters - 1 - clear current queue counters\n"
        "Queue tx counter - read only, transmitted packets through current queue\n"
        "Queue drop counter - read only, dropped packets in current queue\n"
        "Queue pck counter - read only, number of packets in current queue\n" );
    mutex_unlock( &mutexProcTMU );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: end\n", __func__ );
    return 0;
}

static int port_queues_open( struct inode *inode, struct file *file ){
    return single_open( file, port_queues_read, NULL );
}

static const struct file_operations queue_settings_fops = {
    .owner = THIS_MODULE,
    .open  = port_queues_open,
    .read  = seq_read,
    .llseek = seq_lseek,
    .release = single_release,
    .write = port_queues_write,
};

//static const struct file_operations queue_settings_fops = {
//    .owner = THIS_MODULE,
//    .read  = port_queues_read,
//    .write = port_queues_write,
//};

static ssize_t debug_write( struct file *file, const char *buf,
                           size_t count, loff_t *ppos ) {

    int file_id, res, index;
    uint32_t array[3];
    struct sk_buff  *new_skb = NULL;
    fp_dev_info_t   *logical_priv;
    fp_private_t    *real_priv;
    int port, data_to_trm_index, data_size;
    uint8_t *buff;
    size_t result;
    uint16_t *ptr;
    tlite_classq_params_t *tque;
    struct netdev_queue *queue;

    file_id = *( uint32_t* )PDE_DATA( file_inode( file ) );

    if( fp_logical_dev[ 0 ] == NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DEBUG_WRITE: the device doesn't OPEN!\n" );
        return count;
    }

    if( file_id >= MAX_FILES_DEBUG ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DEBUG_WRITE: bad file id!\n" );
        return count;
    }

    result = 0;
    buff = FP_NULL;
    buff = kmalloc( PROC_BUF_SIZE, GFP_KERNEL );
    if( buff == FP_NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DEBUG_WRITE: can not allocate memory for user data\n" );
        goto exit_debug_write;
    }
    memset( buff, 0, PROC_BUF_SIZE );

    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "DEBUG_WRITE: start\n" );

    if( count > ( PROC_BUF_SIZE - 1 ) ){
       count = PROC_BUF_SIZE - 1;
    }

    res = copy_from_user( buff, ( void* )buf, count );
    if( res ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DEBUG_WRITE: can not get data from user\n" );
        goto exit_debug_write;   
    }
    result = count;
    /*------------------------------------------------------------------------*/
    switch( file_id ){
        /*---------- print_message_level ----------*/
        case FILE_DEBUG_PRINT:
            res = sscanf( buff, "fp_debug_flags = 0x%X\nfp_trace_level = %d", &array[ 0 ], &array[ 1 ] );
            if( res != 2 ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DEBUG_WRITE: can not get values from file\n" );
                goto exit_debug_write;
            }
            fp_dbgp_flags = array[ 0 ];
            fp_trace_level = array[ 1 ];
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "DEBUG_WRITE: fp_debug_flags 0x%08X fp_trace_level %d\n", fp_dbgp_flags, fp_trace_level );
            break;
        /*---------- wr_packet ----------*/
        case FILE_DEBUG_WR_PACKET:
            
            logical_priv = netdev_priv( fp_logical_dev[ 0 ] );
            real_priv  = netdev_priv( logical_priv->real_dev );

            if( ( buff[ 0 ] > 0x2F ) && ( buff[ 0 ] < 0x3A ) ){ //port specify in ASCII
                if( buff[ 1 ] == 0x3A ){   /*for this case format is: 2:data_to_trm*/
                    port = ( buff[ 0 ] - 0x30 ) & 0xF;
                    data_to_trm_index = 2;
                }else{   /*for this case: 12:data_to_trm*/
                    port = 10 + ( ( buff[ 1 ] - 0x30 ) & 0xF );
                    port &= 0xF;
                    data_to_trm_index = 3;
                }
            }else{
                port = buff[ 0 ] & 0xF;  //port specify in binary
                data_to_trm_index = 1;
            }
            data_size = count - data_to_trm_index;
            
            if( data_size < 60 ){
                
                for( res = 0; res < ( 60 - data_size ); res++ ){
                    buff[ data_size + res ] = 0;
                }
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "DEBUG_WRITE: %d bytes in packet, it is very small! Add %d bytes in packet\n", data_size, res);
                data_size = 60;
            }
            new_skb = dev_alloc_skb( data_size );

            if( new_skb != NULL ){
                memcpy( new_skb->data, &buff[ data_to_trm_index ], data_size );
                skb_put( new_skb, data_size );
                new_skb->dev = real_priv->dev;
                new_skb->protocol = htons( PROTOCOL_IP );
                new_skb->cb[ CB_PORT_FOR_INJECTION ] = port;
                memcpy( &new_skb->cb[ CB_INJ_TX_FLAG1 ], control_block_injection_flag_values, CB_INJ_TX_FLAG_VALUES_LEN );
                new_skb->cb[ CB_INJ_TX_PORT ] = port;
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "DEBUG_WRITE: write skb to queue xmit, port %d\n", port );
                fp_net_xmit( new_skb, new_skb->dev );
            }else{
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DEBUG_WRITE: can allocate skb buffer for packet\n" );
                goto exit_debug_write;
            }
            break;

        case FILE_DEBUG_AGEING_TIMEOUT:
            res = sscanf( buff, "Timeout: %d", &array[ 0 ] );
            if( res != 1 ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DEBUG_WRITE: can not get values from file, res = %d!\n", res);
                goto exit_debug_write;
            }
            logical_priv = netdev_priv( fp_logical_dev[ 0 ] );
            real_priv  = netdev_priv( logical_priv->real_dev );
            if( ( real_priv->ageing_timeout == 0xFFFFFFFF ) && ( array[ 0 ] != 0xFFFFFFFF ) ){
                real_priv->ageing_timeout = array[ 0 ];
                if( real_priv->ageing_timer_on == 0 ){
                    fp_switch_init_timer( 1 );
                }
            }else{
                real_priv->ageing_timeout = array[ 0 ];
                if(real_priv->ageing_timeout == 0xFFFFFFFF){
                    fp_switch_init_timer( 0 );
                }
            }
            break;

        case FILE_DEBUG_FAR_END_LOOPBACK:
            res = sscanf( buff, "Port: %d\nFarEndLoopback: %d", &array[ 0 ], &array[ 1 ]);
            if( res != 2 ){
                FP_DEBUG(DBGP_FEAT_PROCFS, FP_LOG_ERR, "DEBUG_WRITE: far end loopback, can not get values from file\n" );
                goto exit_debug_write;
            }

            array[ 0 ]--;         //ports: 1 - 16
            array[ 0 ] &= 0xF;    //ports: 0 - 15
            phy_far_loopback = array[ 0 ];
            
            array[ 1 ] &= 1;      //loopback: 0 - disable, 1 - enable

            logical_priv = netdev_priv( fp_logical_dev[ phy_far_loopback ] );

            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "DEBUG_WRITE: far end loopback, phy ic type: 0x%x\n", logical_priv->phy_ic_type );
            if( logical_priv->phy_ic_type == PHY_IC_DP83867 ){

                if( array[ 1 ] == 0 ){  //far end loopback disable

                    array[ 0 ] = fp_mdio_read_phy( logical_priv->mdio_port, logical_priv->phy_addr, DP83867_BISCR );
                    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "DEBUG_WRITE: far end loopback, BISCR 0x%04x\n", array[ 0 ] );
                    array[ 0 ] &= ~( 3 | ( 0xF << 2 ) | ( 1 << 7 ) | ( 0xF << 12 ) );
                    fp_mdio_write_phy( logical_priv->mdio_port, logical_priv->phy_addr, DP83867_BISCR, array[ 0 ] );

                    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "DEBUG_WRITE: far end loopback, port_%d loopback is disable\n", phy_far_loopback );
                    array[ 0 ] = fp_mdio_read_phy( logical_priv->mdio_port, logical_priv->phy_addr, DP83867_BISCR );
                    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "DEBUG_WRITE: far end loopback: after modify BISCR 0x%04x\n", array[ 0 ] );
                }else{  //far end loopback enable
                    array[ 0 ] = fp_mdio_read_phy( logical_priv->mdio_port, logical_priv->phy_addr, DP83867_BISCR );
                    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "DEBUG_WRITE: far end loopback, BISCR 0x%04x\n", array[ 0 ] );
                    array[ 0 ] &= ~( 3 | ( 1 << 7 ) | ( 0xF << 12 ) );
                    array[ 0 ] |= 8 << 2;
                    fp_mdio_write_phy( logical_priv->mdio_port, logical_priv->phy_addr, DP83867_BISCR, array[ 0 ] );

                    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "DEBUG_WRITE: far end loopback, port_%d loopback is enable\n", phy_far_loopback );
                    array[0] = fp_mdio_read_phy(logical_priv->mdio_port, logical_priv->phy_addr, DP83867_BISCR);
                    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "DEBUG_WRITE: far end loopback: after modify BISCR 0x%04x\n", array[ 0 ] );
                }
            }else{
                FP_DEBUG(DBGP_FEAT_PROCFS, FP_LOG_ERR, "DEBUG_WRITE: far end loopback, unavailable condition, ethernet phy does not support this option!\n");
            }
            break;
        
        case FILE_DEBUG_HOST_SYSTEM_LOOPBACK:
            logical_priv = netdev_priv( fp_logical_dev[ 0 ] );
            real_priv  = netdev_priv( logical_priv->real_dev );

            res = sscanf( buff, "Host_system_loopback: %d", &array[ 0 ] );
            if( res != 1 ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DEBUG_WRITE: host system loopback, can not get values from file\n" );
                goto exit_debug_write;
            }
            real_priv->host_system_loopback = array[ 0 ] & 1;
            if( real_priv->host_system_loopback == 0 ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DEBUG_WRITE: host system loopback disable\n" );
            }else{
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DEBUG_WRITE: host system loopback enable\n" );
            }
            break;

        case FILE_DEBUG_JUMBO_ENABLE:
            /*  
            * For this operations need to do next steps:
            * 1. stop ports rx
            * 2. stop host tx
            * 3. wait until all bmu buffers are not empty
            * 4. update jumbo size register
            * 5. update class, tmu thresholds
            * 6. update host queues management settings
            * 7. start host trm
            * 8. start ports rx
            */
            index = 0;
            res = sscanf( buff, "Jumbo frames: %d", &array[ 0 ] );
            if( res != 1 ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DEBUG_WRITE: jumbo frames, can not get enable/disable jumbo from file\n" );
                //result = -EINVAL;
                goto exit_debug_write;
            }
            array[ 0 ] &= 1;
            if( array[ 0 ] ){
                while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
                res = sscanf( &buff[ index ], "Jumbo length: %d", &array[ 1 ] );
                if( res != 1 ){
                    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DEBUG_WRITE: jumbo frames, can not get jumbo length from file\n" );
                    //result = -EINVAL;
                    goto exit_debug_write;
                }
                if( ( array[ 1 ] < 1514 ) || ( array[ 1 ] > JUMBO_MAX_LENGTH ) ){
                    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DEBUG_WRITE: INVALID jumbo frames length\n" );
                    //result = -EINVAL;
                    goto exit_debug_write;
                }
            }

            ptr = FP_NULL;
            tque = FP_NULL;

            ptr = kmalloc( sizeof( uint16_t ) * 8, GFP_KERNEL );
            if( ptr == FP_NULL ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DEBUG_WRITE: can not allocate memory for host queues settings ptr (in jumbo enable code)\n" );
                //result = -ENOMEM;
                goto exit_debug_write;
            }

            tque = kmalloc( sizeof( tlite_classq_params_t ), GFP_KERNEL );
            if( tque == FP_NULL ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DEBUG_WRITE: can not allocate memory for host queues settings tque (in jumbo enable code)\n" );
                //result = -ENOMEM;
                goto exit_debug_write;
            }

            /* Update switch settings */
            logical_priv = netdev_priv( fp_logical_dev[ 0 ] );
            real_priv  = netdev_priv( logical_priv->real_dev );

            /* STEP 1 */
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_INFO, "DEBUG_WRITE: disable EMAC rx...\n" );
            for( port = 0; port < EMAC_NUMS; port++ ){
                array[ 2 ] = fp_reg_read( emac[ port ] );
                array[ 2 ] &= ~( 1 << 2 );  //rx disable
                fp_reg_write( emac[ port ], array[ 2 ] );
            }
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_INFO, "DEBUG_WRITE: disable EMAC rx done\n" );
            /* STEP 2 */
            spin_lock_bh( &real_priv->lock );
            real_priv->txq_stopped = 1;
            spin_unlock_bh( &real_priv->lock );

            /* STEP 3 */
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_INFO, "DEBUG_WRITE: waiting buffers\n" );
            while( fp_reg_read( BMU1_BASE_ADDR + BMU_BUF_CNT ) & 0xFFFF ){}
            while( fp_reg_read( BMU2_BASE_ADDR + BMU_BUF_CNT ) & 0xFFFF ){}
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_INFO, "DEBUG_WRITE: buffers are free\n" );

            /* STEP 4 */
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_INFO, "DEBUG_WRITE: enable/disable jumbo...\n" );
            if( array[ 0 ] ){   //enable jumbo
                for( port = 0; port < EMAC_NUMS; port++ ){
                    fp_reg_write( ( emac[ port ] + EMAC_JUMBO_MAX_LENGTH ), array[ 1 ] );
                    array[ 2 ] = fp_reg_read( emac[ port ] + EMAC_NETWORK_CONFIG );
                    array[ 2 ] |= ( 1 << 3 );  //jumbo enable
                    fp_reg_write( ( emac[ port ] + EMAC_NETWORK_CONFIG ), array[ 2 ] );
                }
                spin_lock_bh( &real_priv->lock );
                real_priv->config_flags |= SWITCH_JUMBO_PROCESSING;
                real_priv->dev->max_mtu = array[ 1 ] - 14;
                spin_unlock_bh( &real_priv->lock );
            }else{  //disable jumbo
                for( port = 0; port < EMAC_NUMS; port++ ){
                    array[ 2 ] = fp_reg_read( emac[ port ] + EMAC_NETWORK_CONFIG );
                    array[ 2 ] &= ~( 1 << 3 );  //jumbo disable
                    fp_reg_write( ( emac[ port ] + EMAC_NETWORK_CONFIG ), array[ 2 ] );
                }
                spin_lock_bh( &real_priv->lock );
                real_priv->config_flags &= ~( SWITCH_JUMBO_PROCESSING );
                real_priv->dev->max_mtu = 1514 - 14;
                spin_unlock_bh( &real_priv->lock );
            }
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_INFO, "DEBUG_WRITE: enable/disable jumbo done\n" );

            /* STEP 5 */
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_INFO, "DEBUG_WRITE: update class, tmu thresholds\n" );
            update_class_tmu_threshold( array[ 0 ], array[ 1 ] );

            /* STEP 6 */
            host_queues_settings( ptr );
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_INFO, "DEBUG_WRITE: update HOST queues settings\n" );

            mutex_lock( &mutexProcTMU );
            pfe_tlite_dynamic_config_enable( TLITE_SWITCH );
            for( res = 1; res <= TLITE_MAX_QUE_NO; res++ ){
                tque->tlcq_classq_updates = 1;
                
                tque->tlcq_dev_id = HOST_PORT_NO + 1;
                tque->tlcq_classq_id = res;
                tque->tlcq_schd_id = fp_tdev_list[ tque->tlcq_dev_id - 1 ].classq_list[ tque->tlcq_classq_id - 1 ];

                tque->tlcq_schd_slot = INVALID_QUEUE0;
                for( index = 0; index < 8; index++ ){
                    if( Qposition[ HOST_PORT_NO ][ tque->tlcq_schd_id - 1 ][ index ] == tque->tlcq_classq_id ){
                        tque->tlcq_schd_slot = index + 1;
                        break;
                    }
                }

                tque->tlcq_weight = 0x500;
                if( tque->tlcq_schd_slot < 9 ){
                    tque->tlcq_weight = fp_reg_read( ( uint32_t )( TLITE_SCH0_Q0_WGHT_ADDR + ( REG_LEN * ( tque->tlcq_schd_slot - 1 ) ) + ( SHED_ADDRESS_MASK * ( tque->tlcq_schd_id - 1 ) ) + ( DEV_ADDRESS_MASK * ( HOST_PORT_NO + 1 ) ) ) );
                }

                array[ 2 ] = read_fasw_context_memory( tque->tlcq_classq_id - 1, tque->tlcq_dev_id - 1, CURQ_QMAX_QMIN_CFG );
                tque->tlcq_max_len = ptr[ res - 1 ];
                tque->tlcq_qmgmt = array[ 2 ] & 3;
                tque->tlcq_min_len = ( array[ 2 ] >> 2 ) & 0x1FF;
                if( ( array[ 2 ] & 3 ) == TLITE_QUE_WRED ){
                    array[ 2 ] = read_fasw_context_memory( tque->tlcq_classq_id - 1, tque->tlcq_dev_id - 1, CURQ_HW_PROB_CFG_TBL0 );
                    tque->tlcq_prob_cfg0 = array[ 2 ] & 0x1F;
                    tque->tlcq_prob_cfg1 = ( array[ 2 ] >>  SHIFT_BY5 ) & 0x1F;
                    tque->tlcq_prob_cfg2 = ( array[ 2 ] >> SHIFT_BY10 ) & 0x1F;
                    tque->tlcq_prob_cfg3 = ( array[ 2 ] >> SHIFT_BY15 ) & 0x1F;
                    tque->tlcq_prob_cfg4 = ( array[ 2 ] >> SHIFT_BY20 ) & 0x1F;
                    tque->tlcq_prob_cfg5 = ( array[ 2 ] >> SHIFT_BY25 ) & 0x1F;

                    array[ 2 ] = read_fasw_context_memory( tque->tlcq_classq_id - 1, tque->tlcq_dev_id - 1, CURQ_HW_PROB_CFG_TBL1 );
                    tque->tlcq_prob_cfg6 = array[ 2 ] & 0x1F;
                    tque->tlcq_prob_cfg7 = ( array[ 2 ] >>  SHIFT_BY5 ) & 0x1F;
                }
            
                if( pcm_tlite_classq_add( tque, TLITE_SWITCH ) < 0 ){
                    FP_DEBUG( DBGP_FEAT_TMU, FP_LOG_ERR, "%s queue add failed! HOST port, queue %d!\n", __func__, ( tque->tlcq_classq_id - 1 ) );
                }
            }
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_INFO, "DEBUG_WRITE: update HOST queues settings done\n" );
            /* HW enable for the tdq fsm */
            pfe_tlite_dynamic_config_done();
            mutex_unlock( &mutexProcTMU );

            kfree( ptr );
            kfree( tque );

            /* STEP 7 */
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_INFO, "DEBUG_WRITE: enable HOST tx\n" );
            spin_lock_bh( &real_priv->lock );
            real_priv->txq_stopped = 0;
            spin_unlock_bh( &real_priv->lock );
            for( res = 0; res < NUM_HIF_CHANNELS; res++ ){
                queue = netdev_get_tx_queue( real_priv->dev, res );
                if( netif_tx_queue_stopped( queue ) ){
                    netif_tx_wake_queue( queue );
                }
            }

            /* STEP 8 */
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_INFO, "DEBUG_WRITE: enable EMACs rx\n" );
            for( port = 0; port < EMAC_NUMS; port++ ){
                array[ 2 ] = fp_reg_read( emac[ port ] );
                array[ 2 ] |= ( 1 << 2 );  //rx enable
                fp_reg_write( emac[ port ], array[ 2 ] );
            }     
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_INFO, "DEBUG_WRITE: jumbo settings done\n" );
            break;

        case FILE_DEBUG_MDIO_SPEED:
            index = 0;
            res = sscanf( buff, "MDIO1 div: %d", &array[ 0 ] );
            if( res != 1 ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DEBUG_WRITE: mdio speed, can not get \"MDIO1 div\" values from file\n" );
                goto exit_debug_write;
            }

            array[ 0 ] &= 7;
            array[ 1 ] = fp_reg_read( EMAC1_BASE_ADDR + 4 );
            array[ 1 ] &= ~( 7 << 18 );
            array[ 1 ] |= ( array[ 0 ] << 18 );
            fp_reg_write( ( EMAC1_BASE_ADDR + 4 ), array[ 1 ] );
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DEBUG_WRITE: mdio speed, mdc1 div %d\n", array[ 0 ] );
            while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
            res = sscanf( &buff[ index ], "MDIO2 div: %d", &array[ 0 ] );
            if( res != 1 ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DEBUG_WRITE: mdio speed, can not get \"MDIO2 div\" values from file\n" );
                goto exit_debug_write;
            }
            array[ 0 ] &= 7;
            array[ 1 ] = fp_reg_read( EMAC2_BASE_ADDR + 4 );
            array[ 1 ] &= ~( 7 << 18 );
            array[ 1 ] |= ( array[ 0 ] << 18 );
            fp_reg_write( ( EMAC2_BASE_ADDR + 4 ), array[ 1 ] );           
            FP_DEBUG(DBGP_FEAT_PROCFS, FP_LOG_ERR, "DEBUG_WRITE: mdio speed, mdc2 div %d\n", array[ 0 ] );
            break;

        case FILE_DEBUG_PHY_TEST_MODE:
            index = 0;
            for( port = 0; port < FP_MAX_SWITCH_PORTS; port++ ){
                if( port == HOST_PORT_NO ){
                    continue;
                }
                res = sscanf( &buff[ index ], "Port: %d test mode: %d channel control: %d", &array[ 0 ], &array[ 1 ], &array[ 2 ] );
                if( res != 3 ){
                    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DEBUG_WRITE: phy_test_mode, can not get values from file\n" );
                    while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
                    continue;
                }

                /* port number in array[0]*/
                array[ 0 ]--;
                array[ 0 ] &= 0xF;

                /* test mode in array[1]*/
                array[ 1 ] &= 7;

                /* channel control in array[2]*/
                array[ 2 ] &= 7;

                logical_priv = netdev_priv( fp_logical_dev[ array[ 0 ] ] );

                if( logical_priv->phy_ic_type == PHY_IC_DP83867 ){
                    if( array[ 1 ] != 0 ){
                        res = fp_mdio_read_phy( logical_priv->mdio_port, logical_priv->phy_addr, DP83867_CFG1 );
                        res &= 0xFFFF;
                        if( ( res & ( 7 << 13 ) ) == 0 ){   //if phy register set in normal mode (TEST MODE field == 0), save it
                            debug_phy_test_mode[ array[ 0 ] ] = res | ( 1 << 16 );
                            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "DEBUG_WRITE: phy_test_mode, save settings 0x%04x reg. DP83867_CFG1 for port %d\n", res, array[ 0 ] );
                        }

                        data_size = ( int )phy_dp83867_mmd_read( logical_priv->mdio_port, logical_priv->phy_addr, DP83867_TMCH_CTRL );
                        data_size &= ~( 7 << 5 );
                        data_size |= array[ 2 ] << 5;
                        phy_dp83867_mmd_write( logical_priv->mdio_port, logical_priv->phy_addr, DP83867_TMCH_CTRL, ( data_size & 0xFFFF ) );
                        data_size = ( int )phy_dp83867_mmd_read( logical_priv->mdio_port, logical_priv->phy_addr, DP83867_IO_MUX_CFG );
                        data_size &= ~( 0x1F << 8 );
                        if( array[ 2 ] < 4 ){
                            data_size |= ( ( 8 + array[ 2 ] ) << 8 );   //Ch A or B or C or D trm clock
                        }else{
                            data_size |= ( 12 << 8 );   //Reference clock (sync to X_I input)
                        }
                        phy_dp83867_mmd_write( logical_priv->mdio_port, logical_priv->phy_addr, DP83867_IO_MUX_CFG, ( data_size & 0xFFFF ) );
                        res &= ~( 7 << 13 );
                        res |= ( array[ 1 ] << 13 );
                        fp_mdio_write_phy( logical_priv->mdio_port, logical_priv->phy_addr, DP83867_CFG1, ( uint32_t )res );
                    }else{
                        if( debug_phy_test_mode[ array[ 0 ] ] & ( 1 << 16 ) ){ //has a data to restore
                            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "DEBUG_WRITE: phy_test_mode, port %d restore settings 0x%04x\n", array[ 0 ], debug_phy_test_mode[ array[ 0 ] ] );
                            fp_mdio_write_phy( logical_priv->mdio_port, logical_priv->phy_addr, DP83867_CFG1, debug_phy_test_mode[ array[ 0 ] ] );
                            debug_phy_test_mode[ array[ 0 ] ] = 0;
                            data_size = ( int )phy_dp83867_mmd_read( logical_priv->mdio_port, logical_priv->phy_addr, DP83867_TMCH_CTRL );
                            data_size &= ~( 7 << 5 );
                            phy_dp83867_mmd_write( logical_priv->mdio_port, logical_priv->phy_addr, DP83867_TMCH_CTRL, ( data_size & 0xFFFF ) );

                            data_size = ( int )phy_dp83867_mmd_read( logical_priv->mdio_port, logical_priv->phy_addr, DP83867_IO_MUX_CFG );
                            data_size &= ~( 0x1F << 8 );
                            data_size |= ( 12 << 8 );
                            phy_dp83867_mmd_write( logical_priv->mdio_port, logical_priv->phy_addr, DP83867_IO_MUX_CFG, ( data_size & 0xFFFF ) );
                        }
                    }

                }else{
                    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DEBUG_WRITE: phy_test_mode: unknown ethernet phy!\n" );
                }
                while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
            }
            break;
    }
    result = count;
exit_debug_write:
    if( buff != FP_NULL ){
        kfree( buff );
    }
    return result;
}

static int debug_read( struct seq_file *f, void *v ) {
    int file_id, res;
    fp_dev_info_t   *logical_priv;
    fp_private_t    *real_priv;
    uint32_t *params;

    if( fp_logical_dev[ 0 ] == NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, the device doesn't OPEN!\n", __func__ );
        return 0;
    }

    file_id = *( uint32_t* )PDE_DATA( file_inode( f->file ) );
    if( file_id >= MAX_FILES_DEBUG ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, invalid file id %d!\n", __func__, file_id );
        return 0;
    }

    params = kmalloc( ( sizeof( uint32_t ) * 32 ), GFP_KERNEL );
    if( params == FP_NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, can not allocate memory for params\n", __func__ );
        return 0;
    }

    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: start\n", __func__ );
    switch( file_id ){
        /*---------- print_message_level ----------*/
        case FILE_DEBUG_PRINT:
            seq_printf( f,  "fp_debug_flags = 0x%X\n"
                            "fp_trace_level = %d\n\n"
                            "--- Debug Flags: ---\n",
                            fp_dbgp_flags, fp_trace_level );
            for( res = 0; res < FP_MAX_DBGP; res++ ){
                if( fp_dbgp_arr[ res ][ 0 ] != 0 ){
                    seq_printf( f, "0x%X - %s\n", ( 1 << res ), fp_dbgp_arr[ res ] );
                }
            }
            seq_printf( f,    "\n--- Trace Level: ---\n"
                            "0  FP_LOG_EMERG\n"
                            "1  FP_LOG_ALERT\n"
                            "2  FP_LOG_CRIT\n"
                            "3  FP_LOG_ERR\n"
                            "4  FP_LOG_WARNING\n"
                            "5  FP_LOG_NOTICE\n"
                            "6  FP_LOG_INFO\n"
                            "7  FP_LOG_DEBUG\n" );

            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: fp_debug_flags = 0x%X\nfp_trace_level = %d\n", __func__, fp_dbgp_flags, fp_trace_level );
            break;            
        /*---------- wr_packet ----------*/
        case FILE_DEBUG_WR_PACKET:
            seq_printf( f,    "Write data to send\n"
                            "Example ASCII format: 1:data to send <- send 'data to send' via port 1\n"
                            "12:data to send 123 <- send 'data to send 123' via port 12\n"
                            "Example bin format: first byte is port number (range 0 - 15), second byte - first transmitt byte...\n" );
            break;
        /*---------- ageing_timeout ----------*/
        case FILE_DEBUG_AGEING_TIMEOUT:
            logical_priv = netdev_priv( fp_logical_dev[ 0 ] );
            real_priv  = netdev_priv( logical_priv->real_dev );
            seq_printf( f,  "Timeout: %d\n"
                            "--------------------\n"
                            "timeout in seconds\n"
                            "value -1 - disable ageing timer, other value - enable it\n",
                            real_priv->ageing_timeout );
            break;
        /*---------- occuped bmu buffers ----------*/
        case FILE_DEBUG_BMU_OCCUPIED_BUFFERS:
            for( res = 0; res < 32; res++ ){
                params[ res ] = fp_reg_read( ( BMU1_BASE_ADDR + 0x60 + ( res << 2 ) ) );
            }
            seq_printf( f, "Number of occupied buffers in BMU1:\n" );
            for( res = 0; res < 32; res++ ){
                seq_printf( f, "\tBMU_MAS%d_BUF_CNT: %d\n", res, params[ res ] );
            }
            for( res = 0; res < 32; res++ ){
                params[ res ] = fp_reg_read( ( BMU2_BASE_ADDR + 0x60 + (res << 2) ) );
            }
            seq_printf( f, "Number of occupied buffers in BMU2:\n" );
            for( res = 0; res < 32; res++ ){
                seq_printf( f, "\tBMU_MAS%d_BUF_CNT: %d\n", res, params[ res ] );              
            }
            break;
        /*---------- far end loopback of ethernet phy ----------*/
        case FILE_DEBUG_FAR_END_LOOPBACK:
            logical_priv = netdev_priv( fp_logical_dev[ phy_far_loopback ] );
            if( logical_priv->phy_ic_type == PHY_IC_DP83867 ){
                params[ 0 ] = fp_mdio_read_phy( logical_priv->mdio_port, logical_priv->phy_addr, DP83867_BISCR );
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: far end loopback: BISCR 0x%04x\n", __func__, params[ 0 ] );
                if( ( params[ 0 ] & ( 0xF << 2 ) ) != 0 ){
                    seq_printf( f,  "Port: %d\n"
                                    "FarEndLoopback: 1\n",
                                    ( phy_far_loopback + 1 ) );
                }else{
                    seq_printf( f,  "Port: %d\n"
                                    "FarEndLoopback: 0\n",
                                    ( phy_far_loopback + 1 ) );
                }
            }else{
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: far end loopback, unavailable condition, ethernet phy does not support this option!\n", __func__ );
                seq_printf( f,  "Port: %d\n"
                                "FarEndLoopback: 0\n"
                                "Ethernet PHY does not support this option\n",
                                ( phy_far_loopback + 1 ) );
            }
            break;
        /*---------- host system loopback ----------*/
        case FILE_DEBUG_HOST_SYSTEM_LOOPBACK:
            logical_priv = netdev_priv( fp_logical_dev[ 0 ] );
            real_priv  = netdev_priv( logical_priv->real_dev );
            seq_printf( f,  "Host_system_loopback: %d\n"
                            "--------------------\n"
                            "0 - disable, 1 - enable\n",
                            real_priv->host_system_loopback );
            break;
        /*---------- jumbo frame settings ----------*/
        case FILE_DEBUG_JUMBO_ENABLE:
            //params[ 0 ] = fp_reg_read( ( EMAC1_BASE_ADDR + EMAC_NETWORK_CONFIG ) );
            //params[ 1 ] = fp_reg_read( ( EMAC1_BASE_ADDR + EMAC_JUMBO_MAX_LENGTH ) );
            //numBytes = snprintf( buff, PROC_BUF_SIZE, "Jumbo frames: %d\nJumbo length: %d\n--------------------\n0 - disable for all ports, 1 - enable for all ports\nMax length: %d\n", ( ( params[ 0 ] & ( 1 << 3 ) ) >> 3 ), params[ 1 ], JUMBO_MAX_LENGTH );
            params[ 0 ] = fp_reg_read( ( EMAC1_BASE_ADDR + EMAC_JUMBO_MAX_LENGTH ) );
            logical_priv = netdev_priv( fp_logical_dev[ 0 ] );
            real_priv  = netdev_priv( logical_priv->real_dev );
            seq_printf( f,  "Jumbo frames: %s\n"
                            "Jumbo length: %d\n"
                            "--------------------\n"
                            "0 - disable for all ports, 1 - enable for all ports\n"
                            "Max length: %d\n",
                            ( real_priv->config_flags & SWITCH_JUMBO_PROCESSING ) ? "1" : "0",
                            params[ 0 ],
                            JUMBO_MAX_LENGTH );
            break;
        /*---------- setup mdio frequency ----------*/
        case FILE_DEBUG_MDIO_SPEED:
            params[ 0 ] = fp_reg_read( ( EMAC1_BASE_ADDR + 4 ) );
            params[ 1 ] = fp_reg_read( ( EMAC2_BASE_ADDR + 4 ) );
            seq_printf( f,  "MDIO1 div: %d\n"
                            "MDIO2 div: %d\n"
                            "--------------------\n"
                            "0: mdc = 25MHz\n"
                            "1: mdc = 12.5MHz\n"
                            "2: mdc = 6.25MHz\n"
                            "3: mdc = 4.17MHz\n"
                            "4: mdc = 3.13MHz\n"
                            "5: mdc = 2.08MHz\n"
                            "6: mdc = 1.56MHz\n"
                            "7: mdc = 0.89MHz\n",
                            ( ( params[ 0 ] & ( 7 << 18 ) ) >> 18 ), ( ( params[ 1 ] & ( 7 << 18 ) ) >> 18 ) );
            break;
        /*---------- print_message_level ----------*/
        case FILE_DEBUG_PHY_TEST_MODE:
            for( params[ 0 ] = 0; params[ 0 ] < FP_MAX_SWITCH_PORTS; params[ 0 ]++ ){
                if( params[ 0 ] == HOST_PORT_NO ){
                    continue;
                }
                logical_priv = netdev_priv( fp_logical_dev[ params[ 0 ] ] );
                if( logical_priv->phy_ic_type == PHY_IC_DP83867 ){
                    params[ 1 ] = fp_mdio_read_phy( logical_priv->mdio_port, logical_priv->phy_addr, DP83867_CFG1 );
                    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: port %d DP83867_PHYCR 0x%04x\n", __func__,
                                params[ 0 ], fp_mdio_read_phy( logical_priv->mdio_port, logical_priv->phy_addr, DP83867_PHYCR ) );
                    params[ 2 ] = phy_dp83867_mmd_read( logical_priv->mdio_port, logical_priv->phy_addr, DP83867_TMCH_CTRL );
                    seq_printf( f,  "Port: %d test mode: %d channel control: %d\n",
                                    ( params[ 0 ] + 1 ), ( ( params[ 1 ] >> 13 ) & 7 ), ( ( params[ 2 ] >> 5 ) & 7 ) );
                }else{
                    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: port %d unknown ethernet phy\n", __func__, params[ 0 ] );
                    seq_printf( f, "Port: %d test mode: 0 channel control: 0 unknown ethernet phy\n",
                                    ( params[ 0 ] + 1 ) );
                }
            }
            seq_printf( f,    "----------------------------------------------------------\n"
                            "Port - port number (range: 1 - 16)\n"
                            "test mode - 0 - Normal operation\n"
                            "          - 1 - Test Mode 1 - Transmit Waveform Test\n"
                            "          - 2 - Test Mode 2 - Transmit Jitter Test (Master Mode)\n"
                            "          - 3 - Test Mode 3 - Transmit Jitter Test (Slave Mode)\n"
                            "          - 4 - Test Mode 4 - Transmit Distortion Test\n"
                            "          - 5 - Test Mode 5 - Scrambled MLT3 Idles\n"
                            "          - 6 - Test Mode 6 - Repetitive 0001 sequence\n"
                            "          - 7 - Test Mode 7 - Repetitive (Pulse, 63 zeros)\n"
                            "channel control - 0 - test mode is driven on channel A\n"
                            "                - 1 - test mode is driven on channel B\n"
                            "                - 2 - test mode is driven on channel C\n"
                            "                - 3 - test mode is driven on channel D\n"
                            "                - 4 - 7 - test mode is driven on all 4 channels\n" );
            break;
        /*---------- MCU firmware version ----------*/
        case FILE_DEBUG_MCU_FW_VERSION:
            params[ 0 ] = get_mcu_firmware_version( );
            seq_printf( f,  "MCU firmware version: %d.%d\n",
                            ( params[ 0 ] & 0xFF ), ( ( params[ 0 ] >> 8 ) & 0xFF ) );
            break;
    }
    kfree( params );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: end\n", __func__ );
    return 0;
}

static int debug_open( struct inode *inode, struct file *file ){
    return single_open( file, debug_read, NULL );
}

static const struct file_operations debug_fops = {
    .owner = THIS_MODULE,
    .open  = debug_open,
    .read  = seq_read,
    .llseek = seq_lseek,
    .release = single_release,
    .write = debug_write,
};

//static const struct file_operations debug_fops = {
//    .owner = THIS_MODULE,
//    .read  = debug_read,
//    .write = debug_write,
//};

static int scheduler_read( struct seq_file *f, void *v ) {
    int file_id, i;
    uint32_t tmp;

    if( fp_logical_dev[ 0 ] == NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, the device doesn't OPEN!\n", __func__ );
        return 0;
    }
    file_id = *( uint32_t* )PDE_DATA( file_inode( f->file ) );
    if( file_id >= MAX_FILES_SCHEDULER_SETTINGS ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, invalid file id!\n", __func__ );
        return 0;
    }
    mutex_lock( &mutexProcTMU );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: start\n", __func__ );
    for( i = 0; i < 2; i++ ){
        seq_printf( f,  "--- Scheduler %d settings: ---\n", ( i + 1 ) );
        tmp = fp_reg_read( ( Tmu_Scheduler_Addr_Regs[ 0 ] + ( SHED_ADDRESS_MASK * i ) + ( ( file_id + 1 ) << 12 ) ) );
        seq_printf( f, "Scheduler algo type: %d\n", ( tmp + 1 ) );
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: sch%d algo type %d\n", __func__, i, tmp );
        if( i == 0 ){
            tmp = fp_reg_read( ( Tmu_Scheduler_Addr_Regs[ 12 ] + ( SHED_ADDRESS_MASK * i ) + ( ( file_id + 1 ) << 12 ) ) );
            seq_printf( f, "Sch2 input to which Sch1 is going: %d\n", ( tmp + 1 ) );
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: sch1 input to which sch0 is mapped: %d\n", __func__, tmp );
            tmp = fp_reg_read( ( Tmu_Scheduler_Addr_Regs[ 1 ] + ( tmp * REG_LEN ) + ( SHED_ADDRESS_MASK * 1 ) + ( ( file_id + 1 ) << 12 ) ) );
            seq_printf( f, "Weight queue of Sch2 input which Sch1 is going: %u\n", tmp );
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: wght queue of sch1 in which mapped sch0 out: %u\n", __func__, tmp );
        }
    }
    seq_printf( f,  "----------------------------------------------------------\n"
                    "Scheduler algo type - algo PRIORITY QUEUE (1), DWRR (3), RR(4), WRR(5)\n"
                    "Sch2 input to which Sch1 is going - Sch2 queue number to which output of Sch1 is connected (range: 1 - 8)\n"
                    "Weight queue of Sch2 input, which Sch1 is going - weight queue of Sch2, which mapped Sch1 (range: 1 - 4294967295)\n" );
    mutex_unlock( &mutexProcTMU );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: end\n", __func__ );
    return 0;
}

static ssize_t scheduler_write( struct file *file, const char *buf,
                           size_t count, loff_t *ppos ) {

    int32_t file_id, res, index;
    uint32_t param, sch;
    tlite_schd_params_t *psch;
    uint8_t *buff;
    size_t result;
    
    file_id = *( uint32_t* )PDE_DATA( file_inode( file ) );

    if( fp_logical_dev[ 0 ] == NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "SCHEDULER_WRITE: the device doesn't OPEN!\n" );
        return 0;
    }

    if( file_id >= MAX_FILES_SCHEDULER_SETTINGS ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "SCHEDULER_WRITE: bad file id!\n" );
        return 0;
    }

    mutex_lock( &mutexProcTMU );
    result = 0;
    buff = FP_NULL;
    psch = FP_NULL;
    buff = kmalloc( PROC_BUF_SIZE, GFP_KERNEL );
    if( buff == FP_NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "SCHEDULER_WRITE: can not allocate memory for user data\n" );
        goto exit_scheduler_write;
    }

    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "SCHEDULER_WRITE: start\n" );
    
    if( count > PROC_BUF_SIZE ){
       count = PROC_BUF_SIZE;
    }

    res = copy_from_user( buff, ( void* )buf, count );
    if( res ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "SCHEDULER_WRITE: can not get data from user\n" );
        goto exit_scheduler_write;   
    }
    buff[ count -1 ] = '\0';

    psch = kmalloc( sizeof( tlite_schd_params_t ), GFP_KERNEL );
    if( psch == NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "SCHEDULER_WRITE: can not memory allocated for tlite_schd_params_t structure\n" );
        goto exit_scheduler_write;
    }

    memset( psch, 0, sizeof( tlite_schd_params_t ) );
    result = count;
    /*--- SETTINGS FOR SCHEDULER 1, 2 ---*/
    index = 0;
    for( sch = 0; sch < 2; sch++ ){
        while( ( buff[ index++ ] != '\n' ) && ( index < (count - 1) ) ){}

        res = sscanf( &buff[ index ], "Scheduler algo type: %d", &param );
        if( res != 1 ){
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "SCHEDULER_WRITE: can not get sch%d \"Scheduler algo type\" value from file!\n", sch );
            goto exit_scheduler_write;
        } 
        psch->tlsd_algo = param;             //algo
        psch->tlsd_schd_id = ( sch + 1 );    //scheduler number
        psch->tlsd_dev_id = ( file_id + 1 ); //port 

        if( psch->tlsd_schd_id == 1 ){

            while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
            res = sscanf( &buff[ index ], "Sch2 input to which Sch1 is going: %d", &param );
            if( res != 1 ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "SCHEDULER_WRITE: can not get sch%d \"Sch2 input to which Sch1 is going\" value from file!\n", sch );
                goto exit_scheduler_write;
            }
            psch->tlsd_position = param;

            while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
            res = sscanf( &buff[ index ], "Weight queue of Sch2 input which Sch1 is going: %d", &param );
            if( res != 1 ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "SCHEDULER_WRITE: can not get scheduler%d \"Weight queue of Sch2 input which mapped Sch1 output\" value from file!\n", sch );
                goto exit_scheduler_write;
            }
            psch->tlsd_weight = param;
            while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
        }

        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "SCHEDULER_WRITE: sch%d settings: algo %d, port %d, input %d, weight %d\n", psch->tlsd_schd_id, psch->tlsd_algo, psch->tlsd_dev_id, psch->tlsd_position, psch->tlsd_weight );

        pfe_tlite_dynamic_config_enable( TLITE_SWITCH );
        pcm_tlite_scheduler_config( psch, TLITE_SWITCH );
        pfe_tlite_dynamic_config_done( );
    }
    result = count;

exit_scheduler_write:
    if( buff != FP_NULL ){
        kfree( buff );
    }
    if( psch != FP_NULL ){
        kfree( psch );
    }
    mutex_unlock( &mutexProcTMU );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "SCHEDULER_WRITE: end\n" );
    return result;
}

static int scheduler_open( struct inode *inode, struct file *file ){
    return single_open( file, scheduler_read, NULL );
}

static const struct file_operations scheduler_fops = {
    .owner = THIS_MODULE,
    .open  = scheduler_open,
    .read  = seq_read,
    .llseek = seq_lseek,
    .release = single_release,
    .write = scheduler_write,
};

//static const struct file_operations scheduler_fops = {
//    .owner = THIS_MODULE,
//    .read  = scheduler_read,
//    .write = scheduler_write,
//};
/*------------------------------------------------------------------------------*/

static int shaper_read( struct seq_file *f, void *v ) {
    int file_id, position;
    uint64_t rate;
    uint8_t *str_ptr;
    uint32_t *params;

    if( fp_logical_dev[ 0 ] == NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, the device doesn't OPEN!\n", __func__ );
        return 0;
    }
    file_id = *( uint32_t* )PDE_DATA( file_inode( f->file ) );
    if( file_id >= MAX_FILES_PORT_SHAPERS ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, invalid file id %d!\n", __func__, file_id );
        return 0;
    }

    mutex_lock( &mutexProcTMU );
    params = FP_NULL;
    str_ptr = FP_NULL;

    params = kmalloc( ( sizeof( uint32_t ) * 5 ), GFP_KERNEL );
    if( params == FP_NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, can not allocate memory for params\n", __func__ );
        return 0;
    }

    str_ptr = kmalloc( 256, GFP_KERNEL );
    if( str_ptr == FP_NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, can not allocate memory for position string\n", __func__ );
        kfree( params );
        return 0;
    }
    memset( str_ptr, 0, 256 );

    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: start\n", __func__ );
    params[ 0 ] = fp_reg_read( ( Tmu_Shaper_Addr_Regs[ 0 ] + ( ( file_id + 1 ) << 12 ) + ( shaper_number[ file_id ] * 0x100 ) ) );
    params[ 1 ] = fp_reg_read( ( Tmu_Shaper_Addr_Regs[ 1 ] + ( ( file_id + 1 ) << 12 ) + ( shaper_number[ file_id ] * 0x100 ) ) );
    params[ 2 ] = fp_reg_read( ( Tmu_Shaper_Addr_Regs[ 2 ] + ( ( file_id + 1 ) << 12 ) + ( shaper_number[ file_id ] * 0x100 ) ) );
    params[ 3 ] = fp_reg_read( ( Tmu_Shaper_Addr_Regs[ 3 ] + ( ( file_id + 1 ) << 12 ) + ( shaper_number[ file_id ] * 0x100 ) ) );
    params[ 4 ] = fp_reg_read( ( Tmu_Shaper_Addr_Regs[ 4 ] + ( ( file_id + 1 ) << 12 ) + ( shaper_number[ file_id ] * 0x100 ) ) );

    if( params[ 3 ] & 1 ){
        /* rate in packet/s */
        rate = ( ( uint64_t )( params[ 1 ] ) * 200 * 1000 * 1000 ) / ( ( 2 << ( params[ 0 ] >> 1 ) ) << 12 );
    }else{
        /* rate in Kbit/s */
        rate = ( ( uint64_t )( params[ 1 ] << 3 ) * 200 * 1000 ) / ( ( 2 << ( params[ 0 ] >> 1 ) ) << 12 );   
    }
    position = ( ( params[ 3 ] >> 1 ) & 0x1F );
    if( params[ 0 ] & 1 ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: shaper position %d\n", __func__, position );
        if( position == 0 ){
            memcpy( str_ptr, "(scheduler 2, output)", sizeof( "(scheduler 2, output)" ) );
        }else if( position == 1 ){
            if( ( 0 == ( fp_reg_read( ( Tmu_Scheduler_Addr_Regs[ 12 ] + ( ( file_id + 1 ) << 12 ) ) ) & 0xF ) ) &&
                ( 7 <  ( fp_reg_read( ( Tmu_Scheduler_Addr_Regs[ 9 ] + SHED_ADDRESS_MASK + ( ( file_id + 1 ) << 12 ) ) ) & 0x1F ) ) ){
                memcpy( str_ptr, "(scheduler 1, output)", sizeof( "(scheduler 1, output)" ) );
            }else{
                memcpy( str_ptr, "(scheduler 2, input 1)", sizeof( "(scheduler 2, input 1)" ) );
            }
        }else if( ( position > 1 )&&( position < 9 ) ){
            snprintf( str_ptr, 256, "(scheduler 2, input %d)", position );
        }else{
            snprintf( str_ptr, 256, "(scheduler 1, input %d)", ( position - 8 ) );
        }
    }else{
        memcpy( str_ptr, "(shaper disable)", sizeof( "(shaper disable)" ) );
    }
    seq_printf( f,  "Shaper: %d\n"
                    "Rate units: %d\n"
                    "Max credit: %u\n"
                    "Min credit: %d\n"
                    "Rate: %lld\n"
                    "Position: %d #%s\n"
                    "Mode: %d\n"
                    "Enable: %d\n",
                    ( shaper_number[ file_id ] + 1 ),
                    ( ( params[ 3 ] & 1 ) + 1 ),
                    ( ( params[ 2 ] >> SHIFT_BY10 ) & 0x3FFFFF ),
                    ( params[ 4 ] & 0x3FFFFF ),
                    rate,
                    ( position + 1 ),
                    str_ptr,
                    ( ( params[ 3 ] >> 6 ) & 1 ),
                    ( params[ 0 ] & 1 ) );

    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: TMU_SHP%d_CTRL = 0x%08X\n", __func__, shaper_number[ file_id ], params[ 0 ] );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: TMU_SHP%d_WGHT = 0x%05X\n", __func__, shaper_number[ file_id ], params[ 1 ] );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: TMU_SHP%d_MAX_CREDIT = 0x%08X\n", __func__, shaper_number[ file_id ], params[ 2 ] );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: TMU_SHP%d_MIN_CREDIT = 0x%06X\n", __func__, shaper_number[ file_id ], params[ 4 ] );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: TMU_SHP%d_CTRL2 = 0x%02X\n", __func__, shaper_number[ file_id ], params[ 3 ] );

    memset( str_ptr, 0, 256 );
    if( file_id < 16 ){
        snprintf( str_ptr, 256, "port%d", ( file_id + 1 ) );
    }else{
        memcpy( str_ptr, "host_port", sizeof( "host_port" ) );
    }
    seq_printf( f,  "----------------------------------------------------------\n"
                    "Shaper - number of shaper (range 1 - 4)\n"
                    "Rate units - Kbit/s (1) or packets/s (2)\n"
                    "Max credit - 22 bits value\n"
                    "Min credit - 22 bits value\n"
                    "Rate - rate in Kbit/s or packets/s (integer value only)\n"
                    "Position - position, where the shaper works (range 1 - 17)\n"
                    "           1 - output of the scheduler 2\n"
                    "           2 ... 9 - inputs of the scheduler 2\n"
                    "           10 ... 17 - inputs of the scheduler 1\n"
                    "           2 - output of the scheduler 1, if \"Sch2 input to which Sch1 is going = 1\" in file \"scheduler_settings/%s_schedulers\" and other queues are disconnected at input 1 of the scheduler 2\n"
                    "Mode - 0 for zero burst mode and 1 for bursty mode\n"
                    "Enable - 0 - shaper disable, 1 - shaper enable\n", str_ptr );

    kfree( params );
    kfree( str_ptr );
    mutex_unlock( &mutexProcTMU );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: end\n", __func__ );
    return 0;
}

static ssize_t shaper_write( struct file *file, const char *buf,
                           size_t count, loff_t *ppos ) {

    int file_id, res, index;
    uint32_t param , en = 1;
    tlite_shpr_params_t *pshp;
    uint8_t *buff;
    size_t result;

    file_id = *( uint32_t* )PDE_DATA( file_inode( file ) );

    if( fp_logical_dev[ 0 ] == NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "SHAPER_WRITE: the device doesn't OPEN!\n" );
        return 0;
    }

    if( file_id >= MAX_FILES_PORT_SHAPERS ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "SHAPER_WRITE: bad file id!\n" );
        return 0;
    }

    mutex_lock( &mutexProcTMU );
    result = 0;
    buff = FP_NULL;
    pshp = FP_NULL;
    buff = kmalloc( PROC_BUF_SIZE, GFP_KERNEL );
    if( buff == FP_NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "SHAPER_WRITE: can not allocate memory for user data\n" );
        goto exit_shaper_write;
    }

    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "SHAPER_WRITE: start\n" );

    if( count > PROC_BUF_SIZE ){
        count = PROC_BUF_SIZE;
    }

    res = copy_from_user( buff, ( void* )buf, count );
    if( res ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "SHAPER_WRITE: can not get data from user\n" );
        goto exit_shaper_write;   
    }
    buff[ count -1 ] = '\0';

    pshp = kmalloc( sizeof( tlite_shpr_params_t ), GFP_KERNEL );
    if( pshp == FP_NULL ){
        FP_DEBUG( DBGP_FEAT_IOCTL, FP_LOG_INFO, "SHAPER_WRITE: can not allocate memory for struct tlite_shpr_params_t\n" );
        goto exit_shaper_write;
    }
    memset( pshp, 0, sizeof( tlite_shpr_params_t ) );
    result = count;
    /* Switch port */
    pshp->tlsp_dev_id = file_id + 1;

    /* Shaper number */
    index = 0;
    res = sscanf( &buff[ index ], "Shaper: %d", &param );
    if( res != 1 ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "SHAPER_WRITE: can not get \"Shaper\" value from file!\n" );
        goto exit_shaper_write;
    }

    if( ( param < TLITE_MIN_SHPR_NUM ) || ( param > TLITE_MAX_SHPR_NUM ) ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "SHAPER_WRITE: \"Shaper\" number is failed!\n" );
        goto exit_shaper_write; 
    }
    pshp->tlsp_shpr_id = param & 7;
    shaper_number[ file_id ] = pshp->tlsp_shpr_id - 1;

    /* Shaper rate units */
    while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}

    result = count;
    res = sscanf( &buff[ index ], "Rate units: %d", &param );
    if( res != 1 ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "SHAPER_WRITE: can not get \"Rate units\" value from file!\n" );
        goto exit_shaper_write;
    }
    pshp->tlsp_rate_units = param & 3;

    /* Shaper max_credit */
    while( ( buff[ index++ ] != '\n') && ( index < ( count - 1 ) ) ){}

    res = sscanf( &buff[ index ], "Max credit: %d", &param );
    if( res != 1 ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "SHAPER_WRITE: can not get \"Max credit\" value from file!\n" );
        goto exit_shaper_write;
    }
    pshp->tlsp_max_credit = param & MAX_MAX_CREDIT;

    /* Shaper min_credit */
    while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}

    res = sscanf( &buff[ index ], "Min credit: %d", &param );
    if( res != 1 ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "SHAPER_WRITE: can not get \"Min credit\" value from file!\n" );
        goto exit_shaper_write;
    }
    pshp->tlsp_min_credit = param & MAX_MAX_CREDIT;

    /* Shaper rate */
    while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}

    res = sscanf( &buff[ index ], "Rate: %d", &param );
    if( res != 1 ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "SHAPER_WRITE: can not get \"Rate\" value from file!\n" );
        goto exit_shaper_write;
    }
    pshp->tlsp_rate = param;

    /* Shaper position */
    while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}

    res = sscanf( &buff[ index ], "Position: %d", &param );
    if( res != 1 ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "SHAPER_WRITE: can not get \"Position\" value from file!\n" );
        goto exit_shaper_write;
    }
    pshp->tlsp_position = param & 0x1F;

    /* Shaper mode */
    while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}

    res = sscanf( &buff[ index ], "Mode: %d", &param );
    if( res != 1 ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "SHAPER_WRITE: can not get \"Mode\" value from file!\n" );
        goto exit_shaper_write;
    }
    pshp->tlsp_mode = param & 1;

    /* Shaper enable */
    while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}

    res = sscanf( &buff[ index ], "Enable: %d", &param );
    if( res != 1 ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "SHAPER_WRITE: can not get \"Enable\" value from file!\n" );
    }else{
        en = param & 1;
    }

    pshp->tlsp_clk_frequency = 200;
    
    pfe_tlite_dynamic_config_enable( TLITE_SWITCH );

    if( en ){
        pcm_tlite_shaper_config( pshp, TLITE_SWITCH, 0 );
    }else{
        pcm_tlite_shaper_disable( pshp, TLITE_SWITCH, 0 );
    }
    pfe_tlite_dynamic_config_done( );

exit_shaper_write:
    if( buff != FP_NULL ){
        kfree( buff );
    }
    if( pshp != FP_NULL ){
        kfree( pshp );
    }
    mutex_unlock( &mutexProcTMU );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "SHAPER_WRITE: end\n" );
    return result;
}

static int shaper_open( struct inode *inode, struct file *file ){
    return single_open( file, shaper_read, NULL );
}

static const struct file_operations shaper_fops = {
    .owner = THIS_MODULE,
    .open  = shaper_open,
    .read  = seq_read,
    .llseek = seq_lseek,
    .release = single_release,
    .write = shaper_write,
};

//static const struct file_operations shaper_fops = {
//    .owner = THIS_MODULE,
//    .read  = shaper_read,
//    .write = shaper_write,
//};


static ssize_t dos_control_write( struct file *file, const char *buf,
                           size_t count, loff_t *ppos ) {
    int file_id, res, index;
    uint32_t param, dos_ctrl_1, dos_ctrl_2, dos_tcp_flagchk;
    size_t result;
    uint8_t *buff;

    file_id = *( uint32_t* )PDE_DATA( file_inode( file ) );

    if( fp_logical_dev[ 0 ] == NULL ){
        FP_DEBUG(DBGP_FEAT_PROCFS, FP_LOG_ERR, "DOS_CONTROL_WRITE: the device doesn't OPEN!\n");
        return count;
    }

    if( file_id >= MAX_FILES_DOS_CONTROL ){
        FP_DEBUG(DBGP_FEAT_PROCFS, FP_LOG_ERR, "DOS_CONTROL_WRITE: bad file id!\n");
        return count;
    }

    FP_DEBUG(DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "DOS_CONTROL_WRITE: start\n");
    
    mutex_lock( &mutexProcClass );
    result = 0;
    buff = FP_NULL;
    buff = kmalloc( PROC_BUF_SIZE, GFP_KERNEL );
    if( buff == FP_NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DOS_CONTROL_WRITE: can not allocate memory for user data\n" );
        goto exit_dos_control_write;
    }

    if( count > PROC_BUF_SIZE ){
       count = PROC_BUF_SIZE - 1;
    }

    res = copy_from_user( buff, (void*)buf, count );
    if( res ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DOS_CONTROL_WRITE: can not get data from user\n" );
        goto exit_dos_control_write;   
    }
    buff[ count -1 ] = '\0';

    dos_ctrl_1 = 0;
    dos_ctrl_2 = 0;
    dos_tcp_flagchk = 0;

    result = count;
    index = 0;
    res = sscanf( &buff[ index ], "Dos enable: %d", &param );
    if( res != 1 ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DOS_CONTROL_WRITE: can not get \"Dos enable\" value from file!\n" );
        goto exit_dos_control_write;
    }
    while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}

    dos_ctrl_1 = param & 1;

    res = sscanf( &buff[ index ], "Dos action: %d", &param );
    if( res != 1 ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DOS_CONTROL_WRITE: can not get \"Dos action\" value from file!\n" );
        goto exit_dos_control_write;
    }
    while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}

    dos_ctrl_1 |= ( ( param & 7 ) << 1 );

    res = sscanf( &buff[ index ], "Dos tcp flags combination enable: 0x%x", &param );
    if( res != 1 ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DOS_CONTROL_WRITE: can not get \"Dos tcp flags combination enable\" value from file!\n" );
        goto exit_dos_control_write;
    }
    while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}

    dos_ctrl_1 |= ( ( param & 0xF ) << 4 );
    /* Gets all data for write it in CLASS_DOS_CONTRL1 register */

    res = sscanf( &buff[ index ], "Dos control 2 bits: 0x%x", &param );
    if( res != 1 ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DOS_CONTROL_WRITE: can not get \"Dos control 2 bits\" value from file!\n" );
        goto exit_dos_control_write;
    }
    while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}

    dos_ctrl_2 = ( param & 0x03FF03FF );
    /* Gets all data for write it in CLASS_DOS_CONTRL2 register */

    res = sscanf( &buff[ index ], "Dos tcp flags comdo 1: 0x%x", &param );
    if( res != 1 ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DOS_CONTROL_WRITE: can not get \"Dos tcp flags comdo 1\" value from file!\n" );
        goto exit_dos_control_write;
    }
    while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1) ) ){}

    dos_tcp_flagchk = ( param & 0x3F );
    
    res = sscanf( &buff[ index ], "Dos tcp flags comdo 2: 0x%x", &param );
    if(res != 1){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DOS_CONTROL_WRITE: can not get \"Dos tcp flags comdo 2\" value from file!\n" );
        goto exit_dos_control_write;
    }
    while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}

    dos_tcp_flagchk |= ( param & 0x3F ) << 6;
    
    res = sscanf( &buff[ index ], "Dos tcp flags comdo 3: 0x%x", &param );
    if( res != 1 ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DOS_CONTROL_WRITE: can not get \"Dos tcp flags comdo 3\" value from file!\n" );
        goto exit_dos_control_write;
    }
    while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}

    dos_tcp_flagchk |= ( param & 0x3F ) << 12;

    res = sscanf( &buff[ index ], "Dos tcp flags comdo 4: 0x%x", &param );
    if( res != 1 ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DOS_CONTROL_WRITE: can not get \"Dos tcp flags comdo 4\" value from file!\n" );
        goto exit_dos_control_write;
    }
    while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}

    dos_tcp_flagchk |= ( param & 0x3F ) << 18;
    /* Gets all data for write it in CLASS_DOS_TCP_FLAGCHK_COMB_VALUE1 register */

    fp_reg_write( CLASS_HW_BASE_ADDR + CLASS_DOS_CONTRL1, dos_ctrl_1 );
    fp_reg_write( CLASS2_HW_BASE_ADDR + CLASS_DOS_CONTRL1, dos_ctrl_1 );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "DOS_CONTROL_WRITE: CLASS1, CLASS2, DOS CONTRL1 write 0x%08X\n", dos_ctrl_1 );
    
    fp_reg_write( CLASS_HW_BASE_ADDR + CLASS_DOS_CONTRL2, dos_ctrl_2 );
    fp_reg_write( CLASS2_HW_BASE_ADDR + CLASS_DOS_CONTRL2, dos_ctrl_2 );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "DOS_CONTROL_WRITE: CLASS1, CLASS2, DOS CONTRL2 write 0x%08X\n", dos_ctrl_2 );

    fp_reg_write( CLASS_HW_BASE_ADDR + CLASS_DOS_TCP_FLAGCHK_COMBO_VALUE1, dos_tcp_flagchk );
    fp_reg_write( CLASS2_HW_BASE_ADDR + CLASS_DOS_TCP_FLAGCHK_COMBO_VALUE1, dos_tcp_flagchk );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "DOS_CONTROL_WRITE: CLASS1, CLASS2, DOS TCP FLAGCHK COMBO VALUE1 write 0x%08X\n", dos_tcp_flagchk );

    res = sscanf( &buff[ index ], "Dos icmp_v4 max packet length: %d", &param );
    if( res != 1 ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "DOS_CONTROL_WRITE: can not get \"Dos icmp_v4 max packet length\" value from file!\n" );
        goto exit_dos_control_write;
    }
    while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
    param &= 0xFFFF;
    /* Gets all data to write in SWITCH */

    fp_reg_write( CLASS_HW_BASE_ADDR + CLASS_DOS_ICMPV4_MAX_PKTLEN, param );
    fp_reg_write( CLASS2_HW_BASE_ADDR + CLASS_DOS_ICMPV4_MAX_PKTLEN, param );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "DOS_CONTROL_WRITE: CLASS1, CLASS2, DOS ICMPV4 MAX PKTLEN write 0x%08X\n", param );

exit_dos_control_write:
    if( buff != FP_NULL ){
        kfree( buff );
    }
    mutex_unlock( &mutexProcClass );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "DOS_CONTROL_WRITE: end\n");
    return result;
}

static int dos_control_read( struct seq_file *f, void *v ) {
    int file_id;
    uint32_t tmp;

    if( fp_logical_dev[ 0 ] == NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, the device doesn't OPEN!\n", __func__ );
        return 0;
    }

    file_id = *( uint32_t* )PDE_DATA( file_inode( f->file ) );
    if( file_id >= MAX_FILES_DOS_CONTROL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, invalid file id %d!\n", __func__, file_id );
        return 0;
    }

    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: start\n", __func__ );
    mutex_lock( &mutexProcClass );
    tmp = fp_reg_read( CLASS_HW_BASE_ADDR + CLASS_DOS_CONTRL1 );
    seq_printf( f,  "Dos enable: %d\n"
                    "Dos action: %d\n"
                    "Dos tcp flags combination enable: 0x%x\n",
                    ( tmp & 1 ),
                    ( ( tmp >> 1 ) & 7 ),
                    ( ( tmp >> 4 ) & 0xF ) );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: CLASS_DOS_CONTRL1 = 0x%08X\n", __func__, tmp );

    tmp = fp_reg_read( CLASS_HW_BASE_ADDR + CLASS_DOS_CONTRL2 );
    seq_printf( f,  "Dos control 2 bits: 0x%08X\n", ( tmp & 0x03FF03FF ) );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: CLASS_DOS_CONTRL2 = 0x%08X\n", __func__, tmp );

    tmp = fp_reg_read( CLASS_HW_BASE_ADDR + CLASS_DOS_TCP_FLAGCHK_COMBO_VALUE1 );
    seq_printf( f,  "Dos tcp flags comdo 1: 0x%02x\n"
                    "Dos tcp flags comdo 2: 0x%02x\n"
                    "Dos tcp flags comdo 3: 0x%02x\n"
                    "Dos tcp flags comdo 4: 0x%02x\n",
                    ( tmp & 0x3F ),
                    ( ( tmp >> 6 ) & 0x3F ),
                    ( ( tmp >> 12 ) & 0x3F ),
                    ( ( tmp >> 18 ) & 0x3F ) );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: CLASS_DOS_TCP_FLAGCHK_COMBO_VALUE1 = 0x%08X\n", __func__, tmp );

    tmp = fp_reg_read( CLASS_HW_BASE_ADDR + CLASS_DOS_ICMPV4_MAX_PKTLEN );
    seq_printf( f, "Dos icmp_v4 max packet length: %d\n", ( tmp & 0xFFFF ) );
    mutex_unlock( &mutexProcClass );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: CLASS_DOS_ICMPV4_MAX_PKTLEN = %d\n", __func__, tmp );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: end\n", __func__ );
    return 0;
}

static int dos_control_open( struct inode *inode, struct file *file ){
    return single_open( file, dos_control_read, NULL );
}

static const struct file_operations dos_fops = {
    .owner = THIS_MODULE,
    .open  = dos_control_open,
    .read  = seq_read,
    .llseek = seq_lseek,
    .release = single_release,
    .write = dos_control_write,
};

//static const struct file_operations dos_fops = {
//    .owner = THIS_MODULE,
//    .read  = dos_control_read,
//    .write = dos_control_write,
//};

static ssize_t pvlan_settings_write( struct file *file, const char *buf,
                           size_t count, loff_t *ppos ) {
    int32_t file_id, res, index;
    uint32_t param, i;
    struct port_s *pport;
    uint8_t *buff;
    size_t result;

    file_id = *( uint32_t* )PDE_DATA( file_inode( file ) );

    if( fp_logical_dev[ 0 ] == NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PVLAN_SETTINGS_WRITE: the device doesn't OPEN!\n" );
        return count;
    }

    if( file_id >= MAX_FILE_PVLAN_PORT ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PVLAN_SETTINGS_WRITE: bad file id!\n" );
        return count;
    }

    mutex_lock( &mutexProcHash );
    result = 0;
    buff = FP_NULL;
    pport = FP_NULL;
    buff = kmalloc( PROC_BUF_SIZE, GFP_KERNEL );
    if( buff == FP_NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PVLAN_SETTINGS_WRITE: can not allocate memory for user data\n" );
        goto exit_pvlan_settings_write;
    }

    FP_DEBUG(DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "PVLAN_SETTINGS_WRITE: start\n");

    if( count > PROC_BUF_SIZE ){
       count = PROC_BUF_SIZE - 1;
    }

    res = copy_from_user( buff, ( void* )buf, count );
    if( res ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PVLAN_SETTINGS_WRITE: can not get data from user\n" );
        goto exit_pvlan_settings_write;   
    }
    buff[ count - 1 ] = '\0';

    pport = kmalloc( sizeof( struct port_s ), GFP_KERNEL );
    if( pport == FP_NULL ){
        FP_DEBUG( DBGP_FEAT_IOCTL, FP_LOG_INFO, "PVLAN_SETTINGS_WRITE: can not allocate memory for struct port_s\n" );
        goto exit_pvlan_settings_write;
    }
    memset( pport, 0, sizeof( struct port_s ) );

    index = 0;
    result = count;

    res = sscanf( &buff[ index ], "Private vlan: %d", &param );
    if( res != 1 ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PVLAN_SETTINGS_WRITE: can not get \"Private vlan\" value from file!\n" );
        goto exit_pvlan_settings_write;
    }

    while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
    pport->pvlan_enable = param & 1;

    res = sscanf( &buff[ index ], "Port type: %d", &param );
    if( res != 1 ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PVLAN_SETTINGS_WRITE: can not get \"Port type\" value from file!\n" );
        goto exit_pvlan_settings_write;
    }

    while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
    pport->pvlan_type = param & 3;

    res = sscanf( &buff[ index ], "Primary vlan: %d", &param );
    if( res != 1 ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PVLAN_SETTINGS_WRITE: can not get \"Primary vlan\" value from file!\n" );
        goto exit_pvlan_settings_write;
    }

    while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
    pport->pvlan_pri = param & 0xFFF;

    switch( pport->pvlan_type ){
        case ISOLATED:
        case COMMUNITY:
            res = sscanf( &buff[ index ], "Secondary vlan: %d", &param );
            if( res != 1 ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PVLAN_SETTINGS_WRITE: can not get \"Secondary vlan\" value for ISOLATED or COMMUNITY port from file!\n" );
                goto exit_pvlan_settings_write;
            }
            pport->num_sec = 1;
            pport->pvlan_sec[ 0 ] = param & 0xFFF;
            break;

        case PROMISCUOUS:
        case INTERSWITCH:
            param = sizeof( "Secondary vlan: " ) - 1;
            i = strncmp( &buff[ index ], "Secondary vlan: ", param );
            if( !i ){
                index += param;
                while( ( pport->num_sec < FP_MAX_SWITCH_PORTS ) && ( index < ( count - 1 ) ) ){
                    res = sscanf( &buff[ index ], "%hd", &pport->pvlan_sec[ pport->num_sec ] );
                    if( res != 1 ){
                        break;
                    }
                    pport->num_sec++;
                    while( ( buff[ index++ ] != ' ' ) && ( index < ( count - 1 ) ) ){}
                }                
            }else{
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PVLAN_SETTINGS_WRITE: can not get \"Secondary vlan\" value for PROMISCUOUS or INTERSWITCH port from file!\nsize %d, eq result %d string %s\n", param , i, &buff[ index ] );
                goto exit_pvlan_settings_write;
            }
            break;
    }
    FP_DEBUG(DBGP_FEAT_PROCFS, FP_LOG_ERR, "PVLAN_SETTINGS_WRITE: total sec vlan is %d\n", pport->num_sec );

    fp_pvlan_config( pport->pvlan_enable, ( file_id + 1 ), pport );

exit_pvlan_settings_write:
    if( buff != FP_NULL ){
        kfree( buff );
    }
    if( pport != FP_NULL ){
        kfree( pport );
    }
    mutex_unlock( &mutexProcHash );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "PVLAN_SETTINGS_WRITE: end\n" );
    return result;
}

static int pvlan_settings_read( struct seq_file *f, void *v ) {
    int file_id;
    uint32_t tmp;
    struct port_s *port;

    if( fp_logical_dev[ 0 ] == NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, the device doesn't OPEN!\n", __func__ );
        return 0;
    }

    file_id = *( uint32_t* )PDE_DATA( file_inode( f->file ) );
    if( file_id >= MAX_FILE_PVLAN_PORT ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, invalid file id %d!\n", __func__, file_id );
        return 0;
    }

    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: start\n", __func__ );
    mutex_lock( &mutexProcHash );
    port = &fp_port[ file_id ];
    seq_printf( f, "Private vlan: %d\n", port->pvlan_enable );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: port_%d pvlan settings: pvlan: %s ", __func__, file_id, ( port->pvlan_enable ? "enable" : "disable" ) );
    seq_printf( f, "Port type: %d\n", port->pvlan_type );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: type: %d\n", __func__, port->pvlan_type );
    seq_printf( f, "Primary vlan: %d\n", port->pvlan_pri );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: prim vlan: %d\n", __func__, port->pvlan_pri );
    
    switch( port->pvlan_type ){
        case ISOLATED:
        case COMMUNITY:
            seq_printf( f, "Secondary vlan: %d\n", port->pvlan_sec[ 0 ] );
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: sec vlan: %d\n", __func__, port->pvlan_sec[ 0 ] );
            break;
        case PROMISCUOUS:
        case INTERSWITCH:
            seq_printf( f, "Secondary vlan:" );
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: sec vlan:\n", __func__ );

            for( tmp = 0; tmp < port->num_sec; tmp++ ){
                seq_printf( f, " %d", port->pvlan_sec[ tmp ] );
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: %d\n", __func__, port->pvlan_sec[ tmp ] );
            }
            seq_printf( f, "\n" );
            break;
        default:
            break;
    }

    seq_printf( f,    "----------------------------------------------------------\n"
                    "Private vlan - enable (1) or disable (0)\n"
                    "Port type: 0 - ISOLATED, 1 - COMMUNITY, 2 - PROMISCUOUS, 3 - INTERSWITCH\n"
                    "Primary vlan: range 2 - 4094\n"
                    "Secondary vlan: range 2 - 4094 (do not match with primary vlan!); for "
                    "PROMISCUOUS and INTERSWITCH port types has a several (but less then 16) "
                    "values with separated by space\n" );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: end\n", __func__ );
    mutex_unlock( &mutexProcHash );
    return 0;
}

static int pvlan_settings_open( struct inode *inode, struct file *file ){
    return single_open( file, pvlan_settings_read, NULL );
}

static const struct file_operations pvlan_settings_fops = {
    .owner = THIS_MODULE,
    .open  = pvlan_settings_open,
    .read  = seq_read,
    .llseek = seq_lseek,
    .release = single_release,
    .write = pvlan_settings_write,
};

//static const struct file_operations pvlan_settings_fops = {
//    .owner = THIS_MODULE,
//    .read  = pvlan_settings_read,
//    .write = pvlan_settings_write,
//};

uint8_t *qos_tc_sel_table_string_read[8] = {    "Dynamic_Untag_notIP:",
                                                "Dynamic_Untag_IP:",
                                                "Dynamic_Tagged_notIP:",
                                                "Dynamic_Tagged_IP:",
                                                "Static_Untag_notIP:",
                                                "Static_Untag_IP:",
                                                "Static_Tagged_notIP:",
                                                "Static_Tagged_IP:" };

uint8_t *qos_tc_sel_table_string_write[8] = {   "Dynamic_Untag_notIP: %d",
                                                "Dynamic_Untag_IP: %d",
                                                "Dynamic_Tagged_notIP: %d",
                                                "Dynamic_Tagged_IP: %d",
                                                "Static_Untag_notIP: %d",
                                                "Static_Untag_IP: %d",
                                                "Static_Tagged_notIP: %d",
                                                "Static_Tagged_IP: %d" };


static ssize_t pcks_to_queue_map_write( struct file *file, const char *buf,
                           size_t count, loff_t *ppos ) {
    int32_t file_id, res, index;
    uint32_t param, i, j;
    uint32_t *param_arrays;
    uint8_t *buff;
    size_t result;

    file_id = *( uint32_t* )PDE_DATA( file_inode( file ) );

    if( fp_logical_dev[ 0 ] == NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PCKS_TO_QUEUE_MAP_WRITE: the device doesn't OPEN!\n" );
        return count;
    }

    if(file_id >= MAX_FILE_PCKS_TO_QUEUES){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PCKS_TO_QUEUE_MAP_WRITE: bad file id!\n" );
        return count;
    }

    mutex_lock( &mutexProcClass );
    result = 0;
    buff = FP_NULL;
    param_arrays = FP_NULL;
    buff = kmalloc( PROC_BUF_SIZE, GFP_KERNEL );
    if( buff == FP_NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PCKS_TO_QUEUE_MAP_WRITE: can not allocate memory for user data\n" );
        goto exit_pcks_to_queue_map_write;
    }

    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "PCKS_TO_QUEUE_MAP_WRITE: start\n" );

    if( count > PROC_BUF_SIZE ){
       count = PROC_BUF_SIZE - 1;
    }

    res = copy_from_user( buff, ( void* )buf, count );
    if( res ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PCKS_TO_QUEUE_MAP_WRITE: can not copy data from user!\n" );
        goto exit_pcks_to_queue_map_write;
    }
    buff[ count - 1 ] = '\0';

    index = 0;
    result = count;

    while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}

    if( file_id == FILE_PCKS_TO_QUEUES_GLOBAL_DSCP_2_TC ){
/*
File format:
--- DSCP_TO_TC_MAP ---
qos global dscp00 - dscp09 to TC: 5 1 8 3 4 5 6 7 8 1
qos global dscp10 - dscp19 to TC: 5 1 8 3 4 5 6 7 8 1
qos global dscp20 - dscp29 to TC: 5 1 8 3 4 5 6 7 8 1
qos global dscp30 - dscp39 to TC: 5 1 8 3 4 5 6 7 8 1
qos global dscp40 - dscp49 to TC: 5 1 8 3 4 5 6 7 8 1
qos global dscp50 - dscp59 to TC: 5 1 8 3 4 5 6 7 8 1
qos global dscp60 - dscp63 to TC: 5 1 8 3
----------------------------------------------------------
DSCP_TO_TC_MAP - range 1 to 8; TC value to which DSCP value of IP header is mapping
For example:
qos global dscp00 - dscp09 to TC: 5 1 8 3 4 5 6 7 8 1
it is mean, for ip packet with DSCP value 0 the TC value is 5,
for ip packet with DSCP value 1 the TC value is 1,
for ip packet with DSCP value 2 the TC value is 8, ...
*/
        param_arrays = kmalloc( ( ( sizeof( uint32_t ) ) * 12 ), GFP_KERNEL );
        if( param_arrays == FP_NULL ){
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PCKS_TO_QUEUE_MAP_WRITE: can not allocate memory for params\n" );
            goto exit_pcks_to_queue_map_write;
        }

        for( i = 0; i < 7; i++ ){
            if( i < 6 ){
                res = sscanf( &buff[ index ], "qos global dscp%d - dscp%d to TC: %d %d %d %d %d %d %d %d %d %d", 
                    &param_arrays[ 0 ], &param_arrays[ 1 ],
                    &param_arrays[ 2 ], &param_arrays[ 3 ], &param_arrays[ 4 ], &param_arrays[ 5 ], &param_arrays[ 6 ], 
                    &param_arrays[ 7 ], &param_arrays[ 8 ], &param_arrays[ 9 ], &param_arrays[ 10 ], &param_arrays[ 11 ] );
                if( res != 12 ){
                    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PCKS_TO_QUEUE_MAP_WRITE: can not get \"qos global dscp%d0 - dscp%d9 to TC\" value from file!\n", i, i );
                    goto exit_pcks_to_queue_map_write;
                }
            }else{
                res = sscanf( &buff[ index ], "qos global dscp%d - dscp%d to TC: %d %d %d %d", 
                    &param_arrays[ 0 ], &param_arrays[ 1 ],
                    &param_arrays[ 2 ], &param_arrays[ 3 ], &param_arrays[ 4 ], &param_arrays[ 5 ] );
                if( res != 6 ){
                    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PCKS_TO_QUEUE_MAP_WRITE: can not get \"qos global dscp60 - dscp63 to TC\" value from file!\n" );
                    goto exit_pcks_to_queue_map_write;
                }
            }

            param = 0;
            switch( i ){
                case 0:
                    if( ( param_arrays[ 0 ] != 0 ) || ( param_arrays[ 1 ] != 9 ) ){
                        param++;
                    }
                    break;
                case 1:
                    if( ( param_arrays[ 0 ] != 10 ) || ( param_arrays[ 1 ] != 19 ) ){
                        param++;
                    }
                    break;
                case 2:
                    if( ( param_arrays[ 0 ] != 20 ) || ( param_arrays[ 1 ] != 29 ) ){
                        param++;
                    }
                    break;
                case 3:
                    if( ( param_arrays[ 0 ] != 30 ) || ( param_arrays[ 1 ] != 39 ) ){
                        param++;
                    }
                    break;
                case 4:
                    if( ( param_arrays[ 0 ] != 40 ) || ( param_arrays[ 1 ] != 49 ) ){
                        param++;
                    }
                    break;
                case 5:
                    if( ( param_arrays[ 0 ] != 50 ) || ( param_arrays[ 1 ] != 59 ) ){
                        param++;
                    }
                    break;
                case 6:
                    if( ( param_arrays[ 0 ] != 60 ) || ( param_arrays[ 1 ] != 63 ) ){
                        param++;
                    }
                    break;
                default:
                    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PCKS_TO_QUEUE_MAP_WRITE: should never come here!\n" );
                    goto exit_pcks_to_queue_map_write;
                    break;
            }

            if(param){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PCKS_TO_QUEUE_MAP_WRITE: bad file text format!\n" );
                goto exit_pcks_to_queue_map_write;
            }

            param = 0;
            if( i < 6 ){
                for( j = 0; j < 10; j++ ){
                    param = param << 3;
                    param |= ( ( param_arrays[ 11 - j ] - 1 ) & 7 );
                }
            }else{
                for( j = 0; j < 4; j++ ){
                    param = param << 3;
                    param |= ( ( param_arrays[ 5 - j ] - 1 ) & 7 );
                }
            }

            fp_reg_write( ( CLASS_HW_BASE_ADDR + CLASS_QOS_GLOBAL_DSCP2TC_MAP0 + ( i << 2 ) ), param );
            fp_reg_write( ( CLASS2_HW_BASE_ADDR + CLASS_QOS_GLOBAL_DSCP2TC_MAP0 + ( i << 2 ) ), param );

            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "PCKS_TO_QUEUE_MAP_WRITE: new value CLASS_QOS_GLOBAL_DSCP2TC_MAP%d 0x%08X\n", i, param );

            while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
        }

    }else{
/*
File format:
--- QOS_TC_SEL_TABLE ---
Dynamic_Untag_notIP: 1
Dynamic_Untag_IP: 2
Dynamic_Tagged_notIP: 3
Dynamic_Tagged_IP: 4
Static_Untag_notIP: 3
Static_Untag_IP: 2
Static_Tagged_notIP: 1
Static_Tagged_IP: 4
--- PCP_TO_TC_MAP ---
PCP0: 1
PCP1: 2
PCP2: 3
PCP3: 4
PCP4: 5
PCP5: 6
PCP6: 7
PCP7: 8
--- PID_TO_TC_MAP ---
Port1: 3
--- TC_TO_QUEUE_MAP ---
TC1: 1
TC2: 2
TC3: 3
TC4: 4
TC5: 5
TC6: 6
TC7: 7
TC8: 8
----------------------------------------------------------
QOS_TC_SEL_TABLE - packet queueing principle (1 - DSCP, 2 - PCP, 3 - MAC, 4 - PID)
For example:
Dynamic_Untag_notIP: 4 - it is mean, if received packet is untagged and not IP packet, and
hasn't static entry in MAC Table for it src MAC, that TC value based by port number,
where it was received.
PCP_TO_TC_MAP - range 1 to 8; TC value to which Prority Code Point (PCP) from vlan tag is mapping
(if QOS_TC_SEL_TABLE equal 2 for this packet)
For example:
PCP5: 1 - it is mean, if received packet has pcp field 5, that TC value is 1
PID_TO_TC_MAP - range 1 to 8; TC value to which receiv Port number is mapping (if QOS_TC_SEL_TABLE equal 4 for this packet)
For example:
Port10: 2 - it is mean, if packet was received by port 10, that TC value is 2
TC_TO_QUEUE_MAP - range 1 to 8; QUEUE number to which packets with specify TC is mapping
For example:
TC5: 2 - it is mean, if packet has TC 5, that it will send through queue 2
DSCP_TO_TC_MAP - for DSCP to TC map you need write data in global_dscp_pcks_to_queues_map file
*/
        param_arrays = kmalloc( ( ( sizeof( uint32_t ) ) * 2 ), GFP_KERNEL );
        if( param_arrays == FP_NULL ){
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PCKS_TO_QUEUE_MAP_WRITE: can not allocate memory for params\n" );
            goto exit_pcks_to_queue_map_write;
        }

        /*--- QOS_TC_SEL_TABLE ---*/
        param = 0;
        for( i = 0; i < 8; i++ ){
            res = sscanf( &buff[ index ], qos_tc_sel_table_string_write[ i ], &param_arrays[ 0 ] );
            if( res != 1 ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PCKS_TO_QUEUE_MAP_WRITE: can not get \"%s\" value from file!\n", qos_tc_sel_table_string_read[ i ] );
                goto exit_pcks_to_queue_map_write;
            }

            param |= ( ( ( param_arrays[ 0 ] - 1 ) & 3 ) << ( i << 1 ) );

            while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
        }

        j = fp_reg_read( hwQosTcSel[ 0 ][ file_id ] );   //get tc_sel table  
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "PCKS_TO_QUEUE_MAP_WRITE: read value CLASS_QOS_PORT%d%d_TC_SEL 0x%08X\n", ( file_id - 1 ), file_id, j );

        if( file_id & 1 ){    //for ports 1, 3, 5, 7, 9, 11, 13, 15, 17
            j = ( ( j & 0xFFFF ) | ( param << 16 ) );
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "PCKS_TO_QUEUE_MAP_WRITE: new value CLASS_QOS_PORT%d%d_TC_SEL 0x%08X\n", ( file_id - 1 ), file_id, j );
        }else{              //for ports 0, 2, 4, 8, 10, 12, 14, 16
            j = ( ( j & 0xFFFF0000 ) | param );
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "PCKS_TO_QUEUE_MAP_WRITE: new value CLASS_QOS_PORT%d%d_TC_SEL 0x%08X\n", file_id, ( file_id + 1 ), j );
        }

        fp_reg_write( hwQosTcSel[ 0 ][ file_id ], j );
        fp_reg_write( hwQosTcSel[ 1 ][ file_id ], j );
        fp_port[ file_id ].tcsel_table = param;

        /*-- PCP_TO_TC_MAP ---*/
        while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
        param = 0;
        for( i = 0; i < 8; i++ ){
            res = sscanf( &buff[ index ], "PCP%d: %d", &param_arrays[ 0 ], &param_arrays[ 1 ] );
            if( res != 2 ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PCKS_TO_QUEUE_MAP_WRITE: can not get \"PCP%d\" value from file!\n", i );
                goto exit_pcks_to_queue_map_write;
            }

            if( param_arrays[ 0 ] != i ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PCKS_TO_QUEUE_MAP_WRITE: bad PCP number, read %d, but range: 1 to 8 !\n", param_arrays[ 0 ] );
                goto exit_pcks_to_queue_map_write;   
            }

            param |= ( ( ( param_arrays[ 1 ] - 1 ) & 7 ) << ( i * 3 ) );

            while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
        }

        fp_reg_write( hwQosPcp2Tc[ 0 ][ file_id ], param );
        fp_reg_write( hwQosPcp2Tc[ 1 ][ file_id ], param );
        fp_port[ file_id ].pcp2tc_table = param;
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "PCKS_TO_QUEUE_MAP_WRITE: new value CLASS_QOS_PORT%d_PCP2TC_MAP 0x%08X\n", file_id, param );

        /*--- PID_TO_TC_MAP ---*/
        while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
        res = sscanf( &buff[ index ], "Port%d: %d", &param_arrays[ 0 ], &param_arrays[ 1 ] );
        if( res != 2 ){
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PCKS_TO_QUEUE_MAP_WRITE: can not get \"Port%d\" value from file!\n", i );
            goto exit_pcks_to_queue_map_write;
        }

        if( ( file_id + 1 ) != param_arrays[ 0 ] ){
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PCKS_TO_QUEUE_MAP_WRITE: bad port number, read %d, but waiting %d\n", param_arrays[ 0 ], ( file_id + 1 ) );
            goto exit_pcks_to_queue_map_write;   
        }

        if( file_id < 10 ){
            i = CLASS_QOS_PORT_PID2TC_MAP;
            param = fp_reg_read( ( CLASS_HW_BASE_ADDR + i ) );
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "PCKS_TO_QUEUE_MAP_WRITE: read value CLASS_QOS_PORT_PID2TC_MAP 0x%08X\n", param );
            param = ( param & ( ~( 7 << ( 3 * file_id ) ) ) ) | ( ( ( param_arrays[ 1 ] - 1 ) & 7 ) << ( 3 * file_id ) );
            fp_reg_write( ( CLASS_HW_BASE_ADDR + i ), param );
            fp_reg_write( ( CLASS2_HW_BASE_ADDR + i ), param );
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "PCKS_TO_QUEUE_MAP_WRITE: new value CLASS_QOS_PORT_PID2TC_MAP 0x%08X\n", param );
        }else{
            i = CLASS_QOS_PORT1_PID2TC_MAP;
            param = fp_reg_read( ( CLASS_HW_BASE_ADDR + i ) );
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "PCKS_TO_QUEUE_MAP_WRITE: read value CLASS_QOS_PORT1_PID2TC_MAP 0x%08X\n", param );
            param = ( param & ( ~( 7 << ( 3 * ( file_id - 10 ) ) ) ) ) | ( ( ( param_arrays[ 1 ] - 1 ) & 7 ) << ( 3 * ( file_id - 10 ) ) );
            fp_reg_write( ( CLASS_HW_BASE_ADDR + i ), param );
            fp_reg_write( ( CLASS2_HW_BASE_ADDR + i ), param );
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "PCKS_TO_QUEUE_MAP_WRITE: new value CLASS_QOS_PORT1_PID2TC_MAP 0x%08X\n", param );
        }
        while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
        /*--- TC_TO_QUEUE_MAP ---*/
        while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
        param = 0;
        for( i = 0; i < 8; i++ ){
            res = sscanf( &buff[ index ], "TC%d: %d", &param_arrays[ 0 ], &param_arrays[ 1 ] );
            if( res != 2 ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PCKS_TO_QUEUE_MAP_WRITE: pcks_to_queue_map_write: can not get \"TC%d\" value from file!\n", i );
                goto exit_pcks_to_queue_map_write;
            }

            if( ( param_arrays[ 0 ] - 1 ) != i ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "PCKS_TO_QUEUE_MAP_WRITE: bad TC number, read %d, but range: 1 to 8 !\n", param_arrays[ 0 ] );
                goto exit_pcks_to_queue_map_write;
            }

            param |= ( ( ( param_arrays[ 1 ] - 1 ) & 7 ) << ( i * 3 ) );

            while( ( buff[ index++ ] != '\n' ) && ( index < ( count - 1 ) ) ){}
        }

        fp_reg_write( hwQosTc2Cos[ 0 ][ file_id ], param );
        fp_reg_write( hwQosTc2Cos[ 1 ][ file_id ], param );
        fp_port[ file_id ].tc2cos_table = param;
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "PCKS_TO_QUEUE_MAP_WRITE: new value CLASS_QOS_PORT%d_TC2COS_MAP 0x%08X\n", file_id, param );

    }

exit_pcks_to_queue_map_write:
    if( buff != FP_NULL ){
        kfree( buff );
    }
    if( param_arrays != FP_NULL ){
        kfree( param_arrays );
    }
    mutex_unlock( &mutexProcClass );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "PCKS_TO_QUEUE_MAP_WRITE: end\n" );
    return result;
}

static int pcks_to_queue_map_read( struct seq_file *f, void *v ) {
    int file_id;
    uint32_t tmp, val;

    if( fp_logical_dev[ 0 ] == NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, the device doesn't OPEN!\n", __func__ );
        return 0;
    }

    file_id = *( uint32_t* )PDE_DATA( file_inode( f->file ) );
    if( file_id >= MAX_FILE_PCKS_TO_QUEUES ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, invalid file id %d!\n", __func__, file_id );
        return 0;
    }

    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: start\n", __func__ );
    mutex_lock( &mutexProcClass );
    if( file_id == FILE_PCKS_TO_QUEUES_GLOBAL_DSCP_2_TC ){
        seq_printf( f, "--- DSCP_TO_TC_MAP ---\n" );
        for( tmp = 0; tmp < 7; tmp++ ){
            val = fp_reg_read( ( CLASS_HW_BASE_ADDR + CLASS_QOS_GLOBAL_DSCP2TC_MAP0 + ( tmp << 2 ) ) );
            if( tmp < 6 ){
                seq_printf( f,
                    "qos global dscp%d0 - dscp%d9 to TC: %d %d %d %d %d %d %d %d %d %d\n",
                    tmp, tmp,
                    ( ( val & 7 ) + 1 ),
                    ( ( ( val >> 3 ) & 7 ) + 1 ),
                    ( ( ( val >> 6 ) & 7 ) + 1 ),
                    ( ( ( val >> 9 ) & 7 ) + 1 ),
                    ( ( ( val >> 12 ) & 7 ) + 1 ),
                    ( ( ( val >> 15 ) & 7 ) + 1 ),
                    ( ( ( val >> 18 ) & 7 ) + 1 ),
                    ( ( ( val >> 21 ) & 7 ) + 1 ),
                    ( ( ( val >> 24 ) & 7 ) + 1 ),
                    ( ( ( val >> 27 ) & 7 ) + 1 ) );
            }else{
                seq_printf( f,
                    "qos global dscp60 - dscp63 to TC: %d %d %d %d\n",
                    ( ( val & 7 ) + 1 ),
                    ( ( ( val >> 3 ) & 7 ) + 1 ),
                    ( ( ( val >> 6 ) & 7 ) + 1 ),
                    ( ( ( val >> 9 ) & 7 ) + 1 ) );
            }
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: CLASS_QOS_GLOBAL_DSCP2TC_MAP%d 0x%08X\n", __func__, tmp, val );
        }
        seq_printf( f,    "----------------------------------------------------------\n"
                        "DSCP_TO_TC_MAP - range 1 to 8; TC value to which DSCP value of IP header is mapping\n"
                        "For example:\n"
                        "qos global dscp00 - dscp09 to TC: 5 1 8 3 4 5 6 7 8 1\n"
                        "it is mean, for ip packet with DSCP value 0 the TC value is 5,\n"
                        "for ip packet with DSCP value 1 the TC value is 1,\n"
                        "for ip packet with DSCP value 2 the TC value is 8, ...\n" );
    }else{
        seq_printf( f, "--- QOS_TC_SEL_TABLE ---\n" );
        val = fp_reg_read( hwQosTcSel[ 0 ][ file_id ] );   //get tc_sel table
        if( file_id & 1 ){
            val = ( ( val & 0xFFFF0000 ) >> 16 );
        }else{
            val = ( val & 0xFFFF );
        }

        for( tmp = 0; tmp < 8; tmp++ ){
            seq_printf( f,  "%s %d\n",
                            qos_tc_sel_table_string_read[ tmp ],
                            ( ( ( val >> ( tmp << 1 ) ) & 3 ) + 1 ) );
        }
        seq_printf( f, "--- PCP_TO_TC_MAP ---\n" );
        val = fp_reg_read( hwQosPcp2Tc[ 0 ][ file_id ] );   //get pcp2tc table
        for( tmp = 0; tmp < 8; tmp++ ){
            seq_printf( f,  "PCP%d: %d\n",
                            tmp,
                            ( ( ( val >> ( tmp * 3 ) ) & 7 ) + 1 ) );
        }
        seq_printf( f, "--- PID_TO_TC_MAP ---\n" );
        if( file_id < 10 ){
            val = fp_reg_read( ( CLASS_HW_BASE_ADDR + CLASS_QOS_PORT_PID2TC_MAP ) );  //get pid2tc
            val = ( val & ( 7 << ( 3 * file_id ) ) ) >> ( 3 * file_id );
        }else{
            val = fp_reg_read( ( CLASS_HW_BASE_ADDR + CLASS_QOS_PORT1_PID2TC_MAP ) ); //get pid2tc
            val = ( val & ( 7 << ( 3 * ( file_id - 10 ) ) ) ) >> ( 3 * ( file_id - 10 ) );
        }
        seq_printf( f,  "Port%d: %d\n",
                        ( file_id + 1 ),
                        ( val + 1 ) );

        seq_printf( f, "--- TC_TO_QUEUE_MAP ---\n" );
        val = fp_reg_read( hwQosTc2Cos[ 0 ][ file_id ] ); //get tc2cos table

        for( tmp = 0; tmp < 8; tmp++ ){
            seq_printf( f,  "TC%d: %d\n",
                            ( tmp + 1 ),
                            ( ( ( val >> ( tmp * 3 ) ) & 7 ) + 1 ) );
        }
        seq_printf( f,    "----------------------------------------------------------\n"
                        "QOS_TC_SEL_TABLE - packet queueing principle (1 - DSCP, 2 - PCP, 3 - MAC, 4 - PID)\n"
                        "For example:\n"
                        "Dynamic_Untag_notIP: 4 - it is mean, if received packet is untagged and not IP packet, and "
                        "hasn't static entry in MAC Table for it src MAC, that TC value based by port number, "
                        "where it was received.\n"
                        "PCP_TO_TC_MAP - range 1 to 8; TC value to which Prority Code Point (PCP) from vlan tag is mapping "
                        "(if QOS_TC_SEL_TABLE equal 2 for this packet)\n"
                        "For example:\n"
                        "PCP5: 1 - it is mean, if received packet has pcp field 5, that TC value is 1\n"
                        "PID_TO_TC_MAP - range 1 to 8; TC value to which receiv Port number is mapping (if QOS_TC_SEL_TABLE equal 4 for this packet)\n"
                        "For example:\n"
                        "Port10: 2 - it is mean, if packet was received by port 10, that TC value is 2\n"
                        "TC_TO_QUEUE_MAP - range 1 to 8; QUEUE number to which packets with specify TC is mapping\n"
                        "For example:\n"
                        "TC5: 2 - it is mean, if packet has TC 5, that it will send through queue 2\n"
                        "DSCP_TO_TC_MAP - for DSCP to TC map you need write data in global_dscp_pcks_to_queues_map file\n" );
    }
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: end\n", __func__ );
    mutex_unlock( &mutexProcClass );
    return 0;
}

static int pcks_to_queue_map_open( struct inode *inode, struct file *file ){
    return single_open( file, pcks_to_queue_map_read, NULL );
}

static const struct file_operations pcks_to_queues_map_fops = {
    .owner = THIS_MODULE,
    .open  = pcks_to_queue_map_open,
    .read  = seq_read,
    .llseek = seq_lseek,
    .release = single_release,
    .write = pcks_to_queue_map_write,
};

//static const struct file_operations pcks_to_queues_map_fops = {
//    .owner = THIS_MODULE,
//    .read  = pcks_to_queue_map_read,
//    .write = pcks_to_queue_map_write,
//};


static ssize_t flow_control_write( struct file *file, const char *buf,
                           size_t count, loff_t *ppos ) {
    int32_t file_id, res;
    uint32_t i, phy_state;
    uint32_t *param_arrays;
    struct port_s *port;
    fp_dev_info_t *logical_priv;
    uint8_t *buff;
    size_t result;

    file_id = *( uint32_t* )PDE_DATA( file_inode( file ) );

    if( fp_logical_dev[ 0 ] == NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "FLOW_CONTROL_WRITE: the device doesn't OPEN!\n" );
        return count;
    }

    if( file_id >= MAX_FILE_FLOW_CONTROL ){
        FP_DEBUG(DBGP_FEAT_PROCFS, FP_LOG_ERR, "FLOW_CONTROL_WRITE: bad file id!\n");
        return count;
    }

    mutex_lock( &mutexProcEMAC );
    result = 0;
    buff = FP_NULL;
    param_arrays = FP_NULL;
    buff = kmalloc( PROC_BUF_SIZE, GFP_KERNEL );
    if( buff == FP_NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "FLOW_CONTROL_WRITE: can not allocate memory for user data\n" );
        goto exit_flow_control_write;
    }

    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "FLOW_CONTROL_WRITE: start\n" );
    
    if( count > PROC_BUF_SIZE ){
       count = PROC_BUF_SIZE;
    }

    res = copy_from_user( buff, ( void* )buf, count );
    if( res ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "FLOW_CONTROL_WRITE: can not get data from user\n" );
        goto exit_flow_control_write;
    }
    buff[ count -1 ] = '\0';

    logical_priv = netdev_priv( fp_logical_dev[ file_id ] );
/*
"----------------------------------------------------------\n\
Enable: 0 - disable flow control,\n\
        1 - enable flow control\n\
Low watermark: - if the value of the occupied buffers goes below watermark, \
PAUSE packet with 0 timeout will be transmitted (range: 0 - 2048)\n\
High watermark: - if the value of the occupied buffers goes after watermark, \
PAUSE packet with timeout will be transmitted (range: 0 - 2048)\n\
Preferred mode: - flow control advertised bits at auto-negotiation\n\
                0 - ASM_DIR - 0, PAUSE - 0 (flow control not suppoted)\n\
                1 - ASM_DIR - 0, PAUSE - 1\n\
                2 - ASM_DIR - 1, PAUSE - 0\n\
                3 - ASM_DIR - 1, PAUSE - 1\n\
Force: 1 - enable forced flow control (even if link partner not supported PAUSE)\n\
       0 - enable flow control only if link partner supports flow control\n\
Status: - status of flow control at local port\n\
Settings Status: - flow control current settings status\n\
                 0 - Tx disable, Rx disable\n\
                 1 - Tx disable, Rx enable\n\
                 2 - Tx enable, Rx disable\n\
                 3 - Tx enbale, Rx enable\n"
*/

    param_arrays = kmalloc( ( ( sizeof( uint32_t ) ) * 5), GFP_KERNEL );
    if( param_arrays == FP_NULL ){
        FP_DEBUG(DBGP_FEAT_PROCFS, FP_LOG_ERR, "FLOW_CONTROL_WRITE: can not allocate memory for params\n" );
        goto exit_flow_control_write;
    }

    result = count;

    res = sscanf( buff, "Enable: %d\nLow watermark: %d\nHigh watermark: %d\nPreferred mode: %d\nForce: %d",
        &param_arrays[ 0 ], &param_arrays[ 1 ], &param_arrays[ 2 ], &param_arrays[ 3 ], &param_arrays[ 4 ] );
    
    if( res != 5 ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "FLOW_CONTROL_WRITE: can not get parameters values from file\n" );
        goto exit_flow_control_write;
    }

    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "FLOW_CONTROL_WRITE: Enable: %d Low: %d High: %d Pref mode: %dForce: %d\n", 
        param_arrays[ 0 ], param_arrays[ 1 ], param_arrays[ 2 ], param_arrays[ 3 ], param_arrays[ 4 ] );

    port = &fp_port[ file_id ];

    if( ( port->fc.setup.enable == param_arrays[ 0 ] ) &&
        ( port->fc.preferred_mode == param_arrays[ 3 ] ) &&
        ( port->fc.low_watermark == param_arrays[ 1 ] ) &&
        ( port->fc.high_watermark == param_arrays[ 2 ] ) && 
        ( port->fc.setup.force == param_arrays[ 4 ] ) ){  //if no change in port flow control settings
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "FLOW_CONTROL_WRITE: no change in flow control settings by Port_%d\n", file_id );
        goto exit_flow_control_write;
    }

    phy_state = phy_dp83867_get_duplex( logical_priv->mdio_port, logical_priv->phy_addr );

    switch( phy_state ){

        case 0:
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "FLOW_CONTROL_WRITE: eth phy auto-negotiation has not completed\n" );
            break;

        case 1:
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "FLOW_CONTROL_WRITE: eth phy auto-negotiation is disabled, half duplex; can not enable flow control\n" );
            goto exit_flow_control_write;
            break;

        case 2:
            if( !( param_arrays[ 4 ] & 1 ) ){    //force == 0
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "FLOW_CONTROL_WRITE: eth phy auto-negotiation is disabled, i don't know link partner capabilities, can not enable flow control\n" );
                goto exit_flow_control_write;
            }
            break;

        case 3:
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "FLOW_CONTROL_WRITE: eth phy in half duplex mode, can not enable flow control\n" );
            goto exit_flow_control_write;
            break;

        case 4:
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "FLOW_CONTROL_WRITE: eth phy in full duplex mode, auto-negotiation enable\n" );
            break;

        default:
            FP_DEBUG(DBGP_FEAT_PROCFS, FP_LOG_ERR, "FLOW_CONTROL_WRITE: should never come here!\n" );
            goto exit_flow_control_write;
            break;
    }

    i = fp_reg_read( ( BMU1_BASE_ADDR + BMU_UCAST_CONFIG ) );
    if( param_arrays[ 1 ] > i ){
        param_arrays[ 1 ] = i;
    }
    port->fc.low_watermark = param_arrays[ 1 ];

    if( param_arrays[ 2 ] > i){
        param_arrays[ 2 ] = i;
    }
    port->fc.high_watermark = param_arrays[ 2 ];

    param_arrays[ 3 ] &= 3;
    port->fc.preferred_mode = param_arrays[ 3 ];

    param_arrays[ 0 ] &= 1;
    param_arrays[ 4 ] &= 1;

    port->fc.setup.force = param_arrays[ 4 ];
    port->fc.setup.enable = param_arrays[ 0 ];

    if( !port->fc.setup.enable ){ //disable flow control
        if( !flow_control_disable( file_id ) ){
            port->fc.high_watermark = 0;
            port->fc.low_watermark = 0;
            port->fc.preferred_mode = 0;
            port->fc.setup.force = 0;
            port->fc.setup.enable = 0;
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "FLOW_CONTROL_WRITE: disable flow control at Port_%d\n", file_id );
        }else{
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "FLOW_CONTROL_WRITE: can not disable flow control at Port_%d!\n", file_id );
        }
        goto exit_flow_control_write;
    }

    if( ( phy_state == 0) || ( phy_state == 4) ){   //if auto-negotiation enable, change fc parameters and restart auto-negotiation
        phy_dp83867_set_fc_parameters( logical_priv->mdio_port, logical_priv->phy_addr, param_arrays[ 3 ] );
        phy_dp83867_restart_aneg( logical_priv->mdio_port, logical_priv->phy_addr );
        port->fc.setup.update_flag = 1;
    }else{ //if auto-negotiation disable, change fc parameters    
        if( port->fc.preferred_mode == 0 ){ //disable flow control 
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "FLOW_CONTROL_WRITE: Port_%d: Enable = 1, but Preferred_mode = 0, incompatible settings, ignore it!\n", file_id );
        }else{
            if( port->fc.preferred_mode == 3 ){
                port->fc.preferred_mode = 1;
            }
            if( !flow_control_enable( file_id, &( port->fc ) ) ){
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "FLOW_CONTROL_WRITE: enable flow control at Port_%d, preffered mode %d\n", file_id, port->fc.preferred_mode );
            }else{
                FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "FLOW_CONTROL_WRITE: can not enable flow control at Port_%d!\n", file_id );
            }
        }
    }

exit_flow_control_write:
    if( buff != FP_NULL ){
        kfree( buff );
    }
    if( param_arrays != FP_NULL ){
        kfree( param_arrays );
    }
    mutex_unlock( &mutexProcEMAC );
    FP_DEBUG(DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "FLOW_CONTROL_WRITE: end\n");
    return result;
}

static int flow_control_read( struct seq_file *f, void *v ) {
    int file_id;
    uint32_t val;
    fp_dev_info_t *logical_priv;
    struct port_s *port;

    if( fp_logical_dev[ 0 ] == NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, the device doesn't OPEN!\n", __func__ );
        return 0;
    }

    file_id = *( uint32_t* )PDE_DATA( file_inode( f->file ) );
    if( file_id >= MAX_FILE_FLOW_CONTROL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, invalid file id %d!\n", __func__, file_id );
        return 0;
    }

    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: start\n", __func__ );
    mutex_lock( &mutexProcEMAC );
    port = &fp_port[ file_id ];
    seq_printf( f,  "Enable: %d\n"
                    "Low watermark: %d\n"
                    "High watermark: %d\n"
                    "Preferred mode: %d\n"
                    "Force: %d\n",
                    port->fc.setup.enable,
                    port->fc.low_watermark,
                    port->fc.high_watermark,
                    port->fc.preferred_mode,
                    port->fc.setup.force );

    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG,
                    "%s: Port_%d "
                    "Enable: %d "
                    "Low watermark: %d "
                    "High watermark: %d "
                    "Preferred mode: %d "
                    "Force: %d\n",
                    __func__,
                    file_id,
                    port->fc.setup.enable,
                    port->fc.low_watermark,
                    port->fc.high_watermark,
                    port->fc.preferred_mode,
                    port->fc.setup.force );
        
    seq_printf( f, "--- Registers values---\n" );

    val = fp_reg_read( egpi[ file_id ] + EGPI_FW_CONTROL );
    seq_printf( f, "Tx pause enable: %d\n", val ? 1 : 0 );

    if( get_mac_register( emac[ file_id ], EMAC_NETWORK_CONFIG, &val ) ){
        val = 0;
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: read EMAC_NETWORK_CONFIG failed!\n", __func__ );
    }else{
        val = ( val & ( 1 << 13 ) ) >> 13;
    }
    seq_printf( f, "Rx pause enable: %d\n", val );        

    val = fp_reg_read( egpi[ file_id ] + EGPI_BMU1_PHY_LOW_WATERMARK );
    seq_printf( f, "Low watermark: %d\n", val );

    val = fp_reg_read( egpi[ file_id ] + EGPI_BMU1_PHY_HIGH_WATERMARK );
    seq_printf( f, "High watermark: %d\n", val );

    logical_priv = netdev_priv( fp_logical_dev[ file_id ] );
    val = phy_dp83867_is_flow_control_advertised( logical_priv->mdio_port, logical_priv->phy_addr );
    seq_printf( f, "Advertised flow control: %d\n", val );
    val = phy_dp83867_is_flow_control_available( logical_priv->mdio_port, logical_priv->phy_addr );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: is FC available at mdio port %d, phy_addr %d, return %d\n", __func__, ( logical_priv->mdio_port + 1 ), logical_priv->phy_addr, val );

    switch(val){
        case 0:
            seq_printf( f, "Status: not supported\n" );
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: %d - FC not supported\n", __func__, val );
            break;
        case 1:
            seq_printf( f, "Status: symmetrical\n" );
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: %d - PAUSE\n", __func__, val );
            break;
        case 2:
            seq_printf( f, "Status: only tx pause frame supported (ASYM PAUSE)\n" );
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: %d - tx only PAUSE\n", __func__, val );
            break;
        case 3:
            seq_printf( f, "Status: only rx pause frame supported (ASYM PAUSE)\n" );
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: %d - rx only PAUSE\n", __func__, val );
            break;
        default:
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: should never come here!\n", __func__ );
            break;
    }
        
    seq_printf( f,  "----------------------------------------------------------\n"
                    "Enable: 0 - disable flow control,\n"
                    "        1 - enable flow control\n"
                    "Low watermark: - if the value of the occupied buffers goes below watermark, "
                    "PAUSE packet with 0 timeout will be transmitted (range: 0 - 2048)\n"
                    "High watermark: - if the value of the occupied buffers goes after watermark, "
                    "PAUSE packet with timeout will be transmitted (range: 0 - 2048)\n"
                    "Preferred mode: - flow control advertised bits at auto-negotiation\n"
                    "                0 - ASM_DIR - 0, PAUSE - 0 (flow control not suppoted)\n"
                    "                1 - ASM_DIR - 0, PAUSE - 1\n"
                    "                2 - ASM_DIR - 1, PAUSE - 0\n"
                    "                3 - ASM_DIR - 1, PAUSE - 1\n"
                    "Force: 1 - enable forced flow control (even if link partner not supports PAUSE)\n"
                    "       0 - enable flow control only if link partner supports flow control\n"
                    "You need write only 5 parameters (Enable, Low watermark, High watermark, Preferred mode, Force) for valid setup of flow control\n"
                    "--- Registers values--- read data from hardware\n"
                    "Tx pause enable: - enable (1) or disable (0) transmitt pause frames\n"
                    "Rx pause enable: - enable (1) or disable (0) processing of received pause frames\n"
                    "Low watermark: - see descripion before\n"
                    "High watermark: - see descripion before\n"
                    "Advertised flow control: - advertised parameters at auto-negotiation\n"
                    "                         0 - ASM_DIR - 0, PAUSE - 0 (flow control not suppoted)\n"
                    "                         1 - ASM_DIR - 0, PAUSE - 1\n"
                    "                         2 - ASM_DIR - 1, PAUSE - 0\n"
                    "                         3 - ASM_DIR - 1, PAUSE - 1\n"
                    "Status: - flow control status after auto-negotiation\n" );

    mutex_unlock( &mutexProcEMAC );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: end\n", __func__ );
    return 0;
}

static int flow_control_open( struct inode *inode, struct file *file ){
    return single_open( file, flow_control_read, NULL );
}

static const struct file_operations flow_control_fops = {
    .owner = THIS_MODULE,
    .open  = flow_control_open,
    .read  = seq_read,
    .llseek = seq_lseek,
    .release = single_release,
    .write = flow_control_write,
};

//static const struct file_operations flow_control_fops = {
//    .owner = THIS_MODULE,
//    .read  = flow_control_read,
//    .write = flow_control_write,
//};


static ssize_t eth_phy_settings_port_write( struct file *file, const char *buf,
                           size_t count, loff_t *ppos ) {

    int32_t file_id, res;
    uint32_t i, speed, duplex;
    struct port_s *port;
    fp_dev_info_t *logical_priv;
    uint8_t *buff;
    uint8_t *param_arrays;
    size_t result;

    file_id = *( uint32_t* )PDE_DATA( file_inode( file ) );

    if( fp_logical_dev[ 0 ] == FP_NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "ETH_PHY_SETTINGS_PORT_WRITE: the device doesn't OPEN!\n");
        return count;
    }

    if( file_id >= MAX_FILE_ETH_PHY_SETTINGS_PORT ){
        FP_DEBUG(DBGP_FEAT_PROCFS, FP_LOG_ERR, "ETH_PHY_SETTINGS_PORT_WRITE: bad file id!\n");
        return count;
    }

    mutex_lock( &mutexProcEMAC );
    result = 0;
    buff = FP_NULL;
    param_arrays = FP_NULL;
    buff = kmalloc( PROC_BUF_SIZE, GFP_KERNEL );
    if( buff == FP_NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "ETH_PHY_SETTINGS_PORT_WRITE: can not allocate memory for user data\n" );
        goto exit_eth_phy_settings_port_write;
    }

    param_arrays = kmalloc( ( ( sizeof( uint8_t ) ) * 8 ), GFP_KERNEL );
    if( param_arrays == FP_NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "ETH_PHY_SETTINGS_PORT_WRITE: can not allocate memory for params\n" );
        goto exit_eth_phy_settings_port_write;
    }

    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "ETH_PHY_SETTINGS_PORT_WRITE: start\n" );

    if( count > PROC_BUF_SIZE ){
       count = PROC_BUF_SIZE;
    }

    res = copy_from_user( buff, ( void* )buf, count );
    if( res ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "ETH_PHY_SETTINGS_PORT_WRITE: can not get data from user\n" );
        goto exit_eth_phy_settings_port_write;   
    }
    buff[ count -1 ] = '\0';

    result = count;

    logical_priv = netdev_priv( fp_logical_dev[ file_id ] );

    res = sscanf( buff, "Enable: %hhx\n"
                        "Auto-negotiation: %hhx\n"
                        "Advertise 10 Mb/s full: %hhx\n"
                        "Advertise 10 Mb/s half: %hhx\n"
                        "Advertise 100 Mb/s full: %hhx\n"
                        "Advertise 100 Mb/s half: %hhx\n"
                        "Advertise 1000 Mb/s full: %hhx\n"
                        "Advertise 1000 Mb/s half: %hhx\n"
                        "Speed: %d\n"
                        "Duplex: %d",
                        &param_arrays[ 0 ],
                        &param_arrays[ 1 ],
                        &param_arrays[ 2 ],
                        &param_arrays[ 3 ],
                        &param_arrays[ 4 ],
                        &param_arrays[ 5 ],
                        &param_arrays[ 6 ],
                        &param_arrays[ 7 ],
                        &speed, &duplex );
    if( res != 10 ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "ETH_PHY_SETTINGS_PORT_WRITE: can not get parameters values from file\n" );
        goto exit_eth_phy_settings_port_write;
    }

    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, 
                    "ETH_PHY_SETTINGS_PORT_WRITE: Enable: %d Aneg: %d "
                    "Adv_10_f: %d Adv_10_h: %d Adv_100_f: %d Adv_100_h %d"
                    "Adv_1000_f: %d Adv_1000_h %d Speed: %d, Duplex: %d\n", 
                    param_arrays[ 0 ], param_arrays[ 1 ], param_arrays[ 2 ],
                    param_arrays[ 3 ], param_arrays[ 4 ], param_arrays[ 5 ],
                    param_arrays[ 6 ], param_arrays[ 7 ], speed, duplex );

    param_arrays[ 0 ] &= 1;
    param_arrays[ 1 ] &= 1;
    param_arrays[ 2 ] &= 1;
    param_arrays[ 3 ] &= 1;
    param_arrays[ 4 ] &= 1;
    param_arrays[ 5 ] &= 1;
    param_arrays[ 6 ] &= 1;
    param_arrays[ 7 ] &= 1;
    duplex &= 1;
    if( !( ( speed == 10 )||( speed == 100 )||( speed == 1000 ) ) ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "ETH_PHY_SETTINGS_PORT_WRITE: bad speed parameter (available values 10 or 100, or 1000)!\n" );
        goto exit_eth_phy_settings_port_write;   
    }

    port = &fp_port[ file_id ];

    if( (     logical_priv->phy_port_state == param_arrays[ 0 ] ) &&
        ( ( ( logical_priv->link_advertising >> ETHTOOL_LINK_MODE_Autoneg_BIT        ) & 1 ) == param_arrays[ 1 ] ) &&
        ( ( ( logical_priv->link_advertising >> ETHTOOL_LINK_MODE_10baseT_Full_BIT   ) & 1 ) == param_arrays[ 2 ] ) &&
        ( ( ( logical_priv->link_advertising >> ETHTOOL_LINK_MODE_10baseT_Half_BIT   ) & 1 ) == param_arrays[ 3 ] ) &&
        ( ( ( logical_priv->link_advertising >> ETHTOOL_LINK_MODE_100baseT_Full_BIT  ) & 1 ) == param_arrays[ 4 ] ) &&
        ( ( ( logical_priv->link_advertising >> ETHTOOL_LINK_MODE_100baseT_Half_BIT  ) & 1 ) == param_arrays[ 5 ] ) && 
        ( ( ( logical_priv->link_advertising >> ETHTOOL_LINK_MODE_1000baseT_Full_BIT ) & 1 ) == param_arrays[ 6 ] ) &&
        ( ( ( logical_priv->link_advertising >> ETHTOOL_LINK_MODE_1000baseT_Half_BIT ) & 1 ) == param_arrays[ 7 ] ) &&
            ( logical_priv->link_speed == speed ) && ( logical_priv->link_full_duplex == duplex ) ){  //if no change in port structure
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "ETH_PHY_SETTINGS_PORT_WRITE: no change in eth phy settings by Port_%d\n", file_id );
        goto exit_eth_phy_settings_port_write;
    }


    /* START PARAMETERS SETUP */
    //need spin lock!
    if( logical_priv->phy_port_state != param_arrays[ 0 ] ){

        logical_priv->phy_port_state = param_arrays[ 0 ];
        if( param_arrays[ 0 ] == 0 ){ //power off
            if( logical_priv->phy_ic_type == PHY_IC_DP83867 ){
                phy_dp83867_set_power_down( logical_priv->mdio_port, logical_priv->phy_addr );
            }else{
                
            }
            logical_priv->link_is_up = 0;
            goto exit_eth_phy_settings_port_write;
        }
        phy_dp83867_set_power_up( logical_priv->mdio_port, logical_priv->phy_addr );
    }

    i = ( param_arrays[ 1 ] << ETHTOOL_LINK_MODE_Autoneg_BIT )|( param_arrays[ 2 ] << ETHTOOL_LINK_MODE_10baseT_Full_BIT )|
        ( param_arrays[ 3 ] << ETHTOOL_LINK_MODE_10baseT_Half_BIT )|( param_arrays[ 4 ] << ETHTOOL_LINK_MODE_100baseT_Full_BIT )|
        ( param_arrays[ 5 ] << ETHTOOL_LINK_MODE_100baseT_Half_BIT )|( param_arrays[ 6 ] << ETHTOOL_LINK_MODE_1000baseT_Full_BIT )|
        ( param_arrays[ 7 ] << ETHTOOL_LINK_MODE_1000baseT_Half_BIT );

    if( ( ( i >> ETHTOOL_LINK_MODE_Autoneg_BIT ) & 1 ) == 0 ){   //autoneg disable
        phy_dp83867_speed_set( logical_priv->mdio_port, logical_priv->phy_addr, speed, duplex, 0 );
    }else{   //autoneg enable
        phy_dp83867_set_advertise_speed_duplex( logical_priv->mdio_port, logical_priv->phy_addr, i );
        phy_dp83867_autoneg_enable( logical_priv->mdio_port, logical_priv->phy_addr );
        phy_dp83867_restart_aneg( logical_priv->mdio_port, logical_priv->phy_addr );
    }
    logical_priv->link_advertising = i;
    /* END PARAMETERS SETUP */    

exit_eth_phy_settings_port_write:
    if( buff != FP_NULL ){
        kfree( buff );
    }
    if( param_arrays != FP_NULL ){
        kfree( param_arrays );
    }
    mutex_unlock( &mutexProcEMAC );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "ETH_PHY_SETTINGS_PORT_WRITE: end\n");
    return result;
}

static int eth_phy_settings_port_read( struct seq_file *f, void *v ) {
    int file_id;
    fp_dev_info_t *logical_priv;

    if( fp_logical_dev[ 0 ] == NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, the device doesn't OPEN!\n", __func__ );
        return 0;
    }

    file_id = *( uint32_t* )PDE_DATA( file_inode( f->file ) );
    if( file_id >= MAX_FILE_ETH_PHY_SETTINGS_PORT ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, invalid file id %d!\n", __func__, file_id );
        return 0;
    }

    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: start\n", __func__ );
    mutex_lock( &mutexProcEMAC );
    logical_priv = netdev_priv( fp_logical_dev[ file_id ] );

    seq_printf( f,  "Enable: %d\n"
                    "Auto-negotiation: %d\n"
                    "Advertise 10 Mb/s full: %d\n"
                    "Advertise 10 Mb/s half: %d\n"
                    "Advertise 100 Mb/s full: %d\n"
                    "Advertise 100 Mb/s half: %d\n"
                    "Advertise 1000 Mb/s full: %d\n"
                    "Advertise 1000 Mb/s half: %d\n"
                    "Speed: %d\n"
                    "Duplex: %d\n"
                    "Link: %s\n", 
                    logical_priv->phy_port_state,
                    ( ( logical_priv->link_advertising >> ETHTOOL_LINK_MODE_Autoneg_BIT ) & 1 ), 
                    ( ( logical_priv->link_advertising >> ETHTOOL_LINK_MODE_10baseT_Full_BIT ) & 1 ),
                    ( ( logical_priv->link_advertising >> ETHTOOL_LINK_MODE_10baseT_Half_BIT ) & 1 ),
                    ( ( logical_priv->link_advertising >> ETHTOOL_LINK_MODE_100baseT_Full_BIT ) & 1 ),
                    ( ( logical_priv->link_advertising >> ETHTOOL_LINK_MODE_100baseT_Half_BIT ) & 1 ),
                    ( ( logical_priv->link_advertising >> ETHTOOL_LINK_MODE_1000baseT_Full_BIT ) & 1 ),
                    ( ( logical_priv->link_advertising >> ETHTOOL_LINK_MODE_1000baseT_Half_BIT ) & 1 ),
                        logical_priv->link_speed,
                        logical_priv->link_full_duplex,
                        logical_priv->link_is_up?"UP":"DOWN" );

    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, 
                    "%s: Port_%d "
                    "Enable: %d "
                    "Auto-negotiation: %d "
                    "Advertise 10 Mb/s full: %d "
                    "Advertise 10 Mb/s half: %d "
                    "Advertise 100 Mb/s full: %d "
                    "Advertise 100 Mb/s half: %d "
                    "Advertise 1000 Mb/s full: %d "
                    "Advertise 1000 Mb/s half: %d "
                    "Speed: %d "
                    "Duplex: %d\n",
                    __func__,
                    file_id, logical_priv->phy_port_state,
                    ( ( logical_priv->link_advertising >> ETHTOOL_LINK_MODE_Autoneg_BIT ) & 1 ), 
                    ( ( logical_priv->link_advertising >> ETHTOOL_LINK_MODE_10baseT_Full_BIT ) & 1 ),
                    ( ( logical_priv->link_advertising >> ETHTOOL_LINK_MODE_10baseT_Half_BIT ) & 1 ),
                    ( ( logical_priv->link_advertising >> ETHTOOL_LINK_MODE_100baseT_Full_BIT ) & 1 ),
                    ( ( logical_priv->link_advertising >> ETHTOOL_LINK_MODE_100baseT_Half_BIT ) & 1 ),
                    ( ( logical_priv->link_advertising >> ETHTOOL_LINK_MODE_1000baseT_Full_BIT ) & 1 ),
                    ( ( logical_priv->link_advertising >> ETHTOOL_LINK_MODE_1000baseT_Half_BIT ) & 1 ),
                        logical_priv->link_speed,
                        logical_priv->link_full_duplex ); 
        
    seq_printf( f,    "----------------------------------------------------------\n"
                    "Enable: 0 - disable physical transceiver of the port,\n"
                    "        1 - enable physical transceiver of the port\n"
                    "Auto-negotiation: 0 - disable auto-negotiation\n"
                    "                  1 - enable auto-negotiation\n"
                    "Advertise 10 Mb/s full: 0 - does not advertise this mode at auto-negotiation\n"
                    "                        1 - advertise this mode at auto-negotiation\n"
                    "Advertise 10 Mb/s half: 0 - does not advertise this mode at auto-negotiation\n"
                    "                        1 - advertise this mode at auto-negotiation\n"
                    "Advertise 100 Mb/s full: 0 - does not advertise this mode at auto-negotiation\n"
                    "                         1 - advertise this mode at auto-negotiation\n"
                    "Advertise 100 Mb/s half: 0 - does not advertise this mode at auto-negotiation\n"
                    "                         1 - advertise this mode at auto-negotiation\n"
                    "Advertise 1000 Mb/s full: 0 - does not advertise this mode at auto-negotiation\n"
                    "                          1 - advertise this mode at auto-negotiation\n"
                    "Advertise 1000 Mb/s half: 0 - does not advertise this mode at auto-negotiation\n"
                    "                          1 - advertise this mode at auto-negotiation\n"
                    "at the write:\n"
                    "Speed: 10 - rate is 10 Mbit/s, if auto-negotiation is disabled\n"
                    "       100 - rate is 100 Mbit/s, if auto-negotiation is disabled\n"
                    "       1000 - rate is 1000 Mbit/s, if auto-negotiation is disabled\n"
                    "Duplex: 0 - duplex is half, if auto-negotiation is disabled\n"
                    "        1 - duplex is full, if auto-negotiation is disabled\n"
                    "at the read:\n"
                    "Speed: real rate of the link (10, 100 or 1000 Mbit/s)\n"
                    "Duplex: real duplex of the link (0 - half, 1 - full)\n"
                    "Link: real state of the link\n" );

    mutex_unlock( &mutexProcEMAC );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: end\n", __func__ );
    return 0;
}

static int eth_phy_settings_port_open( struct inode *inode, struct file *file ){
    return single_open( file, eth_phy_settings_port_read, NULL );
}

static const struct file_operations eth_phy_settings_port_ops = {
    .owner = THIS_MODULE,
    .open  = eth_phy_settings_port_open,
    .read  = seq_read,
    .llseek = seq_lseek,
    .release = single_release,
    .write = eth_phy_settings_port_write,
};

//static const struct file_operations eth_phy_settings_port_ops = {
//    .owner = THIS_MODULE,
//    .read  = eth_phy_settings_port_read,
//    .write = eth_phy_settings_port_write,
//};

uint8_t *get_string_connector_values( uint8_t value ){
    switch( value ){
        case 0x01:
            return "SC";
        case 0x07:
            return "LC";
        case 0x0A:
            return "SG";
        case 0x0B:
            return "Optical pigtail";
        case 0x21:
            return "Copper pigtail";
    }
    return "Unknown or other";
}

uint8_t *get_string_encoding_codes_value( uint8_t value ){
    switch( value ){
        case 0x00:
            return "Unspecified";
        case 0x01:
            return "8B10B";
        case 0x02:
            return "4B5B";
        case 0x03:
            return "NRZ";
        case 0x04:
            return "Manchester";
    }
    return "Reserved";
}

uint8_t *byte_four_module_data[ 8 ] = {
    "OC 48, short reach",
    "OC 48, intermediate reach",
    "OC 48, long reach",
    "Reserved",
    "Reserved",
    "Reserved",
    "Reserved",
    "Reserved"
};

uint8_t *byte_five_module_data[ 8 ] = {
    "OC 3, multi-mode short reach",
    "OC 3, single mode intermediate reach",
    "OC 3, single mode long reach",
    "Reserved",
    "OC 12, multi-mode short reach",
    "OC 12, single mode intermediate reach",
    "OC 12, single mode long reach",
    "Reserved"
};

uint8_t *byte_six_module_data[ 8 ] = {
    "1000BASE-SX",
    "1000BASE-LX",
    "1000BASE-CX",
    "1000BASE-T",
    "Reserved",
    "Reserved",
    "Reserved",
    "Reserved"
};

uint8_t *byte_seven_module_data[ 8 ] = {
    "Electrical inter-enclosure (EL)",
    "Longwave laser (LC)",
    "Reserved",
    "Reserved",
    "long distance (L)",
    "intermediate distance (I)",
    "short distance (S)",
    "very long distance (V)"
};

uint8_t *byte_eight_module_data[ 8 ] = {
    "Reserved",
    "Reserved",
    "Reserved",
    "Reserved",
    "Longwave laser (LL)",
    "Shortwave laser w/ OFC (SL)",
    "Shortwave laser w/o OFC (SN)",
    "Electrical intra-enclosure (EL)"
};

uint8_t *byte_nine_module_data[ 8 ] = {
    "Single Mode (SM)",
    "Reserved",
    "Multi-mode, 50 m (M5)",
    "Multi-mode, 62.5 m (M6)",
    "Video Coax (TV)",
    "Miniature Coax (MI)",
    "Shielded Twisted Pair (TP)",
    "Twin Axial Pair (TW)"
};

uint8_t *byte_ten_module_data[ 8 ] = {
    "100 MByte/Sec",
    "Reserved",
    "200 MByte/Sec",
    "Reserved",
    "400 MByte/Sec",
    "Reserved",
    "Reserved",
    "Reserved"
};

uint8_t **bytes_module_data[ 7 ] = {
    byte_four_module_data,
    byte_five_module_data,
    byte_six_module_data,
    byte_seven_module_data,
    byte_eight_module_data,
    byte_nine_module_data,
    byte_ten_module_data
};

static int optic_module_read( struct seq_file *f, void *v ) {
    int file_id, res, byte_addr;
    struct port_s *port;

    if( fp_logical_dev[ 0 ] == NULL ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, the device doesn't OPEN!\n", __func__ );
        return 0;
    }

    file_id = *( uint32_t* )PDE_DATA( file_inode( f->file ) );
    if( file_id >= MAX_FILE_OPTIC_MODULE_PORT ){
        FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: failed, invalid file id %d!\n", __func__, file_id );
        return 0;
    }

    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: start\n", __func__ );
    port = &fp_port[ file_id ];

    if( !( port->optic_link.status & OPTIC_STATUS_SFP_DATA_READY ) ){
        seq_printf( f, "Not connected\n" );
    }else{
        if( port->optic_link.status & OPTIC_STATUS_COP_SFP_READY ){
            seq_printf( f, "COPPER SFP configured\n" );
        }
        if( port->optic_link.module_descr != FP_NULL ){
            seq_printf( f,  "Identifier: %s\n"
                            "Connector: %s\n"
                            "Transceiver: \n",
                            ( port->optic_link.module_descr[ 0 ] == 0x03 )?"SFP tranceiver":"Unknown",
                            get_string_connector_values( port->optic_link.module_descr[ 2 ] ) );

            for( byte_addr = 4; byte_addr < 11; byte_addr++ ){
                for( res = 0; res < 8; res++ ){
                    if( port->optic_link.module_descr[ byte_addr ] & ( 1 << res ) ){
                        seq_printf( f,  "%d.%d - %s\n", 
                                        byte_addr,
                                        res,
                                        bytes_module_data[ byte_addr - 4 ][ res ] );
                    }
                }
            }

            seq_printf( f,  "Encoding: %s\n",
                            get_string_encoding_codes_value( port->optic_link.module_descr[ 11 ] ) );

            seq_printf( f,  "BR, nominal: %d Mbits/sec\n",
                            ( port->optic_link.module_descr[ 12 ] * 100 ) );

            if( port->optic_link.module_descr[ 14 ] ){
                seq_printf( f,  "Length(9 micron): %d km\n",
                                port->optic_link.module_descr[ 14 ] );
            }

            if( port->optic_link.module_descr[ 15 ] ){
                seq_printf( f,  "Length(9 micron): %d m\n",
                                ( port->optic_link.module_descr[ 15 ] * 100 ) );
            }

            if( port->optic_link.module_descr[ 16 ] ){
                seq_printf( f,  "Length(50 micron): %d m\n",
                                ( port->optic_link.module_descr[ 16 ] *10 ) );
            }

            if( port->optic_link.module_descr[ 17 ] ){
                seq_printf( f,  "Length( 62.5 micron): %d m\n",
                                ( port->optic_link.module_descr[ 17 ] * 10 ) );
            }

            if( port->optic_link.module_descr[ 18 ] ){
                seq_printf( f,  "Length(Copper): %d m\n",
                                port->optic_link.module_descr[ 18 ] );
            }

            seq_printf( f, "Vendor name: %s\n", &port->optic_link.module_descr[ 20 ] );
            seq_printf( f,  "Vendor OUI: %x %x %x\n",
                            port->optic_link.module_descr[ 37 ],
                            port->optic_link.module_descr[ 38 ],
                            port->optic_link.module_descr[ 39 ] );

            seq_printf( f, "Vendor PN: %s\n", &port->optic_link.module_descr[ 40 ] );
            seq_printf( f, "Vendor rev: %s\n", &port->optic_link.module_descr[ 56 ] );
        }else{
            FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_ERR, "%s: poiter to module description data is NULL!\n", __func__ );
            seq_printf( f, "Error occure\n" );
        }
    }
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "%s: end\n", __func__ );
    return 0;
}

static int optic_module_open( struct inode *inode, struct file *file ){
    return single_open( file, optic_module_read, NULL );
}

static const struct file_operations optic_module_ops = {
    .owner = THIS_MODULE,
    .open  = optic_module_open,
    .read  = seq_read,
    .llseek = seq_lseek,
    .release = single_release,
};

//static const struct file_operations optic_module_ops = {
//    .owner = THIS_MODULE,
//    .read  = optic_module_read,
//};

int fp_proc_fs_init( void )
{
    int result = 0, i;
    struct proc_dir_entry *proc_file;

    FP_DEBUG(DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "proc FS init start\n");

    mutex_init( &mutexProcTMU );
    mutex_init( &mutexProcMem );
    mutex_init( &mutexProcStat );
    mutex_init( &mutexProcHash );
    mutex_init( &mutexProcClass );
    mutex_init( &mutexProcEMAC );

    basis_dir = proc_mkdir( "basis", NULL );                                                        // Create basis folder
    if(basis_dir == NULL)
    {
        result = -ENOMEM;
        goto err_dir_basis;
    }

    proc_file = proc_create( "temperature", S_IFREG | S_IRUGO, basis_dir, &temperature_fops );

    mem_dir = proc_mkdir( "mem", basis_dir );                                                       // Create memory folder
    if(mem_dir == NULL)
    {
        result = -ENOMEM;
        goto err_dir_mem;
    }

    proc_file = proc_create( "rd_wr", S_IFREG | S_IRUGO | S_IWUGO, mem_dir, &mem_fops );

    statistics_dir = proc_mkdir( "statistics", basis_dir );                                         // Create statistic folder
    if(statistics_dir == NULL)
    {
        result = -ENOMEM;
        goto err_dir_statistic;
    }

    CREATE_PROC_FILE_ID( proc_file, "em1",   S_IFREG | S_IRUGO | S_IWUGO, statistics_dir, &stat_fops, id_stat, FILE_STAT_EM1 );
    CREATE_PROC_FILE_ID( proc_file, "em2",   S_IFREG | S_IRUGO | S_IWUGO, statistics_dir, &stat_fops, id_stat, FILE_STAT_EM2 );
    CREATE_PROC_FILE_ID( proc_file, "em3",   S_IFREG | S_IRUGO | S_IWUGO, statistics_dir, &stat_fops, id_stat, FILE_STAT_EM3 );
    CREATE_PROC_FILE_ID( proc_file, "em4",   S_IFREG | S_IRUGO | S_IWUGO, statistics_dir, &stat_fops, id_stat, FILE_STAT_EM4 );
    CREATE_PROC_FILE_ID( proc_file, "em5",   S_IFREG | S_IRUGO | S_IWUGO, statistics_dir, &stat_fops, id_stat, FILE_STAT_EM5 );
    CREATE_PROC_FILE_ID( proc_file, "em6",   S_IFREG | S_IRUGO | S_IWUGO, statistics_dir, &stat_fops, id_stat, FILE_STAT_EM6 );
    CREATE_PROC_FILE_ID( proc_file, "em7",   S_IFREG | S_IRUGO | S_IWUGO, statistics_dir, &stat_fops, id_stat, FILE_STAT_EM7 );
    CREATE_PROC_FILE_ID( proc_file, "em8",   S_IFREG | S_IRUGO | S_IWUGO, statistics_dir, &stat_fops, id_stat, FILE_STAT_EM8 );
    CREATE_PROC_FILE_ID( proc_file, "em9",   S_IFREG | S_IRUGO | S_IWUGO, statistics_dir, &stat_fops, id_stat, FILE_STAT_EM9 );
    CREATE_PROC_FILE_ID( proc_file, "em10",  S_IFREG | S_IRUGO | S_IWUGO, statistics_dir, &stat_fops, id_stat, FILE_STAT_EM10 );
    CREATE_PROC_FILE_ID( proc_file, "em11",  S_IFREG | S_IRUGO | S_IWUGO, statistics_dir, &stat_fops, id_stat, FILE_STAT_EM11 );
    CREATE_PROC_FILE_ID( proc_file, "em12",  S_IFREG | S_IRUGO | S_IWUGO, statistics_dir, &stat_fops, id_stat, FILE_STAT_EM12 );
    CREATE_PROC_FILE_ID( proc_file, "em13",  S_IFREG | S_IRUGO | S_IWUGO, statistics_dir, &stat_fops, id_stat, FILE_STAT_EM13 );
    CREATE_PROC_FILE_ID( proc_file, "em14",  S_IFREG | S_IRUGO | S_IWUGO, statistics_dir, &stat_fops, id_stat, FILE_STAT_EM14 );
    CREATE_PROC_FILE_ID( proc_file, "em15",  S_IFREG | S_IRUGO | S_IWUGO, statistics_dir, &stat_fops, id_stat, FILE_STAT_EM15 );
    CREATE_PROC_FILE_ID( proc_file, "em16",  S_IFREG | S_IRUGO | S_IWUGO, statistics_dir, &stat_fops, id_stat, FILE_STAT_EM16 );
    CREATE_PROC_FILE_ID( proc_file, "switch",  S_IFREG | S_IRUGO | S_IWUGO, statistics_dir, &stat_fops, id_stat, FILE_STAT_SWITCH );

    hash_tables_dir = proc_mkdir( "hash_tables", basis_dir );                                        // Create hash tables folder
    if(hash_tables_dir == NULL)
    {
        result = -ENOMEM;
        goto err_dir_hash_tables;
    }

    mac_dir = proc_mkdir( "mac", hash_tables_dir );                                                 // Create mac hash tables folder
    if(mac_dir == NULL)
    {
        result = -ENOMEM;
        goto err_dir_hash_tables_mac;
    }

    CREATE_PROC_FILE_ID( proc_file, "show",   S_IFREG | S_IRUGO, mac_dir, &hash_mac_show_fops, id_hash, FILE_HASH_MAC_SHOW_TABLE );
    CREATE_PROC_FILE_ID( proc_file, "add",    S_IFREG | S_IRUGO | S_IWUGO, mac_dir, &hash_fops, id_hash, FILE_HASH_MAC_ADD );
    CREATE_PROC_FILE_ID( proc_file, "search", S_IFREG | S_IRUGO | S_IWUGO, mac_dir, &hash_fops, id_hash, FILE_HASH_MAC_SEARCH );
    CREATE_PROC_FILE_ID( proc_file, "delete", S_IFREG | S_IRUGO | S_IWUGO, mac_dir, &hash_fops, id_hash, FILE_HASH_MAC_DELETE );
    CREATE_PROC_FILE_ID( proc_file, "clean",  S_IFREG | S_IRUGO | S_IWUGO, mac_dir, &hash_fops, id_hash, FILE_HASH_MAC_CLEAN_TABLE );

    vlan_dir = proc_mkdir( "vlan", hash_tables_dir );                                               // Create vlan hash tables folder
    if(vlan_dir == NULL)
    {
        result = -ENOMEM;
        goto err_dir_hash_tables_vlan;
    }

    CREATE_PROC_FILE_ID( proc_file, "show",   S_IFREG | S_IRUGO, vlan_dir, &hash_vlan_show_fops, id_hash, FILE_HASH_VLAN_SHOW_TABLE );
    CREATE_PROC_FILE_ID( proc_file, "add",    S_IFREG | S_IRUGO | S_IWUGO, vlan_dir, &hash_fops, id_hash, FILE_HASH_VLAN_ADD );
    CREATE_PROC_FILE_ID( proc_file, "search", S_IFREG | S_IRUGO | S_IWUGO, vlan_dir, &hash_fops, id_hash, FILE_HASH_VLAN_SEARCH );
    CREATE_PROC_FILE_ID( proc_file, "delete", S_IFREG | S_IRUGO | S_IWUGO, vlan_dir, &hash_fops, id_hash, FILE_HASH_VLAN_DELETE );
    CREATE_PROC_FILE_ID( proc_file, "clean",  S_IFREG | S_IRUGO | S_IWUGO, vlan_dir, &hash_fops, id_hash, FILE_HASH_VLAN_CLEAN_TABLE );
    CREATE_PROC_FILE_ID( proc_file, "global_vlan_entry",  S_IFREG | S_IRUGO | S_IWUGO, vlan_dir, &hash_fops, id_hash, FILE_HASH_VLAN_GLOBAL_VLAN_ENTRY );

    port_settings_dir = proc_mkdir( "port_settings", basis_dir );                                         // Create statistic folder
    if(port_settings_dir == NULL)
    {
        result = -ENOMEM;
        goto err_dir_port_settings;
    }

    CREATE_PROC_FILE_ID( proc_file, "port1",   S_IFREG | S_IRUGO | S_IWUGO, port_settings_dir, &port_settings_fops, id_port_settings, FILE_SETTINGS_PORT1 );
    CREATE_PROC_FILE_ID( proc_file, "port2",   S_IFREG | S_IRUGO | S_IWUGO, port_settings_dir, &port_settings_fops, id_port_settings, FILE_SETTINGS_PORT2 );
    CREATE_PROC_FILE_ID( proc_file, "port3",   S_IFREG | S_IRUGO | S_IWUGO, port_settings_dir, &port_settings_fops, id_port_settings, FILE_SETTINGS_PORT3 );
    CREATE_PROC_FILE_ID( proc_file, "port4",   S_IFREG | S_IRUGO | S_IWUGO, port_settings_dir, &port_settings_fops, id_port_settings, FILE_SETTINGS_PORT4 );
    CREATE_PROC_FILE_ID( proc_file, "port5",   S_IFREG | S_IRUGO | S_IWUGO, port_settings_dir, &port_settings_fops, id_port_settings, FILE_SETTINGS_PORT5 );
    CREATE_PROC_FILE_ID( proc_file, "port6",   S_IFREG | S_IRUGO | S_IWUGO, port_settings_dir, &port_settings_fops, id_port_settings, FILE_SETTINGS_PORT6 );
    CREATE_PROC_FILE_ID( proc_file, "port7",   S_IFREG | S_IRUGO | S_IWUGO, port_settings_dir, &port_settings_fops, id_port_settings, FILE_SETTINGS_PORT7 );
    CREATE_PROC_FILE_ID( proc_file, "port8",   S_IFREG | S_IRUGO | S_IWUGO, port_settings_dir, &port_settings_fops, id_port_settings, FILE_SETTINGS_PORT8 );
    CREATE_PROC_FILE_ID( proc_file, "port9",   S_IFREG | S_IRUGO | S_IWUGO, port_settings_dir, &port_settings_fops, id_port_settings, FILE_SETTINGS_PORT9 );
    CREATE_PROC_FILE_ID( proc_file, "port10",  S_IFREG | S_IRUGO | S_IWUGO, port_settings_dir, &port_settings_fops, id_port_settings, FILE_SETTINGS_PORT10 );
    CREATE_PROC_FILE_ID( proc_file, "port11",  S_IFREG | S_IRUGO | S_IWUGO, port_settings_dir, &port_settings_fops, id_port_settings, FILE_SETTINGS_PORT11 );
    CREATE_PROC_FILE_ID( proc_file, "port12",  S_IFREG | S_IRUGO | S_IWUGO, port_settings_dir, &port_settings_fops, id_port_settings, FILE_SETTINGS_PORT12 );
    CREATE_PROC_FILE_ID( proc_file, "port13",  S_IFREG | S_IRUGO | S_IWUGO, port_settings_dir, &port_settings_fops, id_port_settings, FILE_SETTINGS_PORT13 );
    CREATE_PROC_FILE_ID( proc_file, "port14",  S_IFREG | S_IRUGO | S_IWUGO, port_settings_dir, &port_settings_fops, id_port_settings, FILE_SETTINGS_PORT14 );
    CREATE_PROC_FILE_ID( proc_file, "port15",  S_IFREG | S_IRUGO | S_IWUGO, port_settings_dir, &port_settings_fops, id_port_settings, FILE_SETTINGS_PORT15 );
    CREATE_PROC_FILE_ID( proc_file, "port16",  S_IFREG | S_IRUGO | S_IWUGO, port_settings_dir, &port_settings_fops, id_port_settings, FILE_SETTINGS_PORT16 );
    CREATE_PROC_FILE_ID( proc_file, "host_port",  S_IFREG | S_IRUGO | S_IWUGO, port_settings_dir, &port_settings_fops, id_port_settings, FILE_SETTINGS_SWITCH );

    queue_settings_dir = proc_mkdir( "queue_settings", basis_dir );                 // Create statistic folder
    if(queue_settings_dir == NULL)
    {
        result = -ENOMEM;
        goto err_dir_queue_settings;
    }

    for(i = 0; i < MAX_FILES_QUEUE_SETTINGS; i++){
        queue_number[i] = 0;
    }

    CREATE_PROC_FILE_ID( proc_file, "port1_queues",   S_IFREG | S_IRUGO | S_IWUGO, queue_settings_dir, &queue_settings_fops, id_queue_settings, FILE_PORT1_QUEUES );
    CREATE_PROC_FILE_ID( proc_file, "port2_queues",   S_IFREG | S_IRUGO | S_IWUGO, queue_settings_dir, &queue_settings_fops, id_queue_settings, FILE_PORT2_QUEUES );
    CREATE_PROC_FILE_ID( proc_file, "port3_queues",   S_IFREG | S_IRUGO | S_IWUGO, queue_settings_dir, &queue_settings_fops, id_queue_settings, FILE_PORT3_QUEUES );
    CREATE_PROC_FILE_ID( proc_file, "port4_queues",   S_IFREG | S_IRUGO | S_IWUGO, queue_settings_dir, &queue_settings_fops, id_queue_settings, FILE_PORT4_QUEUES );
    CREATE_PROC_FILE_ID( proc_file, "port5_queues",   S_IFREG | S_IRUGO | S_IWUGO, queue_settings_dir, &queue_settings_fops, id_queue_settings, FILE_PORT5_QUEUES );
    CREATE_PROC_FILE_ID( proc_file, "port6_queues",   S_IFREG | S_IRUGO | S_IWUGO, queue_settings_dir, &queue_settings_fops, id_queue_settings, FILE_PORT6_QUEUES );
    CREATE_PROC_FILE_ID( proc_file, "port7_queues",   S_IFREG | S_IRUGO | S_IWUGO, queue_settings_dir, &queue_settings_fops, id_queue_settings, FILE_PORT7_QUEUES );
    CREATE_PROC_FILE_ID( proc_file, "port8_queues",   S_IFREG | S_IRUGO | S_IWUGO, queue_settings_dir, &queue_settings_fops, id_queue_settings, FILE_PORT8_QUEUES );
    CREATE_PROC_FILE_ID( proc_file, "port9_queues",   S_IFREG | S_IRUGO | S_IWUGO, queue_settings_dir, &queue_settings_fops, id_queue_settings, FILE_PORT9_QUEUES );
    CREATE_PROC_FILE_ID( proc_file, "port10_queues",  S_IFREG | S_IRUGO | S_IWUGO, queue_settings_dir, &queue_settings_fops, id_queue_settings, FILE_PORT10_QUEUES );
    CREATE_PROC_FILE_ID( proc_file, "port11_queues",  S_IFREG | S_IRUGO | S_IWUGO, queue_settings_dir, &queue_settings_fops, id_queue_settings, FILE_PORT11_QUEUES );
    CREATE_PROC_FILE_ID( proc_file, "port12_queues",  S_IFREG | S_IRUGO | S_IWUGO, queue_settings_dir, &queue_settings_fops, id_queue_settings, FILE_PORT12_QUEUES );
    CREATE_PROC_FILE_ID( proc_file, "port13_queues",  S_IFREG | S_IRUGO | S_IWUGO, queue_settings_dir, &queue_settings_fops, id_queue_settings, FILE_PORT13_QUEUES );
    CREATE_PROC_FILE_ID( proc_file, "port14_queues",  S_IFREG | S_IRUGO | S_IWUGO, queue_settings_dir, &queue_settings_fops, id_queue_settings, FILE_PORT14_QUEUES );
    CREATE_PROC_FILE_ID( proc_file, "port15_queues",  S_IFREG | S_IRUGO | S_IWUGO, queue_settings_dir, &queue_settings_fops, id_queue_settings, FILE_PORT15_QUEUES );
    CREATE_PROC_FILE_ID( proc_file, "port16_queues",  S_IFREG | S_IRUGO | S_IWUGO, queue_settings_dir, &queue_settings_fops, id_queue_settings, FILE_PORT16_QUEUES );
    CREATE_PROC_FILE_ID( proc_file, "host_port_queues",  S_IFREG | S_IRUGO | S_IWUGO, queue_settings_dir, &queue_settings_fops, id_queue_settings, FILE_HOST_QUEUES );

    debug_dir = proc_mkdir( "debug", basis_dir );                 // Create statistic folder
    if(debug_dir == NULL)
    {
        result = -ENOMEM;
        goto err_dir_debug;
    }
    
    for(i = 0; i < FP_MAX_SWITCH_PORTS; i++){
        debug_phy_test_mode[i] = 0;
    }
    CREATE_PROC_FILE_ID( proc_file, "print_message_level",   S_IFREG | S_IRUGO | S_IWUGO, debug_dir, &debug_fops, id_debug, FILE_DEBUG_PRINT );
    CREATE_PROC_FILE_ID( proc_file, "wr_packet",             S_IFREG | S_IRUGO | S_IWUGO, debug_dir, &debug_fops, id_debug, FILE_DEBUG_WR_PACKET );
    CREATE_PROC_FILE_ID( proc_file, "ageing_timeout",        S_IFREG | S_IRUGO | S_IWUGO, debug_dir, &debug_fops, id_debug, FILE_DEBUG_AGEING_TIMEOUT );
    CREATE_PROC_FILE_ID( proc_file, "far_end_loopback",      S_IFREG | S_IRUGO | S_IWUGO, debug_dir, &debug_fops, id_debug, FILE_DEBUG_FAR_END_LOOPBACK );
    CREATE_PROC_FILE_ID( proc_file, "host_system_loopback",  S_IFREG | S_IRUGO | S_IWUGO, debug_dir, &debug_fops, id_debug, FILE_DEBUG_HOST_SYSTEM_LOOPBACK );
    CREATE_PROC_FILE_ID( proc_file, "bmu_occupied_buffers",  S_IFREG | S_IRUGO, debug_dir, &debug_fops, id_debug, FILE_DEBUG_BMU_OCCUPIED_BUFFERS );
    CREATE_PROC_FILE_ID( proc_file, "jumbo_enable",          S_IFREG | S_IRUGO | S_IWUGO, debug_dir, &debug_fops, id_debug, FILE_DEBUG_JUMBO_ENABLE );
    CREATE_PROC_FILE_ID( proc_file, "mdio_speed",            S_IFREG | S_IRUGO | S_IWUGO, debug_dir, &debug_fops, id_debug, FILE_DEBUG_MDIO_SPEED );
    CREATE_PROC_FILE_ID( proc_file, "phy_test_mode",         S_IFREG | S_IRUGO | S_IWUGO, debug_dir, &debug_fops, id_debug, FILE_DEBUG_PHY_TEST_MODE );
    CREATE_PROC_FILE_ID( proc_file, "mcu_fw_version",        S_IFREG | S_IRUGO, debug_dir, &debug_fops, id_debug, FILE_DEBUG_MCU_FW_VERSION );

    scheduler_dir = proc_mkdir("scheduler_settings", basis_dir);
    if(scheduler_dir == NULL){
        result = -ENOMEM;
        goto err_dir_scheduler;
    }
    CREATE_PROC_FILE_ID( proc_file, "port1_schedulers", S_IFREG | S_IRUGO | S_IWUGO, scheduler_dir, &scheduler_fops, id_scheduler_settings, FILE_PORT1_SCHEDULER );
    CREATE_PROC_FILE_ID( proc_file, "port2_schedulers", S_IFREG | S_IRUGO | S_IWUGO, scheduler_dir, &scheduler_fops, id_scheduler_settings, FILE_PORT2_SCHEDULER );
    CREATE_PROC_FILE_ID( proc_file, "port3_schedulers", S_IFREG | S_IRUGO | S_IWUGO, scheduler_dir, &scheduler_fops, id_scheduler_settings, FILE_PORT3_SCHEDULER );
    CREATE_PROC_FILE_ID( proc_file, "port4_schedulers", S_IFREG | S_IRUGO | S_IWUGO, scheduler_dir, &scheduler_fops, id_scheduler_settings, FILE_PORT4_SCHEDULER );
    CREATE_PROC_FILE_ID( proc_file, "port5_schedulers", S_IFREG | S_IRUGO | S_IWUGO, scheduler_dir, &scheduler_fops, id_scheduler_settings, FILE_PORT5_SCHEDULER );
    CREATE_PROC_FILE_ID( proc_file, "port6_schedulers", S_IFREG | S_IRUGO | S_IWUGO, scheduler_dir, &scheduler_fops, id_scheduler_settings, FILE_PORT6_SCHEDULER );
    CREATE_PROC_FILE_ID( proc_file, "port7_schedulers", S_IFREG | S_IRUGO | S_IWUGO, scheduler_dir, &scheduler_fops, id_scheduler_settings, FILE_PORT7_SCHEDULER );
    CREATE_PROC_FILE_ID( proc_file, "port8_schedulers", S_IFREG | S_IRUGO | S_IWUGO, scheduler_dir, &scheduler_fops, id_scheduler_settings, FILE_PORT8_SCHEDULER );
    CREATE_PROC_FILE_ID( proc_file, "port9_schedulers", S_IFREG | S_IRUGO | S_IWUGO, scheduler_dir, &scheduler_fops, id_scheduler_settings, FILE_PORT9_SCHEDULER );
    CREATE_PROC_FILE_ID( proc_file, "port10_schedulers", S_IFREG | S_IRUGO | S_IWUGO, scheduler_dir, &scheduler_fops, id_scheduler_settings, FILE_PORT10_SCHEDULER );
    CREATE_PROC_FILE_ID( proc_file, "port11_schedulers", S_IFREG | S_IRUGO | S_IWUGO, scheduler_dir, &scheduler_fops, id_scheduler_settings, FILE_PORT11_SCHEDULER );
    CREATE_PROC_FILE_ID( proc_file, "port12_schedulers", S_IFREG | S_IRUGO | S_IWUGO, scheduler_dir, &scheduler_fops, id_scheduler_settings, FILE_PORT12_SCHEDULER );
    CREATE_PROC_FILE_ID( proc_file, "port13_schedulers", S_IFREG | S_IRUGO | S_IWUGO, scheduler_dir, &scheduler_fops, id_scheduler_settings, FILE_PORT13_SCHEDULER );
    CREATE_PROC_FILE_ID( proc_file, "port14_schedulers", S_IFREG | S_IRUGO | S_IWUGO, scheduler_dir, &scheduler_fops, id_scheduler_settings, FILE_PORT14_SCHEDULER );
    CREATE_PROC_FILE_ID( proc_file, "port15_schedulers", S_IFREG | S_IRUGO | S_IWUGO, scheduler_dir, &scheduler_fops, id_scheduler_settings, FILE_PORT15_SCHEDULER );
    CREATE_PROC_FILE_ID( proc_file, "port16_schedulers", S_IFREG | S_IRUGO | S_IWUGO, scheduler_dir, &scheduler_fops, id_scheduler_settings, FILE_PORT16_SCHEDULER );
    CREATE_PROC_FILE_ID( proc_file, "host_port_schedulers", S_IFREG | S_IRUGO | S_IWUGO, scheduler_dir, &scheduler_fops, id_scheduler_settings, FILE_HOST_SCHEDULER );

    for(i = 0; i < MAX_FILES_PORT_SHAPERS; i++){
        shaper_number[i] = 0;
    }

    shaper_dir = proc_mkdir( "shaper_settings", basis_dir );                 // Create statistic folder
    if(shaper_dir == NULL)
    {
        result = -ENOMEM;
        goto err_dir_shaper;
    }

    CREATE_PROC_FILE_ID( proc_file, "port1_shapers",   S_IFREG | S_IRUGO | S_IWUGO, shaper_dir, &shaper_fops, id_port_shaper, FILE_PORT1_SHAPERS );
    CREATE_PROC_FILE_ID( proc_file, "port2_shapers",   S_IFREG | S_IRUGO | S_IWUGO, shaper_dir, &shaper_fops, id_port_shaper, FILE_PORT2_SHAPERS );
    CREATE_PROC_FILE_ID( proc_file, "port3_shapers",   S_IFREG | S_IRUGO | S_IWUGO, shaper_dir, &shaper_fops, id_port_shaper, FILE_PORT3_SHAPERS );
    CREATE_PROC_FILE_ID( proc_file, "port4_shapers",   S_IFREG | S_IRUGO | S_IWUGO, shaper_dir, &shaper_fops, id_port_shaper, FILE_PORT4_SHAPERS );
    CREATE_PROC_FILE_ID( proc_file, "port5_shapers",   S_IFREG | S_IRUGO | S_IWUGO, shaper_dir, &shaper_fops, id_port_shaper, FILE_PORT5_SHAPERS );
    CREATE_PROC_FILE_ID( proc_file, "port6_shapers",   S_IFREG | S_IRUGO | S_IWUGO, shaper_dir, &shaper_fops, id_port_shaper, FILE_PORT6_SHAPERS );
    CREATE_PROC_FILE_ID( proc_file, "port7_shapers",   S_IFREG | S_IRUGO | S_IWUGO, shaper_dir, &shaper_fops, id_port_shaper, FILE_PORT7_SHAPERS );
    CREATE_PROC_FILE_ID( proc_file, "port8_shapers",   S_IFREG | S_IRUGO | S_IWUGO, shaper_dir, &shaper_fops, id_port_shaper, FILE_PORT8_SHAPERS );
    CREATE_PROC_FILE_ID( proc_file, "port9_shapers",   S_IFREG | S_IRUGO | S_IWUGO, shaper_dir, &shaper_fops, id_port_shaper, FILE_PORT9_SHAPERS );
    CREATE_PROC_FILE_ID( proc_file, "port10_shapers",  S_IFREG | S_IRUGO | S_IWUGO, shaper_dir, &shaper_fops, id_port_shaper, FILE_PORT10_SHAPERS );
    CREATE_PROC_FILE_ID( proc_file, "port11_shapers",  S_IFREG | S_IRUGO | S_IWUGO, shaper_dir, &shaper_fops, id_port_shaper, FILE_PORT11_SHAPERS );
    CREATE_PROC_FILE_ID( proc_file, "port12_shapers",  S_IFREG | S_IRUGO | S_IWUGO, shaper_dir, &shaper_fops, id_port_shaper, FILE_PORT12_SHAPERS );
    CREATE_PROC_FILE_ID( proc_file, "port13_shapers",  S_IFREG | S_IRUGO | S_IWUGO, shaper_dir, &shaper_fops, id_port_shaper, FILE_PORT13_SHAPERS );
    CREATE_PROC_FILE_ID( proc_file, "port14_shapers",  S_IFREG | S_IRUGO | S_IWUGO, shaper_dir, &shaper_fops, id_port_shaper, FILE_PORT14_SHAPERS );
    CREATE_PROC_FILE_ID( proc_file, "port15_shapers",  S_IFREG | S_IRUGO | S_IWUGO, shaper_dir, &shaper_fops, id_port_shaper, FILE_PORT15_SHAPERS );
    CREATE_PROC_FILE_ID( proc_file, "port16_shapers",  S_IFREG | S_IRUGO | S_IWUGO, shaper_dir, &shaper_fops, id_port_shaper, FILE_PORT16_SHAPERS );
    CREATE_PROC_FILE_ID( proc_file, "host_port_shapers",  S_IFREG | S_IRUGO | S_IWUGO, shaper_dir, &shaper_fops, id_port_shaper, FILE_HOST_SHAPERS );

    dos_dir = proc_mkdir( "dos_control", basis_dir );                 // Create statistic folder
    if(dos_dir == NULL)
    {
        result = -ENOMEM;
        goto err_dir_dos;
    }

    CREATE_PROC_FILE_ID( proc_file, "dos_ctrl",   S_IFREG | S_IRUGO | S_IWUGO, dos_dir, &dos_fops, id_dos_control, FILE_DOS_CONTROL );

    private_vlan_dir = proc_mkdir( "private_vlan", basis_dir );
    if(private_vlan_dir == NULL)
    {
        result = -ENOMEM;
        goto err_dir_private_vlan;
    }

    CREATE_PROC_FILE_ID( proc_file, "port1_pvlan",   S_IFREG | S_IRUGO | S_IWUGO, private_vlan_dir, &pvlan_settings_fops, id_port_pvlan, FILE_PVLAN_PORT1 );
    CREATE_PROC_FILE_ID( proc_file, "port2_pvlan",   S_IFREG | S_IRUGO | S_IWUGO, private_vlan_dir, &pvlan_settings_fops, id_port_pvlan, FILE_PVLAN_PORT2 );
    CREATE_PROC_FILE_ID( proc_file, "port3_pvlan",   S_IFREG | S_IRUGO | S_IWUGO, private_vlan_dir, &pvlan_settings_fops, id_port_pvlan, FILE_PVLAN_PORT3 );
    CREATE_PROC_FILE_ID( proc_file, "port4_pvlan",   S_IFREG | S_IRUGO | S_IWUGO, private_vlan_dir, &pvlan_settings_fops, id_port_pvlan, FILE_PVLAN_PORT4 );
    CREATE_PROC_FILE_ID( proc_file, "port5_pvlan",   S_IFREG | S_IRUGO | S_IWUGO, private_vlan_dir, &pvlan_settings_fops, id_port_pvlan, FILE_PVLAN_PORT5 );
    CREATE_PROC_FILE_ID( proc_file, "port6_pvlan",   S_IFREG | S_IRUGO | S_IWUGO, private_vlan_dir, &pvlan_settings_fops, id_port_pvlan, FILE_PVLAN_PORT6 );
    CREATE_PROC_FILE_ID( proc_file, "port7_pvlan",   S_IFREG | S_IRUGO | S_IWUGO, private_vlan_dir, &pvlan_settings_fops, id_port_pvlan, FILE_PVLAN_PORT7 );
    CREATE_PROC_FILE_ID( proc_file, "port8_pvlan",   S_IFREG | S_IRUGO | S_IWUGO, private_vlan_dir, &pvlan_settings_fops, id_port_pvlan, FILE_PVLAN_PORT8 );
    CREATE_PROC_FILE_ID( proc_file, "port9_pvlan",   S_IFREG | S_IRUGO | S_IWUGO, private_vlan_dir, &pvlan_settings_fops, id_port_pvlan, FILE_PVLAN_PORT9 );
    CREATE_PROC_FILE_ID( proc_file, "port10_pvlan",  S_IFREG | S_IRUGO | S_IWUGO, private_vlan_dir, &pvlan_settings_fops, id_port_pvlan, FILE_PVLAN_PORT10 );
    CREATE_PROC_FILE_ID( proc_file, "port11_pvlan",  S_IFREG | S_IRUGO | S_IWUGO, private_vlan_dir, &pvlan_settings_fops, id_port_pvlan, FILE_PVLAN_PORT11 );
    CREATE_PROC_FILE_ID( proc_file, "port12_pvlan",  S_IFREG | S_IRUGO | S_IWUGO, private_vlan_dir, &pvlan_settings_fops, id_port_pvlan, FILE_PVLAN_PORT12 );
    CREATE_PROC_FILE_ID( proc_file, "port13_pvlan",  S_IFREG | S_IRUGO | S_IWUGO, private_vlan_dir, &pvlan_settings_fops, id_port_pvlan, FILE_PVLAN_PORT13 );
    CREATE_PROC_FILE_ID( proc_file, "port14_pvlan",  S_IFREG | S_IRUGO | S_IWUGO, private_vlan_dir, &pvlan_settings_fops, id_port_pvlan, FILE_PVLAN_PORT14 );
    CREATE_PROC_FILE_ID( proc_file, "port15_pvlan",  S_IFREG | S_IRUGO | S_IWUGO, private_vlan_dir, &pvlan_settings_fops, id_port_pvlan, FILE_PVLAN_PORT15 );
    CREATE_PROC_FILE_ID( proc_file, "port16_pvlan",  S_IFREG | S_IRUGO | S_IWUGO, private_vlan_dir, &pvlan_settings_fops, id_port_pvlan, FILE_PVLAN_PORT16 );
    CREATE_PROC_FILE_ID( proc_file, "host_port_pvlan",  S_IFREG | S_IRUGO | S_IWUGO, private_vlan_dir, &pvlan_settings_fops, id_port_pvlan, FILE_PVLAN_HOST_PORT );

    pcks_to_queues_dir = proc_mkdir( "pcks_to_queues_mapping", basis_dir );
    if(pcks_to_queues_dir == NULL)
    {
        result = -ENOMEM;
        goto err_dir_pcks_to_queues;
    }

    CREATE_PROC_FILE_ID( proc_file, "port1_pcks_to_queues_map",   S_IFREG | S_IRUGO | S_IWUGO, pcks_to_queues_dir, &pcks_to_queues_map_fops, id_pcks_to_queues, FILE_PCKS_TO_QUEUES_PORT1 );
    CREATE_PROC_FILE_ID( proc_file, "port2_pcks_to_queues_map",   S_IFREG | S_IRUGO | S_IWUGO, pcks_to_queues_dir, &pcks_to_queues_map_fops, id_pcks_to_queues, FILE_PCKS_TO_QUEUES_PORT2 );
    CREATE_PROC_FILE_ID( proc_file, "port3_pcks_to_queues_map",   S_IFREG | S_IRUGO | S_IWUGO, pcks_to_queues_dir, &pcks_to_queues_map_fops, id_pcks_to_queues, FILE_PCKS_TO_QUEUES_PORT3 );
    CREATE_PROC_FILE_ID( proc_file, "port4_pcks_to_queues_map",   S_IFREG | S_IRUGO | S_IWUGO, pcks_to_queues_dir, &pcks_to_queues_map_fops, id_pcks_to_queues, FILE_PCKS_TO_QUEUES_PORT4 );
    CREATE_PROC_FILE_ID( proc_file, "port5_pcks_to_queues_map",   S_IFREG | S_IRUGO | S_IWUGO, pcks_to_queues_dir, &pcks_to_queues_map_fops, id_pcks_to_queues, FILE_PCKS_TO_QUEUES_PORT5 );
    CREATE_PROC_FILE_ID( proc_file, "port6_pcks_to_queues_map",   S_IFREG | S_IRUGO | S_IWUGO, pcks_to_queues_dir, &pcks_to_queues_map_fops, id_pcks_to_queues, FILE_PCKS_TO_QUEUES_PORT6 );
    CREATE_PROC_FILE_ID( proc_file, "port7_pcks_to_queues_map",   S_IFREG | S_IRUGO | S_IWUGO, pcks_to_queues_dir, &pcks_to_queues_map_fops, id_pcks_to_queues, FILE_PCKS_TO_QUEUES_PORT7 );
    CREATE_PROC_FILE_ID( proc_file, "port8_pcks_to_queues_map",   S_IFREG | S_IRUGO | S_IWUGO, pcks_to_queues_dir, &pcks_to_queues_map_fops, id_pcks_to_queues, FILE_PCKS_TO_QUEUES_PORT8 );
    CREATE_PROC_FILE_ID( proc_file, "port9_pcks_to_queues_map",   S_IFREG | S_IRUGO | S_IWUGO, pcks_to_queues_dir, &pcks_to_queues_map_fops, id_pcks_to_queues, FILE_PCKS_TO_QUEUES_PORT9 );
    CREATE_PROC_FILE_ID( proc_file, "port10_pcks_to_queues_map",  S_IFREG | S_IRUGO | S_IWUGO, pcks_to_queues_dir, &pcks_to_queues_map_fops, id_pcks_to_queues, FILE_PCKS_TO_QUEUES_PORT10 );
    CREATE_PROC_FILE_ID( proc_file, "port11_pcks_to_queues_map",  S_IFREG | S_IRUGO | S_IWUGO, pcks_to_queues_dir, &pcks_to_queues_map_fops, id_pcks_to_queues, FILE_PCKS_TO_QUEUES_PORT11 );
    CREATE_PROC_FILE_ID( proc_file, "port12_pcks_to_queues_map",  S_IFREG | S_IRUGO | S_IWUGO, pcks_to_queues_dir, &pcks_to_queues_map_fops, id_pcks_to_queues, FILE_PCKS_TO_QUEUES_PORT12 );
    CREATE_PROC_FILE_ID( proc_file, "port13_pcks_to_queues_map",  S_IFREG | S_IRUGO | S_IWUGO, pcks_to_queues_dir, &pcks_to_queues_map_fops, id_pcks_to_queues, FILE_PCKS_TO_QUEUES_PORT13 );
    CREATE_PROC_FILE_ID( proc_file, "port14_pcks_to_queues_map",  S_IFREG | S_IRUGO | S_IWUGO, pcks_to_queues_dir, &pcks_to_queues_map_fops, id_pcks_to_queues, FILE_PCKS_TO_QUEUES_PORT14 );
    CREATE_PROC_FILE_ID( proc_file, "port15_pcks_to_queues_map",  S_IFREG | S_IRUGO | S_IWUGO, pcks_to_queues_dir, &pcks_to_queues_map_fops, id_pcks_to_queues, FILE_PCKS_TO_QUEUES_PORT15 );
    CREATE_PROC_FILE_ID( proc_file, "port16_pcks_to_queues_map",  S_IFREG | S_IRUGO | S_IWUGO, pcks_to_queues_dir, &pcks_to_queues_map_fops, id_pcks_to_queues, FILE_PCKS_TO_QUEUES_PORT16 );
    CREATE_PROC_FILE_ID( proc_file, "host_port_pcks_to_queues_map",  S_IFREG | S_IRUGO | S_IWUGO, pcks_to_queues_dir, &pcks_to_queues_map_fops, id_pcks_to_queues, FILE_PCKS_TO_QUEUES_HOST_PORT );
    CREATE_PROC_FILE_ID( proc_file, "global_dscp_pcks_to_queues_map",  S_IFREG | S_IRUGO | S_IWUGO, pcks_to_queues_dir, &pcks_to_queues_map_fops, id_pcks_to_queues, FILE_PCKS_TO_QUEUES_GLOBAL_DSCP_2_TC );

    flow_control_dir = proc_mkdir( "flow_control", basis_dir );
    if(flow_control_dir == NULL)
    {
        result = -ENOMEM;
        goto err_dir_flow_control;
    }

    CREATE_PROC_FILE_ID( proc_file, "port1_fc",   S_IFREG | S_IRUGO | S_IWUGO, flow_control_dir, &flow_control_fops, id_flow_control, FILE_FLOW_CONTROL_PORT1 );
    CREATE_PROC_FILE_ID( proc_file, "port2_fc",   S_IFREG | S_IRUGO | S_IWUGO, flow_control_dir, &flow_control_fops, id_flow_control, FILE_FLOW_CONTROL_PORT2 );
    CREATE_PROC_FILE_ID( proc_file, "port3_fc",   S_IFREG | S_IRUGO | S_IWUGO, flow_control_dir, &flow_control_fops, id_flow_control, FILE_FLOW_CONTROL_PORT3 );
    CREATE_PROC_FILE_ID( proc_file, "port4_fc",   S_IFREG | S_IRUGO | S_IWUGO, flow_control_dir, &flow_control_fops, id_flow_control, FILE_FLOW_CONTROL_PORT4 );
    CREATE_PROC_FILE_ID( proc_file, "port5_fc",   S_IFREG | S_IRUGO | S_IWUGO, flow_control_dir, &flow_control_fops, id_flow_control, FILE_FLOW_CONTROL_PORT5 );
    CREATE_PROC_FILE_ID( proc_file, "port6_fc",   S_IFREG | S_IRUGO | S_IWUGO, flow_control_dir, &flow_control_fops, id_flow_control, FILE_FLOW_CONTROL_PORT6 );
    CREATE_PROC_FILE_ID( proc_file, "port7_fc",   S_IFREG | S_IRUGO | S_IWUGO, flow_control_dir, &flow_control_fops, id_flow_control, FILE_FLOW_CONTROL_PORT7 );
    CREATE_PROC_FILE_ID( proc_file, "port8_fc",   S_IFREG | S_IRUGO | S_IWUGO, flow_control_dir, &flow_control_fops, id_flow_control, FILE_FLOW_CONTROL_PORT8 );
    CREATE_PROC_FILE_ID( proc_file, "port9_fc",   S_IFREG | S_IRUGO | S_IWUGO, flow_control_dir, &flow_control_fops, id_flow_control, FILE_FLOW_CONTROL_PORT9 );
    CREATE_PROC_FILE_ID( proc_file, "port10_fc",  S_IFREG | S_IRUGO | S_IWUGO, flow_control_dir, &flow_control_fops, id_flow_control, FILE_FLOW_CONTROL_PORT10 );
    CREATE_PROC_FILE_ID( proc_file, "port11_fc",  S_IFREG | S_IRUGO | S_IWUGO, flow_control_dir, &flow_control_fops, id_flow_control, FILE_FLOW_CONTROL_PORT11 );
    CREATE_PROC_FILE_ID( proc_file, "port12_fc",  S_IFREG | S_IRUGO | S_IWUGO, flow_control_dir, &flow_control_fops, id_flow_control, FILE_FLOW_CONTROL_PORT12 );
    CREATE_PROC_FILE_ID( proc_file, "port13_fc",  S_IFREG | S_IRUGO | S_IWUGO, flow_control_dir, &flow_control_fops, id_flow_control, FILE_FLOW_CONTROL_PORT13 );
    CREATE_PROC_FILE_ID( proc_file, "port14_fc",  S_IFREG | S_IRUGO | S_IWUGO, flow_control_dir, &flow_control_fops, id_flow_control, FILE_FLOW_CONTROL_PORT14 );
    CREATE_PROC_FILE_ID( proc_file, "port15_fc",  S_IFREG | S_IRUGO | S_IWUGO, flow_control_dir, &flow_control_fops, id_flow_control, FILE_FLOW_CONTROL_PORT15 );
    CREATE_PROC_FILE_ID( proc_file, "port16_fc",  S_IFREG | S_IRUGO | S_IWUGO, flow_control_dir, &flow_control_fops, id_flow_control, FILE_FLOW_CONTROL_PORT16 );

    eth_phy_settings_port_dir = proc_mkdir( "ports_ethernet_phy_settings", basis_dir );
    if(eth_phy_settings_port_dir == NULL)
    {
        result = -ENOMEM;
        goto err_dir_eth_phy_settings_port;
    }

    CREATE_PROC_FILE_ID( proc_file, "port1_eth_phy_settings",   S_IFREG | S_IRUGO | S_IWUGO, eth_phy_settings_port_dir, &eth_phy_settings_port_ops, id_eth_phy_settings_port, FILE_ETH_PHY_SETTINGS_PORT1 );
    CREATE_PROC_FILE_ID( proc_file, "port2_eth_phy_settings",   S_IFREG | S_IRUGO | S_IWUGO, eth_phy_settings_port_dir, &eth_phy_settings_port_ops, id_eth_phy_settings_port, FILE_ETH_PHY_SETTINGS_PORT2 );
    CREATE_PROC_FILE_ID( proc_file, "port3_eth_phy_settings",   S_IFREG | S_IRUGO | S_IWUGO, eth_phy_settings_port_dir, &eth_phy_settings_port_ops, id_eth_phy_settings_port, FILE_ETH_PHY_SETTINGS_PORT3 );
    CREATE_PROC_FILE_ID( proc_file, "port4_eth_phy_settings",   S_IFREG | S_IRUGO | S_IWUGO, eth_phy_settings_port_dir, &eth_phy_settings_port_ops, id_eth_phy_settings_port, FILE_ETH_PHY_SETTINGS_PORT4 );
    CREATE_PROC_FILE_ID( proc_file, "port5_eth_phy_settings",   S_IFREG | S_IRUGO | S_IWUGO, eth_phy_settings_port_dir, &eth_phy_settings_port_ops, id_eth_phy_settings_port, FILE_ETH_PHY_SETTINGS_PORT5 );
    CREATE_PROC_FILE_ID( proc_file, "port6_eth_phy_settings",   S_IFREG | S_IRUGO | S_IWUGO, eth_phy_settings_port_dir, &eth_phy_settings_port_ops, id_eth_phy_settings_port, FILE_ETH_PHY_SETTINGS_PORT6 );
    CREATE_PROC_FILE_ID( proc_file, "port7_eth_phy_settings",   S_IFREG | S_IRUGO | S_IWUGO, eth_phy_settings_port_dir, &eth_phy_settings_port_ops, id_eth_phy_settings_port, FILE_ETH_PHY_SETTINGS_PORT7 );
    CREATE_PROC_FILE_ID( proc_file, "port8_eth_phy_settings",   S_IFREG | S_IRUGO | S_IWUGO, eth_phy_settings_port_dir, &eth_phy_settings_port_ops, id_eth_phy_settings_port, FILE_ETH_PHY_SETTINGS_PORT8 );
    CREATE_PROC_FILE_ID( proc_file, "port9_eth_phy_settings",   S_IFREG | S_IRUGO | S_IWUGO, eth_phy_settings_port_dir, &eth_phy_settings_port_ops, id_eth_phy_settings_port, FILE_ETH_PHY_SETTINGS_PORT9 );
    CREATE_PROC_FILE_ID( proc_file, "port10_eth_phy_settings",  S_IFREG | S_IRUGO | S_IWUGO, eth_phy_settings_port_dir, &eth_phy_settings_port_ops, id_eth_phy_settings_port, FILE_ETH_PHY_SETTINGS_PORT10 );
    CREATE_PROC_FILE_ID( proc_file, "port11_eth_phy_settings",  S_IFREG | S_IRUGO | S_IWUGO, eth_phy_settings_port_dir, &eth_phy_settings_port_ops, id_eth_phy_settings_port, FILE_ETH_PHY_SETTINGS_PORT11 );
    CREATE_PROC_FILE_ID( proc_file, "port12_eth_phy_settings",  S_IFREG | S_IRUGO | S_IWUGO, eth_phy_settings_port_dir, &eth_phy_settings_port_ops, id_eth_phy_settings_port, FILE_ETH_PHY_SETTINGS_PORT12 );
    CREATE_PROC_FILE_ID( proc_file, "port13_eth_phy_settings",  S_IFREG | S_IRUGO | S_IWUGO, eth_phy_settings_port_dir, &eth_phy_settings_port_ops, id_eth_phy_settings_port, FILE_ETH_PHY_SETTINGS_PORT13 );
    CREATE_PROC_FILE_ID( proc_file, "port14_eth_phy_settings",  S_IFREG | S_IRUGO | S_IWUGO, eth_phy_settings_port_dir, &eth_phy_settings_port_ops, id_eth_phy_settings_port, FILE_ETH_PHY_SETTINGS_PORT14 );
    CREATE_PROC_FILE_ID( proc_file, "port15_eth_phy_settings",  S_IFREG | S_IRUGO | S_IWUGO, eth_phy_settings_port_dir, &eth_phy_settings_port_ops, id_eth_phy_settings_port, FILE_ETH_PHY_SETTINGS_PORT15 );
    CREATE_PROC_FILE_ID( proc_file, "port16_eth_phy_settings",  S_IFREG | S_IRUGO | S_IWUGO, eth_phy_settings_port_dir, &eth_phy_settings_port_ops, id_eth_phy_settings_port, FILE_ETH_PHY_SETTINGS_PORT16 );

    optic_module_dir = proc_mkdir( "sfp_modules", basis_dir );
    if( optic_module_dir == NULL ){
        result = -ENOMEM;
        goto err_dir_optic_module;
    }

    CREATE_PROC_FILE_ID( proc_file, "sfp_port1", S_IFREG | S_IRUGO, optic_module_dir, &optic_module_ops, id_optic_module_port, FILE_OPTIC_MODULE_PORT1 );
    CREATE_PROC_FILE_ID( proc_file, "sfp_port2", S_IFREG | S_IRUGO, optic_module_dir, &optic_module_ops, id_optic_module_port, FILE_OPTIC_MODULE_PORT2 );
    CREATE_PROC_FILE_ID( proc_file, "sfp_port3", S_IFREG | S_IRUGO, optic_module_dir, &optic_module_ops, id_optic_module_port, FILE_OPTIC_MODULE_PORT3 );
    CREATE_PROC_FILE_ID( proc_file, "sfp_port4", S_IFREG | S_IRUGO, optic_module_dir, &optic_module_ops, id_optic_module_port, FILE_OPTIC_MODULE_PORT4 );

    FP_DEBUG(DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "proc FS init end\n");
    return 0;

    /* Errors processing */
err_dir_optic_module:
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port1_eth_phy_settings",  NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port2_eth_phy_settings",  NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port3_eth_phy_settings",  NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port4_eth_phy_settings",  NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port5_eth_phy_settings",  NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port6_eth_phy_settings",  NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port7_eth_phy_settings",  NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port8_eth_phy_settings",  NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port9_eth_phy_settings",  NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port10_eth_phy_settings", NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port11_eth_phy_settings", NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port12_eth_phy_settings", NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port13_eth_phy_settings", NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port14_eth_phy_settings", NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port15_eth_phy_settings", NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port16_eth_phy_settings", NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings", NULL );    
err_dir_eth_phy_settings_port:
    remove_proc_entry( "basis/flow_control/port1_fc",  NULL );
    remove_proc_entry( "basis/flow_control/port2_fc",  NULL );
    remove_proc_entry( "basis/flow_control/port3_fc",  NULL );
    remove_proc_entry( "basis/flow_control/port4_fc",  NULL );
    remove_proc_entry( "basis/flow_control/port5_fc",  NULL );
    remove_proc_entry( "basis/flow_control/port6_fc",  NULL );
    remove_proc_entry( "basis/flow_control/port7_fc",  NULL );
    remove_proc_entry( "basis/flow_control/port8_fc",  NULL );
    remove_proc_entry( "basis/flow_control/port9_fc",  NULL );
    remove_proc_entry( "basis/flow_control/port10_fc", NULL );
    remove_proc_entry( "basis/flow_control/port11_fc", NULL );
    remove_proc_entry( "basis/flow_control/port12_fc", NULL );
    remove_proc_entry( "basis/flow_control/port13_fc", NULL );
    remove_proc_entry( "basis/flow_control/port14_fc", NULL );
    remove_proc_entry( "basis/flow_control/port15_fc", NULL );
    remove_proc_entry( "basis/flow_control/port16_fc", NULL );
    remove_proc_entry( "basis/flow_control", NULL );
err_dir_flow_control:
    remove_proc_entry( "basis/pcks_to_queues_mapping/port1_pcks_to_queues_map",  NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port2_pcks_to_queues_map",  NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port3_pcks_to_queues_map",  NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port4_pcks_to_queues_map",  NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port5_pcks_to_queues_map",  NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port6_pcks_to_queues_map",  NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port7_pcks_to_queues_map",  NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port8_pcks_to_queues_map",  NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port9_pcks_to_queues_map",  NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port10_pcks_to_queues_map", NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port11_pcks_to_queues_map", NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port12_pcks_to_queues_map", NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port13_pcks_to_queues_map", NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port14_pcks_to_queues_map", NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port15_pcks_to_queues_map", NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port16_pcks_to_queues_map", NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/host_port_pcks_to_queues_map",   NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/global_dscp_pcks_to_queues_map", NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping", NULL );
err_dir_pcks_to_queues:
    remove_proc_entry( "basis/private_vlan/port1_pvlan",  NULL );
    remove_proc_entry( "basis/private_vlan/port2_pvlan",  NULL );
    remove_proc_entry( "basis/private_vlan/port3_pvlan",  NULL );
    remove_proc_entry( "basis/private_vlan/port4_pvlan",  NULL );
    remove_proc_entry( "basis/private_vlan/port5_pvlan",  NULL );
    remove_proc_entry( "basis/private_vlan/port6_pvlan",  NULL );
    remove_proc_entry( "basis/private_vlan/port7_pvlan",  NULL );
    remove_proc_entry( "basis/private_vlan/port8_pvlan",  NULL );
    remove_proc_entry( "basis/private_vlan/port9_pvlan",  NULL );
    remove_proc_entry( "basis/private_vlan/port10_pvlan", NULL );
    remove_proc_entry( "basis/private_vlan/port11_pvlan", NULL );
    remove_proc_entry( "basis/private_vlan/port12_pvlan", NULL );
    remove_proc_entry( "basis/private_vlan/port13_pvlan", NULL );
    remove_proc_entry( "basis/private_vlan/port14_pvlan", NULL );
    remove_proc_entry( "basis/private_vlan/port15_pvlan", NULL );
    remove_proc_entry( "basis/private_vlan/port16_pvlan", NULL );
    remove_proc_entry( "basis/private_vlan/host_port_pvlan", NULL );
    remove_proc_entry( "basis/private_vlan", NULL );
err_dir_private_vlan:
    remove_proc_entry( "basis/dos_control/dos_ctrl", NULL );
    remove_proc_entry( "basis/dos_control", NULL );
err_dir_dos:
    remove_proc_entry( "basis/shaper_settings/port1_shapers",  NULL );
    remove_proc_entry( "basis/shaper_settings/port2_shapers",  NULL );
    remove_proc_entry( "basis/shaper_settings/port3_shapers",  NULL );
    remove_proc_entry( "basis/shaper_settings/port4_shapers",  NULL );
    remove_proc_entry( "basis/shaper_settings/port5_shapers",  NULL );
    remove_proc_entry( "basis/shaper_settings/port6_shapers",  NULL );
    remove_proc_entry( "basis/shaper_settings/port7_shapers",  NULL );
    remove_proc_entry( "basis/shaper_settings/port8_shapers",  NULL );
    remove_proc_entry( "basis/shaper_settings/port9_shapers",  NULL );
    remove_proc_entry( "basis/shaper_settings/port10_shapers", NULL );
    remove_proc_entry( "basis/shaper_settings/port11_shapers", NULL );
    remove_proc_entry( "basis/shaper_settings/port12_shapers", NULL );
    remove_proc_entry( "basis/shaper_settings/port13_shapers", NULL );
    remove_proc_entry( "basis/shaper_settings/port14_shapers", NULL );
    remove_proc_entry( "basis/shaper_settings/port15_shapers", NULL );
    remove_proc_entry( "basis/shaper_settings/port16_shapers", NULL );
    remove_proc_entry( "basis/shaper_settings/host_port_shapers", NULL );
    remove_proc_entry( "basis/shaper_settings", NULL );
err_dir_shaper:
    remove_proc_entry( "basis/scheduler_settings/port1_schedulers",  NULL );
    remove_proc_entry( "basis/scheduler_settings/port2_schedulers",  NULL );
    remove_proc_entry( "basis/scheduler_settings/port3_schedulers",  NULL );
    remove_proc_entry( "basis/scheduler_settings/port4_schedulers",  NULL );
    remove_proc_entry( "basis/scheduler_settings/port5_schedulers",  NULL );
    remove_proc_entry( "basis/scheduler_settings/port6_schedulers",  NULL );
    remove_proc_entry( "basis/scheduler_settings/port7_schedulers",  NULL );
    remove_proc_entry( "basis/scheduler_settings/port8_schedulers",  NULL );
    remove_proc_entry( "basis/scheduler_settings/port9_schedulers",  NULL );
    remove_proc_entry( "basis/scheduler_settings/port10_schedulers", NULL );
    remove_proc_entry( "basis/scheduler_settings/port11_schedulers", NULL );
    remove_proc_entry( "basis/scheduler_settings/port12_schedulers", NULL );
    remove_proc_entry( "basis/scheduler_settings/port13_schedulers", NULL );
    remove_proc_entry( "basis/scheduler_settings/port14_schedulers", NULL );
    remove_proc_entry( "basis/scheduler_settings/port15_schedulers", NULL );
    remove_proc_entry( "basis/scheduler_settings/port16_schedulers", NULL );
    remove_proc_entry( "basis/scheduler_settings/host_port_schedulers", NULL );
    remove_proc_entry( "basis/scheduler_settings", NULL );
err_dir_scheduler:
    remove_proc_entry( "basis/debug/print_message_level",  NULL );
    remove_proc_entry( "basis/debug/wr_packet",            NULL );
    remove_proc_entry( "basis/debug/ageing_timeout",       NULL );
    remove_proc_entry( "basis/debug/bmu_occupied_buffers", NULL );
    remove_proc_entry( "basis/debug/far_end_loopback",     NULL );
    remove_proc_entry( "basis/debug/host_system_loopback", NULL );
    remove_proc_entry( "basis/debug/jumbo_enable",         NULL );
    remove_proc_entry( "basis/debug/mdio_speed",           NULL );
    remove_proc_entry( "basis/debug/phy_test_mode",        NULL );
    remove_proc_entry( "basis/debug/mcu_fw_version",       NULL );
    remove_proc_entry( "basis/debug", NULL );
err_dir_debug:
    remove_proc_entry( "basis/queue_settings/port1_queues",  NULL );
    remove_proc_entry( "basis/queue_settings/port2_queues",  NULL );
    remove_proc_entry( "basis/queue_settings/port3_queues",  NULL );
    remove_proc_entry( "basis/queue_settings/port4_queues",  NULL );
    remove_proc_entry( "basis/queue_settings/port5_queues",  NULL );
    remove_proc_entry( "basis/queue_settings/port6_queues",  NULL );
    remove_proc_entry( "basis/queue_settings/port7_queues",  NULL );
    remove_proc_entry( "basis/queue_settings/port8_queues",  NULL );
    remove_proc_entry( "basis/queue_settings/port9_queues",  NULL );
    remove_proc_entry( "basis/queue_settings/port10_queues", NULL );
    remove_proc_entry( "basis/queue_settings/port11_queues", NULL );
    remove_proc_entry( "basis/queue_settings/port12_queues", NULL );
    remove_proc_entry( "basis/queue_settings/port13_queues", NULL );
    remove_proc_entry( "basis/queue_settings/port14_queues", NULL );
    remove_proc_entry( "basis/queue_settings/port15_queues", NULL );
    remove_proc_entry( "basis/queue_settings/port16_queues", NULL );
    remove_proc_entry( "basis/queue_settings/host_port_queues", NULL );
    remove_proc_entry( "basis/queue_settings", NULL );
err_dir_queue_settings:
    remove_proc_entry( "basis/port_settings/port1",  NULL );
    remove_proc_entry( "basis/port_settings/port2",  NULL );
    remove_proc_entry( "basis/port_settings/port3",  NULL );
    remove_proc_entry( "basis/port_settings/port4",  NULL );
    remove_proc_entry( "basis/port_settings/port5",  NULL );
    remove_proc_entry( "basis/port_settings/port6",  NULL );
    remove_proc_entry( "basis/port_settings/port7",  NULL );
    remove_proc_entry( "basis/port_settings/port8",  NULL );
    remove_proc_entry( "basis/port_settings/port9",  NULL );
    remove_proc_entry( "basis/port_settings/port10", NULL );
    remove_proc_entry( "basis/port_settings/port11", NULL );
    remove_proc_entry( "basis/port_settings/port12", NULL );
    remove_proc_entry( "basis/port_settings/port13", NULL );
    remove_proc_entry( "basis/port_settings/port14", NULL );
    remove_proc_entry( "basis/port_settings/port15", NULL );
    remove_proc_entry( "basis/port_settings/port16", NULL );
    remove_proc_entry( "basis/port_settings/host_port", NULL );
    remove_proc_entry( "basis/port_settings", NULL );
err_dir_port_settings:
    remove_proc_entry( "basis/hash_tables/vlan/show",   NULL );
    remove_proc_entry( "basis/hash_tables/vlan/add",    NULL );
    remove_proc_entry( "basis/hash_tables/vlan/search", NULL );
    remove_proc_entry( "basis/hash_tables/vlan/delete", NULL );
    remove_proc_entry( "basis/hash_tables/vlan/clean",  NULL );
    remove_proc_entry( "basis/hash_tables/vlan/global_vlan_entry", NULL );
    remove_proc_entry( "basis/hash_tables/vlan", NULL );
err_dir_hash_tables_vlan:
    remove_proc_entry( "basis/hash_tables/mac/show",   NULL );
    remove_proc_entry( "basis/hash_tables/mac/add",    NULL );
    remove_proc_entry( "basis/hash_tables/mac/search", NULL );
    remove_proc_entry( "basis/hash_tables/mac/delete", NULL );
    remove_proc_entry( "basis/hash_tables/mac/clean",  NULL );
    remove_proc_entry( "basis/hash_tables/mac", NULL );
err_dir_hash_tables_mac:
    remove_proc_entry( "basis/hash_tables", NULL );
err_dir_hash_tables:
    remove_proc_entry( "basis/statistics/em1",  NULL );
    remove_proc_entry( "basis/statistics/em2",  NULL );
    remove_proc_entry( "basis/statistics/em3",  NULL );
    remove_proc_entry( "basis/statistics/em4",  NULL );
    remove_proc_entry( "basis/statistics/em5",  NULL );
    remove_proc_entry( "basis/statistics/em6",  NULL );
    remove_proc_entry( "basis/statistics/em7",  NULL );
    remove_proc_entry( "basis/statistics/em8",  NULL );
    remove_proc_entry( "basis/statistics/em9",  NULL );
    remove_proc_entry( "basis/statistics/em10", NULL );
    remove_proc_entry( "basis/statistics/em11", NULL );
    remove_proc_entry( "basis/statistics/em12", NULL );
    remove_proc_entry( "basis/statistics/em13", NULL );
    remove_proc_entry( "basis/statistics/em14", NULL );
    remove_proc_entry( "basis/statistics/em15", NULL );
    remove_proc_entry( "basis/statistics/em16", NULL );
    remove_proc_entry( "basis/statistics/switch", NULL );
    remove_proc_entry( "basis/statistics", NULL );
err_dir_statistic:
    remove_proc_entry( "basis/mem/rd_wr", NULL );
    remove_proc_entry( "basis/mem", NULL );
err_dir_mem:
    remove_proc_entry( "basis", NULL );
err_dir_basis:
    FP_DEBUG(DBGP_FEAT_PROCFS, FP_LOG_ERR, "proc FS init failed!\n");
    return result;
}

int fp_proc_fs_deinit( void )
{
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_INFO, "proc FS deinit start\n" );

    remove_proc_entry( "basis/statistics/em1",  NULL );
    remove_proc_entry( "basis/statistics/em2",  NULL );
    remove_proc_entry( "basis/statistics/em3",  NULL );
    remove_proc_entry( "basis/statistics/em4",  NULL );
    remove_proc_entry( "basis/statistics/em5",  NULL );
    remove_proc_entry( "basis/statistics/em6",  NULL );
    remove_proc_entry( "basis/statistics/em7",  NULL );
    remove_proc_entry( "basis/statistics/em8",  NULL );
    remove_proc_entry( "basis/statistics/em9",  NULL );
    remove_proc_entry( "basis/statistics/em10", NULL );
    remove_proc_entry( "basis/statistics/em11", NULL );
    remove_proc_entry( "basis/statistics/em12", NULL );
    remove_proc_entry( "basis/statistics/em13", NULL );
    remove_proc_entry( "basis/statistics/em14", NULL );
    remove_proc_entry( "basis/statistics/em15", NULL );
    remove_proc_entry( "basis/statistics/em16", NULL );
    remove_proc_entry( "basis/statistics/switch", NULL );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "remove statistics\n" );
    remove_proc_entry( "basis/statistics", NULL );

    remove_proc_entry( "basis/hash_tables/mac/show",   NULL );
    remove_proc_entry( "basis/hash_tables/mac/add",    NULL );
    remove_proc_entry( "basis/hash_tables/mac/search", NULL );
    remove_proc_entry( "basis/hash_tables/mac/delete", NULL );
    remove_proc_entry( "basis/hash_tables/mac/clean",  NULL );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "remove hash_tables/mac\n" );
    remove_proc_entry( "basis/hash_tables/mac", NULL );

    remove_proc_entry( "basis/hash_tables/vlan/show",   NULL );
    remove_proc_entry( "basis/hash_tables/vlan/add",    NULL );
    remove_proc_entry( "basis/hash_tables/vlan/search", NULL );
    remove_proc_entry( "basis/hash_tables/vlan/delete", NULL );
    remove_proc_entry( "basis/hash_tables/vlan/clean",  NULL );
    remove_proc_entry( "basis/hash_tables/vlan/global_vlan_entry", NULL );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "remove hash_tables/vlan\n" );
    remove_proc_entry( "basis/hash_tables/vlan", NULL );

    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "remove hash_tables\n" );
    remove_proc_entry( "basis/hash_tables", NULL );

    remove_proc_entry( "basis/mem/rd_wr", NULL );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "remove mem\n" );
    remove_proc_entry( "basis/mem", NULL );

    remove_proc_entry( "basis/temperature", NULL );

    remove_proc_entry( "basis/port_settings/port1",  NULL );
    remove_proc_entry( "basis/port_settings/port2",  NULL );
    remove_proc_entry( "basis/port_settings/port3",  NULL );
    remove_proc_entry( "basis/port_settings/port4",  NULL );
    remove_proc_entry( "basis/port_settings/port5",  NULL );
    remove_proc_entry( "basis/port_settings/port6",  NULL );
    remove_proc_entry( "basis/port_settings/port7",  NULL );
    remove_proc_entry( "basis/port_settings/port8",  NULL );
    remove_proc_entry( "basis/port_settings/port9",  NULL );
    remove_proc_entry( "basis/port_settings/port10", NULL );
    remove_proc_entry( "basis/port_settings/port11", NULL );
    remove_proc_entry( "basis/port_settings/port12", NULL );
    remove_proc_entry( "basis/port_settings/port13", NULL );
    remove_proc_entry( "basis/port_settings/port14", NULL );
    remove_proc_entry( "basis/port_settings/port15", NULL );
    remove_proc_entry( "basis/port_settings/port16", NULL );
    remove_proc_entry( "basis/port_settings/host_port", NULL );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "remove port_settings\n" );
    remove_proc_entry( "basis/port_settings", NULL );

    remove_proc_entry( "basis/queue_settings/port1_queues",  NULL );
    remove_proc_entry( "basis/queue_settings/port2_queues",  NULL );
    remove_proc_entry( "basis/queue_settings/port3_queues",  NULL );
    remove_proc_entry( "basis/queue_settings/port4_queues",  NULL );
    remove_proc_entry( "basis/queue_settings/port5_queues",  NULL );
    remove_proc_entry( "basis/queue_settings/port6_queues",  NULL );
    remove_proc_entry( "basis/queue_settings/port7_queues",  NULL );
    remove_proc_entry( "basis/queue_settings/port8_queues",  NULL );
    remove_proc_entry( "basis/queue_settings/port9_queues",  NULL );
    remove_proc_entry( "basis/queue_settings/port10_queues", NULL );
    remove_proc_entry( "basis/queue_settings/port11_queues", NULL );
    remove_proc_entry( "basis/queue_settings/port12_queues", NULL );
    remove_proc_entry( "basis/queue_settings/port13_queues", NULL );
    remove_proc_entry( "basis/queue_settings/port14_queues", NULL );
    remove_proc_entry( "basis/queue_settings/port15_queues", NULL );
    remove_proc_entry( "basis/queue_settings/port16_queues", NULL );
    remove_proc_entry( "basis/queue_settings/host_port_queues", NULL );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "remove queue_settings\n" );
    remove_proc_entry( "basis/queue_settings", NULL );

    remove_proc_entry( "basis/debug/print_message_level", NULL );
    remove_proc_entry( "basis/debug/wr_packet", NULL );
    remove_proc_entry( "basis/debug/ageing_timeout", NULL );
    remove_proc_entry( "basis/debug/bmu_occupied_buffers", NULL );
    remove_proc_entry( "basis/debug/far_end_loopback", NULL );
    remove_proc_entry( "basis/debug/host_system_loopback", NULL );
    remove_proc_entry( "basis/debug/jumbo_enable", NULL );
    remove_proc_entry( "basis/debug/mdio_speed", NULL );
    remove_proc_entry( "basis/debug/phy_test_mode", NULL );
    remove_proc_entry( "basis/debug/mcu_fw_version", NULL );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "remove debug\n" );
    remove_proc_entry( "basis/debug", NULL );

    remove_proc_entry( "basis/scheduler_settings/port1_schedulers",  NULL );
    remove_proc_entry( "basis/scheduler_settings/port2_schedulers",  NULL );
    remove_proc_entry( "basis/scheduler_settings/port3_schedulers",  NULL );
    remove_proc_entry( "basis/scheduler_settings/port4_schedulers",  NULL );
    remove_proc_entry( "basis/scheduler_settings/port5_schedulers",  NULL );
    remove_proc_entry( "basis/scheduler_settings/port6_schedulers",  NULL );
    remove_proc_entry( "basis/scheduler_settings/port7_schedulers",  NULL );
    remove_proc_entry( "basis/scheduler_settings/port8_schedulers",  NULL );
    remove_proc_entry( "basis/scheduler_settings/port9_schedulers",  NULL );
    remove_proc_entry( "basis/scheduler_settings/port10_schedulers", NULL );
    remove_proc_entry( "basis/scheduler_settings/port11_schedulers", NULL );
    remove_proc_entry( "basis/scheduler_settings/port12_schedulers", NULL );
    remove_proc_entry( "basis/scheduler_settings/port13_schedulers", NULL );
    remove_proc_entry( "basis/scheduler_settings/port14_schedulers", NULL );
    remove_proc_entry( "basis/scheduler_settings/port15_schedulers", NULL );
    remove_proc_entry( "basis/scheduler_settings/port16_schedulers", NULL );
    remove_proc_entry( "basis/scheduler_settings/host_port_schedulers", NULL );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "remove scheduler_settings\n" );
    remove_proc_entry( "basis/scheduler_settings", NULL );

    remove_proc_entry( "basis/shaper_settings/port1_shapers",  NULL );
    remove_proc_entry( "basis/shaper_settings/port2_shapers",  NULL );
    remove_proc_entry( "basis/shaper_settings/port3_shapers",  NULL );
    remove_proc_entry( "basis/shaper_settings/port4_shapers",  NULL );
    remove_proc_entry( "basis/shaper_settings/port5_shapers",  NULL );
    remove_proc_entry( "basis/shaper_settings/port6_shapers",  NULL );
    remove_proc_entry( "basis/shaper_settings/port7_shapers",  NULL );
    remove_proc_entry( "basis/shaper_settings/port8_shapers",  NULL );
    remove_proc_entry( "basis/shaper_settings/port9_shapers",  NULL );
    remove_proc_entry( "basis/shaper_settings/port10_shapers", NULL );
    remove_proc_entry( "basis/shaper_settings/port11_shapers", NULL );
    remove_proc_entry( "basis/shaper_settings/port12_shapers", NULL );
    remove_proc_entry( "basis/shaper_settings/port13_shapers", NULL );
    remove_proc_entry( "basis/shaper_settings/port14_shapers", NULL );
    remove_proc_entry( "basis/shaper_settings/port15_shapers", NULL );
    remove_proc_entry( "basis/shaper_settings/port16_shapers", NULL );
    remove_proc_entry( "basis/shaper_settings/host_port_shapers", NULL );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "remove shaper_settings\n" );
    remove_proc_entry( "basis/shaper_settings", NULL );

    remove_proc_entry( "basis/dos_control/dos_ctrl", NULL );
    remove_proc_entry( "basis/dos_control", NULL );

    remove_proc_entry( "basis/private_vlan/port1_pvlan",  NULL );
    remove_proc_entry( "basis/private_vlan/port2_pvlan",  NULL );
    remove_proc_entry( "basis/private_vlan/port3_pvlan",  NULL );
    remove_proc_entry( "basis/private_vlan/port4_pvlan",  NULL );
    remove_proc_entry( "basis/private_vlan/port5_pvlan",  NULL );
    remove_proc_entry( "basis/private_vlan/port6_pvlan",  NULL );
    remove_proc_entry( "basis/private_vlan/port7_pvlan",  NULL );
    remove_proc_entry( "basis/private_vlan/port8_pvlan",  NULL );
    remove_proc_entry( "basis/private_vlan/port9_pvlan",  NULL );
    remove_proc_entry( "basis/private_vlan/port10_pvlan", NULL );
    remove_proc_entry( "basis/private_vlan/port11_pvlan", NULL );
    remove_proc_entry( "basis/private_vlan/port12_pvlan", NULL );
    remove_proc_entry( "basis/private_vlan/port13_pvlan", NULL );
    remove_proc_entry( "basis/private_vlan/port14_pvlan", NULL );
    remove_proc_entry( "basis/private_vlan/port15_pvlan", NULL );
    remove_proc_entry( "basis/private_vlan/port16_pvlan", NULL );
    remove_proc_entry( "basis/private_vlan/host_port_pvlan", NULL );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "remove private_vlan\n" );
    remove_proc_entry( "basis/private_vlan", NULL );

    remove_proc_entry( "basis/pcks_to_queues_mapping/port1_pcks_to_queues_map",  NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port2_pcks_to_queues_map",  NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port3_pcks_to_queues_map",  NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port4_pcks_to_queues_map",  NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port5_pcks_to_queues_map",  NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port6_pcks_to_queues_map",  NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port7_pcks_to_queues_map",  NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port8_pcks_to_queues_map",  NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port9_pcks_to_queues_map",  NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port10_pcks_to_queues_map", NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port11_pcks_to_queues_map", NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port12_pcks_to_queues_map", NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port13_pcks_to_queues_map", NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port14_pcks_to_queues_map", NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port15_pcks_to_queues_map", NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/port16_pcks_to_queues_map", NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/host_port_pcks_to_queues_map", NULL );
    remove_proc_entry( "basis/pcks_to_queues_mapping/global_dscp_pcks_to_queues_map", NULL );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "remove pcks_to_queues_mapping\n" );
    remove_proc_entry( "basis/pcks_to_queues_mapping", NULL );

    remove_proc_entry( "basis/flow_control/port1_fc",  NULL );
    remove_proc_entry( "basis/flow_control/port2_fc",  NULL );
    remove_proc_entry( "basis/flow_control/port3_fc",  NULL );
    remove_proc_entry( "basis/flow_control/port4_fc",  NULL );
    remove_proc_entry( "basis/flow_control/port5_fc",  NULL );
    remove_proc_entry( "basis/flow_control/port6_fc",  NULL );
    remove_proc_entry( "basis/flow_control/port7_fc",  NULL );
    remove_proc_entry( "basis/flow_control/port8_fc",  NULL );
    remove_proc_entry( "basis/flow_control/port9_fc",  NULL );
    remove_proc_entry( "basis/flow_control/port10_fc", NULL );
    remove_proc_entry( "basis/flow_control/port11_fc", NULL );
    remove_proc_entry( "basis/flow_control/port12_fc", NULL );
    remove_proc_entry( "basis/flow_control/port13_fc", NULL );
    remove_proc_entry( "basis/flow_control/port14_fc", NULL );
    remove_proc_entry( "basis/flow_control/port15_fc", NULL );
    remove_proc_entry( "basis/flow_control/port16_fc", NULL );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "remove flow_control\n" );
    remove_proc_entry( "basis/flow_control", NULL );

    remove_proc_entry( "basis/ports_ethernet_phy_settings/port1_eth_phy_settings",  NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port2_eth_phy_settings",  NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port3_eth_phy_settings",  NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port4_eth_phy_settings",  NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port5_eth_phy_settings",  NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port6_eth_phy_settings",  NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port7_eth_phy_settings",  NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port8_eth_phy_settings",  NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port9_eth_phy_settings",  NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port10_eth_phy_settings", NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port11_eth_phy_settings", NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port12_eth_phy_settings", NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port13_eth_phy_settings", NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port14_eth_phy_settings", NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port15_eth_phy_settings", NULL );
    remove_proc_entry( "basis/ports_ethernet_phy_settings/port16_eth_phy_settings", NULL );

    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "remove ports_ethernet_phy_settings\n" );
    remove_proc_entry( "basis/ports_ethernet_phy_settings", NULL );

    remove_proc_entry( "basis/sfp_modules/sfp_port1", NULL );
    remove_proc_entry( "basis/sfp_modules/sfp_port2", NULL );
    remove_proc_entry( "basis/sfp_modules/sfp_port3", NULL );
    remove_proc_entry( "basis/sfp_modules/sfp_port4", NULL );

    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "remove sfp_modules\n" );
    remove_proc_entry( "basis/sfp_modules", NULL );

    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_DEBUG, "remove basis\n" );
    remove_proc_entry( "basis", NULL );
    FP_DEBUG( DBGP_FEAT_PROCFS, FP_LOG_INFO, "proc FS deinit end\n" );

    return 0;
}
